/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.biblioteca.util;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author P_6414
 *
 */
public class CaminhoAbsoluto
{
    /**
     * Logger for this class
     */
    private static final Log log = LogFactory.getLog(CaminhoAbsoluto.class.getName());

    private static ClassLoader obterClassLoader()
    {
        if (log.isDebugEnabled())
        {
            log.debug("obterClassLoader() - start");
        }

        ClassLoader returnClassLoader = Thread.currentThread().getContextClassLoader();
        if (log.isDebugEnabled())
        {
            log.debug("obterClassLoader() - end - return value=" + returnClassLoader);
        }
        return returnClassLoader;
    }

    public static String obterReferencias(String strDestino)
    {
        if (log.isDebugEnabled())
        {
            log.debug("obterReferencias(String=" + strDestino + ") - start");
        }

        String caminho = null;

        ClassLoader objClassloader = obterClassLoader();

        Enumeration<URL> objURLcaminhos = null;

        try
        {
            objURLcaminhos = objClassloader.getResources(strDestino);
        }
        catch (IOException io)
        {
            log.error("obterReferencias(String)", io);

            // Pacincia
        }

        if (objURLcaminhos != null)
        {
            StringBuilder strCaminho = new StringBuilder();
            while (objURLcaminhos.hasMoreElements())
            {
                URL objURLCaminho = (URL) objURLcaminhos.nextElement();
                String strCaminhoAux = obter(objURLCaminho);
                if (strCaminhoAux != null)
                {
                    strCaminho.append(strCaminhoAux);
                    strCaminho.append(File.pathSeparator);
                }
            }
            if (!"".equals(strCaminho.toString()))
            {
                strCaminho.deleteCharAt(strCaminho.length() - File.pathSeparator.length());
                caminho = strCaminho.toString();
            }

        }

        if (log.isDebugEnabled())
        {
            log.debug("obterReferencias(String=" + strDestino + ") - end - return value=" + caminho);
        }

        return caminho;
    }

    public static String obter(String strDestino)
    {
        if (log.isDebugEnabled())
        {
            log.debug("obter(String=" + strDestino + ") - start");
        }

        String strCaminho = null;

        URL objURLCaminho = null;
        ClassLoader objClassloader = obterClassLoader();
        objURLCaminho = objClassloader.getResource(strDestino);
        if (objURLCaminho == null)
        {
            // Tentar com File
            File destino = new File(strDestino);
            try
            {
                objURLCaminho = destino.toURL();
            }
            catch (MalformedURLException e)
            {
                // Pacincia, ser tratado mais abaixo...
            }
        }

        // O destino foi encontrado ?
        if (objURLCaminho != null)
        {
            strCaminho = obter(objURLCaminho);
        }

        if (log.isDebugEnabled())
        {
            log.debug("obter(String=" + strDestino + ") - end - return value=" + strCaminho);
        }
        return strCaminho;
    }

    /**
     * 
     * @return Devolve sempre uma lista, mesmo que vazia, mas nunca null
     */
    private static List<URL> obterUrlsQueContem(String conteudoQueDeveTer, List<URL> urlRecursos)
    {
        List<URL> encontrados = new ArrayList<URL>();

        if (urlRecursos != null)
        {
            for (URL urlEncontrado : urlRecursos)
            {
                if (urlEncontrado.getPath().contains(conteudoQueDeveTer))
                {
                    encontrados.add(urlEncontrado);
                }
            }
        }

        return encontrados;
    }

    /**
     * 
     * @return Devolve sempre uma lista, mesmo que vazia, mas nunca null
     */
    public static List<URL> obterTodosRecursosDoClassPath()
    {
        ClassLoader objClassloader = obterClassLoader();

        List<URL> urlRecursos = new ArrayList<URL>();
        try
        {
            Enumeration<URL> enumUrlRecursos = objClassloader.getResources("");
            if (enumUrlRecursos.hasMoreElements())
            {
                while (enumUrlRecursos.hasMoreElements())
                {
                    urlRecursos.add(enumUrlRecursos.nextElement());
                }
            }
        }
        catch (IOException e)
        {
            // Paciencia...
        }

        return urlRecursos;
    }

    private static String obter(URL urlDestino)
    {
        if (log.isDebugEnabled())
        {
            log.debug("obter(URL=" + urlDestino + ") - start");
        }

        String strCaminho = null;

        // O destino foi encontrado ?
        if (urlDestino != null)
        {
            File filDestino = new File(urlDestino.getPath());
            if (filDestino.exists())
            {
                strCaminho = obter(filDestino);
            }
        }

        if (log.isDebugEnabled())
        {
            log.debug("obter(URL=" + urlDestino + ") - end - return value=" + strCaminho);
        }
        return strCaminho;
    }

    private static String obter(File filDestino)
    {
        if (log.isDebugEnabled())
        {
            log.debug("obter(File filDestino=" + filDestino + ") - start");
        }

        String strCaminho = null;

        try
        {
            strCaminho = filDestino.getCanonicalPath();
            if (log.isDebugEnabled())
            {
                log.debug("obter(File) - getAbsolutePath()  = " + filDestino.getAbsolutePath());
                log.debug("obter(File) - getCanonicalPath() = " + filDestino.getCanonicalPath());
            }
        }
        catch (IOException ioe)
        {
            // Ocorreu um erro... Pacincia...
            log.error("Ocorreu um erro ao tentar obter o caminho Canonical para o arquivo " + filDestino.toString());
        }

        if (log.isDebugEnabled())
        {
            log.debug("obter(File filDestino=" + filDestino + ") - end - return value=" + strCaminho);
        }
        return strCaminho;
    }

    public static String obterArquivos(String strDiretorio, String strFiltro)
    {
        if (log.isDebugEnabled())
        {
            log.debug("obterArquivos(String strDiretorio=" + strDiretorio + ", String strFiltro=" + strFiltro + ") - start");
        }

        String strCaminhoRetorno = null;

        String strCaminho = obter(strDiretorio);
        if (strCaminho != null)
        {
            File filDiretorio = new File(strCaminho);
            if (filDiretorio.isDirectory())
            {
                StringBuilder strCaminhoRetornoTemp = new StringBuilder();

                File[] filArquivos = filDiretorio.listFiles();
                for (int i = 0; i < filArquivos.length; i++)
                {
                    try
                    {
                        String strCaminhoCanonical = filArquivos[i].getCanonicalPath();
                        if (strFiltro == null || strCaminhoCanonical.indexOf(strFiltro) > -1)
                        {
                            strCaminhoRetornoTemp.append(strCaminhoCanonical);
                            strCaminhoRetornoTemp.append("/");
                        }
                    }
                    catch (IOException ioe)
                    {
                        log.error("obterArquivos(String, String)", ioe);

                        // Ocorreu um erro recuperando o caminho canonical especificado. Pacincia...
                    }
                }

                if (!"".equals(strCaminhoRetornoTemp.toString()))
                {
                    strCaminhoRetorno = strCaminhoRetornoTemp.deleteCharAt(strCaminhoRetornoTemp.length() - "/".length()).toString();
                }

            }
        }

        if (log.isDebugEnabled())
        {
            log.debug("obterArquivos(String strDiretorio=" + strDiretorio + ", String strFiltro=" + strFiltro + ") - end - return value=" + strCaminhoRetorno);
        }
        return strCaminhoRetorno;
    }

    public static File obterAPartirDoClasses(String strLocalizacao)
    {
        String[] caminhosDesejados = new String[] { "classes", "WEB-INF/config" }; // "WEB-INF/classes"

        String strLocalizacaoTratada = strLocalizacao.replaceAll("\\\\", "/");

        String[] partesDoCaminhoRelativoParaArtefato = null;
        partesDoCaminhoRelativoParaArtefato = strLocalizacaoTratada.split("/");

        File arquivoEncontrado = null;

        List<URL> recursosNoClasspath = obterTodosRecursosDoClassPath();

        if (!recursosNoClasspath.isEmpty())
        {
            for (int i = 0; i < caminhosDesejados.length; i++)
            {
                List<URL> caminhosClasses = obterUrlsQueContem(caminhosDesejados[i], recursosNoClasspath);
                if (!caminhosClasses.isEmpty())
                {
                    for (URL caminhoAbsolutoInicial : caminhosClasses)
                    {
                        String caminhoRelatorios = montarCaminhoAbsolutoProvavelAPartirDoCaminhoRelativo(partesDoCaminhoRelativoParaArtefato, caminhoAbsolutoInicial);

                        arquivoEncontrado = recuperarArquivoSeExiste(caminhoRelatorios);
                        if (arquivoEncontrado != null)
                        {
                            break;
                        }
                    }
                    if (arquivoEncontrado != null)
                    {
                        break;
                    }
                }
            }
        }

        return arquivoEncontrado;
    }

    private static File recuperarArquivoSeExiste(String caminhoRelatorios)
    {
        File arquivoEncontrado;
        arquivoEncontrado = new File(caminhoRelatorios);
        if (!arquivoEncontrado.exists())
        {
            arquivoEncontrado = null;
        }
        return arquivoEncontrado;
    }

    private static String montarCaminhoAbsolutoProvavelAPartirDoCaminhoRelativo(String[] partesDoCaminhoRelativoParaArtefato, URL caminhoAbsolutoInicial)
    {
        List<String> novoClassPath = new ArrayList<String>(Arrays.asList(caminhoAbsolutoInicial.getPath().split("/")));

        // Processar eventuais ".." na localizao especificada e montar o caminho final
        for (int j = 0; partesDoCaminhoRelativoParaArtefato.length > j; j++)
        {
            if ("..".equals(partesDoCaminhoRelativoParaArtefato[j]))
            {
                novoClassPath.remove(novoClassPath.size() - 1);
            }
            else if (!".".equals(partesDoCaminhoRelativoParaArtefato[j]))
            {
                novoClassPath.add(partesDoCaminhoRelativoParaArtefato[j]);
            }
        }

        String caminhoRelatorios = novoClassPath.toString().replaceAll(", ", "/");
        caminhoRelatorios = caminhoRelatorios.substring(1, caminhoRelatorios.length() - 1);
        return caminhoRelatorios;
    }
}
