/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.biblioteca.util;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import br.gov.camara.exception.CDException;

/**
 * Classe para poder obter cpia de objetos JavaBean
 */
public class Copia
{
    private Copia()
    {}

    public static void criar(Object objOriginal, Object objCopia) throws CDException
    {
        try
        {
            Class<? extends Object> classeOriginal = objOriginal.getClass();
            Class<? extends Object> classeCopia = objCopia.getClass();

            Method[] metodosDaClasseOriginal = classeOriginal.getMethods();

            boolean temMetodoValido = false;
            for (Method metodoDaClasseOriginal : metodosDaClasseOriginal)
            {
                if (metodoValidoParaCopia(metodoDaClasseOriginal))
                {
                    temMetodoValido = true;
                    Object valorRetornadoDoMetodoParaCopiar = metodoDaClasseOriginal.invoke(objOriginal, (Object[]) null);

                    String nomeMetodoParaAtribuir = "set" + nomeMetodoParaCopiarSemPrefixoGet(metodoDaClasseOriginal);

                    chamarMetodoParaAtribuicaoDoParametro(objCopia, nomeMetodoParaAtribuir, valorRetornadoDoMetodoParaCopiar);
                }
            }

            if (!temMetodoValido)
            {
                String mensagem = "O objeto '"
                        + objOriginal
                        + "' da classe '"
                        + classeOriginal
                        + "' no tem um mtodo vlido para cpia. Pode ser um erro... Analisar!";
                throw new CDException(mensagem);
            }
        }
        catch (Exception e)
        {
            CDException.dispararExcecao(e);
        }
    }

    private static void chamarMetodoParaAtribuicaoDoParametro(Object objeto, String nomeMetodo, Object parametro) throws ClassNotFoundException, CDException, IllegalAccessException, InvocationTargetException
    {
        if (parametro == null)
        {
            return;
        }

        Class classeDoParametro = obterAClasseDoObjeto(parametro);
        Class classeDoObjeto = obterAClasseDoObjeto(objeto);

        Method metodoDoObjetoCopiaParaAtribuirValorRetornado = obterMetodoDaClasseEComONomeInformadosConformeClasseDoParametro(classeDoObjeto, nomeMetodo, classeDoParametro);

        if (metodoDoObjetoCopiaParaAtribuirValorRetornado == null)
        {
            throw new CDException("No foi possvel encontrar um mtodo com o nome '"
                    + nomeMetodo
                    + "' para atribuir o objeto do tipo '"
                    + classeDoParametro.getName()
                    + "'.");
        }

        metodoDoObjetoCopiaParaAtribuirValorRetornado.invoke(objeto, new Object[] { parametro });
    }

    private static Method obterMetodoDaClasseEComONomeInformadosConformeClasseDoParametro(Class classe, String nomeDoMetodo, Class classeDoParametro)
    {
        Method metodoDoObjetoCopiaParaAtribuirObjetoRetornadoDoOriginal = null;

        // Primeiro tenta chamar o mtodo cuja cuja assinatura  classe em questo
        try
        {
            metodoDoObjetoCopiaParaAtribuirObjetoRetornadoDoOriginal = classe.getMethod(nomeDoMetodo, new Class[] { classeDoParametro });
        }
        catch (NoSuchMethodException nsme)
        {
            // Sero feitas tentativas diferentes abaixo
        }

        // Verifica se o mtodo na assinatura  um dos pais da classe original
        if (metodoDoObjetoCopiaParaAtribuirObjetoRetornadoDoOriginal == null)
        {
            classeDoParametro = classeDoParametro.getSuperclass();
            while (metodoDoObjetoCopiaParaAtribuirObjetoRetornadoDoOriginal == null && classeDoParametro != null)
            {
                try
                {
                    metodoDoObjetoCopiaParaAtribuirObjetoRetornadoDoOriginal = classe.getMethod(nomeDoMetodo, new Class[] { classeDoParametro });
                }
                catch (NoSuchMethodException nsme)
                {
                    classeDoParametro = classeDoParametro.getSuperclass();
                    // tentar novamente com a nova classe
                }
            }
        }

        return metodoDoObjetoCopiaParaAtribuirObjetoRetornadoDoOriginal;
    }

    private static String nomeMetodoParaCopiarSemPrefixoGet(Method metodoParaCopiar)
    {
        String nomeMetodoParaCopiar = metodoParaCopiar.getName();

        return nomeMetodoParaCopiar.substring(3, nomeMetodoParaCopiar.length());
    }

    private static Class obterAClasseDoObjeto(Object objetoRetornoDoMetodoParaCopiar) throws ClassNotFoundException
    {
        Class classeDoObjeto = obterClasseDoObjetoPrimitivo(objetoRetornoDoMetodoParaCopiar);

        if (classeDoObjeto == null)
        {
            classeDoObjeto = ajustarClasseDoObjeto(objetoRetornoDoMetodoParaCopiar);
        }

        if (classeDoObjeto != null)
        {
            classeDoObjeto = tratarClasseDeObjetoDeManipuladoPorBytecode(classeDoObjeto);
        }

        return classeDoObjeto;
    }

    private static Class tratarClasseDeObjetoDeManipuladoPorBytecode(Class classeDoObjeto) throws ClassNotFoundException
    {
        if (classeDoObjeto.toString().indexOf("_$$_") >= 0)
        {
            // JAVASSIST
            classeDoObjeto = Class.forName(classeDoObjeto.toString().substring(6, classeDoObjeto.toString().indexOf("_$$_")));
        }
        else if (classeDoObjeto.toString().indexOf('$') >= 0)
        {
            // CGLIB ?
            classeDoObjeto = Class.forName(classeDoObjeto.toString().substring(6, classeDoObjeto.toString().indexOf('$')));
        }

        return classeDoObjeto;
    }

    private static Class ajustarClasseDoObjeto(Object objeto) throws ClassNotFoundException
    {
        Class classeDoObjeto = objeto.getClass();
        String nomeClasse = classeDoObjeto.getName();

        if ("java.util.GregorianCalendar".equals(nomeClasse))
        {
            classeDoObjeto = Class.forName("java.util.Calendar");
        }
        else if ("net.sf.hibernate.collection.Set".equals(nomeClasse) || "org.hibernate.collection.PersistentSet".equals(nomeClasse))
        {
            classeDoObjeto = Class.forName("java.util.Set");
        }
        else if ("java.sql.Timestamp".equals(nomeClasse))
        {
            classeDoObjeto = Class.forName("java.util.Date");
        }
        else if (nomeClasse.contains("ArrayList"))
        {
            classeDoObjeto = Class.forName("java.util.List");
        }
        return classeDoObjeto;
    }

    private static Class obterClasseDoObjetoPrimitivo(Object objetoRetornoDoMetodoParaCopiar)
    {
        Class classeDoObjeto = null;

        if (objetoRetornoDoMetodoParaCopiar.getClass().isPrimitive())
        {
            if (objetoRetornoDoMetodoParaCopiar instanceof Boolean)
            {
                classeDoObjeto = Boolean.TYPE;
            }
            else if (objetoRetornoDoMetodoParaCopiar instanceof Character)
            {
                classeDoObjeto = Character.TYPE;
            }
            else if (objetoRetornoDoMetodoParaCopiar instanceof Byte)
            {
                classeDoObjeto = Byte.TYPE;
            }
            else if (objetoRetornoDoMetodoParaCopiar instanceof Short)
            {
                classeDoObjeto = Short.TYPE;
            }
            else if (objetoRetornoDoMetodoParaCopiar instanceof Integer)
            {
                classeDoObjeto = Integer.TYPE;
            }
            else if (objetoRetornoDoMetodoParaCopiar instanceof Long)
            {
                classeDoObjeto = Long.TYPE;
            }
            else if (objetoRetornoDoMetodoParaCopiar instanceof Float)
            {
                classeDoObjeto = Float.TYPE;
            }
            else if (objetoRetornoDoMetodoParaCopiar instanceof Double)
            {
                classeDoObjeto = Double.TYPE;
            }
        }

        return classeDoObjeto;
    }

    private static boolean metodoValidoParaCopia(Method metodoDaClasse)
    {
        String[] metodosIniciadosComGetQueNaoDevemSerTratados = new String[] { "getClass", "getCallback", "getChaveEntidade", "getHandler", "getHibernate", "getBytes" };

        if (metodoDaClasse == null)
        {
            return false;
        }

        Class[] tiposDeParametrosDoMetodo = metodoDaClasse.getParameterTypes();

        if (tiposDeParametrosDoMetodo.length > 0)
        {
            return false;
        }

        String nomeMetodoParaCopiar = metodoDaClasse.getName();
        if (nomeMetodoParaCopiar == null || "".equals(nomeMetodoParaCopiar))
        {
            return false;
        }

        if (!nomeMetodoParaCopiar.startsWith("get"))
        {
            return false;
        }

        for (String nomeMetodoIgnorar : metodosIniciadosComGetQueNaoDevemSerTratados)
        {
            if (nomeMetodoParaCopiar.startsWith(nomeMetodoIgnorar))
            {
                return false;
            }
        }

        return true;

    }
}
