/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.biblioteca.util;

/**
 * Classe responsvel por retornar campos no formatdo HTML
 *
 * @author P_999326
 *
 */
public class HtmlUtil
{
    public static final String NEW_LINE = "\n";
    public static final String CARIAGE_RETURN_NEW_LINE = "\r\n";
    public static final String ESPACO = " ";
    public static final String HTML_BREAK = "<BR>";
    public static final String HTML_QUOTE = "&quot;";
    public static final String HTML_LESS_THAN = "&lt;";
    public static final String HTML_GREATER_THAN = "&gt;";
    public static final String HTML_NON_BREAKING_SPACE = "&nbsp;";

    private static final String arrScriptInjection[][] = { { ">", HTML_GREATER_THAN }, { "<", HTML_LESS_THAN } };

    public static String nbspSeNulo(Object objeto)
    {
        return StringUtil.seNulo(objeto, HTML_NON_BREAKING_SPACE);
    }

    /**
     * Substitui a quebra de linha do Java "\n" por "<BR>" que  quebra de linha do HTML
     * Substitui os primeiros espaos em branco da linha por "&nbsp;"
     * Dessa forma o texto no distorce no formato HTML
     *
     * @param texto
     *
     * @return texto formatado em HTML
     */
    public static String formatarTextoEmHtml(String texto)
    {
        StringBuilder strRetorno = new StringBuilder();

        if (texto == null)
        {
            //pode ser q o Obj String esteja null
            texto = "";
        }
        else
        {
            // substituindo o sinal de  > por uma tag(html) &gt;
            texto = texto.replaceAll(">", HTML_GREATER_THAN);

            // substituindo o sinal de  < por uma tag(html) &lt;
            //texto = texto.replaceAll("<", "&lt;");

            // substituindo aspas simples ' por aspas como tag(html)
            if (texto.indexOf("'") >= 0)
            {
                texto = texto.replaceAll("'", "\\\\'");
            }

            // substituindo aspas " por aspas duplas como tag(html) &quot;
            if (texto.indexOf("\"") >= 0)
            {
                texto = texto.replaceAll("\"", HTML_QUOTE);
            }
        }

        if (texto.indexOf(NEW_LINE) > 0)
        {
            // substituindo a quebra de linha do Java por uma tag <BR>
            texto = texto.replaceAll(CARIAGE_RETURN_NEW_LINE, HTML_BREAK);

            //recuperando as linhas do texto
            String linhasTexto[] = texto.split(HTML_BREAK);

            //percorrendo todas as linhas
            for (int i = 0; i < linhasTexto.length; i++)
            {
                String linha = linhasTexto[i];

                //tranformando a linha em um array de caracter
                String espacao[] = linha.split(ESPACO);

                StringBuilder aux = new StringBuilder();

                //percorrendo os caracteres da linha
                for (int j = 0; j < espacao.length; j++)
                {
                    String caracter = espacao[j];

                    //enquanto os primeiros caracter for espao em branco coloca-se "&nbsp;" 
                    if ("".equals(caracter))
                    {
                        aux.append(HTML_NON_BREAKING_SPACE);
                    }
                    else
                    {
                        break;
                    }

                }

                strRetorno.append(aux.toString());
                strRetorno.append(linha);
                strRetorno.append(HTML_BREAK);
            }
        }
        else
        {
            strRetorno.append(texto);
        }
        return strRetorno.toString();
    }

    /**
     * Realiza substituies de texto e de caracteres '>' e '<' em texto html 
     * para evitar problema de script injection 
     * @param texto - string a ser convertida
     * @return string com o texto substituido
     */

    public static String evitarScriptInjection(String texto)
    {
        String strRetorno = texto;
        for (int i = 0; i < arrScriptInjection.length; i++)
        {
            // Efetua as substituies
            strRetorno = strRetorno.replaceAll(arrScriptInjection[i][0], arrScriptInjection[i][1]);
        }

        return strRetorno;
    }

    /**
     * Realiza substituies de texto e desfaz as alteraes de caracteres '>' e '<' em texto html 
     * @param texto - string a ser convertida
     * @return string com o texto substituido
     */

    public static String desfazerAlteracoesScriptInjection(String texto)
    {
        String strRetorno = texto;
        for (int i = 0; i < arrScriptInjection.length; i++)
        {
            // substituindo o sinal de '&gt;' por uma tag(html) '>' 
            strRetorno = strRetorno.replaceAll(arrScriptInjection[i][1], arrScriptInjection[i][0]);
        }
        return strRetorno;
    }

    /**
     * Prepara a String para ser utilizada 
     *
     * @param texto 
     *
     * @return texto formatado em HTML
     */
    public static String formatarTextoParaJavascript(String texto)
    {
        StringBuilder strRetorno = new StringBuilder();

        if (texto == null)
        {
            //pode ser q o Obj String esteja null
            texto = "";
        }
        else
        {
            // substituindo o sinal de  > por uma tag(html) &gt;
            texto = texto.replaceAll(">", HTML_GREATER_THAN);

            // substituindo o sinal de  < por uma tag(html) &lt;
            //texto = texto.replaceAll("<", "&lt;");

            // substituindo aspas simples ' por aspas como tag(html)
            if (texto.indexOf("'") >= 0)
            {
                texto = texto.replaceAll("'", "\\\\'");
            }

            // substituindo aspas " por aspas duplas como tag(html) &quot;
            if (texto.indexOf("\"") >= 0)
            {
                texto = texto.replaceAll("\"", HTML_QUOTE);
            }
        }

        if (texto.indexOf(NEW_LINE) > 0)
        {
            // substituindo a quebra de linha do Java por uma tag <BR>
            texto = texto.replaceAll(CARIAGE_RETURN_NEW_LINE, HTML_BREAK);

            //recuperando as linhas do texto
            String linhasTexto[] = texto.split(HTML_BREAK);

            //percorrendo todas as linhas
            for (int i = 0; i < linhasTexto.length; i++)
            {
                String linha = linhasTexto[i];

                //tranformando a linha em um array de caracter
                String espacao[] = linha.split(ESPACO);

                StringBuilder aux = new StringBuilder();

                //percorrendo os caracteres da linha
                for (int j = 0; j < espacao.length; j++)
                {
                    String caracter = espacao[j];

                    //enquanto os primeiros caracter for espao em branco coloca-se "&nbsp;" 
                    if ("".equals(caracter))
                    {
                        aux.append(HTML_NON_BREAKING_SPACE);
                    }
                    else
                    {
                        break;
                    }

                }

                strRetorno.append(aux);
                strRetorno.append(linha);
                strRetorno.append(HTML_BREAK);
            }
        }
        else
        {
            strRetorno.append(texto);
        }
        return strRetorno.toString();
    }
}
