/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.exception;

import java.text.DecimalFormat;
import java.util.Random;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Classe padro de excees
 */
public class CDException extends Exception
{
    private Log log = LogFactory.getLog(CDException.class);

    public static final String ERRO_INESPERADO = "Ocorreu um erro inesperado.";

    private static final long serialVersionUID = 1661419714317374637L;

    private static final int MAX_NIVEIS_PARA_TENTAR_INTERPRETAR_CAUSA_ERRO = 3;

    private int idErroCDException = 0;
    private String mensagemUsuario = null;

    public enum NivelLog
    {
        DEBUG, INFO, WARN, ERROR
    }

    /**
     * Cria um novo objeto CDException
     */
    public CDException()
    {
        super(ERRO_INESPERADO);
        gerarLog();
    }

    /**
     * Cria um novo objeto CDException
     * 
     * 
     */
    public CDException(boolean noLog)
    {
        super(ERRO_INESPERADO);
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     */
    public CDException(String strMensagem)
    {
        super(strMensagem);
        gerarLog();
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     */
    public CDException(String strMensagem, int quantidadeStackTrace, NivelLog nivelLog)
    {
        super(strMensagem);
        gerarLog(quantidadeStackTrace, nivelLog);
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * 
     */
    public CDException(String strMensagem, boolean noLog)
    {
        super(strMensagem);
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param strMensagemUsuario Mensagem para o usurio
     */
    public CDException(String strMensagem, String strMensagemUsuario)
    {
        super(strMensagem);
        this.mensagemUsuario = strMensagemUsuario;
        gerarLog();
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param strMensagemUsuario Mensagem para o usurio
     * 
     */
    public CDException(String strMensagem, String strMensagemUsuario, boolean noLog)
    {
        super(strMensagem);
        this.mensagemUsuario = strMensagemUsuario;
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param cause Causa da exceo
     */
    public CDException(Throwable cause)
    {
        super(getMensagem(cause), cause);
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
            mensagemUsuario = ((CDException) cause).getMensagemUsuario();
        }
        gerarLog();
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param cause Causa da exceo
     * 
     */
    public CDException(Throwable cause, boolean noLog)
    {
        super(getMensagem(cause), cause);
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
            mensagemUsuario = ((CDException) cause).getMensagemUsuario();
        }
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param cause Causa da exceo
     */
    public CDException(String strMensagem, Throwable cause)
    {
        super(strMensagem, cause);
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
            mensagemUsuario = ((CDException) cause).getMensagemUsuario();
        }
        gerarLog();
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param cause Causa da exceo
     * 
     */
    public CDException(String strMensagem, Throwable cause, boolean noLog)
    {
        super(strMensagem, cause);
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
            mensagemUsuario = ((CDException) cause).getMensagemUsuario();
        }
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param cause Causa da exceo
     */
    public CDException(String strMensagem, Throwable cause, String strMensagemUsuario)
    {
        super(strMensagem, cause);
        mensagemUsuario = strMensagemUsuario;
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
        }
        gerarLog();
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param cause Causa da exceo
     * 
     */
    public CDException(String strMensagem, Throwable cause, String strMensagemUsuario, boolean noLog)
    {
        super(strMensagem, cause);
        mensagemUsuario = strMensagemUsuario;
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
        }
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param cause Causa da exceo
     */
    public CDException(Throwable cause, String strMensagemUsuario)
    {
        super(getMensagem(cause), cause);
        mensagemUsuario = strMensagemUsuario;
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
        }
        gerarLog();
    }

    /**
     * Cria um novo objeto CDException
     *
     * @param strMensagem Mensagem da exceo
     * @param cause Causa da exceo
     * 
     */
    public CDException(Throwable cause, String strMensagemUsuario, boolean noLog)
    {
        super(getMensagem(cause), cause);
        mensagemUsuario = strMensagemUsuario;
        if (cause instanceof CDException)
        {
            idErroCDException = ((CDException) cause).getIdentificadorErro();
        }
    }

    /**
     * Gera mensagem de log
     *
     */
    private void gerarLog()
    {
        gerarLog(5, NivelLog.ERROR);
    }

    /**
     * Gera mensagem de log
     *
     */
    private void gerarLog(int quantidadeStackTrace, NivelLog nivelLog)
    {
        // Gera log
        if (log.isErrorEnabled())
        {
            DecimalFormat dcfFormato = new DecimalFormat("000000");

            // Somente gerar um novo identificador, caso no tenha sido gerado um ainda.
            if (idErroCDException == 0)
            {
                Random objRandom = new Random();
                idErroCDException = 1 + objRandom.nextInt(999999);
            }

            StackTraceElement[] steException = getStackTrace();
            int i;
            for (i = 0; i < steException.length; i++)
            {
                if (i > quantidadeStackTrace)
                {
                    break;
                }
                else
                {
                    logar(dcfFormato.format(idErroCDException) + " | ST: " + steException[i], nivelLog);
                }

            }
            logar(dcfFormato.format(idErroCDException)
                    + " | GM: "
                    + (i > quantidadeStackTrace ? "(...) " : "")
                    + getMensagemParaLog(this)
                    + " | "
                    + getClass(), nivelLog);
        }
    }

    private void logar(String mensagem, NivelLog nivel)
    {
        switch (nivel)
        {
            case DEBUG:
                log.debug(mensagem);
                break;

            case INFO:
                log.info(mensagem);
                break;

            case WARN:
                log.warn(mensagem);
                break;

            default:
                log.error(mensagem);
                break;
        }
    }

    /**
     * Mtodo getter para mensagem de usurio
     *
     * @return String contendo a mensagem para o usurio
     */
    protected String getMensagemUsuario()
    {
        return mensagemUsuario;
    }

    /**
     * Mtodo getter para identificador do erro
     *
     * @return Identificado do erro
     */
    public int getIdentificadorErro()
    {
        return idErroCDException;
    }

    /**
     * Obtm mensagem gerada para o usurio
     *
     * @return String contendo a mensagem
     * @deprecated Usar getMensagem
     */
    public String obterMensagem()
    {
        return getMensagem();
    }

    public String getMensagem()
    {
        return getMensagem(this);
    }

    protected static String getMensagem(Throwable throwable)
    {
        return getMensagemParaLog(throwable, false);
    }

    protected static String getMensagemParaLog(Throwable throwable)
    {
        return getMensagemParaLog(throwable, true);
    }

    private static String getMensagemParaLog(Throwable throwable, boolean paraLog)
    {
        StringBuilder mensagem = new StringBuilder();
        if (throwable instanceof CDException)
        {
            boolean temMensagemUsuario = false;

            if (((CDException) throwable).getMensagemUsuario() != null)
            {
                mensagem.append(((CDException) throwable).getMensagemUsuario());
                temMensagemUsuario = true;
            }
            else
            {
                mensagem.append(((CDException) throwable).getMessage());
            }

            if (((CDException) throwable).getIdentificadorErro() != 0 && !paraLog)
            {
                mensagem.append(" Ocorrncia #");
                mensagem.append(Integer.toString(((CDException) throwable).getIdentificadorErro()));
                mensagem.append(".");
            }

            if (paraLog && temMensagemUsuario)
            {
                mensagem.append(" CAUSA: ");
                mensagem.append(throwable.getMessage());
                mensagem.append(".");
            }
        }
        else
        {
            Throwable causaInterpretada = getCausaInterpretada(throwable, MAX_NIVEIS_PARA_TENTAR_INTERPRETAR_CAUSA_ERRO);
            mensagem.append(causaInterpretada.getMessage());
        }

        return mensagem.toString();
    }

    private static final Throwable getCausaInterpretada(Throwable causaInicial, int nivel)
    {
        // Validaes iniciais

        if (nivel == 0)
        {
            return causaInicial;
        }

        if (causaInicial.getCause() == null)
        {
            return causaInicial;
        }

        // Interpretar o nvel atual

        if (causaInicial.getClass().getName().startsWith("org.hibernate."))
        {
            // Devolver a causa do erro que gerou a mensagem do hibernate, se existir
            if (causaInicial.getCause() == null)
            {
                return causaInicial;
            }
            return causaInicial.getCause();
        }

        // Descer mais um nvel para tentar interpretar 

        int proximoNivel = nivel - 1;
        return getCausaInterpretada(causaInicial, proximoNivel);
    }

    /**
     * Obtm toda a pilha de excees ocorrida
     *
     * @return String com a mensagem da pilha
     * 
     */
    public String obterExcecoes()
    {
        StringBuilder strRetorno = new StringBuilder();
        strRetorno.append(getMensagem());
        strRetorno.append(" | ");

        Throwable throwable = getCause();
        while (throwable != null)
        {
            strRetorno.append(getMensagem(throwable));
            throwable = throwable.getCause();
            if (throwable != null)
            {
                strRetorno.append(" | ");
            }
        }

        return strRetorno.toString();
    }

    public static void dispararExcecao(Throwable e) throws CDException
    {
        if (e instanceof CDException)
        {
            throw (CDException) e;
        }
        else
        {
            throw new CDException(getMensagem(e), e);
        }
    }

    public static void dispararExcecao(String mensagem) throws CDException
    {
        throw new CDException(mensagem);
    }

    public static void dispararExcecao(String mensagem, int quantidadeStackTrace, NivelLog nivel) throws CDException
    {
        throw new CDException(mensagem, quantidadeStackTrace, nivel);
    }
}
