/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.hibernate;

import java.sql.Connection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.impl.SessionImpl;
import org.hibernate.jdbc.JDBCContext;
import org.hibernate.transform.ResultTransformer;

import br.gov.camara.hibernate.exception.HibernateExceptionCD;
import br.gov.camara.negocio.ParametroDAO;

class ConexaoJDBC
{
    /**
    * Logger for this class
    */
    private static final Log log = LogFactory.getLog(ConexaoJDBC.class);

    private Session sessao = null;

    ConexaoJDBC(Session sessao)
    {
        try
        {
            this.sessao = sessao;
        }
        catch (Exception e)
        {
            log.error("Ocorreu um erro recuperando a sesso hibernate: " + e.getMessage(), e);
        }
    }

    static void resolverFaltaAutocommit(Session sessaoHibernate)
    {
        try
        {
            JDBCContext contexto = ((SessionImpl) sessaoHibernate).getJDBCContext();

            Connection conexao = contexto.connection();

            if (!conexao.isClosed() && !conexao.getAutoCommit() && contexto.isTransactionInProgress())
            {
                conexao.commit();
            }
        }
        catch (Throwable sqle)
        {
            if (sqle.getMessage().contains("The user must supply a JDBC connection"))
            {
                log.warn("Ignorando o comando para fechar a conexo porque ela  aparentemente virtual: " + sqle.getMessage());
            }
            else
            {
                throw new HibernateExceptionCD(sqle);
            }
        }
    }

    int executar(String comandoSql)
    {
        return executarComParametros(comandoSql, null);
    }

    int executarComParametros(String comandoSql, ParametroDAO parametros)
    {
        Query querySql = criarQuerySQL(comandoSql, parametros, null);

        return querySql.executeUpdate();
    }

    List recuperar(String comandoSql)
    {
        return recuperarComParametrosPorPagina(0, 0, comandoSql, null, AliasToMapResultTransformer.INSTANCE);
    }

    List recuperar(String comandoSql, ResultTransformer transformadorDeResultado)
    {
        return recuperarComParametrosPorPagina(0, 0, comandoSql, null, transformadorDeResultado);
    }

    List recuperarComParametros(String comandoSql, ParametroDAO parametros)
    {
        return recuperarComParametrosPorPagina(0, 0, comandoSql, parametros, AliasToMapResultTransformer.INSTANCE);
    }

    List recuperarComParametros(String comandoSql, ParametroDAO parametros, ResultTransformer transformadorDeResultado)
    {
        return recuperarComParametrosPorPagina(0, 0, comandoSql, parametros, transformadorDeResultado);
    }

    List recuperarComParametrosPorPagina(int intNumeroPagina, int intMaximoPagina, String comandoSql, ParametroDAO parametros)
    {
        return recuperarComParametrosPorPagina(intNumeroPagina, intMaximoPagina, comandoSql, parametros, AliasToMapResultTransformer.INSTANCE);
    }

    List recuperarComParametrosPorPagina(int intNumeroPagina, int intMaximoPagina, String comandoSql, ParametroDAO parametros, ResultTransformer transformadorDeResultado)
    {
        Query querySql = criarQuerySQL(comandoSql, parametros, transformadorDeResultado);

        if (intMaximoPagina > 0)
        {
            querySql.setMaxResults(intMaximoPagina);
            if (intNumeroPagina < 1)
            {
                intNumeroPagina = 1;
            }
            querySql.setFirstResult((intNumeroPagina - 1) * intMaximoPagina);
        }

        return querySql.list();
    }

    private Query criarQuerySQL(String comandoSql, ParametroDAO parametros, ResultTransformer transformadorDeResultado)
    {
        Query querySql = sessao.createSQLQuery(comandoSql);

        if (parametros != null && !parametros.isEmpty())
        {
            querySql = parametros.setarParametroQuery(querySql);
        }

        if (transformadorDeResultado != null)
        {
            querySql.setResultTransformer(transformadorDeResultado);
        }

        return querySql;
    }
}
