/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.hibernate;

import java.util.ArrayList;
import java.util.List;

/**
 * Camara dos Deputados
 * Projeto: HibernateUtil
 * Data Criao: 08/10/2008
 * Breve Descrio:  Classe responsavel em retornar uma implementacao 
 *                   da FacadeChamadora
* 
* @author p_6987
* 
* */
public abstract class DelimitadoraSessaoTransacao
{
    // Inicia a procura na 3a posio, que  onde, no mais cedo, estar a assinatura procurada 
    private static final int POSICAO_PESQUISA_INICIAL = 3;

    /**
     * Retorna a assinatura da primeira classe/mtodo que chamou um DAO de acordo com a nomenclatura definida pela Delimitadora.
     *  necessrio que a nomenclatura seja respeitada devido aos inicioTransao que podem ser chamados antes das Facades
     * (exemplo testes unitrios, onde a BaseTestCase inicia a transao, mas a primeira classe/mtodo antes do DAO  uma Facade).
     * @return
     */
    public final String obterAssinaturaOperadorSessaoTransacao()
    {
        StackTraceElement[] stackTrace = obterElementosStackTrace();
        String assinatura = null;

        int posicao = POSICAO_PESQUISA_INICIAL;
        posicao = encontraPosicaoClasseDelimitadora(stackTrace, posicao);
        if (posicao > -1)
        {
            assinatura = getAssinaturaOperadorSessaoTransacao(stackTrace[posicao]);
            if (assinatura == null)
            {
                int qtdeElementos = stackTrace.length;
                while (assinatura == null && qtdeElementos > ++posicao)
                {
                    assinatura = getAssinaturaOperadorSessaoTransacao(stackTrace[posicao]);
                }
            }
        }

        return assinatura;
    }

    // Ver: http://bugs.java.com/bugdatabase/view_bug.do?bug_id=6375302
    private StackTraceElement[] obterElementosStackTrace()
    {
        return new Throwable().getStackTrace();
    }

    /**
     * @return nulo se o elemento no satisfaz a assinatura desejada.
     */
    protected abstract String getAssinaturaOperadorSessaoTransacao(StackTraceElement stackTraceElement);

    protected abstract boolean isClassePreDelimitadora(StackTraceElement stackTraceElement);

    private int encontraPosicaoClasseDelimitadora(StackTraceElement[] stackTrace, int inicioPesquisa)
    {
        int posicaoEncontrada = -1;
        int tamanho = stackTrace.length;

        boolean encontrado = false;
        for (int i = inicioPesquisa; i < tamanho; i++)
        {
            if (isClassePreDelimitadora(stackTrace[i]))
            {
                encontrado = true;
            }
            else if (encontrado)
            {
                posicaoEncontrada = i;
                break;
            }
        }

        return posicaoEncontrada;
    }

    // Para indicar que houve recursividade
    private int procuraNovaOcorrenciaComMesmaAssinatura(StackTraceElement[] stackTrace, int inicioPesquisa, String assinaturaProcurada)
    {
        int posicaoEncontrada = -1;
        int tamanho = stackTrace.length;

        for (int i = inicioPesquisa; i < tamanho; i++)
        {
            String assinaturaElemento = getAssinaturaOperadorSessaoTransacao(stackTrace[i]);
            if (assinaturaProcurada.equals(assinaturaElemento))
            {
                posicaoEncontrada = i;
                break;
            }
        }

        return posicaoEncontrada;
    }

    /**
     * Mtodo que retorna todas as ocorrncias 
     * TODO Porque procurar todas as ocorrncias????
     * @return
     */
    public List<String> obterTodosOperadoresSessaoTransacao()
    {
        List<String> relacaoOperadores = new ArrayList<String>();

        StackTraceElement[] stackTrace = obterElementosStackTrace();

        int posicaoClasseDelimitadora = encontraPosicaoClasseDelimitadora(stackTrace, POSICAO_PESQUISA_INICIAL);
        if (posicaoClasseDelimitadora >= 0)
        {
            String assinaturaElemento = getAssinaturaOperadorSessaoTransacao(stackTrace[posicaoClasseDelimitadora]);
            if (assinaturaElemento == null)
            {
                // LLD: O primeiro elemento no satisfaz a assinatura, mas colocar o nome da classe mesmo assim para indicar isso
                relacaoOperadores.add(stackTrace[posicaoClasseDelimitadora].getClassName() + "." + stackTrace[posicaoClasseDelimitadora].getMethodName());
            }
            else
            {
                relacaoOperadores.add(assinaturaElemento);

                if (!isDelimitadoraPrimeiraAssinatura())
                {
                    int posicao = procuraNovaOcorrenciaComMesmaAssinatura(stackTrace, posicaoClasseDelimitadora + 1, assinaturaElemento);
                    if (posicao > -1)
                    {
                        String assinatura = getAssinaturaOperadorSessaoTransacao(stackTrace[posicao]);
                        if (assinatura != null)
                        {
                            relacaoOperadores.add(assinatura);
                        }
                    }
                }
            }
        }

        if (relacaoOperadores.size() == 0)
        {
            relacaoOperadores = null;
        }

        return relacaoOperadores;
    }

    protected abstract boolean isDelimitadoraPrimeiraAssinatura();
}