/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.hibernate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;

import br.gov.camara.hibernate.exception.HibernateExceptionCD;
import br.gov.camara.hibernate.interceptor.session.SessionInterceptorManager;

public class FabricaSessaoHibernate
{
    /**
    * Logger for this class
    */
    private static final Log log = LogFactory.getLog(FabricaSessaoHibernate.class);

    protected static final int SESSAO_POSICAO_SESSION = 0;
    protected static final int SESSAO_POSICAO_FACADE_CHAMADORA = 1;
    protected static final int SESSAO_POSICOES_TOTAL = 2;

    public static final ThreadLocal<Map<String, List<Object>>> threadLocalMapSessionAbertas = new ThreadLocal<Map<String, List<Object>>>();

    private FabricaSessaoHibernate()
    {}

    /**
     * Obtem uma sesso do hibernate Se j existe uma sesso aberta, a mesma  retornada Seno, uma nova sesso  criada
     * Observao: a sesso fica registrada na ThreadLocal
     * 
     * @param request The requeset we are servicing
     * @return An open session
     * @throws HibernateExceptionCD
     */
    public static Session obterSessao(String nomeConexao) throws HibernateExceptionCD
    {
        if (nomeConexao == null)
        {
            nomeConexao = GerenciadorConfiguracaoHibernate.obterNomeConexaoPadrao();
        }

        if (log.isDebugEnabled())
        {
            log.debug("Obtendo a sesso do Hibernate para a conexo '" + nomeConexao + "'");
        }

        // Verificar se a sesso ou a transao Hibernate existem na sesso
        Session hseHibernateSession = obterSessaoIniciada(nomeConexao);

        try
        {
            if (hseHibernateSession != null)
            {
                if (log.isDebugEnabled())
                {
                    log.debug("J existe sesso aberta para a conexo '" + nomeConexao + "'. Retornando a mesma...");
                }
            }
            else
            {
                if (log.isDebugEnabled())
                {
                    log.debug("No existe sesso aberta para a conexo '" + nomeConexao + "'. Criando uma nova");
                }

                // Como estar sendo criada uma nova sesso, descartar qualquer
                // transao existente
                Transaction trxTransacao = FabricaTransacaoHibernate.obterTransacaoIniciada(nomeConexao);

                if (trxTransacao != null)
                {
                    if (log.isWarnEnabled())
                    {
                        log.warn("No existia nenhuma sesso aberta, mas havia uma transao aberta para a conexo '" + nomeConexao + "'. Desfazendo-a...");
                    }
                    try
                    {
                        trxTransacao.rollback();
                    }
                    catch (Exception e2)
                    {
                        if (log.isInfoEnabled())
                        {
                            // No precisa fazer nada... Pacincia...
                            log.info("Ocorreu um erro desfazendo a transao aberta (sem sesso)" + e2.getMessage());
                        }
                    }
                    trxTransacao = null;
                    FabricaTransacaoHibernate.removerTransacaoIniciada(nomeConexao);
                }

                // Obter nova sesso Hibernate
                hseHibernateSession = iniciarNovaSessao(nomeConexao);
            }
        }
        catch (Exception e)
        {
            // Oooppsss... Ocorreu um erro... :-(
            if (log.isErrorEnabled())
            {
                log.error("...Ocorreu o seguinte erro tentando obter uma nova sesso do Hibernate: " + e.getMessage());
            }

            if (hseHibernateSession != null)
            {
                try
                {
                    if (hseHibernateSession.isOpen())
                    {
                        hseHibernateSession.close();
                    }
                }
                catch (Exception e2)
                {
                    if (log.isInfoEnabled())
                    {
                        // No precisa fazer nada... Pacincia...
                        log.info("Ocorreu um erro fechando a sesso Hibernate com erro na criao: " + e2.getMessage());
                    }
                }
                hseHibernateSession = null;
            }

            if (obterSessaoIniciada(nomeConexao) != null)
            {
                removerSessaoIniciada(nomeConexao);
            }

            if (e instanceof HibernateExceptionCD)
            {
                throw (HibernateExceptionCD) e;
            }
            else
            {
                throw new HibernateExceptionCD(e);
            }
        }

        if (log.isDebugEnabled())
        {
            log.debug("Retornando a sesso do Hibernate para a conexo '" + nomeConexao + "'");
        }

        return hseHibernateSession;
    }

    /**
     * Finaliza a sesso do hibernate Se no existir sesso (e transao), nada  feito Se existir transao aberta, ela
     *  desfeita, a sesso  finalizada e um erro  gerado
     * 
     * @param request The requeset we are servicing
     * @throws HibernateExceptionCD
     */
    public static void fecharSessao(String nomeConexao) throws HibernateExceptionCD
    {
        fecharSessao(nomeConexao, null);
    }

    private static void fecharSessao(String nomeConexao, Iterator<String> iteracaoConexaoASerRemovida) throws HibernateExceptionCD
    {
        if (nomeConexao == null)
        {
            nomeConexao = GerenciadorConfiguracaoHibernate.obterNomeConexaoPadrao();
        }

        if (log.isDebugEnabled())
        {
            log.debug("Tentando fechar a sesso do Hibernate da conexo '" + nomeConexao + "'...");
        }

        // Verificar existncia da sesso e da transao Hibernate
        String comTransacao = null;

        // Verificar se a sesso Hibernate existe na sesso
        Session hseHibernateSession = obterSessaoIniciada(nomeConexao);

        // Verificar se a Facade chamadora  a mesma que iniciou
        if (hseHibernateSession != null)
        {
            if (!isMesmaFacadeIniciouSessao(nomeConexao) && iteracaoConexaoASerRemovida == null)
            {
                return;
            }
        }

        // Obter transao existente
        Transaction trxTransacao = FabricaTransacaoHibernate.obterTransacaoIniciada(nomeConexao);
        List<Exception> errosOcorridos = new ArrayList<Exception>();

        // Finalizar a sesso Hibernate
        try
        {
            // Desfazer transao se houver uma em andamento...
            if (trxTransacao != null)
            {
                try
                {
                    comTransacao = FabricaTransacaoHibernate.forcarFechamentoTransacao(nomeConexao);
                }
                catch (Exception e)
                {
                    log.error("Ocorreu um erro tentando forar o fechamento da transao existente para a conexo '" + nomeConexao + "': " + e.getMessage());
                    // No precisa fazer nada
                    errosOcorridos.add(e);
                }
            }

            if (hseHibernateSession == null)
            {
                if (log.isDebugEnabled())
                {
                    log.debug("...No  possvel executar um flush da sesso Hibernate para a conexo '"
                            + nomeConexao
                            + "' porque no foi encontrada nenhuma sesso aberta...");
                }
            }
            else
            {
                if (log.isDebugEnabled())
                {
                    log.debug("...Executando um flush da sesso Hibernate para a conexo '" + nomeConexao + "'...");
                }

                // ... Fazendo um flush da sesso Hibernate
                hseHibernateSession.flush();

                // "Gato" para resolver o problema do Ingres que abre uma transao automaticamente...
                try
                {
                    // "Gato" para resolver o problema do Ingres que abre uma transao automaticamente...
                    // Parece que esse problema acontece tambm com o Oracle
                    ConexaoJDBC.resolverFaltaAutocommit(hseHibernateSession);
                }
                catch (Exception e)
                {
                    log.error("Ocorreu um erro tentando realizar a transao (por causa do Ingres que abre uma automaticamente) para a conexo '"
                            + nomeConexao
                            + "': "
                            + e.getMessage());
                    e.printStackTrace();
                    // Pacincia... NO MAIS !!!!
                    errosOcorridos.add(e);
                }
            }

            if (!errosOcorridos.isEmpty())
            {
                StringBuilder erros = new StringBuilder();
                for (int i = 0; i < errosOcorridos.size(); i++)
                {
                    erros.append(i);
                    erros.append(") ");
                    erros.append(((Throwable) errosOcorridos.get(i)).getMessage());
                    erros.append(".\n");
                }
                throw new HibernateException(erros.toString());
            }
        }
        catch (Exception e)
        {
            String mensagemDeErro = e.getMessage();
            if (mensagemDeErro != null && mensagemDeErro.contains("The user must supply a JDBC connection"))
            {
                log.info("No foi possvel fechar a conexo '" + nomeConexao + "' porque ela  aparentemente virtual: " + e.getMessage());
            }
            else
            {
                if (log.isErrorEnabled())
                {
                    log.error("...Ocorreu o seguinte erro executando o flush para a conexo '" + nomeConexao + "': " + e.getMessage());
                    // e.printStackTrace();
                }

                HibernateExceptionCD.dispararExcecao(e);
            }
        }
        finally
        {
            if (log.isDebugEnabled())
            {
                log.debug("...Finalmente, fechando a sesso Hibernate para a conexo '"
                        + nomeConexao
                        + "' e removendo as referncias (sesso e transao Hibernate)...");
            }

            // Fecha a sesso Hibernate
            try
            {
                if (hseHibernateSession == null)
                {
                    if (log.isDebugEnabled())
                    {
                        log.debug("...No  possvel fechar a sesso Hibernate para a conexo '"
                                + nomeConexao
                                + "' porque no foi encontrada nenhuma sesso aberta...");
                    }
                }
                else
                {
                    // Executar interceptor
                    try
                    {
                        // Foi definido algum interceptor de sesso ?
                        SessionInterceptorManager sessionInterceptorManager = GerenciadorConfiguracaoHibernate.obterSessionInterceptorManagerConexao(nomeConexao);
                        if (sessionInterceptorManager != null)
                        {
                            sessionInterceptorManager.executarAntesFechar(hseHibernateSession, nomeConexao);
                        }
                    }
                    catch (Exception e)
                    {
                        // Opsss ocorreu um erro ao executar o interceptor
                        log.error("Ocorreu um erro executando o sessionInterceptor antes de fechar a conexo '" + nomeConexao + "': " + e.getMessage());
                    }

                    hseHibernateSession.close();
                }
            }
            catch (Exception e)
            {
                log.error("Ocorreu um erro fechando a sesso Hibernate para a conexo '" + nomeConexao + "': " + e.getMessage());

                // No precisa fazer nada... Pacincia...
            }

            // Remover o ponteiro para a sesso Hibernate
            hseHibernateSession = null;
            if (iteracaoConexaoASerRemovida != null)
            {
                removerSessaoIniciada(iteracaoConexaoASerRemovida);
            }
            else if (obterSessaoIniciada(nomeConexao) != null)
            {
                removerSessaoIniciada(nomeConexao);
            }

            if (log.isDebugEnabled())
            {
                log.debug("...Concludo o fechamento da sesso e transao Hibernate para a conexo '" + nomeConexao + "'");
            }

            if (comTransacao != null && !"".equals(comTransacao))
            {
                String mensagem = "A sesso hibernate para a conexo '"
                        + nomeConexao
                        + "' foi fechada, contudo havia uma transao em andamento ("
                        + comTransacao
                        + ") que foi desfeita";

                if (log.isErrorEnabled())
                {
                    log.error(mensagem);
                }

                throw new HibernateExceptionCD(mensagem);
            }
        }

        if (log.isDebugEnabled())
        {
            log.debug("Fim do mtodo");
        }
    }

    private static void removerSessaoIniciada(String nomeConexao) throws HibernateExceptionCD
    {
        Map<String, List<Object>> mapSessionAbertas = obterMapSessoesIniciadas();
        mapSessionAbertas.remove(GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao));
    }

    private static void removerSessaoIniciada(Iterator<String> iteracaoConexaoASerRemovida) throws HibernateExceptionCD
    {
        iteracaoConexaoASerRemovida.remove();
    }

    private static boolean isMesmaFacadeIniciouSessao(String nomeConexao) throws HibernateExceptionCD
    {
        boolean mesmaFacade = false;

        // Verificar se a Facade chamadora  a mesma que iniciou
        List<String> strFacadeChamadora = GerenciadorConfiguracaoHibernate.obterDelimitadoraSessaoTransacaoConexao(nomeConexao).obterTodosOperadoresSessaoTransacao();
        if (strFacadeChamadora == null || strFacadeChamadora.size() > 1)
        {
            mesmaFacade = false;
        }
        else
        {
            String strFacadeChamadoraInicial = obterFacadeChamadoraSessaoIniciada(nomeConexao);
            if (strFacadeChamadoraInicial != null && strFacadeChamadoraInicial.equals(strFacadeChamadora.get(0)))
            {
                mesmaFacade = true;
                if (log.isDebugEnabled())
                {
                    log.debug("A facade chamadora  a mesma que iniciou (" + strFacadeChamadora + ").");
                }
            }
            else
            {
                mesmaFacade = false;
                if (log.isDebugEnabled())
                {
                    log.debug("A facade chamadora atual  "
                            + strFacadeChamadora
                            + ", mas quem iniciou foi "
                            + strFacadeChamadoraInicial
                            + ". Ignorando este mtodo.");
                }
            }
        }

        return mesmaFacade;
    }

    private static Map<String, List<Object>> obterMapSessoesIniciadas()
    {
        Map<String, List<Object>> mapSessionAbertas = (Map<String, List<Object>>) threadLocalMapSessionAbertas.get();
        if (mapSessionAbertas == null)
        {
            synchronized (threadLocalMapSessionAbertas)
            {
                mapSessionAbertas = (Map<String, List<Object>>) threadLocalMapSessionAbertas.get();
                if (mapSessionAbertas == null)
                {
                    mapSessionAbertas = new HashMap<String, List<Object>>();
                    threadLocalMapSessionAbertas.set(mapSessionAbertas);
                }
            }
        }

        return mapSessionAbertas;
    }

    static Session obterSessaoIniciada(String nomeConexao) throws HibernateExceptionCD
    {
        Map<String, List<Object>> mapSessionAbertas = obterMapSessoesIniciadas();
        Session hseHibernateSession = null;
        List<Object> lstSessaoAberta = null;

        lstSessaoAberta = mapSessionAbertas.get(GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao));

        if (lstSessaoAberta != null)
        {
            hseHibernateSession = (Session) lstSessaoAberta.get(SESSAO_POSICAO_SESSION);
        }

        return hseHibernateSession;
    }

    private static String obterFacadeChamadoraSessaoIniciada(String nomeConexao) throws HibernateExceptionCD
    {
        Map<String, List<Object>> mapSessionAbertas = obterMapSessoesIniciadas();
        List<Object> lstSessaoAberta = null;
        String strFacadeChamadora = null;
        lstSessaoAberta = (List<Object>) mapSessionAbertas.get(GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao));
        if (lstSessaoAberta != null)
        {
            strFacadeChamadora = (String) lstSessaoAberta.get(SESSAO_POSICAO_FACADE_CHAMADORA);
        }

        return strFacadeChamadora;
    }

    private static Session iniciarNovaSessao(String nomeConexao) throws HibernateExceptionCD
    {
        // Obtm a nova sesso
        Session hseHibernateSession = null;
        try
        {
            hseHibernateSession = GerenciadorConfiguracaoHibernate.obterSessionFactory(GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao)).openSession();
        }
        catch (HibernateException he)
        {
            if (hseHibernateSession != null && hseHibernateSession.isOpen())
            {
                hseHibernateSession.close();
            }

            if (he instanceof HibernateExceptionCD)
            {
                throw (HibernateExceptionCD) he;
            }
            else
            {
                throw new HibernateExceptionCD(he);
            }
        }

        DelimitadoraSessaoTransacao delimitadoraSessaoTransacao = GerenciadorConfiguracaoHibernate.obterDelimitadoraSessaoTransacaoConexao(nomeConexao);
        String strFacadeChamadora = delimitadoraSessaoTransacao.obterAssinaturaOperadorSessaoTransacao();
        if (strFacadeChamadora == null)
        {
            // No deveria acontecer. Provavelmente  porque a implementao da DelimitadoraSessaoTransacao no est enxergando corretamente quem iniciou a sesso/transao.
            String mensagemErro = "No conseguimos identificar quem pediu para inicar a sesso/transao para a conexo '"
                    + nomeConexao
                    + "'. DelimitadoraSessaoTransacao = '"
                    + delimitadoraSessaoTransacao.getClass().getName()
                    + "'";
            log.error(mensagemErro);
            throw new RuntimeException(mensagemErro);
        }

        // Foi definido algum interceptor ?
        try
        {
            // Foi definido algum interceptor de sesso ?
            SessionInterceptorManager sessionInterceptorManager = GerenciadorConfiguracaoHibernate.obterSessionInterceptorManagerConexao(nomeConexao);
            if (sessionInterceptorManager != null)
            {
                sessionInterceptorManager.executarAposObterNovaSessao(hseHibernateSession, nomeConexao);
            }
        }
        catch (Exception e)
        {
            // Opsss ocorreu um erro ao executar o interceptor - FECHAR A SESSO
            if (hseHibernateSession != null && hseHibernateSession.isOpen())
            {
                hseHibernateSession.close();
            }

            log.error("Ocorreu um erro executando o sessionInterceptor aps obter a conexo '" + nomeConexao + "': " + e.getMessage());

            if (e instanceof HibernateExceptionCD)
            {
                throw (HibernateExceptionCD) e;
            }
            else
            {
                throw new HibernateExceptionCD(e);
            }
        }

        List<Object> arlSessao = new ArrayList<Object>(SESSAO_POSICOES_TOTAL);
        arlSessao.add(SESSAO_POSICAO_SESSION, hseHibernateSession);
        arlSessao.add(SESSAO_POSICAO_FACADE_CHAMADORA, strFacadeChamadora);

        // Registrar a sesso do Hibernate no map da ThreadLocal
        Map<String, List<Object>> mapSessionAbertas = obterMapSessoesIniciadas();
        mapSessionAbertas.put(GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao), arlSessao);

        if (log.isDebugEnabled())
        {
            log.debug("Iniciando uma nova sesso chamada pela Facade: " + strFacadeChamadora);
        }

        return hseHibernateSession;
    }

    static String forcarFechamentoSessoesAbertas()
    {
        StringBuilder strNomeSessoesAbertas = new StringBuilder();

        // Recupera os dados das transaes abertas
        Map<String, List<Object>> mapSessoesAbertas = obterMapSessoesIniciadas();

        Set<String> setSessoesAbertas = mapSessoesAbertas.keySet();
        if (setSessoesAbertas != null && setSessoesAbertas.size() > 0)
        {
            Iterator<String> itrSessoesAbertas = setSessoesAbertas.iterator();
            while (itrSessoesAbertas.hasNext())
            {
                String numeroConexao = (String) itrSessoesAbertas.next();
                String nomeConexao = GerenciadorConfiguracaoHibernate.obterNomeConexao(numeroConexao);
                String facadeChamadora = obterFacadeChamadoraSessaoIniciada(nomeConexao);
                strNomeSessoesAbertas.append(nomeConexao).append("-").append(facadeChamadora).append(", ");

                fecharSessao(nomeConexao, itrSessoesAbertas);
            }

            if (!"".equals(strNomeSessoesAbertas.toString()))
            {
                strNomeSessoesAbertas.deleteCharAt(strNomeSessoesAbertas.length() - 1).deleteCharAt(strNomeSessoesAbertas.length() - 1);
                log.error("Sesses abertas: " + strNomeSessoesAbertas.toString());
            }
        }

        return strNomeSessoesAbertas.toString();
    }

    @Override
    protected void finalize() throws Throwable
    {
        // Teoricamente no precisa...
        log.info("Destruindo o Threadlocal das sesses do Hibernate.");

        threadLocalMapSessionAbertas.remove();
        super.finalize();
    }

    static Session obterSessaoExclusiva(String nomeConexao) throws HibernateExceptionCD
    {
        // Obtm a nova sesso
        Session hseHibernateSession = null;
        try
        {
            hseHibernateSession = GerenciadorConfiguracaoHibernate.obterSessionFactory(GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao)).openSession();
        }
        catch (HibernateException he)
        {
            if (hseHibernateSession != null && hseHibernateSession.isOpen())
            {
                hseHibernateSession.close();
            }

            if (he instanceof HibernateExceptionCD)
            {
                throw (HibernateExceptionCD) he;
            }
            else
            {
                throw new HibernateExceptionCD(he);
            }
        }

        // Foi definido algum interceptor ?
        try
        {
            // Foi definido algum interceptor de sesso ?
            SessionInterceptorManager sessionInterceptorManager = GerenciadorConfiguracaoHibernate.obterSessionInterceptorManagerConexao(nomeConexao);
            if (sessionInterceptorManager != null)
            {
                sessionInterceptorManager.executarAposObterNovaSessao(hseHibernateSession, nomeConexao);
            }
        }
        catch (Exception e)
        {
            // Opsss ocorreu um erro ao executar o interceptor - FECHAR A SESSO
            if (hseHibernateSession != null && hseHibernateSession.isOpen())
            {
                hseHibernateSession.close();
            }

            log.error("Ocorreu um erro executando o sessionInterceptor aps obter a conexo '" + nomeConexao + "': " + e.getMessage());

            if (e instanceof HibernateExceptionCD)
            {
                throw (HibernateExceptionCD) e;
            }
            else
            {
                throw new HibernateExceptionCD(e);
            }
        }

        registrarSessaoExclusiva(nomeConexao, hseHibernateSession);

        return hseHibernateSession;
    }

    static void fecharSessaoExclusiva(String nomeConexao, Session sessaoExclusiva) throws HibernateExceptionCD
    {
        if (nomeConexao == null)
        {
            throw new HibernateExceptionCD("O nome da conexo deve ser informado");
        }

        if (sessaoExclusiva == null)
        {
            return;
        }

        if (log.isDebugEnabled())
        {
            log.debug("Tentando fechar a sesso exclusiva do Hibernate da conexo '" + nomeConexao + "'...");
        }

        // Verificar se a sesso Hibernate existe na sesso
        Session hseHibernateSession = sessaoExclusiva;

        List<Exception> errosOcorridos = new ArrayList<Exception>();
        // Finalizar a sesso Hibernate
        try
        {
            if (log.isDebugEnabled())
            {
                log.debug("...Executando um flush da sesso Hibernate para a conexo '" + nomeConexao + "'...");
            }

            // ... Fazendo um flush da sesso Hibernate
            hseHibernateSession.flush();

            // "Gato" para resolver o problema do Ingres que abre uma transao automaticamente...
            try
            {
                // "Gato" para resolver o problema do Ingres que abre uma transao automaticamente...
                // Parece que esse problema acontece tambm com o Oracle
                ConexaoJDBC.resolverFaltaAutocommit(hseHibernateSession);
            }
            catch (Exception e)
            {
                log.error("Ocorreu um erro tentando realizar a transao (por causa do Ingres que abre uma automaticamente) para a conexo '"
                        + nomeConexao
                        + "': "
                        + e.getMessage());
                e.printStackTrace();
                // Pacincia... NO MAIS !!!!
                errosOcorridos.add(e);
            }

            if (!errosOcorridos.isEmpty())
            {
                StringBuilder erros = new StringBuilder();
                for (int i = 0; i < errosOcorridos.size(); i++)
                {
                    erros.append(i);
                    erros.append(") ");
                    erros.append(((Throwable) errosOcorridos.get(i)).getMessage());
                    erros.append(".\n");
                }
                throw new HibernateException(erros.toString());
            }
        }
        catch (Exception e)
        {
            String mensagemDeErro = e.getMessage();
            if (mensagemDeErro != null && mensagemDeErro.contains("The user must supply a JDBC connection"))
            {
                log.info("No foi possvel fechar a conexo '" + nomeConexao + "' porque ela  aparentemente virtual: " + e.getMessage());
            }
            else
            {
                if (log.isErrorEnabled())
                {
                    log.error("...Ocorreu o seguinte erro executando o flush para a conexo '" + nomeConexao + "': " + e.getMessage());
                    // e.printStackTrace();
                }

                HibernateExceptionCD.dispararExcecao(e);
            }
        }
        finally
        {
            if (log.isDebugEnabled())
            {
                log.debug("...Finalmente, fechando a sesso Hibernate para a conexo '"
                        + nomeConexao
                        + "' e removendo as referncias (sesso e transao Hibernate)...");
            }

            // Fecha a sesso Hibernate
            try
            {
                // Executar interceptor
                try
                {
                    // Foi definido algum interceptor de sesso ?
                    SessionInterceptorManager sessionInterceptorManager = GerenciadorConfiguracaoHibernate.obterSessionInterceptorManagerConexao(nomeConexao);
                    if (sessionInterceptorManager != null)
                    {
                        sessionInterceptorManager.executarAntesFechar(hseHibernateSession, nomeConexao);
                    }
                }
                catch (Exception e)
                {
                    // Opsss ocorreu um erro ao executar o interceptor
                    log.error("Ocorreu um erro executando o sessionInterceptor antes de fechar a conexo '" + nomeConexao + "': " + e.getMessage());
                }

                hseHibernateSession.close();
            }
            catch (Exception e)
            {
                log.error("Ocorreu um erro fechando a sesso Hibernate para a conexo '" + nomeConexao + "': " + e.getMessage());

                // No precisa fazer nada... Pacincia...
            }

            removerRegistroSessaoExclusiva(nomeConexao, hseHibernateSession);
        }

        if (log.isDebugEnabled())
        {
            log.debug("Fim do mtodo");
        }
    }

    private static void registrarSessaoExclusiva(String nomeConexao, Session sessaoExclusiva)
    {
        List<Object> arlSessao = new ArrayList<Object>(SESSAO_POSICOES_TOTAL);
        arlSessao.add(SESSAO_POSICAO_SESSION, sessaoExclusiva);
        arlSessao.add(SESSAO_POSICAO_FACADE_CHAMADORA, "Sesso exclusiva");

        // Registrar a sesso do Hibernate no map da ThreadLocal
        Map<String, List<Object>> mapSessionAbertas = obterMapSessoesIniciadas();
        mapSessionAbertas.put(gerarIdSessaoExclusiva(nomeConexao, sessaoExclusiva), arlSessao);
    }

    private static void removerRegistroSessaoExclusiva(String nomeConexao, Session sessaoExclusiva) throws HibernateExceptionCD
    {
        Map<String, List<Object>> mapSessionAbertas = obterMapSessoesIniciadas();
        mapSessionAbertas.remove(gerarIdSessaoExclusiva(nomeConexao, sessaoExclusiva));
    }

    private static String gerarIdSessaoExclusiva(String nomeConexao, Session sessaoExclusiva)
    {
        return nomeConexao + "#SessaoExclusiva@" + Integer.toHexString(sessaoExclusiva.hashCode());
    }
}
