/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.hibernate;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.MappingException;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.dialect.function.ClassicAvgFunction;
import org.hibernate.dialect.function.ClassicCountFunction;
import org.hibernate.dialect.function.ClassicSumFunction;
import org.hibernate.engine.SessionFactoryImplementor;
import org.hibernate.jdbc.util.SQLStatementLogger;
import org.hibernate.mapping.Table;

import br.gov.camara.biblioteca.util.ClasseDinamica;
import br.gov.camara.exception.CDException;
import br.gov.camara.exception.CDException.NivelLog;
import br.gov.camara.hibernate.exception.HibernateExceptionCD;
import br.gov.camara.hibernate.interceptor.TipoDeInterceptor;
import br.gov.camara.hibernate.interceptor.operation.OperationInterceptorManager;
import br.gov.camara.hibernate.interceptor.session.SessionInterceptorManager;
import sigesp.comum.util.hibernate.ColecaoDadosConexao;
import sigesp.comum.util.hibernate.DadosConexao;

public class GerenciadorConfiguracaoHibernate
{
    /**
    * Logger for this class
    */
    private static final Log log = LogFactory.getLog(GerenciadorConfiguracaoHibernate.class);

    protected static final int CONEXAO_POSICAO_DADOS_CONEXAO = 0;
    protected static final int CONEXAO_POSICAO_SESSION_FACTORY = 1;
    protected static final int CONEXAO_POSICAO_SQL_LOGGER = 2;
    protected static final int CONEXAO_POSICOES_TOTAL = 3;

    protected static final String CHAVE_INTERCEPTOR_MANAGER_SESSION = "sessionInterceptorManager";
    protected static final String CHAVE_DELIMITADORA_SESSAO_TRANSACAO_IMPL = "delimitadoraSessaoTransacaoImpl";

    protected static final String CHAVE_REALIZAR_FLUSH_IMEDIATAMENTE = "realizar.flush.imediatamente";

    protected static final String CHAVE_HIBERNATE_SQL_SHOW = "hibernate.show_sql";
    protected static final String CHAVE_HIBERNATE_SQL_FORMAT = "hibernate.format_sql";

    protected static final String CLASSE_PADRAO_DELIMITADORA_SESSAO_TRANSACAO = "br.gov.camara.hibernate.FacadeDelimitadora";

    protected static Map<String, List<Object>> mapPersistenciaSessionFactory;
    protected static Map<String, String> mapNomesConexoes;
    protected static List<List<String>> nomesDasConexoes;
    protected static String nomeConexaoPadrao;

    private static boolean inicializado = false;

    private GerenciadorConfiguracaoHibernate()
    {
        super();
    }

    protected synchronized static boolean iniciar(ColecaoDadosConexao conexoesDados)
    {
        if (inicializado)
        {
            if (log.isInfoEnabled())
            {
                log.info("Gerenciador de configuraes do Hibernate j inicializado");
            }

            return false;
        }
        else
        {
            if (log.isInfoEnabled())
            {
                log.info("Inicializando o gerenciador de configuraes do Hibernate...");
            }

            String strNomeConexaoPadrao = (String) conexoesDados.obterConexaoPeloIndice(0).getNomesConexao().get(0);
            Iterator<DadosConexao> iteratorConexoes = conexoesDados.obterIterator();

            Map<String, List<Object>> mapPersistSF = new HashMap<String, List<Object>>();
            Map<String, String> mapNomesConex = new HashMap<String, String>();
            List<List<String>> listaComOsNomesDasConexoes = new ArrayList<List<String>>();

            List<String> listaGeralTabelas = new ArrayList<String>();

            int numeroConexao = 0;
            while (iteratorConexoes.hasNext())
            {
                DadosConexao dadosConexao = iteratorConexoes.next();
                if (log.isInfoEnabled())
                {
                    log.info("Inicializando a conexo '" + numeroConexao + "' - " + dadosConexao.getNomesConexao().toString());
                }
                Configuration configuracaoHibernate = criarConfiguracaoHibernate(dadosConexao);

                // Configura os Interceptadores com operaes de persistncia
                List<String> interceptorsOperation = dadosConexao.getInterceptorsPorTipo(TipoDeInterceptor.OPERACAO);
                if (interceptorsOperation != null)
                {
                    configuracaoHibernate.setInterceptor(new OperationInterceptorManager(interceptorsOperation));
                }

                // Para manter a compatibilidade (o retorno continuar sendo Integer ao invs de Long, como foi alterado)
                configuracaoHibernate.addSqlFunction("count", new ClassicCountFunction());
                configuracaoHibernate.addSqlFunction("avg", new ClassicAvgFunction());
                configuracaoHibernate.addSqlFunction("sum", new ClassicSumFunction());

                // Para manter a compatibilidade com o comportamento das sesses na verso 2
                configuracaoHibernate.setProperty("hibernate.connection.release_mode", "on_close");

                // Constroi a fbrica de sesso do Hibernate
                SessionFactory sessionFactory = configuracaoHibernate.buildSessionFactory();

                // Cria um mapa de todas as tabelas referenciadas nesta conexo
                List<String> mapTabelas = dadosConexao.getMapeamentosTabelas();
                Iterator<Table> itrTabelas = configuracaoHibernate.getTableMappings();
                while (itrTabelas.hasNext())
                {
                    String nomeTable = itrTabelas.next().getName().toLowerCase();

                    if (mapTabelas.contains(nomeTable))
                    {
                        if (log.isWarnEnabled())
                        {
                            log.warn("*** ATENO: O nome da tabela '"
                                    + nomeTable
                                    + "' est sendo referenciado mais de uma vez na conexo '"
                                    + dadosConexao.getNomesConexao().toString()
                                    + "'.");
                        }
                    }
                    if (listaGeralTabelas.contains(nomeTable))
                    {
                        if (log.isWarnEnabled())
                        {
                            log.warn("*** ATENO: O nome da tabela '" + nomeTable + "' est sendo referenciado em MAIS DE UMA conexo distinta.");
                        }
                    }
                    mapTabelas.add(nomeTable);
                }
                listaGeralTabelas.addAll(mapTabelas);

                // Configura os Interceptadores de sesso
                List<String> interceptorsSession = dadosConexao.getInterceptorsPorTipo(TipoDeInterceptor.SESSAO);
                if (interceptorsSession != null && interceptorsSession.size() > 0)
                {
                    dadosConexao.setObjetoConexao(CHAVE_INTERCEPTOR_MANAGER_SESSION, new SessionInterceptorManager(interceptorsSession));
                }

                // Configura a Delimitadora da Facade (responsvel por controlar as conexes e as transaes)
                DelimitadoraSessaoTransacao objFacadeChamadora = null;
                String nomeDelimitadoraSessaoTransacaoImpl = obterPropriedadeConexao(dadosConexao, CHAVE_DELIMITADORA_SESSAO_TRANSACAO_IMPL);
                if (nomeDelimitadoraSessaoTransacaoImpl == null || "".equals(nomeDelimitadoraSessaoTransacaoImpl))
                {
                    nomeDelimitadoraSessaoTransacaoImpl = System.getProperty("delimitadoraSessaoTransacaoImpl");
                    if (nomeDelimitadoraSessaoTransacaoImpl == null || "".equals(nomeDelimitadoraSessaoTransacaoImpl))
                    {
                        nomeDelimitadoraSessaoTransacaoImpl = CLASSE_PADRAO_DELIMITADORA_SESSAO_TRANSACAO;
                    }
                }
                try
                {
                    objFacadeChamadora = (DelimitadoraSessaoTransacao) new ClasseDinamica().obterInstancia(nomeDelimitadoraSessaoTransacaoImpl);
                }
                catch (CDException e)
                {
                    throw new RuntimeException("Erro ao instanciar a classe delimitadora das sesses e transaes: " + e.getMessage(), e);
                }
                dadosConexao.setObjetoConexao(CHAVE_DELIMITADORA_SESSAO_TRANSACAO_IMPL, objFacadeChamadora);

                // Configura o SQL Logger da conexo
                boolean sqlLog = "true".equals(obterPropriedadeConexao(dadosConexao, CHAVE_HIBERNATE_SQL_SHOW));
                boolean sqlFormat = "true".equals(obterPropriedadeConexao(dadosConexao, CHAVE_HIBERNATE_SQL_FORMAT));
                SQLStatementLogger statementLogger = new SQLStatementLogger(sqlLog, sqlFormat);

                // Guarda os objetos da conexo
                List<Object> conexao = new ArrayList<Object>(CONEXAO_POSICOES_TOTAL);
                conexao.add(CONEXAO_POSICAO_DADOS_CONEXAO, dadosConexao);
                conexao.add(CONEXAO_POSICAO_SESSION_FACTORY, sessionFactory);
                conexao.add(CONEXAO_POSICAO_SQL_LOGGER, statementLogger);

                // Cria o ponteiro com os diversos nomes das conexes
                List<String> lstNomesConexao = dadosConexao.getNomesConexao();
                listaComOsNomesDasConexoes.add(Collections.unmodifiableList(lstNomesConexao));
                String strNomeConexao;
                for (int i = 0; i < lstNomesConexao.size(); i++)
                {
                    strNomeConexao = lstNomesConexao.get(i);
                    mapNomesConex.put(strNomeConexao, Integer.toString(numeroConexao));
                }

                mapPersistSF.put(Integer.toString(numeroConexao), conexao);

                numeroConexao++;
            }

            if (mapPersistSF.size() == 0)
            {
                throw new HibernateExceptionCD("Nenhuma conexo Hibernate foi inicializada");
            }

            mapPersistenciaSessionFactory = mapPersistSF;
            mapNomesConexoes = mapNomesConex;
            nomesDasConexoes = Collections.unmodifiableList(listaComOsNomesDasConexoes);

            nomeConexaoPadrao = strNomeConexaoPadrao;

            inicializado = true;

            if (log.isInfoEnabled())
            {
                log.info("... fim da inicializao do gerenciador de configuraes do Hibernate...");
            }

            return inicializado;
        }
    }

    /**
     * Retorna o identificador da conexo associado com o nome da conexo informado (alias)
     * 
     * @return nmero da conexo informada
     */
    protected static String obterPonteiroConexao(String nomeConexao) throws HibernateExceptionCD
    {
        if (log.isDebugEnabled())
        {
            log.debug("Obtendo o ponteiro para a conexo '" + nomeConexao + "'");
        }

        if (!inicializado)
        {
            throw new HibernateExceptionCD("O Hibernate ainda no foi inicializado.");
        }

        String ponteiroConexao;

        Map<String, String> mapNomes = mapNomesConexoes;
        ponteiroConexao = mapNomes.get(nomeConexao);

        if (ponteiroConexao == null)
        {
            throw new HibernateExceptionCD("No foi encontrada nenhuma conexo com o nome '" + nomeConexao + "'");
        }

        if (log.isDebugEnabled())
        {
            log.debug("Retornando o nome do ponteiro para a conexo '" + nomeConexao + "' = '" + ponteiroConexao + "'");
        }

        return ponteiroConexao;
    }

    private static DadosConexao obterDadosConexao(String nomeConexao)
    {
        Map<String, List<Object>> mapPersistSF = mapPersistenciaSessionFactory;
        List<Object> lstconexao = null;

        try
        {
            String strPonteiro = GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao);
            lstconexao = mapPersistSF.get(strPonteiro);
        }
        catch (HibernateException he)
        {
            // Sem problemas ... Retornar nulo
        }

        DadosConexao dadosConexao = null;
        if (lstconexao != null)
        {
            dadosConexao = (DadosConexao) lstconexao.get(CONEXAO_POSICAO_DADOS_CONEXAO);
        }

        return dadosConexao;
    }

    public static SQLStatementLogger obterGeradorLogSQL(String nomeConexao)
    {
        Map<String, List<Object>> mapPersistSF = mapPersistenciaSessionFactory;
        List<Object> lstconexao = null;

        try
        {
            if (nomeConexao == null)
            {
                nomeConexao = nomeConexaoPadrao;
            }

            String strPonteiro = GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao);
            lstconexao = mapPersistSF.get(strPonteiro);
        }
        catch (HibernateException he)
        {
            // Sem problemas ... Retornar nulo
        }

        SQLStatementLogger dadosConexao = null;
        if (lstconexao != null)
        {
            dadosConexao = (SQLStatementLogger) lstconexao.get(CONEXAO_POSICAO_SQL_LOGGER);
        }

        return dadosConexao;
    }

    /**
     * Fetch the SessionFactory from application scope.
     * 
     * @param request The requeste we are servicing
     * @return The SessionFactory for this application session
     * @throws HibernateExceptionCD
     */
    protected static SessionFactory obterSessionFactory(String nomeConexao) throws HibernateExceptionCD
    {
        if (nomeConexao == null)
        {
            nomeConexao = nomeConexaoPadrao;
        }

        if (log.isDebugEnabled())
        {
            log.debug("Tentando recuperar a SessionFactory da conexo '" + nomeConexao + "'...");
        }

        if (!inicializado)
        {
            throw new HibernateExceptionCD("O Hibernate ainda no foi inicializado.");
        }

        Map<String, List<Object>> mapConexoes = mapPersistenciaSessionFactory;

        List<Object> lstConexao = mapConexoes.get(nomeConexao);

        if (lstConexao == null)
        {
            if (log.isErrorEnabled())
            {
                log.error("No foi encontrada a SessionFactory da conexo '" + nomeConexao + "'");
            }
            throw new HibernateExceptionCD("No foi encontrada a SessionFactory da conexo '" + nomeConexao + "'");
        }

        SessionFactory hsfSessionFactory = (SessionFactory) lstConexao.get(CONEXAO_POSICAO_SESSION_FACTORY);

        if (hsfSessionFactory == null)
        {
            if (log.isErrorEnabled())
            {
                log.error("A SessionFactory recuperada da conexo '" + nomeConexao + "'  nula");
            }
            throw new HibernateExceptionCD("A SessionFactory recuperada da conexo '" + nomeConexao + "'  nula");
        }

        if (log.isDebugEnabled())
        {
            log.debug("A SessionFactory da conexo '" + nomeConexao + "' foi recuperada com sucesso");
        }

        return hsfSessionFactory;
    }

    private static Configuration criarConfiguracaoHibernate(DadosConexao dadosConexao) throws HibernateExceptionCD
    {
        // Mandar para o hibernate cada configurao lida
        Configuration configuracaoHibernate = new Configuration();

        // Mandar os parmetros da conexo
        configuracaoHibernate.addProperties(dadosConexao.getPropriedadesConexao());

        // Mandar os mapeamentos da conexo
        Iterator<String> iteratorMapeamentos = dadosConexao.getMapeamentosConexao().iterator();

        String arquivoMapeamento = null;
        while (iteratorMapeamentos.hasNext())
        {
            arquivoMapeamento = iteratorMapeamentos.next();
            try
            {
                configuracaoHibernate.addResource(arquivoMapeamento);
            }
            catch (MappingException me)
            {
                throw new HibernateExceptionCD(me);
            }
        }

        return configuracaoHibernate;
    }

    /**
     * Obtm o contedo da chave da conexo informada
     * 
     * @param nome da conexo
     * @param chave a ser recuperada
     * @return contedo da chave recuperada
     */
    static DelimitadoraSessaoTransacao obterDelimitadoraSessaoTransacaoConexao(String nomeConexao)
    {
        DelimitadoraSessaoTransacao retorno = null;

        if (nomeConexao == null)
        {
            nomeConexao = nomeConexaoPadrao;
        }

        DadosConexao dadosConexao = obterDadosConexao(nomeConexao);
        if (dadosConexao != null)
        {
            retorno = (DelimitadoraSessaoTransacao) dadosConexao.getObjetoConexao(CHAVE_DELIMITADORA_SESSAO_TRANSACAO_IMPL);
        }

        return retorno;
    }

    /**
     * Obtm o contedo da chave da conexo informada
     * 
     * @param nome da conexo
     * @param chave a ser recuperada
     * @return contedo da chave recuperada
     */
    static SessionInterceptorManager obterSessionInterceptorManagerConexao(String nomeConexao)
    {
        SessionInterceptorManager retorno = null;

        if (nomeConexao == null)
        {
            nomeConexao = nomeConexaoPadrao;
        }

        DadosConexao dadosConexao = obterDadosConexao(nomeConexao);
        if (dadosConexao != null)
        {
            retorno = (SessionInterceptorManager) dadosConexao.getObjetoConexao(CHAVE_INTERCEPTOR_MANAGER_SESSION);
        }

        return retorno;
    }

    /**
     * Obtm o nome da conexo padro
     * 
     * @return nome da conexo padro
     */
    public static String obterNomeConexaoPadrao()
    {
        return nomeConexaoPadrao;
    }

    /**
     * Retorna nome de uma das conexes associadas com o identificador (nmero) informado
     * 
     * @return nome da conexo
     */
    protected static String obterNomeConexao(String ponteiroConexao) throws HibernateExceptionCD
    {
        if (log.isDebugEnabled())
        {
            log.debug("Obtendo o nome de uma conexo que aponta para '" + ponteiroConexao + "'");
        }

        if (!inicializado)
        {
            throw new HibernateExceptionCD("O Hibernate ainda no foi inicializado.");
        }

        Map<String, String> mapNomes = mapNomesConexoes;

        Set<String> setTransacoesAbertas = mapNomes.keySet();
        String strNomeConexao = null;

        // Recupera os dados das transaes abertas
        if (setTransacoesAbertas != null)
        {
            Iterator<String> itrTransacoesAbertas = setTransacoesAbertas.iterator();
            while (itrTransacoesAbertas.hasNext())
            {
                String nomeConexao = itrTransacoesAbertas.next();
                String strPonteiro = GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao);

                if (ponteiroConexao.equals(strPonteiro))
                {
                    strNomeConexao = nomeConexao;
                    break;
                }
            }
        }

        if (strNomeConexao == null)
        {
            throw new HibernateExceptionCD("No foi encontrada nenhuma conexo que aponta para '" + ponteiroConexao + "'");
        }

        if (log.isDebugEnabled())
        {
            log.debug("Retornando o nome da conexo '" + strNomeConexao + "' que aponta para o ponteiro '" + ponteiroConexao + "'");
        }

        return strNomeConexao;
    }

    /**
     * Obtm o dialeto especificado para a conexo padro
     * 
     * @return nome do dialeto
     */
    public static String obterDialeto()
    {
        return obterDialeto(nomeConexaoPadrao);
    }

    /**
     * Obtm o dialeto especificado para a conexo especificada
     * 
     * @return nome do dialeto
     */
    public static String obterDialeto(String nomeConexao)
    {
        if (nomeConexao == null)
        {
            nomeConexao = nomeConexaoPadrao;
        }

        String strDialeto = null;

        try
        {
            SessionFactoryImplementor objSessionFactoryImpl = (SessionFactoryImplementor) GerenciadorConfiguracaoHibernate.obterSessionFactory(GerenciadorConfiguracaoHibernate.obterPonteiroConexao(nomeConexao));
            if (objSessionFactoryImpl != null)
            {
                strDialeto = objSessionFactoryImpl.getDialect().toString();
            }
        }
        catch (HibernateException he)
        {
            // No precisa fazer nada... Pacincia...
        }

        return strDialeto;
    }

    /**
     * Obtm a propriedade que indica se  para realizar o flush imediato ou no
     * 
     * @return indicativo se  para executar o flush imediatamente depois de alguns comandos hibernate
     */
    public static boolean realizarFlushImediatamente(String nomeConexao)
    {
        if (nomeConexao == null)
        {
            nomeConexao = nomeConexaoPadrao;
        }

        String valor = null;
        boolean retorno = false;

        DadosConexao dadosConexao = obterDadosConexao(nomeConexao);

        valor = obterPropriedadeConexao(dadosConexao, CHAVE_REALIZAR_FLUSH_IMEDIATAMENTE);
        if (valor != null && "true".equals(valor))
        {
            retorno = true;
        }

        return retorno;
    }

    /**
     * Obtm o contedo da propriedade da conexo informada
     * 
     * @param nome da conexo
     * @param chave a ser recuperada
     * @return contedo da chave recuperada
     */
    public static String obterPropriedadeConexao(String nomeConexao, String chave)
    {
        if (nomeConexao == null)
        {
            nomeConexao = nomeConexaoPadrao;
        }

        String valor = null;

        DadosConexao dadosConexao = obterDadosConexao(nomeConexao);

        valor = obterPropriedadeConexao(dadosConexao, chave);
        return valor;
    }

    /**
     * Obtm o contedo da propriedade da conexo informada
     * 
     * @param dadosConexao Dados da conexo
     * @param chave a ser recuperada
     * @return contedo da chave recuperada
     */
    private static String obterPropriedadeConexao(DadosConexao dadosConexao, String chave)
    {
        String valor = null;

        if (dadosConexao != null)
        {
            Properties propriedades = dadosConexao.getPropriedadesConexao();
            valor = propriedades.getProperty(chave);
        }

        return valor;
    }

    /**
     * Obtm o contedo da chave da conexo informada
     * 
     * @param nome da conexo
     * @param chave a ser recuperada
     * @return contedo da chave recuperada
     */
    public static String obterPropriedadeSubstituicao(String nomeConexao, String chave)
    {
        if (nomeConexao == null)
        {
            nomeConexao = nomeConexaoPadrao;
        }

        String valor = null;

        DadosConexao dadosConexao = obterDadosConexao(nomeConexao);
        if (dadosConexao != null)
        {
            Properties propriedades = dadosConexao.getSubstituicoesConexao();
            valor = propriedades.getProperty(chave);
        }

        return valor;
    }

    public synchronized static void finalizar()
    {
        if (mapPersistenciaSessionFactory != null)
        {
            // Primeiro forcar o fechamento de qualquer conexo aberta
            String strNomeTransacoesAbertas = FabricaTransacaoHibernate.forcarFechamentoTransacoesAbertas();
            String strNomeSessoesAbertas = FabricaSessaoHibernate.forcarFechamentoSessoesAbertas();
            if (!"".equals(strNomeSessoesAbertas) || !"".equals(strNomeTransacoesAbertas))
            {
                try
                {
                    CDException.dispararExcecao("Existiam sesses ("
                            + strNomeSessoesAbertas
                            + ") e/ou transaes ("
                            + strNomeTransacoesAbertas
                            + ") abertas que foram finalizadas.", 5, NivelLog.ERROR);
                }
                catch (Exception e)
                {
                    // No precisa fazer nada. Foi s para registro...
                }
            }

            Set<String> nomeConfiguracoes = mapPersistenciaSessionFactory.keySet();
            if (nomeConfiguracoes != null)
            {
                Iterator<String> iteratorNomeConfiguracoes = nomeConfiguracoes.iterator();

                while (iteratorNomeConfiguracoes.hasNext())
                {
                    String numeroConexao = iteratorNomeConfiguracoes.next();
                    SessionFactory sessionFactory = null;

                    try
                    {
                        sessionFactory = obterSessionFactory(numeroConexao);
                    }
                    catch (Exception e)
                    {
                        if (log.isWarnEnabled())
                        {
                            log.warn("Ocorreu o seguinte erro obtendo a SessionFactory da conexo '" + numeroConexao + "': " + e.getMessage());
                        }
                    }

                    if (log.isInfoEnabled())
                    {
                        log.info("...Fechando a SessionFactory da conexo '" + numeroConexao + "'...");
                    }

                    try
                    {
                        if (sessionFactory != null)
                        {
                            sessionFactory.close();
                        }
                    }
                    catch (HibernateException e)
                    {
                        if (log.isDebugEnabled())
                        {
                            log.debug("...Ocorreu o seguinte erro fechando a ultima SessionFactory: " + e.getMessage());
                        }
                        // too late now
                    }

                    sessionFactory = null;
                }

                mapPersistenciaSessionFactory = null;
            }
        }
        else
        {
            if (log.isDebugEnabled())
            {
                log.debug("...No existe nenhuma SessionFactory registrada...");
            }
        }

        inicializado = false;
    }

    protected static String obterNomeConexaoTabela(String nomeTabela) throws HibernateExceptionCD
    {
        String nomeConexaoRetorno = null;
        if (mapPersistenciaSessionFactory != null)
        {
            Set<String> nomeConfiguracoes = mapPersistenciaSessionFactory.keySet();
            if (nomeConfiguracoes != null)
            {
                Iterator<String> iteratorNomeConfiguracoes = nomeConfiguracoes.iterator();

                while (iteratorNomeConfiguracoes.hasNext())
                {
                    String nomeConexao = obterNomeConexao(iteratorNomeConfiguracoes.next());

                    DadosConexao dadosConexao = obterDadosConexao(nomeConexao);
                    if (dadosConexao.getMapeamentosTabelas().contains(nomeTabela.toLowerCase()))
                    {
                        nomeConexaoRetorno = nomeConexao;
                        break;
                    }

                }
            }
        }

        return nomeConexaoRetorno;
    }

    protected static List<List<String>> obterNomesDasConexoes()
    {
        return nomesDasConexoes;
    }
}
