/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.hibernate;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.transform.ResultTransformer;

import br.gov.camara.hibernate.exception.HibernateExceptionCD;
import br.gov.camara.negocio.ParametroDAO;
import sigesp.comum.util.hibernate.ColecaoDadosConexao;
import sigesp.comum.util.hibernate.PersistenciaConfiguracao;

/**
 * Initialize the Hibernate SessionFactory for this project as an application scope object.
 * 
 * @author Ted Husted
 * @version $Revision: 1.3 $ $Date: 2003/03/14 21:59:41 $
 */
public class HibernateUtilCD
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(HibernateUtilCD.class.getName());

    public static final String INDICATIVO_QUE_ASSUMIU_RESPONSABILIDADE_PARA_FINALIZAR_O_HIBERNATE = "AssumiuResponsabilidadeParaFinalizarOHibernate";

    public static final String CHAVE_QUERY_TIMESTAMP_BD = "queryConsultaTimeStampBD";

    /**
     * The classes with mappings to add to the Configuration are enumerated here. There should be a "${class}.hbm.xml"
     * mapping file for each class stored with each compiled class file.
     * <p>
     * To complete the Hibernate setup, there must be a valid "hiberate.properties" file under the "classes" folder
     * (root of the classpath), which specifies the details of the database hookup.
     * </p>
     * <p>
     * The mapping documents and properties file is all that Hibernate requires.
     * </p>
     * <p>
     * A JDBC Driver is not included in this distribution and must be available on your server's or container's
     * classpath (e.g., the Tomcat common/lib directory).
     * </p>
     * 
     * @return A Configuration object
     * @throws org.hibernate.MappingException if any the mapping documents can be rendered.
     */
    protected static final ColecaoDadosConexao lerArquivoConfiguracaoHibernate(String strCaminhoEArquivoHibernateConfig) throws HibernateExceptionCD
    {
        if (log.isDebugEnabled())
        {
            log.debug("lerArquivoConfiguracaoHibernate() - start");
        }

        ColecaoDadosConexao colecaoDadosConexoes = null;

        try
        {
            PersistenciaConfiguracao persistencia = null;
            persistencia = new PersistenciaConfiguracao();
            colecaoDadosConexoes = persistencia.doConfiguracao(strCaminhoEArquivoHibernateConfig);
        }
        catch (HibernateException e)
        {
            if (log.isErrorEnabled())
            {
                log.error("Ocorreu o seguinte erro tentando obter a configurao do Hibernate:" + e.getMessage());
            }
            throw new HibernateExceptionCD(e);
        }

        if (log.isDebugEnabled())
        {
            log.debug("lerArquivoConfiguracaoHibernate() - end");
        }
        return colecaoDadosConexoes;
    }

    /**
     * DOCUMENTAR!
     * 
     * @param servlet DOCUMENTAR!
     * @param config DOCUMENTAR!
     */
    public synchronized boolean inicializar(String strCaminhoEArquivoHibernateConfig) throws HibernateExceptionCD
    {
        try
        {
            return GerenciadorConfiguracaoHibernate.iniciar(lerArquivoConfiguracaoHibernate(strCaminhoEArquivoHibernateConfig));
        }
        catch (HibernateException e)
        {
            if (log.isErrorEnabled())
            {
                log.error("...Ocorreu o seguinte erro criando a SessionFactory do Hibernate: " + e.getMessage());
            }
            throw new HibernateExceptionCD(e);
        }
    }

    /**
     * DOCUMENTAR!
     * 
     * @param servlet DOCUMENTAR!
     * @param config DOCUMENTAR!
     */
    public synchronized boolean inicializar() throws HibernateExceptionCD
    {
        return inicializar(null);
    }

    /**
     * DOCUMENTAR!
     */
    public synchronized void finalizar()
    {
        if (log.isInfoEnabled())
        {
            log.info("Finalizando o HibernatePlugIn...");
        }

        GerenciadorConfiguracaoHibernate.finalizar();

        if (log.isDebugEnabled())
        {
            log.debug("O HibernatePlugIn foi finalizado");
        }
    }

    public static void verificarSessoesTransacoesNaoFinalizadas() throws HibernateExceptionCD
    {
        String strNomeTransacoesAbertas = FabricaTransacaoHibernate.forcarFechamentoTransacoesAbertas();
        String strNomeSessoesAbertas = FabricaSessaoHibernate.forcarFechamentoSessoesAbertas();

        if (!"".equals(strNomeSessoesAbertas) || !"".equals(strNomeTransacoesAbertas))
        {
            throw new HibernateExceptionCD("Existiam sesses ("
                    + strNomeSessoesAbertas
                    + ") e/ou transaes ("
                    + strNomeTransacoesAbertas
                    + ") abertas que foram finalizadas.");
        }
        else
        {
            if (log.isDebugEnabled())
            {
                log.debug("Todas as sesses e transaes do Hibernate foram finalizadas.");
            }
        }
    }

    public static int executarSQLComParametros(String nomeConexao, String strSQL, ParametroDAO parametros) throws HibernateExceptionCD
    {
        int retorno = 0;

        if (strSQL != null && !"".equals(strSQL))
        {
            ConexaoJDBC conexao = new ConexaoJDBC(FabricaSessaoHibernate.obterSessao(nomeConexao));
            retorno = conexao.executarComParametros(strSQL, parametros);
        }

        return retorno;
    }

    public static int executarSQL(String nomeConexao, String strSQL) throws HibernateExceptionCD
    {
        return executarSQLComParametros(nomeConexao, strSQL, null);
    }

    public static List carregarPorSQL(String nomeConexao, String strSQL) throws HibernateExceptionCD
    {
        return carregarPorSQLComParametros(nomeConexao, strSQL, null);
    }

    public static List carregarPorSQLComParametros(String nomeConexao, String strSQL, ParametroDAO parametrosConsulta) throws HibernateExceptionCD
    {
        return carregarPorSQLComParametros(nomeConexao, strSQL, parametrosConsulta, null);
    }

    public static List carregarPorSQLComParametros(String nomeConexao, String strSQL, ParametroDAO parametrosConsulta, ResultTransformer transformadorDeResultado) throws HibernateExceptionCD
    {
        // Declaraes
        List lstResultado = null;

        if (strSQL != null && !"".equals(strSQL))
        {
            //exibirSQL(nomeConexao, strSQL);
            ConexaoJDBC conexao = new ConexaoJDBC(FabricaSessaoHibernate.obterSessao(nomeConexao));
            if (transformadorDeResultado == null)
            {
                lstResultado = conexao.recuperarComParametros(strSQL, parametrosConsulta);
            }
            else
            {
                lstResultado = conexao.recuperarComParametros(strSQL, parametrosConsulta, transformadorDeResultado);
            }
        }

        return lstResultado;
    }

    public static List carregarPorSQLComParametrosPorPagina(String nomeConexao, int intNumeroPagina, int intMaximoPagina, String strSQL, ParametroDAO parametrosConsulta) throws HibernateExceptionCD
    {
        // Declaraes
        List lstResultado = null;

        if (strSQL != null && !"".equals(strSQL))
        {
            //exibirSQL(nomeConexao, strSQL);
            ConexaoJDBC conexao = new ConexaoJDBC(FabricaSessaoHibernate.obterSessao(nomeConexao));
            lstResultado = conexao.recuperarComParametrosPorPagina(intNumeroPagina, intMaximoPagina, strSQL, parametrosConsulta);
        }

        return lstResultado;
    }

    public static List carregarPorSQLComParametrosPorPagina(String nomeConexao, int intNumeroPagina, int intMaximoPagina, String strSQL, ParametroDAO parametrosConsulta, ResultTransformer transformadorDeRegistro) throws HibernateExceptionCD
    {
        // Declaraes
        List lstResultado = null;

        if (strSQL != null && !"".equals(strSQL))
        {
            //exibirSQL(nomeConexao, strSQL);
            ConexaoJDBC conexao = new ConexaoJDBC(FabricaSessaoHibernate.obterSessao(nomeConexao));
            lstResultado = conexao.recuperarComParametrosPorPagina(intNumeroPagina, intMaximoPagina, strSQL, parametrosConsulta, transformadorDeRegistro);
        }

        return lstResultado;
    }

    public static Session obterSessao(String nomeConexao) throws HibernateExceptionCD
    {
        return FabricaSessaoHibernate.obterSessao(nomeConexao);
    }

    public static Session obterSessaoExclusiva(String nomeConexao) throws HibernateExceptionCD
    {
        return FabricaSessaoHibernate.obterSessaoExclusiva(nomeConexao);
    }

    public static void fecharSessaoExclusiva(String nomeConexao, Session sessaoExclusiva) throws HibernateExceptionCD
    {
        FabricaSessaoHibernate.fecharSessaoExclusiva(nomeConexao, sessaoExclusiva);
    }

    /**
     * Devolve uma sesso Hibernate caso exista uma j iniciada. NO CRIA UMA NOVA SESSO!!
     * @param nomeConexao
     * @return
     * @throws HibernateExceptionCD
     */
    public static Session obterSessaoIniciada(String nomeConexao) throws HibernateExceptionCD
    {
        return FabricaSessaoHibernate.obterSessaoIniciada(nomeConexao);
    }

    public static String obterPropriedadeSubstituicao(String nomeConexao, String chave) throws HibernateExceptionCD
    {
        return GerenciadorConfiguracaoHibernate.obterPropriedadeSubstituicao(nomeConexao, chave);
    }

    public static String obterDialeto(String nomeConexao) throws HibernateExceptionCD
    {
        return GerenciadorConfiguracaoHibernate.obterDialeto(nomeConexao);
    }

    public static boolean realizarFlushImediatamente(String nomeConexao) throws HibernateExceptionCD
    {
        return GerenciadorConfiguracaoHibernate.realizarFlushImediatamente(nomeConexao);
    }

    public static String obterNomeConexaoPadrao() throws HibernateExceptionCD
    {
        return GerenciadorConfiguracaoHibernate.obterNomeConexaoPadrao();
    }

    public static void iniciarTransacao(String nomeConexao) throws HibernateExceptionCD
    {
        FabricaTransacaoHibernate.iniciarTransacao(nomeConexao);
    }

    public static void realizarTransacao(String nomeConexao) throws HibernateExceptionCD
    {
        FabricaTransacaoHibernate.realizarTransacao(nomeConexao);
    }

    public static void desfazerTransacao(String nomeConexao) throws HibernateExceptionCD
    {
        FabricaTransacaoHibernate.desfazerTransacao(nomeConexao);
    }

    public static void fecharSessao(String nomeConexao) throws HibernateExceptionCD
    {
        FabricaSessaoHibernate.fecharSessao(nomeConexao);
    }

    public static String obterNomeConexaoTabela(String nomeTabela) throws HibernateExceptionCD
    {
        return GerenciadorConfiguracaoHibernate.obterNomeConexaoTabela(nomeTabela);
    }

    public static MetadadosDaConexao obterMetadadosConexao(String nomeConexao)
    {
        return new MetadadosDaConexao(FabricaSessaoHibernate.obterSessao(nomeConexao));
    }

    public static List<List<String>> obterNomesDasConexoes()
    {
        return GerenciadorConfiguracaoHibernate.obterNomesDasConexoes();
    }
}
