/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.hibernate;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Session;
import org.hibernate.engine.SessionFactoryImplementor;

public class MetadadosDaConexao
{
    /**
    * Logger for this class
    */
    private static final Log LOGGER = LogFactory.getLog(MetadadosDaConexao.class);

    private String url = null;
    private String nomeUsuario = null;
    private String nomeDriver = null;
    private String versaoDriver = null;
    private String nomeBancoDados = null;
    private String versaoBancoDados = null;

    MetadadosDaConexao(Session sessao)
    {
        DatabaseMetaData metadadosBancoDeDados = null;
        try
        {
            metadadosBancoDeDados = obterMetadadosDaSessao(sessao);
        }
        catch (Exception e)
        {
            LOGGER.warn("Ocorreu um erro ao tentar recuperar os metadados da conexao: " + e.getMessage());
        }

        if (metadadosBancoDeDados == null)
        {
            return;
        }

        try
        {
            this.url = metadadosBancoDeDados.getURL();
        }
        catch (SQLException e)
        {
            LOGGER.debug("No foi possvel obter a url do banco de dados atravs da conexo");
        }

        try
        {
            this.nomeUsuario = metadadosBancoDeDados.getUserName();
        }
        catch (SQLException e)
        {
            LOGGER.debug("No foi possvel obter o nome do usurio de banco de dados atravs da conexo");
        }

        try
        {
            this.nomeDriver = metadadosBancoDeDados.getDriverName();
        }
        catch (SQLException e)
        {
            LOGGER.debug("No foi possvel obter o nome do driver de banco de dados atravs da conexo");
        }

        try
        {
            this.versaoDriver = metadadosBancoDeDados.getDriverVersion();
        }
        catch (SQLException e)
        {
            LOGGER.debug("No foi possvel obter a verso do driver de banco de dados atravs da conexo");
        }

        try
        {
            this.nomeBancoDados = metadadosBancoDeDados.getDatabaseProductName();
        }
        catch (SQLException e)
        {
            LOGGER.debug("No foi possvel obter o nome do banco de dados atravs da conexo");
        }

        try
        {
            this.versaoBancoDados = metadadosBancoDeDados.getDatabaseProductVersion();
        }
        catch (SQLException e)
        {
            LOGGER.debug("No foi possvel obter a verso do banco de dados atravs da conexo");
        }
    }

    private DatabaseMetaData obterMetadadosDaSessao(Session sessao) throws SQLException
    {
        Connection conexao = sessao.connection();
        if (((SessionFactoryImplementor) sessao.getSessionFactory()).getDialect().toString().toLowerCase().contains("ingres"))
        {
            // Para recuperar o usurio REAL da conexo
            conexao.createStatement().execute("SET SESSION AUTHORIZATION INITIAL_USER");
        }

        DatabaseMetaData metadadosBancoDeDados = conexao.getMetaData();
        return metadadosBancoDeDados;
    }

    public String getUrl()
    {
        return url;
    }

    public String getNomeUsuario()
    {
        return nomeUsuario;
    }

    public String getNomeDriver()
    {
        return nomeDriver;
    }

    public String getVersaoDriver()
    {
        return versaoDriver;
    }

    public String getNomeBancoDados()
    {
        return nomeBancoDados;
    }

    public String getVersaoBancoDados()
    {
        return versaoBancoDados;
    }
}
