/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.NonUniqueResultException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.transform.ResultTransformer;

import br.gov.camara.biblioteca.exception.DataException;
import br.gov.camara.biblioteca.util.DataNova;
import br.gov.camara.biblioteca.util.StringUtil;
import br.gov.camara.hibernate.AliasLowerToMapResultTransformer;
import br.gov.camara.hibernate.AliasToMapResultTransformer;
import br.gov.camara.hibernate.HibernateUtilCD;
import br.gov.camara.hibernate.MetadadosDaConexao;
import br.gov.camara.negocio.exception.DAOException;

/**
 * Classe abstrata para DAOs
 */
public abstract class DAO
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(DAO.class.getName());

    // private Session hbsSession;
    protected final String strNomeClasse;

    protected final String nomeConexao;

    protected String dialeto = null;
    protected Boolean isDialetoIngres = null;
    protected Boolean isDialetoOracle = null;
    protected Boolean isDialetoMySQL = null;
    protected Boolean isDialetoH2 = null;

    protected String strCampoData = null;
    protected String strCampoDataNaoPreenchida = null;
    protected String strCampoDataNulo = null;

    protected final String TIPO_CAMPO_MES_TRUNCADO = "M";
    protected final String TIPO_CAMPO_ANO_TRUNCADO = "Y";

    private static final String METODO_NAO_IMPLEMENTADO = "Mtodo no implementado.";

    /**
     * Cria um novo objeto
     * 
     * @param strNomeClasse Para nomear a classe
     */
    public DAO(String strNomeClasse)
    {
        this.strNomeClasse = strNomeClasse;
        this.nomeConexao = null;
    }

    /**
     * Cria um novo objeto
     * 
     * @param strNomeClasse Para nomear a classe
     */
    public DAO(String strNomeClasse, String strNomeConexao)
    {
        this.strNomeClasse = strNomeClasse;
        this.nomeConexao = strNomeConexao;
    }

    /**
     * Abre uma sesso Hibernate e inicia uma transao (BeginTrans)
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void iniciarTransacao(String nomeConexao) throws DAOException
    {
        // Inicializa o ambiente de persistncia
        try
        {
            HibernateUtilCD.iniciarTransacao(nomeConexao);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, HibernateUtilCD.obterDialeto(nomeConexao));
        }
    }

    /**
     * Abre uma sesso Hibernate e inicia uma transao (BeginTrans)
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void iniciarTransacao() throws DAOException
    {
        iniciarTransacao(HibernateUtilCD.obterNomeConexaoPadrao());
    }

    /**
     * Fecha a sesso hibernate
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void desconectar(String nomeConexao) throws DAOException
    {
        try
        {
            HibernateUtilCD.fecharSessao(nomeConexao);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, HibernateUtilCD.obterDialeto(nomeConexao));
        }
    }

    /**
     * Fecha a sesso hibernate
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void desconectar() throws DAOException
    {
        desconectar(HibernateUtilCD.obterNomeConexaoPadrao());
    }

    /**
     * Fecha a sesso atual Hibernate e realiza uma transao (CommitTrans)
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void realizarTransacao(String nomeConexao) throws DAOException
    {
        // Realiza transao
        try
        {
            HibernateUtilCD.realizarTransacao(nomeConexao);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, HibernateUtilCD.obterDialeto(nomeConexao));
        }
    }

    /**
     * Fecha a sesso atual Hibernate e realiza uma transao (CommitTrans)
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void realizarTransacao() throws DAOException
    {
        // Realiza transao
        realizarTransacao(HibernateUtilCD.obterNomeConexaoPadrao());
    }

    /**
     * Fecha a sesso atual Hibernate e desfaz uma transao (RollbackTrans)
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void desfazerTransacao(String nomeConexao) throws DAOException
    {
        // Desfaz transao
        try
        {
            HibernateUtilCD.desfazerTransacao(nomeConexao);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, HibernateUtilCD.obterDialeto(nomeConexao));
        }
    }

    /**
     * Fecha a sesso atual Hibernate e desfaz uma transao (RollbackTrans)
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void desfazerTransacao() throws DAOException
    {
        // Desfaz transao
        desfazerTransacao(HibernateUtilCD.obterNomeConexaoPadrao());
    }

    /**
     * Obtm o total de registros
     * 
     * @param strConsulta String contendo a consulta para contar os registros
     * @return List Contendo os POJOs representando os registro obtidos
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @deprecated Utilize {@link obterTotalRegistrosPorHQL(String, ParametroDAO)}
     * @see 
     */
    public final int obterTotalRegistros(String strConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        int intRetorno = 0;

        try
        {
            // Recupera os dados
            intRetorno = ((Integer) obterQueryPorHQLComParametros(strConsulta, null).uniqueResult()).intValue();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return intRetorno;
    }

    /**
     * Obtm o total de registros
     * 
     * @param strConsulta String contendo a consulta para contar os registros
     * @return List Contendo os POJOs representando os registro obtidos
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @see 
     */
    public final int obterTotalRegistrosPorHQL(ManipuladorSql manipuladorSql) throws DAOException
    {
        return obterTotalRegistrosPorHQL(manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros());
    }

    public final int obterTotalRegistrosPorHQL(String strConsulta, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        Integer intResultado = (Integer) obterObjetoPorHQL(strConsulta, parametros);
        int retorno = 0;

        if (intResultado != null)
        {
            retorno = intResultado.intValue();
        }
        return retorno;
    }

    public final int obterTotalDeRegistrosPorSQL(ManipuladorSql manipuladorSql) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        Integer intResultado = (Integer) obterObjetoPorSQL(manipuladorSql, TransformadorGenericoDeMapaDeInteiroParaInteiro.INSTANCE);
        int retorno = 0;

        if (intResultado != null)
        {
            retorno = intResultado.intValue();
        }
        return retorno;
    }

    /**
     * Obtm registros de acordo com o filtro especificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * @param strConsulta Contendo a consulta
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @deprecated Use {@link obterListaPorHQL(String, ParametroDAO)} ou {@link obterObjetoPorHQL(String, ParametroDAO)}
     */
    protected final List obter(String strConsulta) throws DAOException
    {
        return obterListaPorHQL(strConsulta, null);
    }

    /**
     * Obtm registros de acordo com o filtro especificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * @param strConsulta Contendo a consulta
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final List obterListaPorHQL(String strConsulta, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List resultado = null;

        try
        {
            // Recupera os dados
            Query query = obterQueryPorHQLComParametros(strConsulta, parametros);

            resultado = query.list();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return resultado;
    }

    protected final List obterListaPorHQL(ManipuladorSql manipuladorSql) throws DAOException
    {
        return obterListaPorHQL(manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros());
    }

    protected final List obterListaComMapasPorHQL(String strConsulta, ParametroDAO parametros) throws DAOException
    {
        return obterListaComMapasPorHQLPorPagina(1, 0, strConsulta, parametros);
    }

    protected final List obterListaComMapasPorHQLPorPagina(int intNumeroPagina, int intMaximoPagina, String strConsulta, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List lstRetorno = null;

        try
        {
            // Recupera os dados
            Query qryConsulta = obterQueryPorHQLComParametros(strConsulta, parametros);

            if (log.isDebugEnabled())
            {
                log.debug("Conexo: '" + this.nomeConexao + "' - Consulta: " + strConsulta);
            }

            if (intMaximoPagina > 0)
            {
                qryConsulta.setMaxResults(intMaximoPagina);
                if (intNumeroPagina < 1)
                {
                    intNumeroPagina = 1;
                }
                qryConsulta.setFirstResult((intNumeroPagina - 1) * intMaximoPagina);
            }
            qryConsulta.setResultTransformer(AliasLowerToMapResultTransformer.INSTANCE);

            lstRetorno = qryConsulta.list();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return lstRetorno;
    }

    /**
     * Obtm UM registro de acordo com o filtro especificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * @param strConsulta Contendo a consulta
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final Object obterObjetoPorHQL(String strConsulta, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        Object resultado = null;

        try
        {
            // Recupera os dados
            Query query = obterQueryPorHQLComParametros(strConsulta, parametros);

            resultado = query.uniqueResult();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return resultado;
    }

    protected final Object obterObjetoPorHQL(ManipuladorSql manipuladorSql) throws DAOException
    {
        // Declaraes
        Object resultado = null;

        try
        {
            // Recupera os dados
            Query query = obterQueryPorHQLComParametros(manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros());

            resultado = query.uniqueResult();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return resultado;
    }

    protected final Object obterObjetoPorSQL(ManipuladorSql manipuladorSql, ResultTransformer transformadorDeResultado) throws DAOException
    {
        // Declaraes
        Object resultado = null;

        try
        {
            // Declaraes
            List resultados = null;

            // Executa SQL
            if (transformadorDeResultado == null)
            {
                resultados = HibernateUtilCD.carregarPorSQLComParametros(this.nomeConexao, manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros());
            }
            else
            {
                resultados = HibernateUtilCD.carregarPorSQLComParametros(this.nomeConexao, manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros(), transformadorDeResultado);
            }

            resultado = elementoUnico(resultados);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return resultado;
    }

    /**
     * Obtm UM registro de acordo com o filtro especificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * @param strConsulta Contendo a consulta
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     * @deprecated Deixou de ser usado porque, quando no realizvamos o flush imediatamente, obtnhamos um erro hibernate to tipo
     * "org.hibernate.AssertionFailure: possible nonthreadsafe access to session"
     */
    protected final Object obterObjetoPorHQLComBloqueio(String strConsulta, ParametroDAO parametros, String aliasDoObjetoABloquear) throws DAOException
    {
        DAOException.dispararExcecao("Este mtodo causa um erro e, por isso, no deve mais ser usado.");

        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        Object resultado = null;

        try
        {
            // Recupera os dados
            Query query = obterQueryPorHQLComParametros(strConsulta, parametros, aliasDoObjetoABloquear);

            resultado = query.uniqueResult();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return resultado;
    }

    /**
     * Obtm os registros de determinada pgina, de acordo com o filtro especificado
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * @param strConsulta Contendo a consulta
     * @return List contendo o resultado da pesquisa
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final List obterPorPagina(int intNumeroPagina, int intMaximoPagina, String strConsulta) throws DAOException
    {
        return obterPorPaginaComParametro(intNumeroPagina, intMaximoPagina, strConsulta, null);
    }

    /**
     * Obtm os registros de determinada pgina, de acordo com o filtro especificado
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * @param strConsulta Contendo a consulta
     * @return List contendo o resultado da pesquisa
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final List obterPorPaginaComParametro(int intNumeroPagina, int intMaximoPagina, ManipuladorSql manipuladorSql) throws DAOException
    {
        return obterPorPaginaComParametro(intNumeroPagina, intMaximoPagina, manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros());
    }

    /**
     * Obtm os registros de determinada pgina, de acordo com o filtro especificado
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * @param strConsulta Contendo a consulta
     * @return List contendo o resultado da pesquisa
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final List obterPorPaginaComParametro(int intNumeroPagina, int intMaximoPagina, String strConsulta, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List lstRetorno = null;

        try
        {
            // Recupera os dados
            Query qryConsulta = obterQueryPorHQLComParametros(strConsulta, parametros);

            if (log.isDebugEnabled())
            {
                log.debug("Conexo: '" + this.nomeConexao + "' - Consulta: " + strConsulta);
            }

            if (intMaximoPagina > 0)
            {
                qryConsulta.setMaxResults(intMaximoPagina);
                if (intNumeroPagina < 1)
                {
                    intNumeroPagina = 1;
                }
                qryConsulta.setFirstResult((intNumeroPagina - 1) * intMaximoPagina);
            }

            lstRetorno = qryConsulta.list();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return lstRetorno;
    }

    /**
     * Obtm todos os registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @deprecated
     */
    public List obterTodos() throws DAOException
    {
        throw new DAOException(METODO_NAO_IMPLEMENTADO);
    }

    /**
     * Obtm um registro a partir da chave
     * 
     * @param strChave Chave do registro a ser obtido
     * @return Object POJO representando o registro obtido
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @deprecated Utilize {@link obterPelaChave(Object)}
     */
    public Object obterPelaChave(String strChave) throws DAOException
    {
        throw new DAOException(METODO_NAO_IMPLEMENTADO);
    }

    /**
     * Obtm um registro a partir da chave
     * 
     * @param objPOJO POJO contendo o(s) identificador(es) preenchidos para a recuperao do objeto
     * @return Object POJO representando o registro obtido
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public Object obterPelaChave(Object objPOJO) throws DAOException
    {
        throw new DAOException(METODO_NAO_IMPLEMENTADO);
    }

    /**
     * Verifica se  necessrio obter sesso do Hibernate, e chama a implementao de incluso
     * 
     * @param objPOJO POJO representando o objeto a ser includo
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @return retorna a chave do objeto que foi incluido
     */
    public final Serializable incluir(Object objPOJO) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        Serializable idGerado = null;

        try
        {
            Session hbsSession = HibernateUtilCD.obterSessao(this.nomeConexao);

            // Grava este novo objeto
            idGerado = hbsSession.save(objPOJO);

            if (HibernateUtilCD.realizarFlushImediatamente(this.nomeConexao))
            {
                // Executa os comandos pendentes
                hbsSession.flush();
            }
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return idGerado;
    }

    public final <T> T carregarPeloId(Class<T> tipo, Serializable id) throws DAOException
    {
        try
        {
            Session hbsSession = HibernateUtilCD.obterSessao(this.nomeConexao);

            // Grava este novo objeto
            return (T) hbsSession.get(tipo, id);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return null;
    }

    /**
     * Verifica se  necessrio obter sesso do Hibernate, e chama a implementao de alterao
     * 
     * @param objPOJO POJO representando o objeto a ser alterado
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public final void alterar(Object objPOJO) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            // Obtm sesso
            Session hbsSession = HibernateUtilCD.obterSessao(this.nomeConexao);

            // Atualiza objeto
            hbsSession.update(objPOJO);

            if (HibernateUtilCD.realizarFlushImediatamente(this.nomeConexao))
            {
                // Fora a execuo do comando SQL
                hbsSession.flush();
            }
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }
    }

    /**
     * Verifica se  necessrio obter sesso do Hibernate, e chama a implementao de excluso
     * 
     * @param objPOJO POJO representando o objeto a ser excludo
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public final void excluir(Object objPOJO) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            // Obtm sesso
            Session hbsSession = HibernateUtilCD.obterSessao(this.nomeConexao);

            // Inicia transao
            String strConsulta = this.excluirImpl(objPOJO);

            // Despersiste objeto
            if (strConsulta != null)
            {
                log.warn("A excluso chamando 'excluirImpl' no deve ser utilizada porque os INTERCEPTORs de operacao (como LOG) no so chamados por este mecanismo. Query: "
                        + strConsulta);
                this.excluir(strConsulta);
            }
            else
            {
                hbsSession.delete(objPOJO);
                if (HibernateUtilCD.realizarFlushImediatamente(this.nomeConexao))
                {
                    // Fora a execuo do comando SQL 
                    hbsSession.flush();
                }
            }

        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }
    }

    /**
     * Verifica se  necessrio obter sesso do Hibernate, e efetua uma excluso partir de um HQL passado
     * 
     * @param objPOJO POJO representando o objeto a ser excludo
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @deprecated No deve ser utilizado porque os INTERCEPTORs de OPERAO (como log) no so disparados com este mecanismo.
     */
    protected final void excluir(String strConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            // Obtm sesso
            Session hbsSession = HibernateUtilCD.obterSessao(this.nomeConexao);

            // Exclui
            if (strConsulta.indexOf("DELETE ") == -1)
            {
                strConsulta = "DELETE " + strConsulta;
            }

            Query query = obterQueryPorHQLComParametros(strConsulta, null);
            if (query != null)
            {
                log.warn("A excluso chamando 'excluir(String)' no deve ser utilizada porque os INTERCEPTORs de operacao (como LOG) no so chamados por este mecanismo. Query: "
                        + strConsulta);
                query.executeUpdate();
            }

            if (log.isDebugEnabled())
            {
                log.debug("Conexo: '" + this.nomeConexao + "' - Consulta: " + strConsulta);
            }

            if (HibernateUtilCD.realizarFlushImediatamente(this.nomeConexao))
            {
                // Fora a execuo do comando SQL
                hbsSession.flush();
            }
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }
    }

    /**
     * Executa um comando SQL informado como parmetroo
     * 
     * @param objPOJO POJO representando o SQL a ser executado
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @deprecated Utilize preferencial {@link executarSQLComParametros(String, ParametroDAO)}
     */
    public final int executarSQL(String objPOJO) throws DAOException
    {
        int retorno = 0;
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            // Executa SQL
            retorno = HibernateUtilCD.executarSQL(this.nomeConexao, objPOJO);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return retorno;
    }

    /**
     * Executa um comando SQL informado como parmetroo
     * 
     * @param objPOJO POJO representando o SQL a ser executado
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public final int executarSQLComParametros(String objPOJO, ParametroDAO parametros) throws DAOException
    {
        int retorno = 0;
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            // Executa SQL
            retorno = HibernateUtilCD.executarSQLComParametros(this.nomeConexao, objPOJO, parametros);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return retorno;
    }

    public final int executarSQLComParametros(ManipuladorSql manipuladorSql) throws DAOException
    {
        int retorno = 0;
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            // Executa SQL
            retorno = HibernateUtilCD.executarSQLComParametros(this.nomeConexao, manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros());
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return retorno;
    }

    /**
     * Mtodo que monta um comando SQL para a excluso do registro relacionado com o objeto informado no parmetro
     * 
     * @param SituacaoServidor POJO representando o objeto a ser excludo
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @return String com o comando SQL para excluir o registro relacionado com o parmetro
     * @deprecated No deve ser utilizado porque os INTERCEPTORs de OPERAO (como log) no so disparados com este mecanismo.
     */
    protected String excluirImpl(Object objPOJO) throws DAOException
    {
        // Retorna null por default para informar que a excluso no ser feita por SQL 
        return null;
    }

    /**
     * Inicializa o relacionamento do objeto passado
     * 
     * @param objRelacionado Objeto desejado
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final void inicializarRelacionamento(Object objRelacionado) throws DAOException
    {
        try
        {
            Hibernate.initialize(objRelacionado);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }
    }

    /**
     * Informa se o relacionamento est ou no iniciliazado
     * 
     * @param objRelacionado Objeto desejado
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final boolean estaInicializado(Object objRelacionado) throws DAOException
    {
        boolean retorno = false;
        try
        {
            retorno = Hibernate.isInitialized(objRelacionado);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he);
        }

        return retorno;
    }

    /**
     * Obtm registros de acordo com um comando SQL especificado
     * 
     * @return List Contendo os POJOs representando os registros obtidos
     * @param strConsulta SQL contendo a consulta
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * @deprecated Utilize preferencialmente {@link #obterPorSQLComParametros(String, ParametroDAO)}
     */
    public final List obterPorSQL(String strConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List lstResultado = null;

        try
        {
            // Executa SQL
            lstResultado = HibernateUtilCD.carregarPorSQL(this.nomeConexao, strConsulta);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return lstResultado;
    }

    /**
     * Obtm registros de acordo com um comando SQL especificado e respectivos parmetros
     * 
     * @param strConsulta SQL contendo a consulta
     * @param parametros Parmetros da consulta
     * @return List Contendo os POJOs representando os registros obtidos
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public final List obterPorSQLComParametros(String strConsulta, ParametroDAO parametros) throws DAOException
    {
        return obterPorSQLComParametros(strConsulta, parametros, null);
    }

    public final List obterPorSQLComParametros(ManipuladorSql manipuladorSql) throws DAOException
    {
        return obterPorSQLComParametros(manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros(), null);
    }

    public final List obterPorSQLComParametros(ManipuladorSql manipuladorSql, ResultTransformer transformadorDeResultado) throws DAOException
    {
        return obterPorSQLComParametros(manipuladorSql.gerarSqlFinal(), manipuladorSql.getParametros(), transformadorDeResultado);
    }

    public final List obterPorSQLComParametros(String strConsulta, ParametroDAO parametros, ResultTransformer transformadorDeResultado) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List lstResultado = null;

        try
        {
            // Executa SQL
            if (transformadorDeResultado == null)
            {
                lstResultado = HibernateUtilCD.carregarPorSQLComParametros(this.nomeConexao, strConsulta, parametros);
            }
            else
            {
                lstResultado = HibernateUtilCD.carregarPorSQLComParametros(this.nomeConexao, strConsulta, parametros, transformadorDeResultado);
            }
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return lstResultado;
    }

    public final List obterListaPorSQLComParametrosPorPagina(int intNumeroPagina, int intMaximoPagina, ManipuladorSql sql, ResultTransformer transformadorDeResultado) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List lstRetorno = null;

        try
        {
            // Executa SQL
            if (transformadorDeResultado == null)
            {
                lstRetorno = HibernateUtilCD.carregarPorSQLComParametrosPorPagina(this.nomeConexao, intNumeroPagina, intMaximoPagina, sql.gerarSqlFinal(), sql.getParametros());
            }
            else
            {
                lstRetorno = HibernateUtilCD.carregarPorSQLComParametrosPorPagina(this.nomeConexao, intNumeroPagina, intMaximoPagina, sql.gerarSqlFinal(), sql.getParametros(), transformadorDeResultado);
            }
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return lstRetorno;
    }

    public final List obterListaComMapasPorSQLComParametrosPorPagina(int intNumeroPagina, int intMaximoPagina, String strConsulta, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List lstRetorno = null;

        try
        {
            // Executa SQL
            lstRetorno = HibernateUtilCD.carregarPorSQLComParametrosPorPagina(this.nomeConexao, intNumeroPagina, intMaximoPagina, strConsulta, parametros);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return lstRetorno;
    }

    public final String realizarSubstituicao(String mascara, String... parametros)
    {
        return StringUtil.realizarSubstituicao(mascara, parametros);
    }

    public final String obterPropriedadeSubstituicao(String chave)
    {
        return HibernateUtilCD.obterPropriedadeSubstituicao(this.nomeConexao, chave);
    }

    public boolean isDialetoTratado() throws DAOException
    {
        boolean dialetoTratado = false;
        dialetoTratado = (this.isIngres() ? true : dialetoTratado || false);
        dialetoTratado = (this.isOracle() ? true : dialetoTratado || false);
        dialetoTratado = (this.isMySQL() ? true : dialetoTratado || false);
        dialetoTratado = (this.isH2() ? true : dialetoTratado || false);
        return dialetoTratado;
    }

    public final String obterCampoDataAtual() throws DAOException
    {
        if (this.strCampoData == null)
        {
            if (this.isIngres())
            {
                this.strCampoData = "Date('today')";
            }
            else if (this.isOracle())
            {
                this.strCampoData = "trunc(SYSDATE)"; // "TO_DATE(TO_CHAR(SYSDATE, 'DD/MM/YYYY'), 'DD/MM/YYYY')"; // "TO_CHAR(SYSDATE, 'DD/MM/YYYY')";
            }
            else if (this.isMySQL())
            {
                this.strCampoData = "STR_TO_DATE(DATE_FORMAT(NOW(), '%d/%m/%Y'), '%d/%m/%Y')"; // "DATE_FORMAT(NOW(), '%d/%m/%Y')";
            }
            else if (this.isH2())
            {
                this.strCampoData = "CURRENT_DATE()";
            }
            else
            {
                throw new DAOException("O dialeto Hibernate em uso, " + obterDialeto() + ", ainda no  tratado por este mtodo.");
            }
        }

        return this.strCampoData;
    }

    public final String obterCampoDataHoraAtual() throws DAOException
    {
        if (this.strCampoData == null)
        {
            if (this.isIngres())
            {
                this.strCampoData = "Date('now')";
            }
            else if (this.isOracle())
            {
                this.strCampoData = "SYSDATE";
            }
            else if (this.isMySQL())
            {
                this.strCampoData = "STR_TO_DATE(DATE_FORMAT(NOW(), '%d/%m/%Y %H:%i:%s'), '%d/%m/%Y %H:%i:%s')"; // "DATE_FORMAT(NOW(), '%d/%m/%Y')";
            }
            else if (this.isH2())
            {
                this.strCampoData = "CURRENT_TIMESTAMP()";
            }
            else
            {
                throw new DAOException("O dialeto Hibernate em uso, " + obterDialeto() + ", ainda no  tratado por este mtodo.");
            }
        }

        return this.strCampoData;
    }

    public final String obterCampoDataNaoPreenchida() throws DAOException
    {
        if (this.strCampoDataNaoPreenchida == null)
        {
            if (this.isIngres())
            {
                this.strCampoDataNaoPreenchida = "''";
            }
            else if (this.isOracle())
            {
                this.strCampoDataNaoPreenchida = "null";
            }
            else if (this.isMySQL())
            {
                this.strCampoDataNaoPreenchida = "null";
            }
            else if (this.isH2())
            {
                this.strCampoDataNaoPreenchida = "null";
            }
            else
            {
                throw new DAOException("O dialeto Hibernate em uso, " + obterDialeto() + ", ainda no  tratado por este mtodo.");
            }
        }

        return this.strCampoDataNaoPreenchida;
    }

    public final String obterCampoDataTruncado(String nomeCampo, String tipoCampo) throws DAOException
    {
        String retorno = null;

        if (this.isIngres())
        {
            if (TIPO_CAMPO_MES_TRUNCADO.equals(tipoCampo))
            {
                retorno = "date_trunc('month', " + nomeCampo + ")";
            }
            else if (TIPO_CAMPO_ANO_TRUNCADO.equals(tipoCampo))
            {
                retorno = "date_trunc('year', " + nomeCampo + ")";
            }
        }
        else if (this.isOracle())
        {
            if (TIPO_CAMPO_MES_TRUNCADO.equals(tipoCampo))
            {
                retorno = "trunc(" + nomeCampo + ", 'MM')";
            }
            else if (TIPO_CAMPO_ANO_TRUNCADO.equals(tipoCampo))
            {
                retorno = "trunc(" + nomeCampo + ", 'YYYY')";
            }
        }
        else if (this.isH2())
        {
            if (TIPO_CAMPO_MES_TRUNCADO.equals(tipoCampo))
            {
                retorno = "PARSEDATETIME(FORMATDATETIME(" + nomeCampo + ", '01/MM/yyyy'), 'dd/MM/yyyy')";
            }
            else if (TIPO_CAMPO_ANO_TRUNCADO.equals(tipoCampo))
            {
                retorno = "PARSEDATETIME(FORMATDATETIME(" + nomeCampo + ", '01/01/yyyy'), 'dd/MM/yyyy')";
            }
        }

        if (retorno == null)
        {
            throw new DAOException("O dialeto Hibernate em uso, " + obterDialeto() + ", ou o tipo de campo informado, ainda no  tratado por este mtodo.");
        }

        return retorno;
    }

    public final String obterCampoDataNulo() throws DAOException
    {
        if (this.strCampoDataNulo == null)
        {
            if (this.isIngres())
            {
                this.strCampoDataNulo = "date(null)";
            }
            else if (this.isOracle())
            {
                this.strCampoDataNulo = "To_Date(null)";
            }
            else if (this.isMySQL())
            {
                this.strCampoDataNulo = "CAST(null AS DATE)";
            }
            else if (this.isH2())
            {
                this.strCampoDataNulo = "CAST(null AS DATE)";
            }
            else
            {
                throw new DAOException("O dialeto Hibernate em uso, " + obterDialeto() + ", ainda no  tratado por este mtodo.");
            }
        }

        return this.strCampoDataNulo;
    }

    protected final String obterDialeto()
    {
        if (this.dialeto == null)
        {
            this.dialeto = HibernateUtilCD.obterDialeto(this.nomeConexao);
        }

        return this.dialeto;
    }

    private boolean isIngres()
    {
        if (this.isDialetoIngres == null)
        {
            if (obterDialeto().toLowerCase().indexOf("ingres") > -1)
            {
                this.isDialetoIngres = Boolean.TRUE;
                this.isDialetoOracle = Boolean.FALSE;
                this.isDialetoMySQL = Boolean.FALSE;
                this.isDialetoH2 = Boolean.FALSE;
            }
            else
            {
                this.isDialetoIngres = Boolean.FALSE;
            }
        }
        return this.isDialetoIngres;
    }

    private boolean isOracle()
    {
        if (this.isDialetoOracle == null)
        {
            if (obterDialeto().toLowerCase().indexOf("oracle") > -1)
            {
                this.isDialetoIngres = Boolean.FALSE;
                this.isDialetoOracle = Boolean.TRUE;
                this.isDialetoMySQL = Boolean.FALSE;
                this.isDialetoH2 = Boolean.FALSE;
            }
            else
            {
                this.isDialetoOracle = Boolean.FALSE;
            }
        }
        return this.isDialetoOracle;
    }

    private boolean isMySQL()
    {
        if (this.isDialetoMySQL == null)
        {
            if (obterDialeto().toLowerCase().indexOf("mysql") > -1)
            {
                this.isDialetoIngres = Boolean.FALSE;
                this.isDialetoOracle = Boolean.FALSE;
                this.isDialetoMySQL = Boolean.TRUE;
                this.isDialetoH2 = Boolean.FALSE;
            }
            else
            {
                this.isDialetoMySQL = Boolean.FALSE;
            }
        }
        return this.isDialetoMySQL;
    }

    private boolean isH2()
    {
        if (this.isDialetoH2 == null)
        {
            if (obterDialeto().toLowerCase().indexOf("h2") > -1)
            {
                this.isDialetoIngres = Boolean.FALSE;
                this.isDialetoOracle = Boolean.FALSE;
                this.isDialetoMySQL = Boolean.FALSE;
                this.isDialetoH2 = Boolean.TRUE;
            }
            else
            {
                this.isDialetoH2 = Boolean.FALSE;
            }
        }
        return this.isDialetoH2;
    }

    private String formatarParametroDate(Date datParametro, boolean comHora) throws DAOException
    {
        String strFormato = null;
        String retorno = null;

        if (this.isIngres())
        {
            strFormato = "MM/dd/yyyy";
            if (comHora)
            {
                strFormato += " hh:mm:ss";
            }
        }
        else if (this.isOracle())
        {
            strFormato = "dd/MM/yyyy";
            if (comHora)
            {
                strFormato += " hh:mm:ss";
            }
        }
        else if (this.isMySQL())
        {
            strFormato = "yyyy-MM-dd";
            if (comHora)
            {
                strFormato += " HH:mm:ss";
            }
        }
        else if (this.isH2())
        {
            strFormato = "yyyy-MM-dd";
            if (comHora)
            {
                strFormato += " HH:mm:ss";
            }
        }
        else
        {
            throw new DAOException("O dialeto Hibernate em uso, " + obterDialeto() + ", ainda no  tratado por este mtodo.");
        }

        try
        {
            retorno = DataNova.formatarData(datParametro, strFormato);
        }
        catch (DataException de)
        {
            throw new DAOException("A data informada (" + datParametro + ") no  vlida.");
        }
        return retorno;
    }

    public final String formatarParametroData(Date datParametro) throws DAOException
    {
        return this.formatarParametroDate(datParametro, false);
    }

    public final String formatarParametroDataHora(Date datParametro) throws DAOException
    {
        return this.formatarParametroDate(datParametro, true);
    }

    public static final String obterNomeConexaoTabela(String nomeTabela) throws DAOException
    {
        return HibernateUtilCD.obterNomeConexaoTabela(nomeTabela);
    }

    /**
     * Executa a query HQL especificada e retorna quantos registros foram afetados
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * @param strConsulta Contendo a consulta
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final int executarHQL(String strConsulta) throws DAOException
    {
        return executarHQLComParametros(strConsulta, null);
    }

    /**
     * Executa a query HQL especificada e retorna quantos registros foram afetados
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * @param strConsulta Contendo a consulta
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    protected final int executarHQLComParametros(String strConsulta, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        int lstRetorno = 0;

        try
        {
            // Recupera os dados
            Query query = obterQueryPorHQLComParametros(strConsulta, parametros);
            lstRetorno = query.executeUpdate();
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return lstRetorno;
    }

    /**
     * Obtm registros de acordo com um comando SQL especificado e respectivos parmetros
     * 
     * @param strConsulta SQL contendo a consulta
     * @param parametros Parmetros da consulta
     * @return List Contendo os POJOs representando os registros obtidos
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    private Query obterQueryPorHQLComParametros(String consultaHQL, ParametroDAO parametros) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        Query query = null;

        try
        {
            Session hbsSession = HibernateUtilCD.obterSessao(this.nomeConexao);

            if (log.isDebugEnabled())
            {
                log.debug("Conexo: '" + this.nomeConexao + "' - Consulta: " + consultaHQL);
            }

            query = criarQuery(consultaHQL, parametros, hbsSession);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return query;
    }

    private Query criarQuery(String consultaHQL, ParametroDAO parametros, Session hbsSession)
    {
        Query query = hbsSession.createQuery(consultaHQL);

        if (parametros != null && !parametros.isEmpty())
        {
            query = parametros.setarParametroQuery(query);
        }

        return query;
    }

    /**
     * Obtm registros de acordo com um comando SQL especificado e respectivos parmetros
     * 
     * @param strConsulta SQL contendo a consulta
     * @param parametros Parmetros da consulta
     * @return List Contendo os POJOs representando os registros obtidos
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     * @deprecated Deixou de ser usado porque, quando no realizvamos o flush imediatamente, obtnhamos um erro hibernate to tipo
     * "org.hibernate.AssertionFailure: possible nonthreadsafe access to session"
     */
    private Query obterQueryPorHQLComParametros(String consultaHQL, ParametroDAO parametros, String aliasDoObjetoABloquear) throws DAOException
    {
        DAOException.dispararExcecao("Este mtodo causa um erro e, por isso, no deve mais ser usado.");

        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        Query query = null;

        try
        {
            Session hbsSession = HibernateUtilCD.obterSessao(this.nomeConexao);

            if (log.isDebugEnabled())
            {
                log.debug("Conexo: '" + this.nomeConexao + "' - Consulta: " + consultaHQL);
            }

            query = criarQuery(consultaHQL, parametros, hbsSession);

            if (aliasDoObjetoABloquear != null)
            {
                query.setLockMode(aliasDoObjetoABloquear, LockMode.UPGRADE);
            }
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, obterDialeto());
        }

        return query;
    }

    /**
     * Remove a instncia de um objeto do cache do hibernate.
     * ATENO: ateno ao utilizar esta soluo, pois se existe alguma atualizao pendente que ainda no foi "commitada"
     * (sobretudo com a utilizao do !realizarFlushImediato), a atualizao no ser persistida no banco de dados.
     * 
     * @param SituacaoServidor POJO representando o objeto a ser alterado
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public final void desconectarObjetoDaPersistenciaIgnorandoQualquerOperacaoPendente(Object objPOJO) throws DAOException
    {
        desconectarObjetoDaPersistenciaIgnorandoQualquerOperacaoPendente(this.nomeConexao, objPOJO);
    }

    /**
     * Remove a instncia de um objeto do cache do hibernate.
     * ATENO: ateno ao utilizar esta soluo, pois se existe alguma atualizao pendente que ainda no foi "commitada"
     * (sobretudo com a utilizao do !realizarFlushImediato), a atualizao no ser persistida no banco de dados.
     * 
     * @param SituacaoServidor POJO representando o objeto a ser alterado
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public static final void desconectarObjetoDaPersistenciaIgnorandoQualquerOperacaoPendente(String nomeConexao, Object objPOJO) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            // Obtm sesso
            Session hbsSession = HibernateUtilCD.obterSessao(nomeConexao);

            // Remover do cache da sesso hibernate
            hbsSession.evict(objPOJO);
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, HibernateUtilCD.obterDialeto(nomeConexao));
        }
    }

    protected String obterNomeConexao()
    {
        return this.nomeConexao;
    }

    public static final void efetivarOperacoesPendentes(String nomeConexao) throws DAOException
    {
        try
        {
            // Obtm sesso
            Session hbsSession = HibernateUtilCD.obterSessaoIniciada(nomeConexao);
            if (hbsSession != null)
            {
                hbsSession.flush();
            }
        }
        catch (HibernateException he)
        {
            DAOException.dispararExcecao(he, HibernateUtilCD.obterDialeto(nomeConexao));
        }
    }

    public static final List<List<String>> obterNomesDasConexoes()
    {
        return HibernateUtilCD.obterNomesDasConexoes();
    }

    public static final MetadadosDaConexao obterMetadadosDaConexao(String nomeConexao)
    {
        return HibernateUtilCD.obterMetadadosConexao(nomeConexao);
    }

    protected static Object elementoUnico(List list) throws NonUniqueResultException
    {
        int size = list.size();
        if (size == 0)
            return null;
        Object first = list.get(0);
        for (int i = 1; i < size; i++)
        {
            if (list.get(i) != first)
            {
                throw new NonUniqueResultException(list.size());
            }
        }
        return first;
    }

    protected static class TransformadorGenericoDeMapaDeInteiroParaInteiro extends AliasToMapResultTransformer
    {
        public static final AliasToMapResultTransformer INSTANCE = new TransformadorGenericoDeMapaDeInteiroParaInteiro();

        private static final long serialVersionUID = 1L;

        @Override
        public Integer transformarRegistro(Map<String, Object> registro)
        {
            if (registro.size() > 1)
            {
                throw new IllegalArgumentException("O registro recuperado no pode ter mais de uma coluna para utilizar este conversor genrico");
            }

            Integer retorno = null;

            for (Entry<String, Object> colunaValor : registro.entrySet())
            {
                Object valor = colunaValor.getValue();
                if (valor == null)
                {
                    continue;
                }

                if (!(valor instanceof Number))
                {
                    throw new IllegalArgumentException("A coluna recuperada deve ser do tipo numrico para poder utilizar este conversor genrico");
                }

                retorno = ((Number) valor).intValue();
            }

            return retorno;
        }
    }

    protected static class TransformadorGenericoDeMapaDeLongParaLong extends AliasToMapResultTransformer
    {
        public static final AliasToMapResultTransformer INSTANCE = new TransformadorGenericoDeMapaDeLongParaLong();

        private static final long serialVersionUID = 1L;

        @Override
        public Long transformarRegistro(Map<String, Object> registro)
        {
            if (registro.size() > 1)
            {
                throw new IllegalArgumentException("O registro recuperado no pode ter mais de uma coluna para utilizar este conversor genrico");
            }

            Long retorno = null;

            for (Entry<String, Object> colunaValor : registro.entrySet())
            {
                Object valor = colunaValor.getValue();
                if (valor == null)
                {
                    continue;
                }

                if (!(valor instanceof Number))
                {
                    throw new IllegalArgumentException("A coluna recuperada deve ser do tipo numrico para poder utilizar este conversor genrico");
                }

                retorno = ((Number) valor).longValue();
            }

            return retorno;
        }
    }

    protected static class TransformadorGenericoDeMapaDeStringParaString extends AliasToMapResultTransformer
    {
        public static final AliasToMapResultTransformer INSTANCE = new TransformadorGenericoDeMapaDeStringParaString();

        private static final long serialVersionUID = 1L;

        @Override
        public String transformarRegistro(Map<String, Object> registro)
        {
            if (registro.size() > 1)
            {
                throw new IllegalArgumentException("O registro recuperado no pode ter mais de uma coluna para utilizar este conversor genrico");
            }

            String retorno = null;

            for (Entry<String, Object> colunaValor : registro.entrySet())
            {
                Object valor = colunaValor.getValue();
                if (valor == null)
                {
                    continue;
                }

                if (!(valor instanceof String))
                {
                    throw new IllegalArgumentException("A coluna recuperada deve ser do tipo String para poder utilizar este conversor genrico");
                }

                retorno = (String) valor;
            }

            return retorno;
        }
    }

}
