/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.hibernate.type.DateType;

import br.gov.camara.hibernate.exception.HibernateExceptionCD;
import br.gov.camara.negocio.exception.DAOException;

public class ParametroDAO extends HashMap<Object, Object>
{
    /**
    * Logger for this class
    */
    private static Log log = LogFactory.getLog(ParametroDAO.class);

    public ParametroDAO()
    {}

    public ParametroDAO(String id, Object valor) throws DAOException
    {
        adicionar(id, valor);
    }

    /**
     * 
     */
    private static final long serialVersionUID = -6305931874878925483L;

    // -1 -> Tipo de parmetro indefinido
    // 0 -> No definido, ou seja, ainda no utilizado
    // 1 -> Nomeado
    // 2 -> Posicional
    private int tipoParametro = 0;

    public ParametroDAO adicionar(String id, Object valor) throws DAOException
    {
        if (tipoParametro == 0)
        {
            tipoParametro = 1;
        }
        else if (tipoParametro != 1)
        {
            DAOException.dispararExcecao("No se pode misturar os tipos de parmetros.");
        }

        if (log.isDebugEnabled())
        {
            log.debug("Identificador: " + id + " - Valor: " + valor);
        }

        super.put(id, valor);
        return this;
    }

    public ParametroDAO adicionar(Integer id, Object valor) throws DAOException
    {
        if (tipoParametro == 0)
        {
            tipoParametro = 2;
        }
        else if (tipoParametro != 2)
        {
            DAOException.dispararExcecao("No se pode misturar os tipos de parmetros.");
        }

        if (log.isDebugEnabled())
        {
            log.debug("Identificador: " + id + " - Valor: " + valor);
        }

        super.put(id, valor);
        return this;
    }

    public Object put(Object id, Object valor)
    {
        Object retorno = null;

        if (id instanceof String)
        {
            try
            {
                retorno = adicionar((String) id, valor);
            }
            catch (DAOException e)
            {
                throw new RuntimeException(e.getMensagem());
            }
        }
        else if (id instanceof Integer)
        {
            try
            {
                retorno = adicionar((Integer) id, valor);
            }
            catch (DAOException e)
            {
                throw new RuntimeException(e.getMensagem());
            }
        }
        else
        {
            if (tipoParametro == 0)
            {
                tipoParametro = -1;
            }
            else if (tipoParametro != -1)
            {
                throw new RuntimeException("No se pode misturar os tipos de parmetros dentro de um ParametroDAO.");
            }

            if (log.isDebugEnabled())
            {
                log.debug("Identificador: " + id + " - Valor: " + valor);
            }
            retorno = super.put(id, valor);
        }

        return retorno;
    }

    public Query setarParametroQuery(Query query)
    {
        return setarParametroQuery(query, this);
    }

    public static Query setarParametroQuery(Query query, ParametroDAO parametros)
    {
        Set<Entry<Object, Object>> entrySet = (parametros == null ? null : parametros.entrySet());

        if (entrySet == null || entrySet.isEmpty())
        {
            return query;
        }

        //obtem os parametros caso algum tenha sido passado
        Iterator<Entry<Object, Object>> it = entrySet.iterator();
        Query queryComParametros = null;
        while (it.hasNext())
        {
            Entry<Object, Object> entry = it.next();
            queryComParametros = setarParametroQuery(query, entry.getKey(), entry.getValue());
        }
        return queryComParametros;
    }

    private static Query setarParametroQuery(Query query, Object chave, Object valor)
    {
        try
        {
            if (chave instanceof Integer || chave instanceof Long)
            {
                // Posicional
                int parameterIndex = Integer.parseInt(chave.toString());
                query.setParameter(parameterIndex - 1, valor);
            }
            else if (chave instanceof String)
            {
                // Por nome
                String parameterName = chave.toString();
                Date valorData = null;
                boolean pendenteTratamentoData = false;
                if (valor instanceof Collection)
                {
                    query.setParameterList(parameterName, (Collection) valor);
                }
                else if (valor instanceof Object[])
                {
                    query.setParameterList(parameterName, (Object[]) valor);
                }
                else if (valor instanceof java.sql.Date)
                {
                    query.setParameter(parameterName, valor, new DateType());
                }
                else if (valor instanceof Calendar)
                {
                    valorData = ((Calendar) valor).getTime();
                    pendenteTratamentoData = true;
                }
                else if (valor instanceof Date)
                {
                    valorData = (Date) valor;
                    pendenteTratamentoData = true;
                }
                else
                {
                    query.setParameter(parameterName, valor);
                }

                if (pendenteTratamentoData)
                {
                    if (valorData.getHours() != 0 || valorData.getMinutes() != 0 || valorData.getSeconds() != 0)
                    {
                        // Tratar utilizando o padro do Hibernate
                        query.setParameter(parameterName, valor);
                    }
                    else
                    {
                        // Forar a indicao de data
                        query.setParameter(parameterName, valorData, new DateType());
                    }
                }
            }
            else
            {
                // Tipo de parmetro no tratado
                throw new HibernateExceptionCD("O tipo de chave informada como parmetro ainda no est sendo tratado");
            }
        }
        catch (Throwable throwable)
        {
            log.error("Erro ao setar parametro " + chave + " com o valor " + valor);
            HibernateExceptionCD.dispararExcecao(throwable);
            //System.out.print(true);
        }

        return query;
    }
}
