/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.autenticacaoperfil.facade;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import br.gov.camara.biblioteca.util.Copia;
import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.autenticacaoperfil.dao.FuncionalidadeSistemaDAO;
import br.gov.camara.negocio.autenticacaoperfil.pojo.FuncionalidadeSistema;
import br.gov.camara.negocio.autenticacaoperfil.pojo.FuncionalidadeSistemaVisualizacao;
import br.gov.camara.negocio.autenticacaoperfil.pojo.Sistema;
import br.gov.camara.negocio.autenticacaoperfil.pojo.UsuarioSistema;

/**
 * Facade para atributo de talento
 */
public class FuncionalidadeSistemaFacade extends Facade
{
    private final int ELEMENTO_SUPERIOR = 1;
    private final int ELEMENTO_INFERIOR = 2;
    private List lstEncaminhamento = new ArrayList();
    private List lstAcao = new ArrayList();

    // Variveis de instncia
    private static Log log = LogFactory.getLog(FuncionalidadeSistemaFacade.class.getName());

    public FuncionalidadeSistemaFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param objFuncionalidadeSistema POJO contendo a chave de pesquisa
     * 
     * @return FuncionalidadeSistema POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public FuncionalidadeSistema obterPelaChave(FuncionalidadeSistema objFuncionalidadeSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        FuncionalidadeSistema objRetorno = null;
        try
        {
            objRetorno = (FuncionalidadeSistema) objFuncionalidadeSistemaDAO.obterPelaChave(objFuncionalidadeSistema);
            objFuncionalidadeSistemaDAO.inicializarSistema(objRetorno);
            if (objRetorno.getFuncionalidadeAgrupadora() != null)
            {
                objFuncionalidadeSistemaDAO.inicializarFuncionalidadeAgrupadora(objRetorno);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objRetorno;
    }

    /**
     * Inclui um registro, verificando se o usurio passado  administrador do sistema ou se o objeto 
     * controlado j est cadastrado
     *
     c     * @param objUsuarioSistema POJO representando os dados do usurio que ser validado como administrador do sistema
     *
     * @return retorna a chave do objeto que foi incluido
    
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public String incluir(FuncionalidadeSistema objFuncionalidadeSistema, UsuarioSistema objUsuarioSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        String strChave = null;

        // Instancia DAO e inclui registro
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        GestorSistemaFacade objGestorSistemaFacade = new GestorSistemaFacade(obterNomeConexao());
        try
        {
            // Verifica se o usurio  administrador do sistema
            if (!objGestorSistemaFacade.isAdministradorSistema(objFuncionalidadeSistema.getSistema(), objUsuarioSistema))
            {
                throw new CDException("Somente administradores deste sistema podem incluir uma Funcionalidade");
            }
            // Verifica se o objeto controlado existe
            else if (objFuncionalidadeSistemaDAO.obterPeloObjetoControlado(objFuncionalidadeSistema.getObjetoControlado()) != null)
            {
                throw new CDException("J existe uma funcionalidade com o Objeto controlado informado");
            }

            // Realiza incluso
            strChave = objFuncionalidadeSistemaDAO.incluir(objFuncionalidadeSistema).toString();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return strChave;
    }

    /**
     * Altera um registro, verificando se o usurio passado  administrador do sistema ou se o objeto 
     * controlado j est cadastrado
     *
     * @param objFuncionalidadeSistema POJO representando o objeto a ser alterado
     * @param objUsuarioSistema POJO representando os dados do usurio que ser validado como administrador do sistema
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterar(FuncionalidadeSistema objFuncionalidadeSistema, UsuarioSistema objUsuarioSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e altera registro
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        GestorSistemaFacade objGestorSistemaFacade = new GestorSistemaFacade(obterNomeConexao());
        try
        {
            // Verifica se o usurio  administrador do sistema
            if (!objGestorSistemaFacade.isAdministradorSistema(objFuncionalidadeSistema.getSistema(), objUsuarioSistema))
            {
                throw new CDException("Somente administradores deste sistema podem alterar uma Funcionalidade");
            }
            // Verifica se o objeto controlado existe
            else if (objFuncionalidadeSistemaDAO.isObjetoControladoCadastrado(objFuncionalidadeSistema))
            {
                throw new CDException("J existe uma funcionalidade com o Objeto controlado informado");
            }

            // Realiza alterao
            objFuncionalidadeSistemaDAO.alterar(objFuncionalidadeSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Altera a funcionalidade agrupadora das funcionalidades passadas na lista, 
     * verificando se o usurio passado  administrador do sistema ou se o objeto 
     * controlado j est cadastrado
     *
     * @param lstFuncionalidades Lista de funcionalidades que tero seu agrupador alterado 
     * @param objFuncAgrupadora POJO representando a funcionalidade agrupadora a ser atribuda
     * @param objUsuarioSistema POJO representando os dados do usurio que ser validado como administrador do sistema
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterarFuncionalidadeSuperior(List lstFuncionalidades, FuncionalidadeSistema objFuncAgrupadora, UsuarioSistema objUsuarioSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            DAO.iniciarTransacao(obterNomeConexao());
            for (int i = 0; i < lstFuncionalidades.size(); i++)
            {
                FuncionalidadeSistema objFunc = (FuncionalidadeSistema) lstFuncionalidades.get(i);

                // Obtm objeto a ser alterado
                FuncionalidadeSistema objFuncionalidadeSistema = obterPelaChave(objFunc);
                // Seta o agrupador
                objFuncionalidadeSistema.setFuncionalidadeAgrupadora(objFuncAgrupadora);
                // Realiza alterao 
                alterar(objFuncionalidadeSistema, objUsuarioSistema);

                objFuncionalidadeSistema = null;
            }
            DAO.realizarTransacao(obterNomeConexao());

        }
        catch (Exception daoe)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Exclui um registro, verificando se o usurio passado  administrador do sistema 
     *
     * @param objFuncionalidadeSistema POJO representando o objeto a ser excludo
     * @param objUsuarioSistema POJO representando os dados do usurio que ser validado como administrador do sistema
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void excluir(FuncionalidadeSistema objFuncionalidadeSistema, UsuarioSistema objUsuarioSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e exclui registro
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        GestorSistemaFacade objGestorSistemaFacade = new GestorSistemaFacade(obterNomeConexao());
        try
        {
            // Verifica se o usurio  administrador do sistema
            if (!objGestorSistemaFacade.isAdministradorSistema(objFuncionalidadeSistema.getSistema(), objUsuarioSistema))
            {
                throw new CDException("Somente administradores deste sistema podem excluir uma Funcionalidade");
            }

            // Realiza excluso
            objFuncionalidadeSistemaDAO.excluir(objFuncionalidadeSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Realiza excluso de uma ou vrias funcionalidades, 
     * verificando se o usurio passado  administrador do sistema
     *
     * @param objUsuarioSistema POJO representando os dados do usurio que ser validado como administrador do sistema
     * @param strtFunc Objeto contendo as funcionalidades a serem excludas
     *
     * @return Lista de funcionalidades excludas
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List excluir(UsuarioSistema objUsuarioLogado, StringTokenizer strtFunc) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        List lstRetorno = new ArrayList();

        try
        {
            DAO.iniciarTransacao(obterNomeConexao());
            while (strtFunc.hasMoreElements())
            {
                FuncionalidadeSistema objFuncionalidadeSistema = new FuncionalidadeSistema();

                // Obtm funcionalidade a ser excluda
                Integer intIdentificador = new Integer(strtFunc.nextElement().toString());
                objFuncionalidadeSistema.setIdentificador(intIdentificador);
                objFuncionalidadeSistema = obterPelaChave(objFuncionalidadeSistema);

                // Realiza excluso da funcionalidade
                excluir(objFuncionalidadeSistema, objUsuarioLogado);
                lstRetorno.add(objFuncionalidadeSistema);
            }
            DAO.realizarTransacao(obterNomeConexao());
        }
        catch (Exception daoe)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm o total de registros
     * 
     * @param objFuncionalidadeSistema POJO contendo os parmetros da consulta
     * 
     * @return int Contendo o total de registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistros(FuncionalidadeSistema objFuncionalidadeSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objFuncionalidadeSistemaDAO.obterTotalRegistros(objFuncionalidadeSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm os registros de determinada pgina
     * 
     * @param intNumeroPagina Nmero da pgina a ser mostrada
     * @param intMaximoPagina Quantidade de registros por pgina
     * @param objFuncionalidadeSistema POJO contendo os parmetros da consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina, FuncionalidadeSistema objFuncionalidadeSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFuncionalidadeSistemaDAO.obterPorPagina(intNumeroPagina, intMaximoPagina, objFuncionalidadeSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm todas as funcionalidades
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTodos() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFuncionalidadeSistemaDAO.obterTodos();
        }
        catch (Exception e)
        {
            throw new CDException(e.getMessage(), e);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm as funcionalidades filhas de uma determinada funcionalidade
     * 
     * @param objFuncionalidadeSistema POJO contendo os parmetros da consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPelaFuncionalidadeAgrupadora(FuncionalidadeSistema objFuncionalidadeSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFuncionalidadeSistemaDAO.obterPelaFuncionalidadeAgrupadora(objFuncionalidadeSistema);
            lstRetorno = obterVisualizacao(lstRetorno);
        }
        catch (Exception e)
        {
            throw new CDException(e.getMessage(), e);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm as funcionalidades filhas de uma determinada funcionalidade
     * 
     * @param intIdentificador objeto contendo a chave do agrupador
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPelaFuncionalidadeAgrupadora(Integer intIdentificador) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFuncionalidadeSistemaDAO.obterPelaFuncionalidadeAgrupadora(intIdentificador);
            objFuncionalidadeSistemaDAO.inicializarSistema(lstRetorno);
            lstRetorno = obterVisualizacao(lstRetorno);
        }
        catch (Exception e)
        {
            throw new CDException(e.getMessage(), e);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm todas as funcionalidades quem no possuem pai (esto no topo da hierarquia)
     * 
     * @param objFuncionalidadeSistema objeto contendo os parmetros da consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPeloSistemaAgrupadorNull(FuncionalidadeSistema objFuncionalidadeSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFuncionalidadeSistemaDAO.obterPeloSistemaAgrupadorNull(objFuncionalidadeSistema);
            lstRetorno = obterVisualizacao(lstRetorno);
        }
        catch (Exception e)
        {
            throw new CDException(e.getMessage(), e);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm lista da consulta de acordo com parmetros passados
     * 
     * @param objFuncionalidadeSistema objeto contendo os parmetros da consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPelaConsulta(FuncionalidadeSistema objFuncionalidadeSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFuncionalidadeSistemaDAO.obterPelaConsulta(objFuncionalidadeSistema);
            lstRetorno = obterVisualizacao(lstRetorno);
        }
        catch (Exception e)
        {
            throw new CDException(e.getMessage(), e);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm objeto de visualizao
     * 
     * @param lstFuncionalidadeSistema objeto contendo as funcionalidades a serem copiadas
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    private List obterVisualizacao(List lstFuncionalidadeSistema) throws CDException
    {
        List lstRetorno = new ArrayList();
        for (int i = 0; i < lstFuncionalidadeSistema.size(); i++)
        {
            FuncionalidadeSistema objFuncionalidade = (FuncionalidadeSistema) lstFuncionalidadeSistema.get(i);
            FuncionalidadeSistemaVisualizacao objFuncionalidadeSistemaVisualizacao = new FuncionalidadeSistemaVisualizacao();
            Copia.criar(objFuncionalidade, objFuncionalidadeSistemaVisualizacao);
            objFuncionalidadeSistemaVisualizacao.setRetorno(objFuncionalidade.getIdentificador().intValue() + ";" + objFuncionalidade.getDescricao());
            lstRetorno.add(objFuncionalidadeSistemaVisualizacao);
        }
        return lstRetorno;
    }

    /**
     * 
     * Realiza importao das funcionalidades a partir de um arquivo XML
     * 
     * @param ipsArquivo Arquivo XML contendo os dados das funcionalidades a serem importadas
     * @param objUsuarioSistema Pojo contendo os dados do usurio a ser validado como administrador
     * @param lstErros Lista contendo as funcionalidades que deram erro durante a importao
     * @param lstInclusoes Lista contendo as funcionalidades que foram includas durante a  importao
     * @param lstExistentes Lista contendo as funcionalidades que j existem.
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public void importar(InputStream ipsArquivo, Sistema objSistema, UsuarioSistema objUsuarioSistema, List lstErros, List lstInclusoes, List lstExistentes) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        GestorSistemaFacade objGestorSistemaFacade = new GestorSistemaFacade(obterNomeConexao());
        try
        {
            // Verifica se o usurio  administrador do sistema
            if (!objGestorSistemaFacade.isAdministradorSistema(objSistema, objUsuarioSistema))
            {
                throw new CDException("Somente administradores deste sistema podem importar Funcionalidades");
            }

            //  Obtm XML escolhido
            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(ipsArquivo);

            // Recupera o primeiro nvel do XML
            Element root = doc.getRootElement();

            DAO.iniciarTransacao(obterNomeConexao());

            FuncionalidadeSistema objFuncionalidadeModulo = obterFuncionalidadeModulo(root);

            gravarFuncionalidades(root.getChildren("mapeamentos"), objFuncionalidadeModulo, objSistema, objFuncionalidadeModulo, lstErros, lstInclusoes, lstExistentes);

            if (lstErros.size() > 0)
            {
                DAO.desfazerTransacao(obterNomeConexao());
            }
            else
            {
                DAO.realizarTransacao(obterNomeConexao());
            }
        }
        catch (CDException cde)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            throw cde;
        }
        catch (IOException e)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            throw new CDException("Erro ao processar arquivo XML");
        }
        catch (JDOMException e)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            throw new CDException("Erro ao processar arquivo XML");
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * 
     * Grava as funcionalidades importadas do arquivo XML
     * 
     * @param lstFilhos Lista contendo os filhos de um determinado n da arvore
     * @param objFuncionalidadeAgrupadora Funcionalidade pai/superior da lista de filhos
     * @param objSistema POJO contendo o sistema das funcionalidades
     * @param objFuncionalidadeModulo Funcionalidade mdulo da importao
     * @param lstErros Lista contendo as funcionalidades que deram erro durante a importao
     * @param lstInclusoes Lista contendo as funcionalidades que foram includas durante a  importao
     * @param lstExistentes Lista contendo as funcionalidades que j existem.
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    private void gravarFuncionalidades(List lstFilhos, FuncionalidadeSistema objFuncAgrupadora, Sistema objSistema, FuncionalidadeSistema objFuncionalidadeModulo, List lstErros, List lstInclusoes, List lstExistentes) throws CDException
    {
        if (lstFilhos != null)
        {
            try
            {
                FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
                for (int i = 0; i < lstFilhos.size(); i++)
                {
                    FuncionalidadeSistema objFunc = new FuncionalidadeSistema();
                    Element mapeamento = (Element) lstFilhos.get(i);
                    if (mapeamento.getName().equals("mapeamentos"))
                    {
                        gravarFuncionalidades(mapeamento.getChildren("mapeamento"), objFuncAgrupadora, objSistema, objFuncionalidadeModulo, lstErros, lstInclusoes, lstExistentes);
                        break;
                    }

                    objFunc.setNome(!"".equals(mapeamento.getAttributeValue("nome")) ? mapeamento.getAttributeValue("nome") : mapeamento.getAttributeValue("objetoControlado"));
                    objFunc.setDescricao(!"".equals(mapeamento.getAttributeValue("descricao")) ? mapeamento.getAttributeValue("descricao") : mapeamento.getAttributeValue("objetoControlado"));
                    objFunc.setObjetoControlado(mapeamento.getAttributeValue("objetoControlado"));
                    objFunc.setFuncionalidadeAgrupadora(objFuncAgrupadora);
                    objFunc.setSistema(objFuncionalidadeModulo.getSistema());

                    // Inclui, caso o objeto controlado ainda no exista
                    FuncionalidadeSistema objFuncionalidadeExistente = objFuncionalidadeSistemaDAO.obterPeloObjetoControlado(mapeamento.getAttributeValue("objetoControlado"));
                    if (objFuncionalidadeExistente == null && objFunc.getObjetoControlado() != null)
                    {
                        objFuncionalidadeSistemaDAO.incluir(objFunc);
                        lstInclusoes.add(objFunc);
                    }
                    else
                    {
                        FuncionalidadeSistema objFuncionalidadeCadastrada = obterModulo(objFuncionalidadeExistente, objFuncionalidadeModulo);
                        if ((objFuncionalidadeCadastrada != null
                                && objFuncionalidadeModulo.getIdentificador().intValue() == objFuncionalidadeCadastrada.getIdentificador().intValue())
                                || objFuncionalidadeCadastrada == null)
                        {
                            lstExistentes.add(objFuncionalidadeExistente);
                        }
                        else
                        {
                            objFuncionalidadeExistente.setDescricao(objFuncionalidadeCadastrada.getNome());
                            objFuncionalidadeSistemaDAO.inicializarSistema(objFuncionalidadeExistente);
                            lstErros.add(objFuncionalidadeExistente);
                        }
                        objFunc = objFuncionalidadeExistente;
                    }
                    gravarFuncionalidades(mapeamento.getChildren("mapeamento"), objFunc, objSistema, objFuncionalidadeModulo, lstErros, lstInclusoes, lstExistentes);
                }
            }
            catch (CDException cde)
            {
                DAO.desfazerTransacao(obterNomeConexao());
                throw cde;
            }
            finally
            {
                DAO.desconectar(obterNomeConexao());
            }
        }
    }

    /**
     * 
     * Obtm a funcionalidade mdulo de acordo com os parmetros
     * 
     * @param objFuncionalidadeExistente POJO contendo os dados da funcionalidade existente
     * @param objFuncionalidadeModulo POJO contendo os dados da funcionalidade mdulo
     *
     * @return FuncionalidadeSistema POJO contendo os dados obtidos na consulta
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    private FuncionalidadeSistema obterModulo(FuncionalidadeSistema objFuncionalidadeExistente, FuncionalidadeSistema objFuncionalidadeModulo) throws CDException
    {
        FuncionalidadeSistema objRetorno = null;
        try
        {
            FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());

            if (objFuncionalidadeExistente != null)
            {
                while (objFuncionalidadeExistente.getFuncionalidadeAgrupadora() != null)
                {
                    objFuncionalidadeExistente = (FuncionalidadeSistema) objFuncionalidadeSistemaDAO.obterPelaChave(objFuncionalidadeExistente.getFuncionalidadeAgrupadora());
                }
                objRetorno = objFuncionalidadeExistente;
            }
        }
        catch (CDException cde)
        {
            throw cde;
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return objRetorno;
    }

    /**
     * 
     * Obtm a funcionalidade mdulo de acordo com parametro
     * 
     * @param root elemento do XML que contm a descrio do mdulo a ser consultado
     *
     * @return FuncionalidadeSistema POJO contendo os dados obtidos na consulta
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    private FuncionalidadeSistema obterFuncionalidadeModulo(Element root) throws CDException
    {
        // Declaraes
        FuncionalidadeSistema objFuncionalidadeSistema = new FuncionalidadeSistema();

        // Instancia DAO e obtm os registros da pgina
        FuncionalidadeSistemaDAO objFuncionalidadeSistemaDAO = new FuncionalidadeSistemaDAO(obterNomeConexao());
        try
        {
            String strModulo = root.getChild("modulo").getAttributeValue("objetoControlado");

            objFuncionalidadeSistema = objFuncionalidadeSistemaDAO.obterPeloObjetoControladoAgrupadorNull(strModulo);
            if (objFuncionalidadeSistema == null)
            {
                throw new CDException("No existe uma funcionalidade com o nome do mdulo para a devida associao");
            }
            objFuncionalidadeSistemaDAO.inicializarSistema(objFuncionalidadeSistema);
        }
        catch (CDException cde)
        {
            throw cde;
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objFuncionalidadeSistema;

    }

    /**
     * 
     * Adiciona um encaminhamendo ao XML
     * 
     * @param strNome String que contm o valor a ser atribudo ao nome
     * @param strDescricao String que contm o valor a ser atribudo ao descrio
     * @param strObjetoControlado String que contm o valor a ser atribudo ao objetoControlado
     * 
     */
    public void adicionarEncaminhamento(String strNome, String strDescricao, String strObjetoControlado)
    {
        lstEncaminhamento.add(new String[] { strNome, strDescricao, strObjetoControlado });
    }

    /**
     * 
     * Adiciona uma ao ao XML
     * 
     * @param strNome String que contm o valor a ser atribudo ao nome
     * @param strDescricao String que contm o valor a ser atribudo ao descrio
     * @param strObjetoControlado String que contm o valor a ser atribudo ao objetoControlado
     * 
     */
    public void adicionarAcao(String strNome, String strDescricao, String strObjetoControlado)
    {
        lstAcao.add(new String[] { strNome, strDescricao, strObjetoControlado });
    }

    /**
     * 
     * Cria XML a partir de uma funcionalidade e sua hierarquia
     * 
     * @param objFuncionalidadeSistema POJO representando o objeto a ser obtido
     * 
     * @return Document Objeto XML obtido pela consulta
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public Document obterDocumentoXML(FuncionalidadeSistema objFuncionalidadeSistema) throws CDException
    {
        Document docXML = null;
        try
        {

            // cria um elemento 
            Element sistema = new Element("sistema");

            //cria uma instncia no documento
            docXML = new Document(sistema);

            FuncionalidadeSistema objFunc = obterPelaChave(objFuncionalidadeSistema);
            Element objElementEncaminhamento = adicionarMapeamento(objFunc, null, ELEMENTO_SUPERIOR);

            // Obtm encaminhamentos inferiores
            List lstFilhos = obterPelaFuncionalidadeAgrupadora(objFunc.getIdentificador());

            obterFilhos(lstFilhos, objElementEncaminhamento);

            // Obtm os encaminhamentos superiores
            if (objFunc.getFuncionalidadeAgrupadora() != null)
            {
                do
                {
                    objFunc = obterPelaChave(objFunc.getFuncionalidadeAgrupadora());
                    if (objFunc.getFuncionalidadeAgrupadora() != null)
                    {
                        objElementEncaminhamento = adicionarMapeamento(objFunc, objElementEncaminhamento, ELEMENTO_SUPERIOR);
                    }
                }
                while (objFunc.getFuncionalidadeAgrupadora() != null);
            }

            // cria a aplicao
            Element objElementAplicacao = new Element("aplicacao");
            objElementAplicacao.setAttribute(new Attribute("nome", objFuncionalidadeSistema.getSistema().getDescricao()));
            sistema.addContent(objElementAplicacao);

            // cria o mdulo
            Element objElementModulo = new Element("modulo");
            objElementModulo.setAttribute(new Attribute("nome", objFunc.getNome()));
            objElementModulo.setAttribute(new Attribute("descricao", objFunc.getDescricao()));
            objElementModulo.setAttribute(new Attribute("objetoControlado", objFunc.getObjetoControlado()));
            sistema.addContent(objElementModulo);

            Element objElementMapeamentos = new Element("mapeamentos");
            objElementMapeamentos.addContent(objElementEncaminhamento);

            sistema.addContent(objElementMapeamentos);

        }
        catch (Exception e)
        {
            throw new CDException(e.getMessage(), e);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return docXML;
    }

    /**
     * 
     * Obtm os filhos de um elemento no XML 
     * 
     * @param lstFilhos Obtm os filhos de uma lista de funcionalidades recursivamente
     * @param objElement Elemento do XML
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    private void obterFilhos(List lstFilhos, Element objElement) throws CDException
    {
        if (lstFilhos != null)
        {
            for (int i = 0; i < lstFilhos.size(); i++)
            {
                FuncionalidadeSistema objFuncionalidadeSistema = (FuncionalidadeSistema) lstFilhos.get(i);
                Element objElementFilha = adicionarMapeamento(objFuncionalidadeSistema, objElement, ELEMENTO_INFERIOR);
                List lstNetos = obterPelaFuncionalidadeAgrupadora(objFuncionalidadeSistema.getIdentificador());
                obterFilhos(lstNetos, objElementFilha);
            }
        }
    }

    /**
     * 
     * Adiciona um elemento ao XML
     * 
     * @param lstFilhos Obtm os filhos de uma lista de funcionalidades recursivamente
     * @param objElement Elemento do XML
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    private Element adicionarMapeamento(FuncionalidadeSistema objFuncionalidadeSistema, Element objElement, int intTipoAdicao)
    {
        Element objRetorno = new Element("mapeamento");
        objRetorno.setAttribute(new Attribute("nome", objFuncionalidadeSistema.getNome()));
        objRetorno.setAttribute(new Attribute("descricao", objFuncionalidadeSistema.getDescricao()));
        objRetorno.setAttribute(new Attribute("objetoControlado", objFuncionalidadeSistema.getObjetoControlado()));
        if (objElement != null)
        {
            if (intTipoAdicao == ELEMENTO_SUPERIOR)
            {
                objRetorno.addContent(objElement);
            }
            else if (intTipoAdicao == ELEMENTO_INFERIOR)
            {
                objElement.addContent(objRetorno);
            }
        }

        return objRetorno;
    }

}
