/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.autenticacaoperfil.facade;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.autenticacaoperfil.dao.AgrupadorPerfilDAO;
import br.gov.camara.negocio.autenticacaoperfil.dao.PerfilSistemaAgrupadorPerfilDAO;
import br.gov.camara.negocio.autenticacaoperfil.dao.PerfilSistemaDAO;
import br.gov.camara.negocio.autenticacaoperfil.dao.SistemaDAO;
import br.gov.camara.negocio.autenticacaoperfil.pojo.AgrupadorPerfil;
import br.gov.camara.negocio.autenticacaoperfil.pojo.AgrupadorPerfilUsuario;
import br.gov.camara.negocio.autenticacaoperfil.pojo.PerfilSistema;
import br.gov.camara.negocio.autenticacaoperfil.pojo.PerfilSistemaAgrupadorPerfil;
import br.gov.camara.negocio.autenticacaoperfil.pojo.UsuarioSistema;

/**
 * Facade para atributo de talento
 */
public class PerfilSistemaAgrupadorPerfilFacade extends Facade
{
    // Variveis de instncia
    private static Log log = LogFactory.getLog(PerfilSistemaAgrupadorPerfilFacade.class.getName());

    /**
     * @param nomeConexao
     *
     */
    public PerfilSistemaAgrupadorPerfilFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistros() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objPerfilSistemaAgrupadorPerfilDAO.obterTotalRegistros();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm os registros de determinada pgina
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objPerfilSistemaAgrupadorPerfilDAO.obterPorPagina(intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return PerfilSistemaAgrupadorPerfil POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public PerfilSistemaAgrupadorPerfil obterPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        PerfilSistemaAgrupadorPerfil objPerfilSistemaAgrupadorPerfil = null;
        try
        {
            objPerfilSistemaAgrupadorPerfil = (PerfilSistemaAgrupadorPerfil) objPerfilSistemaAgrupadorPerfilDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objPerfilSistemaAgrupadorPerfil;
    }

    /**
     * Inclui um registro
     *
     * @param objPerfilSistemaAgrupadorPerfil POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     * @return retorna a chave do objeto que foi incluido
     *
     */
    public String incluir(PerfilSistemaAgrupadorPerfil objPerfilSistemaAgrupadorPerfil) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        String strChave = null;

        // Instancia Facade
        AgrupadorPerfilFacade objAgrupadorPerfilFacade = new AgrupadorPerfilFacade(obterNomeConexao());
        UsuarioCriterioAcessoPerfilFacade objUsuarioCriterioAcessoPerfilFacade = null;

        // Instancia DAO e inclui registro
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        try
        {
            // Obtm os usurios associados ao agrupador para associ-los ao critrio de acesso
            List lstUsuarios = objAgrupadorPerfilFacade.obterUsuariosAssociados(objPerfilSistemaAgrupadorPerfil.getAgrupadorPerfil());
            if (lstUsuarios != null && lstUsuarios.size() > 0)
            {
                objUsuarioCriterioAcessoPerfilFacade = new UsuarioCriterioAcessoPerfilFacade(obterNomeConexao());
                for (int i = 0; i < lstUsuarios.size(); i++)
                {
                    AgrupadorPerfilUsuario objAgrupadorPerfilUsuario = (AgrupadorPerfilUsuario) lstUsuarios.get(i);
                    objUsuarioCriterioAcessoPerfilFacade.incluir(objPerfilSistemaAgrupadorPerfil.getPerfilSistema(), objAgrupadorPerfilUsuario.getUsuarioSistema());
                }
            }

            strChave = objPerfilSistemaAgrupadorPerfilDAO.incluir(objPerfilSistemaAgrupadorPerfil).toString();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return strChave;
    }

    /**
     * Altera um registro
     *
     * @param objPerfilSistemaAgrupadorPerfil POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterar(PerfilSistemaAgrupadorPerfil objPerfilSistemaAgrupadorPerfil) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e altera registro
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        try
        {
            objPerfilSistemaAgrupadorPerfilDAO.alterar(objPerfilSistemaAgrupadorPerfil);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Exclui um registro
     *
     * @param objPerfilSistemaAgrupadorPerfil POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void excluir(PerfilSistemaAgrupadorPerfil objPerfilSistemaAgrupadorPerfil) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia Facade
        AgrupadorPerfilFacade objAgrupadorPerfilFacade = new AgrupadorPerfilFacade(obterNomeConexao());
        UsuarioCriterioAcessoPerfilFacade objUsuarioCriterioAcessoPerfilFacade = null;

        // Instancia DAO e exclui registro
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        try
        {
            // Obtm os usurios associados ao agrupador para associ-los ao critrio de acesso
            List lstUsuarios = objAgrupadorPerfilFacade.obterUsuariosAssociados(objPerfilSistemaAgrupadorPerfil.getAgrupadorPerfil());
            if (lstUsuarios != null && lstUsuarios.size() > 0)
            {
                objUsuarioCriterioAcessoPerfilFacade = new UsuarioCriterioAcessoPerfilFacade(obterNomeConexao());
                for (int i = 0; i < lstUsuarios.size(); i++)
                {
                    AgrupadorPerfilUsuario objAgrupadorPerfilUsuario = (AgrupadorPerfilUsuario) lstUsuarios.get(i);
                    objUsuarioCriterioAcessoPerfilFacade.excluir(objPerfilSistemaAgrupadorPerfil.getPerfilSistema(), objAgrupadorPerfilUsuario.getUsuarioSistema());
                }
            }

            objPerfilSistemaAgrupadorPerfilDAO.excluir(objPerfilSistemaAgrupadorPerfil);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Obtm os registros de determinada pgina
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTodos() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objPerfilSistemaAgrupadorPerfilDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistrosPerfisDisponiveis(PerfilSistemaAgrupadorPerfil objPerfilSistemaAgrupadorPerfil, UsuarioSistema objUsuarioSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objPerfilSistemaAgrupadorPerfilDAO.obterTotalRegistrosPerfisDisponiveis(objPerfilSistemaAgrupadorPerfil, objUsuarioSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    public List obterPerfisDisponiveisPorPagina(PerfilSistemaAgrupadorPerfil objPerfilSistemaAgrupadorPerfil, UsuarioSistema objUsuarioSistema, int intPagina, int intTotalRegistros) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        PerfilSistemaDAO objPerfilSistemaDAO = new PerfilSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objPerfilSistemaAgrupadorPerfilDAO.obterPerfisDisponiveisPorPagina(objPerfilSistemaAgrupadorPerfil, objUsuarioSistema, intPagina, intTotalRegistros);
            if (lstRetorno != null && lstRetorno.size() > 0)
            {
                for (int i = 0; i < lstRetorno.size(); i++)
                {
                    objPerfilSistemaDAO.inicializarPerfilSistema((PerfilSistema) lstRetorno.get(i));
                }
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistrosPerfisSelecionados(AgrupadorPerfil objAgrupadorPerfil) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objPerfilSistemaAgrupadorPerfilDAO.obterTotalRegistrosPerfisSelecionados(objAgrupadorPerfil);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    public List obterPerfisSelecionadosPorPagina(AgrupadorPerfil objAgrupadorPerfil, int intPagina, int intTotalRegistros) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        PerfilSistemaDAO objPerfilSistemaDAO = new PerfilSistemaDAO(obterNomeConexao());
        SistemaDAO objSistemaDAO = new SistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objPerfilSistemaAgrupadorPerfilDAO.obterPerfisSelecionadosPorPagina(objAgrupadorPerfil, intPagina, intTotalRegistros);
            if (lstRetorno != null && lstRetorno.size() > 0)
            {
                for (int i = 0; i < lstRetorno.size(); i++)
                {
                    objPerfilSistemaDAO.inicializarPerfilSistema((PerfilSistema) lstRetorno.get(i));
                    objSistemaDAO.inicializarSistema(((PerfilSistema) lstRetorno.get(i)).getSistema());
                }
            }

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    public void atualizarPerfis(String strOperacao, PerfilSistemaAgrupadorPerfil objPerfilSistemaAgrupadorPerfil) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            if ("E".equals(strOperacao))
            {
                excluir(objPerfilSistemaAgrupadorPerfil);
            }
            else if ("I".equals(strOperacao))
            {
                incluir(objPerfilSistemaAgrupadorPerfil);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

    }

    public List obterPeloPerfil(PerfilSistema objPerfilSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        PerfilSistemaAgrupadorPerfilDAO objPerfilSistemaAgrupadorPerfilDAO = new PerfilSistemaAgrupadorPerfilDAO(obterNomeConexao());
        AgrupadorPerfilDAO objAgrupadorPerfilDAO = new AgrupadorPerfilDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objPerfilSistemaAgrupadorPerfilDAO.obterPeloPerfil(objPerfilSistema);
            if (lstRetorno != null && lstRetorno.size() > 0)
            {
                for (int i = 0; i < lstRetorno.size(); i++)
                {
                    objAgrupadorPerfilDAO.inicializarAgrupadorPerfil(((PerfilSistemaAgrupadorPerfil) lstRetorno.get(i)).getAgrupadorPerfil());
                }
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }
}
