/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.autenticacaoperfil.facade;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.biblioteca.util.Copia;
import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.autenticacaoperfil.dao.UsuarioSistemaDAO;
import br.gov.camara.negocio.autenticacaoperfil.pojo.UsuarioSistema;
import br.gov.camara.negocio.autenticacaoperfil.pojo.UsuarioSistemaVisualizacao;

/**
 * Facade para a exibio do currculo de determinada pessoa
 * 
 */
public class UsuarioSistemaFacade extends Facade
{
    // Variveis de instncia
    private static Log log = LogFactory.getLog(UsuarioSistemaFacade.class.getName());

    /**
     * @param nomeConexao
     *
     */
    public UsuarioSistemaFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Obtm o usurio pela chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return UsuarioSistema POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public UsuarioSistema obterPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        UsuarioSistema objUsuarioSistema = null;
        try
        {
            objUsuarioSistema = (UsuarioSistema) objUsuarioSistemaDAO.obterPelaChave(strChave);
            if (objUsuarioSistema != null)
            {
                objUsuarioSistemaDAO.inicializarGrupo(objUsuarioSistema);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objUsuarioSistema;
    }

    /**
     * Obtm o usurio pela chave
     *
     * @param strLogin login do registro a ser obtido
     * 
     * @return UsuarioSistema POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public UsuarioSistema obterPeloLogin(String strLogin) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        UsuarioSistema objUsuarioSistema = null;
        try
        {
            objUsuarioSistema = (UsuarioSistema) objUsuarioSistemaDAO.obterPeloLogin(strLogin);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objUsuarioSistema;
    }

    /**
     * Obtm lista de registros pelo nome por pagina  
     *
     * @param String nome a ser consultado
     * @param int nmero da pgina a ser consultada
     * @param int mximo de registro por pgina
     * 
     * @return List Lista com os POJOs representando o registros obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPeloNomePorPagina(String strNome, int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        List lstUsuarioSistema = null;
        try
        {
            lstUsuarioSistema = objUsuarioSistemaDAO.obterPeloNomePorPagina(strNome, intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstUsuarioSistema;
    }

    /**
     * Obtm todos os registro por pgina 
     *
     * @param int nmero da pgina a ser consultada
     * @param int mximo de registro por pgina
     * 
     * @return List Lista com os POJOs representando o registros obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTodosPorPagina(int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        List lstUsuarioSistema = null;
        try
        {
            lstUsuarioSistema = objUsuarioSistemaDAO.obterTodosPorPagina(intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstUsuarioSistema;
    }

    /**
     * Obtm total de registro
     *
     * @return int Total de registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistros() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objUsuarioSistemaDAO.obterTotalRegistros();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm total de registro pelo nome
     *
     * @param String Nome a ser consultado
     * 
     * @return int Contendo o total de registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistrosPeloNome(String strNome) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objUsuarioSistemaDAO.obterTotalRegistrosPeloNome(strNome);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm lista de registros por pgina  
     *
     * @param int nmero da pgina a ser consultada
     * @param int mximo de registro por pgina
     * @param objUsuarioSistema parametro do usurio a serem consultados
     * 
     * @return List Lista com os POJOs representando o registros obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina, UsuarioSistema objUsuarioSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        List lstUsuarioSistema = null;
        List lstRetorno = null;
        try
        {
            lstUsuarioSistema = objUsuarioSistemaDAO.obterPorPagina(intNumeroPagina, intMaximoPagina, objUsuarioSistema);
            lstRetorno = obterVisualizacao(lstUsuarioSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm total de registro
     *
     * @param objUsuarioSistema parametros para consulta
     * 
     * @return int Contendo o total de registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistros(UsuarioSistema objUsuarioSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objUsuarioSistemaDAO.obterTotalRegistros(objUsuarioSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    private List obterVisualizacao(List lstUsuarioSistema) throws CDException
    {
        List lstRetorno = new ArrayList();
        for (int i = 0; i < lstUsuarioSistema.size(); i++)
        {
            UsuarioSistema objUsuario = (UsuarioSistema) lstUsuarioSistema.get(i);
            UsuarioSistemaVisualizacao objUsuarioSistemaVisualizacao = new UsuarioSistemaVisualizacao();
            Copia.criar(objUsuario, objUsuarioSistemaVisualizacao);
            objUsuarioSistemaVisualizacao.setRetorno(objUsuario.getLogin() + ";" + objUsuario.getNome());
            lstRetorno.add(objUsuarioSistemaVisualizacao);
        }
        return lstRetorno;
    }

    /**
     * Verifica se existe um relacionamento entre um usurio e uma funcionalidade de sistema  
     *
     */
    public boolean verificarPermissaoUsuarioFuncionalidade(String strIdentificadorUsuario, String strIdentificadorGrupoUsuario, String strFuncionalidade, Map mapPerfisCriterioAcesso) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        boolean blnRetorno = false;

        try
        {
            UsuarioSistemaDAO usuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
            blnRetorno = usuarioSistemaDAO.validarPermissaoFuncionalidade(strIdentificadorUsuario, strIdentificadorGrupoUsuario, strFuncionalidade, mapPerfisCriterioAcesso);
        }
        catch (Exception daoe)
        {
            log.error("Ocorreu um erro verificando a permisso do usurio " + strIdentificadorUsuario + " na funcionalidade " + strFuncionalidade);
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return blnRetorno;
    }

    /**
     * Verifica se existe um relacionamento entre um usurio e uma funcionalidade de sistema  
     *
     */
    public boolean verificarPermissaoUsuarioFuncionalidade(String strIdentificadorUsuario, String strIdentificadorGrupoUsuario, String strFuncionalidade) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        boolean blnRetorno = false;

        try
        {
            UsuarioSistemaDAO usuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());
            blnRetorno = usuarioSistemaDAO.validarPermissaoFuncionalidade(strIdentificadorUsuario, strIdentificadorGrupoUsuario, strFuncionalidade);
        }
        catch (Exception daoe)
        {
            log.error("Ocorreu um erro verificando a permisso do usurio " + strIdentificadorUsuario + " na funcionalidade " + strFuncionalidade);
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return blnRetorno;
    }

}
