/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.consultatalento;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.joda.time.LocalDate;
import org.joda.time.LocalDateTime;

import br.gov.camara.hibernate.AliasToMapResultTransformer;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.ManipuladorSql;
import br.gov.camara.negocio.exception.DAOException;

class ResultadoConsultaTalentoDAO extends DAO
{
    public ResultadoConsultaTalentoDAO(String nomeConexao)
    {
        super(ResultadoConsultaTalentoDAO.class.getSimpleName(), nomeConexao);
    }

    public List<ResultadoConsultaTalento> recuperarResultadoConsultaTalento(List<Integer> pontos) throws DAOException
    {
        ManipuladorSql sql = new ManipuladorSql();
        sql.adicionarSql("select p.IDEPESSOAL, p.NOMPESSOAL, l.DESLOTACAO, l.DATTERMINO, g.CODGRUPO, g.DESGRUPO, l.DATTERMINO, max(t.DATLANCAMENTO) as datultimolancamento");
        sql.adicionarSql("from SIGESPPESSOAL p");
        sql.adicionarSql("left join GRUPO g on p.CODGRUPO = g.CODGRUPO");
        sql.adicionarSql("left join SIGESPPESSOALLOTACAO l on l.IDEPESSOAL = p.IDEPESSOAL and l.DATINICIO = (SELECT MAX(li.datinicio) from SIGESPPESSOALLOTACAO li where li.idepessoal=p.IDEPESSOAL)");
        sql.adicionarSql("left join TALENTO t on t.IDEPESSOAL = p.IDEPESSOAL");
        sql.adicionarSql("where p.IDEPESSOAL IN ( :matriculas )");
        sql.adicionarSql("group by p.IDEPESSOAL, p.NOMPESSOAL, l.DESLOTACAO, l.DATTERMINO, g.CODGRUPO, g.DESGRUPO, l.DATTERMINO");
        sql.adicionarSql("having max(t.DATLANCAMENTO) is not null");
        sql.adicionarSql("order by 1");

        sql.comParametro("matriculas", pontos);

        return obterPorSQLComParametros(sql, TransformadorParaListaDeResultadoConsultaTalento.INSTANCE);
    }

    static class TransformadorParaListaDeResultadoConsultaTalento extends AliasToMapResultTransformer
    {
        static final AliasToMapResultTransformer INSTANCE = new TransformadorParaListaDeResultadoConsultaTalento();

        private static final long serialVersionUID = 1L;

        @Override
        public ResultadoConsultaTalento transformarRegistro(Map<String, Object> registro)
        {
            int ponto = ((Number) registro.get("idepessoal")).intValue();
            String nome = (String) registro.get("nompessoal");
            String lotacao = (String) registro.get("deslotacao");
            Date dataFimLotacao = (Date) registro.get("dattermino");
            LocalDate fimLotacao = dataFimLotacao == null ? null : LocalDate.fromDateFields(dataFimLotacao);
            int codigoGrupo = ((Number) registro.get("codgrupo")).intValue();
            String nomeGrupo = (String) registro.get("desgrupo");
            Date dataDesligamento = (Date) registro.get("dt_deslig");
            LocalDate desligamento = dataDesligamento == null ? null : LocalDate.fromDateFields(dataDesligamento);
            Date dataHoraUltimoLancamento = (Date) registro.get("datultimolancamento");
            LocalDateTime ultimoLancamento = dataHoraUltimoLancamento == null ? null : LocalDateTime.fromDateFields(dataHoraUltimoLancamento);

            return ResultadoConsultaTalento.com(ponto, nome, codigoGrupo, nomeGrupo, lotacao, fimLotacao, desligamento, ultimoLancamento);
        }
    }

}
