/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.consultatalento;

import java.util.ArrayList;
import java.util.List;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.bancotalentos.pojo.Pessoa;

public class ResultadoConsultaTalentoFacade extends Facade
{
    public ResultadoConsultaTalentoFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    public List<ResultadoConsultaTalento> recuperarResultadoConsultaTalentoDasPessoas(List<Pessoa> pessoas) throws CDException
    {
        List<ResultadoConsultaTalento> lstConsultaTalentos = null;

        try
        {
            List<Integer> listaMatriculas = transformarParaListaDeMatriculas(pessoas);

            lstConsultaTalentos = recuperarResultadoConsultaTalentoDasMatriculas(listaMatriculas);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return lstConsultaTalentos;
    }

    public List<ResultadoConsultaTalento> recuperarResultadoConsultaTalentoDasMatriculas(List<Integer> matriculas) throws CDException
    {
        List<ResultadoConsultaTalento> lstConsultaTalentos = null;

        try
        {
            ResultadoConsultaTalentoDAO objConsultaTalentoDAO = new ResultadoConsultaTalentoDAO(obterNomeConexao());
            if (matriculas.isEmpty())
            {
                lstConsultaTalentos = new ArrayList<ResultadoConsultaTalento>();
            }
            else
            {
                lstConsultaTalentos = objConsultaTalentoDAO.recuperarResultadoConsultaTalento(matriculas);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return lstConsultaTalentos;
    }

    private List<Integer> transformarParaListaDeMatriculas(List<Pessoa> pessoas)
    {
        if (pessoas == null)
        {
            return null;
        }

        if (pessoas.isEmpty())
        {
            return new ArrayList<Integer>(0);
        }

        List<Integer> listaMatriculas = new ArrayList<Integer>();
        for (Pessoa pessoa : pessoas)
        {
            listaMatriculas.add(pessoa.getIdentificador());
        }

        return listaMatriculas;
    }
}
