/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.dao;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.ManipuladorSql;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoOpcao;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.negocio.util.Consulta;
import br.gov.camara.negocio.util.ConsultaComum;

/**
 * Classe DAO para a tabela AtributoTalentoOpcao
 * 
 */
public class AtributoTalentoOpcaoDAO extends DAO implements ConsultaComum
{

    // Variveis de instncia
    private static Log log = LogFactory.getLog(AtributoTalentoDAO.class.getName());

    /**
     * Construtor sem argumentos
     */
    public AtributoTalentoOpcaoDAO(String nomeConexao)
    {
        super("Opes de atributo de talento", nomeConexao);
    }

    /**
     * Obtm todos os registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

        return obter(strConsulta);
    }

    /**
     * Obtm todos os registros relacionados a determinado atributo de talento
     * 
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorAtributoTalento(AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE" + " atributoTalentoOpcao.atributoTalento.identificador = " + objAtributoTalento.getIdentificador() +

                " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

        return obter(strConsulta);
    }

    /**
     * Verifica se existem registros relacionados a determinado atributo de talento
     * 
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return boolean Contendo a verificao
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public boolean verificarExistenciaPorAtributoTalento(AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT" + " COUNT(*)" +

                " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE" + " atributoTalentoOpcao.atributoTalento.identificador = " + objAtributoTalento.getIdentificador() +

                " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

        // Conta registros
        int intTotalRegistros = obterTotalRegistros(strConsulta);
        if (intTotalRegistros > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Object POJO representando o registro obtido
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public Object obterPelaChave(String strChave) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE" + " atributoTalentoOpcao.identificador = " + strChave;

        List lstConsulta = obter(strConsulta);
        if (!lstConsulta.isEmpty())
        {
            return (AtributoTalentoOpcao) lstConsulta.get(0);
        }
        else
        {
            return null;
        }
    }

    /**
     * Obtm o total de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT COUNT(*)" +

                " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao";

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm o total de registros relacionados a determinado atributo de talento
     *
     * @param objAtributoTalento Atributo de talento desejado
     *
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistrosPorAtributoTalento(AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT" + " COUNT(*)" +

                " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE" + " atributoTalentoOpcao.atributoTalento.identificador = " + objAtributoTalento.getIdentificador();

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm os registros de determinada pgina
     *
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta consulta
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " ORDER BY" + " atributoTalento.descricao ASC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Obtm os registros por descrio, apenas os que estiverem associados aos atributos de talento especificados, 
     * de determinada pgina
     *
     * @param String Descrio da opo
     * @param List Atributos de talento que as opes devem estar relacionadas
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorDescricaoAtributosTalentoPorPagina(String strDescricao, List lstAtributosTalento, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        // Obtm identificadores dos atributos
        List<Integer> strAtributosTalento = concatenarIdentificadoresSeparadosPorVirgula(lstAtributosTalento);

        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("FROM AtributoTalentoOpcao atributoTalentoOpcao");
        hql.adicionarSql("WHERE atributoTalentoOpcao.atributoTalento.identificador IN (:idsAtributosTalentos)");
        hql.adicionarSql("AND upper(atributoTalentoOpcao.descricao) LIKE upper(:descricao)");
        hql.adicionarSql("ORDER BY atributoTalentoOpcao.descricao ASC");

        hql.comParametro("idsAtributosTalentos", strAtributosTalento);
        hql.comParametro("descricao", "%" + strDescricao + "%");

        // Recupera os dados
        return obterPorPaginaComParametro(intNumeroPagina, intMaximoPagina, hql);
    }

    public int obterTotalRegistrosPorDescricaoAtributosTalentoPorPagina(String strDescricao, List lstAtributosTalento) throws DAOException
    {
        List<Integer> strAtributosTalento = concatenarIdentificadoresSeparadosPorVirgula(lstAtributosTalento);

        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("SELECT COUNT(*)");
        hql.adicionarSql("FROM AtributoTalentoOpcao atributoTalentoOpcao");
        hql.adicionarSql("WHERE atributoTalentoOpcao.atributoTalento.identificador IN (:idsAtributosTalentos)");
        hql.adicionarSql("AND upper(atributoTalentoOpcao.descricao) LIKE upper(:descricao)");

        hql.comParametro("idsAtributosTalentos", strAtributosTalento);
        hql.comParametro("descricao", "%" + strDescricao + "%");

        // Recupera os dados
        return obterTotalRegistrosPorHQL(hql);
    }

    private List<Integer> concatenarIdentificadoresSeparadosPorVirgula(List<AtributoTalento> lstAtributosTalento)
    {
        List<Integer> identificadoresConcatenados = new ArrayList<Integer>();
        for (AtributoTalento atributo : lstAtributosTalento)
        {
            identificadoresConcatenados.add(atributo.getIdentificador());
        }
        return identificadoresConcatenados;
    }

    /**
     * Obtm os registros de determinada pgina relacionados a determinado atributo de talento
     *
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPaginaPorAtributoTalento(AtributoTalento objAtributoTalento, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta consulta
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE" + " atributoTalentoOpcao.atributoTalento.identificador = " + objAtributoTalento.getIdentificador() +

                " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Obtm candidatos a pai, ou seja, todas as opes do atributos requerido, exceto a que est sendo passado
     * 
     * @param objAtributoTalento Atributo de talento para buscar as opes
     * @param objAtributoTalentoOpcao Opo de atributo de talento que se deseja excluir
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterCandidatosAPai(AtributoTalento objAtributoTalento, AtributoTalentoOpcao objAtributoTalentoOpcao) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE" + " atributoTalentoOpcao.atributoTalento.identificador = " + objAtributoTalento.getAtributoTalentoPai().getIdentificador();

        if (objAtributoTalentoOpcao != null)
        {
            strConsulta += " AND" + " atributoTalentoOpcao.identificador != " + objAtributoTalentoOpcao.getIdentificador();
        }

        strConsulta += " ORDER BY" + " atributoTalentoOpcao.descricao";

        // Recupera os dados 
        return obter(strConsulta);
    }

    /**
     * Obtm as opes de determinado atributo de talento
     * 
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List contendo as opes de atributo de talento
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List obterPeloAtributo(AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE " + " atributoTalentoOpcao.atributoTalento.identificador = " + objAtributoTalento.getIdentificador() +

                " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm opes de determinado atributo de talento, e que no tenham pai
     *  
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List contendo as opes de atributo de talento
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List obterPeloAtributoSemPai(AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE " +

                " atributoTalentoOpcao.atributo.identificador = " + objAtributoTalento.getIdentificador() + " AND atributoTalentoOpcao.atributoPai IS NULL " +

                " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

        return obter(strConsulta);
    }

    /**
     * Obtm opes de atributo pelo pai
     * 
     * @param objAtributoTalentoOpcao Opo de atributo de talento a ser consultada
     * 
     * @return List contendo as opes de atributo de talento
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List obterPeloPaiEAtributoTalento(AtributoTalentoOpcao objAtributoTalentoOpcaoPai, AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE atributoTalentoOpcao.atributoTalentoOpcaoPai.identificador = "
                + objAtributoTalentoOpcaoPai.getIdentificador()
                + " AND atributoTalentoOpcao.atributoTalento.identificador = "
                + objAtributoTalento.getIdentificador()
                + " ORDER BY "
                + " atributoTalentoOpcao.descricao ASC";

        // Retorna
        return obter(strConsulta);

    }

    /**
     * Obtm opes de atributo pelo pai
     * 
     * @param objAtributoTalentoOpcao Opo de atributo de talento a ser consultada
     * 
     * @return List contendo as opes de atributo de talento
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List obterPeloPai(AtributoTalentoOpcao objAtributoTalentoOpcaoPai) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE " + " atributoTalentoOpcao.atributoTalentoOpcaoPai.identificador = " + objAtributoTalentoOpcaoPai.getIdentificador() +

                " ORDER BY " + " atributoTalentoOpcao.descricao ASC";

        // Retorna
        return obter(strConsulta);

    }

    /**
     * Exclui um registro
     *
     * @param Object POJO representando o objeto a ser excludo
     * 
     * @return String Contendo a consulta de excluso 
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public String excluirImpl(Object objAtributoTalentoOpcao) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Retorna consulta de excluso
        return " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao" +

                " WHERE" + " atributoTalentoOpcao.identificador = " + ((AtributoTalentoOpcao) objAtributoTalentoOpcao).getIdentificador();
    }

    /**
     * Mtodo utilizado para efetuar consultas genricas, por pgina
     * 
     * @param objConsulta Objeto de consulta contendo os parmetros necessrios para montar a query
     * @param intNumeroPagina Nmero da pgina a ser exibida
     * @param intMaximoPagina Nmero total de registros da pgina
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List consultar(Consulta objConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Declaraes
        List lstRetorno = null;

        try
        {
            // Monta query
            String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao";

            // Verifica filtro
            if (!"".equals(objConsulta.montarFiltro()))
            {
                strConsulta += " WHERE " + objConsulta.montarFiltro();

            }

            // Ordenao            
            strConsulta += " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

            // Recupera os dados 
            lstRetorno = obter(strConsulta);

            // Retorna
            return lstRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse);
        }
    }

    /**
     * Obtm total de registros da consulta
     *
     * @param Consulta objeto contendo os filtros para consulta
     *  
     * @return List contendo o resultado da pesquisa
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros(Consulta objConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        int intRetorno = 0;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " SELECT COUNT(*)" + " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }

            // Recupera os dados
            intRetorno = super.obterTotalRegistros(strConsulta);

            // Retorna
            return intRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros por pgina na classe " + strNomeClasse, cde);
        }
    }

    /**
     * Mtodo utilizado para efetuar consultas genricas, por pgina
     * 
     * @param objConsulta Objeto de consulta contendo os parmetros necessrios para montar a query
     * @param intNumeroPagina Nmero da pgina a ser exibida
     * @param intMaximoPagina Nmero total de registros da pgina
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List consultar(Consulta objConsulta, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        List lstRetorno = null;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }
            strConsulta += " ORDER BY" + " atributoTalentoOpcao.descricao ASC";

            // Recupera os dados 
            lstRetorno = obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);

            // Retorna
            return lstRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros por pgina na classe " + strNomeClasse, cde);
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalentoOpcaoPai, que  lazy
     * 
     * @param objAtributoTalentoOpcao Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalentoOpcaoPai(AtributoTalentoOpcao objAtributoTalentoOpcao) throws DAOException
    {
        inicializarRelacionamento(objAtributoTalentoOpcao.getAtributoTalentoOpcaoPai());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalentoOpcaoPai, que  lazy
     * 
     * @param lstAtributoTalentoOpcoes Lista de objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalentoOpcaoPai(List lstAtributoTalentoOpcoes) throws DAOException
    {
        Iterator itrAtributoTalentoOpcoes = lstAtributoTalentoOpcoes.iterator();
        while (itrAtributoTalentoOpcoes.hasNext())
        {
            inicializarAtributoTalentoOpcaoPai((AtributoTalentoOpcao) itrAtributoTalentoOpcoes.next());
        }
    }

    /**
     * Mtodo que pede a inicializao da propriedade atributoTalento, que  lazy
     * 
     * @param objAtributoTalentoOpcao Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalento(AtributoTalentoOpcao objAtributoTalentoOpcao) throws DAOException
    {
        inicializarRelacionamento(objAtributoTalentoOpcao.getAtributoTalento());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalento, que  lazy
     * 
     * @param lstAtributoTalentoOpcoes Lista de objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalento(List lstAtributoTalentoOpcoes) throws DAOException
    {
        Iterator itrAtributoTalentoOpcoes = lstAtributoTalentoOpcoes.iterator();
        while (itrAtributoTalentoOpcoes.hasNext())
        {
            inicializarAtributoTalento((AtributoTalentoOpcao) itrAtributoTalentoOpcoes.next());
        }
    }
}
