/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.dao;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.ManipuladorSql;
import br.gov.camara.negocio.ParametroDAO;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoOpcao;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoValorado;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaAtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.Talento;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.negocio.util.Consulta;
import br.gov.camara.negocio.util.ConsultaComum;

/**
 * Classe DAO para a tabela AtributoTalentoValorado
 */
public class AtributoTalentoValoradoDAO extends DAO implements ConsultaComum
{
    private static Log log = LogFactory.getLog(AtributoTalentoValoradoDAO.class.getName());

    /**
     * Construtor sem argumentos
     */
    public AtributoTalentoValoradoDAO(String nomeConexao)
    {
        super("Valoraes de talento", nomeConexao);
    }

    /**
     * Obtm todos os registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoValorado atributoTalentoValorado" + " ORDER BY" + " atributoTalentoValorado.identificador ASC";

        return super.obter(strConsulta);
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Object POJO representando o registro obtido
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public Object obterPelaChave(String strChave) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        String strConsulta = " FROM" + " AtributoTalentoValorado atributoTalentoValorado" + " WHERE" + " atributoTalentoValorado.identificador = " + strChave;

        return (AtributoTalentoValorado) super.obter(strConsulta).get(0);

    }

    /**
     * Obtm as valoraes de determinado talento
     * 
     * @param objTalento Talento desejado
     * 
     * @return List contendo as valoraes
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List obterPeloTalento(Talento objTalento) throws DAOException
    {
        ManipuladorSql sql = new ManipuladorSql();

        // Monta query
        StringBuffer consulta = new StringBuffer();
        consulta.append(" FROM AtributoTalentoValorado atributoTalentoValorado");
        consulta.append(" LEFT JOIN FETCH atributoTalentoValorado.categoriaAtributoTalento categoriaAtributoTalento ");
        consulta.append(" WHERE atributoTalentoValorado.talento.identificador = :idTalento");
        consulta.append(" ORDER BY atributoTalentoValorado.categoriaAtributoTalento.sequencialOrdenacao ASC");
        consulta.append(" , atributoTalentoValorado.valoracao ASC");

        ParametroDAO parametro = new ParametroDAO();
        parametro.adicionar("idTalento", objTalento.getIdentificador());

        // Retorna
        return obterListaPorHQL(consulta.toString(), parametro);
    }

    public List obterPeloTalentoComCategoriaAtributoTalentoEAtributoTalentoETipoHtml(Talento objTalento) throws DAOException
    {
        ManipuladorSql sql = new ManipuladorSql();

        StringBuffer consulta = new StringBuffer();
        consulta.append(" FROM AtributoTalentoValorado atributoTalentoValorado");
        consulta.append(" LEFT JOIN FETCH atributoTalentoValorado.categoriaAtributoTalento categoriaAtributoTalento ");
        consulta.append(" LEFT JOIN FETCH categoriaAtributoTalento.atributoTalento atributoTalento ");
        consulta.append(" LEFT JOIN FETCH atributoTalento.tipoHTML tipo ");
        consulta.append(" WHERE atributoTalentoValorado.talento.identificador = :idTalento");
        consulta.append(" ORDER BY atributoTalentoValorado.categoriaAtributoTalento.sequencialOrdenacao ASC");
        consulta.append(" , atributoTalentoValorado.valoracao ASC");

        ParametroDAO parametro = new ParametroDAO();
        parametro.adicionar("idTalento", objTalento.getIdentificador());

        // Retorna
        return obterListaPorHQL(consulta.toString(), parametro);
    }

    /**
     * Obtm quantidade de valorao de talentos por categoria/atributo
     * 
     * @param objCategoriaAtributoTalento Categoria/atributo desejado
     * 
     * @return int Com o total de registros 
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados 
     */
    public int obterTotalRegistrosPorCategoriaAtributoTalento(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT" + " COUNT(*)" +

                " FROM" + " AtributoTalentoValorado atributoTalentoValorado" +

                " WHERE " + " atributoTalentoValorado.categoriaAtributoTalento.identificador = " + objCategoriaAtributoTalento.getIdentificador();

        // Retorna
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm valorao por talento e categoria/atributo de talento
     * 
     * @param objTalento Talento desejado
     * @param objCategoriaAtributoTalento Categoria/atributo desejado
     * 
     * @return Object contendo a valorao 
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados 
     */
    public Object obterPorTalentoCategoriaAtributoTalento(Talento objTalento, CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " AtributoTalentoValorado atributoTalentoValorado"
                +

                " WHERE "
                + " atributoTalentoValorado.talento.identificador = "
                + objTalento.getIdentificador()
                + " AND atributoTalentoValorado.categoriaAtributo.identificador = "
                + objCategoriaAtributoTalento.getIdentificador()
                +

                " ORDER BY"
                + " atributoTalentoValorado.identificador ASC";

        // Retorna
        return (AtributoTalentoValorado) super.obter(strConsulta).get(0);
    }

    /**
     * Obtm valoraes pela opo
     * 
     * @param objAtributoTalentoOpcao Opo de atributo de talento para recuperao
     * 
     * @return List Contendo as valoraes 
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados 
     */
    public List obterPorCategoriaAtributoTalentoOpcao(AtributoTalentoOpcao objAtributoTalentoOpcao) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " AtributoTalentoValorado atributoTalentoValorado" +

                " WHERE " + " atributoTalentoValorado.atributoTalentoOpcao.identificador = " + objAtributoTalentoOpcao.getIdentificador() +

                " ORDER BY" + " atributoTalentoValorado.identificador ASC";

        // Retorna
        return obter(strConsulta);
    }

    /**
     * Exclui um registro
     *
     * @param Object POJO representando o objeto a ser excludo
     * 
     * @return String Contendo a consulta de excluso 
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public String excluirImpl(Object objAtributoTalentoValorado) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Retorna consulta de excluso
        return " FROM" + " AtributoTalentoValorado atributoTalentoValorado" +

                " WHERE" + " atributoTalentoValorado.identificador = " + ((AtributoTalentoValorado) objAtributoTalentoValorado).getIdentificador();
    }

    /**
     * Exclui registros pela sua categoria/atributo
     *
     * @param CategoriaAtributoTalento Com a categoria/atributo desejado
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public void excluirPorCategoriaAtributoTalento(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta consulta de excluso
        String strConsulta = " FROM" + " AtributoTalentoValorado atributoTalentoValorado" +

                " WHERE" + " atributoTalentoValorado.categoriaAtributoTalento.identificador = " + objCategoriaAtributoTalento.getIdentificador();

        //Executa
        excluir(strConsulta);
    }

    /**
     * Exclui registros pelo seu talento
     *
     * @param Talento Com o talento desejado
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public void excluirPorTalento(Talento objTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta consulta de excluso
        String strConsulta = " FROM" + " AtributoTalentoValorado atributoTalentoValorado" +

                " WHERE" + " atributoTalentoValorado.talento.identificador = " + objTalento.getIdentificador();

        //Executa
        excluir(strConsulta);
    }

    /**
     * Obtm total de registros da consulta
     *
     * @param Consulta objeto contendo os filtros para consulta
     *  
     * @return List contendo o resultado da pesquisa
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros(Consulta objConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        int intRetorno = 0;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " SELECT COUNT(*)" + " FROM" + " AtributoTalentoValorado atributoTalentoValorado";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }

            // Recupera os dados
            intRetorno = obterTotalRegistros(strConsulta);

            // Retorna
            return intRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     * Mtodo utilizado para efetuar consultas genricas, por pgina
     * 
     * @param objConsulta Objeto de consulta contendo os parmetros necessrios para montar a query
     * @param intNumeroPagina Nmero da pgina a ser exibida
     * @param intMaximoPagina Nmero total de registros da pgina
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List consultar(Consulta objConsulta, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        List lstRetorno = null;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " FROM" + " AtributoTalentoValorado atributoTalentoValorado";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }
            strConsulta += " ORDER BY" + " atributoTalentoValorado.identificador ASC";

            // Recupera os dados 
            lstRetorno = obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);

            // Retorna
            return lstRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade talento, que  lazy
     * 
     * @param objAtributoTalentoValorado Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarTalento(AtributoTalentoValorado objAtributoTalentoValorado) throws DAOException
    {
        inicializarRelacionamento(objAtributoTalentoValorado.getTalento());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade talento, que  lazy
     * 
     * @param List lstAtributosTalentoValorados Lista com os objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarTalento(List lstAtributosTalentoValorados) throws DAOException
    {
        Iterator itrAtributosTalentoValorados = lstAtributosTalentoValorados.iterator();
        while (itrAtributosTalentoValorados.hasNext())
        {
            inicializarTalento((AtributoTalentoValorado) itrAtributosTalentoValorados.next());
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade categoriaAtributoTalento, que  lazy
     * 
     * @param AtributoTalentoValorado objAtributoTalentoValorado Classe que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarCategoriaAtributoTalento(AtributoTalentoValorado objAtributoTalentoValorado) throws DAOException
    {
        inicializarRelacionamento(objAtributoTalentoValorado.getCategoriaAtributoTalento());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade categoriaAtributoTalento, que  lazy
     * 
     * @param List lstAtributosTalentoValorados Lista com os objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarCategoriaAtributoTalento(List lstAtributosTalentoValorados) throws DAOException
    {
        Iterator itrAtributosTalentoValorados = lstAtributosTalentoValorados.iterator();
        while (itrAtributosTalentoValorados.hasNext())
        {
            inicializarCategoriaAtributoTalento((AtributoTalentoValorado) itrAtributosTalentoValorados.next());
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalentoOpcao, que  lazy
     * 
     * @param objAtributoTalentoValorado Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalentoOpcao(AtributoTalentoValorado objAtributoTalentoValorado) throws DAOException
    {
        inicializarRelacionamento(objAtributoTalentoValorado.getAtributoTalentoOpcao());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalentoOpcao, que  lazy
     * 
     * @param lstAtributosTalentoValorados Lista com os objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalentoOpcao(List lstAtributosTalentoValorados) throws DAOException
    {
        Iterator itrAtributosTalentoValorados = lstAtributosTalentoValorados.iterator();
        while (itrAtributosTalentoValorados.hasNext())
        {
            inicializarAtributoTalentoOpcao((AtributoTalentoValorado) itrAtributosTalentoValorados.next());
        }
    }

    public List obterPeloTalentoComAtributoTalentoEAtributoTalentoOpcoes(Talento objTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        StringBuffer consulta = new StringBuffer();
        consulta.append(" FROM AtributoTalentoValorado atributoTalentoValorado");
        consulta.append(" LEFT JOIN FETCH atributoTalentoValorado.categoriaAtributoTalento categoriaAtributoTalento ");
        consulta.append(" LEFT JOIN FETCH atributoTalentoValorado.categoriaAtributoTalento.atributoTalento ");
        consulta.append(" LEFT JOIN FETCH atributoTalentoValorado.atributoTalentoOpcao ");
        consulta.append(" WHERE atributoTalentoValorado.talento.identificador = :idTalento");
        consulta.append(" ORDER BY atributoTalentoValorado.categoriaAtributoTalento.sequencialOrdenacao ASC");
        consulta.append(" , atributoTalentoValorado.valoracao ASC");

        ParametroDAO parametro = new ParametroDAO();
        parametro.adicionar("idTalento", objTalento.getIdentificador());

        // Retorna
        return obterListaPorHQL(consulta.toString(), parametro);
    }
}
