/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.dao;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.ManipuladorSql;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaAtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaTalento;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.negocio.util.Consulta;
import br.gov.camara.negocio.util.ConsultaComum;

/**
 * Classe DAO para a tabea CategoriaAtributoTalento
 */

public class CategoriaAtributoTalentoDAO extends DAO implements ConsultaComum
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(CategoriaAtributoTalentoDAO.class.getName());

    /**
     * Construtor sem argumentos
     *
     */
    public CategoriaAtributoTalentoDAO(String nomeConexao)
    {
        super("Categoria/atributos de talento", nomeConexao);
    }

    /**
     * Obtm todos os registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " ORDER BY" + " categoriaAtributoTalento.sequencialPreenchimento ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm todos os registros relacionados a determinada categoria de talento
     * 
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorCategoriaTalento(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("FROM CategoriaAtributoTalento categoriaAtributoTalento");
        hql.adicionarSql("LEFT JOIN FETCH categoriaAtributoTalento.atributoTalento atributoTalento");
        hql.adicionarSql("LEFT JOIN FETCH atributoTalento.tipoHTML tipo");
        hql.adicionarSql("WHERE categoriaAtributoTalento.categoriaTalento.identificador = :idCategoriaTalento");
        hql.adicionarSql("ORDER BY categoriaAtributoTalento.sequencialOrdenacao ASC");
        hql.comParametro("idCategoriaTalento", objCategoriaTalento.getIdentificador());

        return obterListaPorHQL(hql);
    }

    /**
     * Obtm todos os registros para consulta relacionados a determinada categoria de talento
     * 
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterParaConsultaPorCategoriaTalento(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT DISTINCT "
                + " categoriaAtributoTalento "
                +

                " FROM"
                + " CategoriaAtributoTalento categoriaAtributoTalento"
                +

                " WHERE"
                + " categoriaAtributoTalento.categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador()
                //                + " AND (categoriaAtributoTalento.atributoTalento.tipoHTML.multiplicidade IN('U','M') "
                //                + " OR categoriaAtributoTalento.atributoTalento.tipoDado IN('D','N')) "
                +

                " ORDER BY"
                + " categoriaAtributoTalento.sequencialOrdenacao ASC";

        return obter(strConsulta);
    }

    /**
     * Obtm todos os registros filhos de determinada categoria/atributo de talento
     * 
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodosOsFilhos(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("FROM CategoriaAtributoTalento categoriaAtributoTalento");
        hql.adicionarSql("LEFT JOIN FETCH categoriaAtributoTalento.atributoTalento atributoTalento");
        hql.adicionarSql("LEFT JOIN FETCH atributoTalento.tipoHTML tipoHtml");
        hql.adicionarSql("WHERE categoriaAtributoTalento.categoriaTalento.identificador = :idCategoriaTalento");
        hql.adicionarSql("AND categoriaAtributoTalento.atributoTalento IN (");
        hql.adicionarSql("FROM AtributoTalento atributoTalento");
        hql.adicionarSql("WHERE atributoTalento.atributoTalentoPai.identificador = :idAtributoTalentoPai");
        hql.adicionarSql(")");
        hql.comParametro("idCategoriaTalento", objCategoriaAtributoTalento.getCategoriaTalento().getIdentificador());
        hql.comParametro("idAtributoTalentoPai", objCategoriaAtributoTalento.getAtributoTalento().getIdentificador());

        // Retorna consulta
        return obterListaPorHQL(hql);
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Object POJO representando o registro obtido
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public Object obterPelaChave(String strChave) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        String strConsulta = " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " WHERE " + "categoriaAtributoTalento.identificador = " + strChave;

        // Consulta
        List lstResultado = obter(strConsulta);
        if (!lstResultado.isEmpty())
        {
            return lstResultado.get(0);
        }
        else
        {
            return null;
        }
    }

    public Object obterPelaChaveComAtributoTalentoETipoHtml(String strChave) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        StringBuffer consulta = new StringBuffer();
        consulta.append(" FROM");
        consulta.append(" CategoriaAtributoTalento categoriaAtributoTalento");
        consulta.append(" LEFT JOIN FETCH categoriaAtributoTalento.atributoTalento atributoTalento");
        consulta.append(" LEFT JOIN FETCH atributoTalento.tipoHTML tipoHtml");
        consulta.append(" WHERE ");
        consulta.append("categoriaAtributoTalento.identificador = ");
        consulta.append(strChave);

        // Consulta
        List lstResultado = obterListaPorHQL(consulta.toString(), null);
        if (!lstResultado.isEmpty())
        {
            return lstResultado.get(0);
        }
        else
        {
            return null;
        }
    }

    /**
     * Obtm o total de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT COUNT(*)" +

                " FROM" + " AtributoTalentoOpcao atributoTalentoOpcao";

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm o total de registros relacionados a determinada categoria de talento
     *
     * @param objCategoriaTalento Categoria de talento desejada
     *
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistrosPorCategoriaTalento(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT" + " COUNT(*)" +

                " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " WHERE" + " categoriaAtributoTalento.categoriaTalento.identificador = " + objCategoriaTalento.getIdentificador();

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm os registros de determinada pgina
     *
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        // Monta consulta
        String strConsulta = " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " ORDER BY" + " categoriaAtributoTalento.atributoTalento.nome ASC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Obtm os registros de determinada pgina relacionados a determinada categoria de talento
     *
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPaginaPorCategoriaTalento(CategoriaTalento objCategoriaTalento, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        // Monta consulta
        String strConsulta = " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " WHERE" + " categoriaAtributoTalento.categoriaTalento.identificador = " + objCategoriaTalento.getIdentificador() +

                " ORDER BY" + " categoriaAtributoTalento.sequencialOrdenacao ASC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Obtm o ltimo sequencial de ordenao para a categoria passada
     *
     * @param objCategoriaTalento Categoria de talento a se verificar o ltimo sequencial
     *
     * @return int contendo o valor requerido
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public int obterUltimoSequencialOrdenacao(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        //Declaraes
        int intRetorno = 0;

        // Monta query
        String strConsulta = " SELECT " + " MAX(categoriaAtributoTalento.sequencialOrdenacao)" +

                " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " WHERE" + " categoriaAtributoTalento.categoriaTalento.identificador = " + objCategoriaTalento.getIdentificador();

        // Consulta
        List lstResultado = obter(strConsulta);
        if (!lstResultado.isEmpty() && lstResultado.get(0) != null)
        {
            intRetorno = Integer.parseInt(String.valueOf(lstResultado.get(0)));
        }
        else
        {
            intRetorno = 1;
        }

        // Retorna
        return intRetorno;

    }

    /**
     * Exclui um registro
     *
     * @param Object POJO representando o objeto a ser excludo
     * 
     * @return String Contendo a consulta de excluso 
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public String excluirImpl(Object objCategoriaAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Retorna consulta de excluso
        return " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " WHERE" + " categoriaAtributoTalento.identificador = " + ((CategoriaAtributoTalento) objCategoriaAtributoTalento).getIdentificador();
    }

    /**
     * Obtm total de registros da consulta
     *
     * @param Consulta objeto contendo os filtros para consulta
     *  
     * @return List contendo o resultado da pesquisa
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros(Consulta objConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        int intRetorno = 0;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " SELECT COUNT(*)" + " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }

            // Recupera os dados
            intRetorno = obterTotalRegistros(strConsulta);

            // Retorna
            return intRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     * Mtodo utilizado para efetuar consultas genricas, por pgina
     * 
     * @param objConsulta Objeto de consulta contendo os parmetros necessrios para montar a query
     * @param intNumeroPagina Nmero da pgina a ser exibida
     * @param intMaximoPagina Nmero total de registros da pgina
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List consultar(Consulta objConsulta, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        List lstRetorno = null;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }
            strConsulta += " ORDER BY" + " categoriaAtributoTalento.seqPreenchimento ASC";

            // Recupera os dados 
            lstRetorno = obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);

            // Retorna
            return lstRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     * Verifica a existncia da relao categoria/atributo 
     * 
     * @param objCategoriaTalento Categoria de talento a ser verificada
     * @param objAtributoTalento Atributo de talento a ser verificado
     * 
     * @return boolean Com a verificao
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public boolean verificarExistenciaAtributoRelacionado(CategoriaTalento objCategoriaTalento, AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT "
                + "COUNT(*)"
                +

                " FROM"
                + " CategoriaAtributoTalento categoriaAtributoTalento"
                +

                " WHERE"
                + " categoriaAtributoTalento.categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador()
                + " AND categoriaAtributoTalento.atributoTalento.identificador = "
                + objAtributoTalento.getIdentificador();

        // Recupera os dados
        int intTotalRegistros = obterTotalRegistros(strConsulta);
        if (intTotalRegistros > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Verifica a existncia de atributos talento para talento
     * 
     * @param objCategoriaAtributoTalento Categoria/atributo de talento a ser verificada
     * 
     * @return int total de talentos associados
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public int verificarExistenciaCategoriaAtributosTalentoTalento(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT " + "COUNT(*)" +

                " FROM" + " Talento talento" +

                " WHERE" + " talento.categoriaTalento.identificador = " + objCategoriaAtributoTalento.getCategoriaTalento().getIdentificador();

        // Recupera os dados
        int intTotalRegistros = obterTotalRegistros(strConsulta);

        return intTotalRegistros;
    }

    /**
     * Verifica a existncia de atributos filho para a categoria/atributo desejada  
     * 
     * @param objCategoriaAtributoTalento Categoria/atributo de talento a ser verificada
     * 
     * @return boolean Com a verificao
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public boolean verificarExistenciaCategoriaAtributosTalentoFilhos(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT "
                + "COUNT(*)"
                +

                " FROM"
                + " CategoriaAtributoTalento categoriaAtributoTalento"
                +

                " WHERE"
                + " categoriaAtributoTalento.categoriaTalento.identificador = "
                + objCategoriaAtributoTalento.getCategoriaTalento().getIdentificador()
                + " AND categoriaAtributoTalento.atributoTalento.identificador IN "
                + " (SELECT atributoTalento.identificador"
                + " FROM AtributoTalento atributoTalento"
                + " WHERE atributoTalento.atributoTalentoPai.identificador = "
                + objCategoriaAtributoTalento.getAtributoTalento().getIdentificador()
                + ")";

        // Recupera os dados
        int intTotalRegistros = obterTotalRegistros(strConsulta);
        if (intTotalRegistros > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * 
     * Obtm categoria/atributo de acordo com a categoria e o atributo passados
     * 
     * @param objCategoriaTalento Categoria de talento a ser consultada
     * @param objAtributoTalento Atributo de talento a ser consultado
     * 
     * @return CategoriaAtributoTalento Contendo a categoria/atributo
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public CategoriaAtributoTalento obterPorCategoriaTalentoAtributoTalento(CategoriaTalento objCategoriaTalento, AtributoTalento objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " CategoriaAtributoTalento categoriaAtributoTalento"
                +

                " WHERE"
                + " categoriaAtributoTalento.categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador()
                + " AND categoriaAtributoTalento.atributoTalento.identificador = "
                + objAtributoTalento.getIdentificador();

        // Recupera os dados
        List lstResultado = obter(strConsulta);
        if (!lstResultado.isEmpty())
        {
            return (CategoriaAtributoTalento) lstResultado.get(0);
        }
        else
        {
            return null;
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade categoriaTalento, que  lazy
     * 
     * @param objCategoriaAtributoTalento Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarCategoriaTalento(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        inicializarRelacionamento(objCategoriaAtributoTalento.getCategoriaTalento());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade categoriaTalento, que  lazy
     * 
     * @param List lstCategoriaAtributosTalento Lista com os objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarCategoriaTalento(List lstCategoriaAtributosTalento) throws DAOException
    {
        Iterator itrCategoriaAtributosTalento = lstCategoriaAtributosTalento.iterator();
        while (itrCategoriaAtributosTalento.hasNext())
        {
            inicializarCategoriaTalento((CategoriaAtributoTalento) itrCategoriaAtributosTalento.next());
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalento, que  lazy
     * 
     * @param  objCategoriaAtributoTalento Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalento(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        inicializarRelacionamento(objCategoriaAtributoTalento.getAtributoTalento());
        inicializarRelacionamento(objCategoriaAtributoTalento.getAtributoTalento().getTipoHTML());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalento, que  lazy
     * 
     * @param List lstCategoriaAtributosTalento Lista com os objetos que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarAtributoTalento(List lstCategoriaAtributosTalento) throws DAOException
    {
        Iterator itrCategoriaAtributosTalento = lstCategoriaAtributosTalento.iterator();
        while (itrCategoriaAtributosTalento.hasNext())
        {
            inicializarAtributoTalento((CategoriaAtributoTalento) itrCategoriaAtributosTalento.next());
        }
    }
}
