/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.dao;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.ManipuladorSql;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaTalento;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.negocio.util.Consulta;
import br.gov.camara.negocio.util.ConsultaComum;

/**
 * Classe DAO para a tabela CategoriaTalento
 */

public class CategoriaTalentoDAO extends DAO implements ConsultaComum
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(CategoriaTalentoDAO.class.getName());

    /**
     * Construtor sem argumentos
     *
     */
    public CategoriaTalentoDAO(String nomeConexao)
    {
        super("Categorias de talentos", nomeConexao);
    }

    /**
     * Obtm todos os registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " CategoriaTalento categoriaTalento" +

                " ORDER BY" + " categoriaTalento.sequencialOrdenacao";

        return obter(strConsulta);

    }

    /**
     * Obtm todos os registros disponveis para determinado grupo 
     * 
     * @param objGrupo Grupo a ser pesquisado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterAssociadasAoGrupo(int codigoGrupo) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("SELECT DISTINCT categoriaTalento");
        hql.adicionarSql("FROM CategoriaTalento categoriaTalento");
        hql.adicionarSql("INNER JOIN categoriaTalento.gruposCategoriaTalento gruposCategoriaTalento");
        hql.adicionarSql("WHERE gruposCategoriaTalento.codigo IN (:grupos)");
        hql.comParametro("grupos", codigoGrupo);
        hql.adicionarSql("ORDER BY categoriaTalento.sequencialOrdenacao");

        return obterListaPorHQL(hql);
    }

    /**
     * Obtm todos os registros disponveis para consulta 
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterDisponiveisParaConsultaPorPagina(String identificadorPessoa, boolean filtrarPorGrupo, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        List<Integer> grupos = null;

        String strGrupos = null;
        // Verifica se h filtro por grupo
        if (filtrarPorGrupo)
        {
            FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
            grupos = objFiltroConsultaDAO.obterListaCodigosGruposPermitidos(identificadorPessoa);
        }

        return obterAssociadasAAtributosPorGrupoPorPagina(grupos, intNumeroPagina, intMaximoPagina);
    }

    public List obterAssociadasAAtributosPorGrupoPorPagina(List<Integer> grupos, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("SELECT DISTINCT categoriaTalento");
        hql.adicionarSql("FROM CategoriaAtributoTalento categoriaAtributoTalento");
        hql.adicionarSql("INNER JOIN categoriaAtributoTalento.categoriaTalento categoriaTalento");

        if (grupos != null && !grupos.isEmpty())
        {
            hql.adicionarSql("INNER JOIN categoriaTalento.gruposCategoriaTalento gruposCategoriaTalento");
            hql.adicionarSql("WHERE gruposCategoriaTalento.codigo IN (:grupos)");
            hql.comParametro("grupos", grupos);
        }

        hql.adicionarSql("ORDER BY categoriaTalento.sequencialOrdenacao");

        return obterPorPaginaComParametro(intNumeroPagina, intMaximoPagina, hql);
    }

    public int obterQtdeAssociadasAAtributosPorGrupo(Integer grupo) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("SELECT COUNT(DISTINCT categoriaTalento)");
        hql.adicionarSql("FROM CategoriaAtributoTalento categoriaAtributoTalento");
        hql.adicionarSql("INNER JOIN categoriaAtributoTalento.categoriaTalento categoriaTalento");

        if (grupo != null)
        {
            hql.adicionarSql("INNER JOIN categoriaTalento.gruposCategoriaTalento gruposCategoriaTalento");
            hql.adicionarSql("WHERE gruposCategoriaTalento.codigo = :grupo");
            hql.comParametro("grupo", grupo);
        }

        return obterTotalRegistrosPorHQL(hql);
    }

    public List obterTodasPorGrupoPorPagina(List<Integer> grupos, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("SELECT DISTINCT categoriaTalento");
        hql.adicionarSql("FROM CategoriaTalento categoriaTalento");

        if (grupos != null && !grupos.isEmpty())
        {
            hql.adicionarSql("INNER JOIN categoriaTalento.gruposCategoriaTalento gruposCategoriaTalento");
            hql.adicionarSql("WHERE gruposCategoriaTalento.codigo IN (:grupos)");
            hql.comParametro("grupos", grupos);
        }

        hql.adicionarSql("ORDER BY categoriaTalento.sequencialOrdenacao");

        return obterPorPaginaComParametro(intNumeroPagina, intMaximoPagina, hql);
    }

    /**
     * Obtm o total de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterQtdeTodasPorGrupo(String grupoFiltro) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("SELECT COUNT(DISTINCT categoriaTalento)");
        hql.adicionarSql("FROM CategoriaTalento categoriaTalento");

        Integer grupos = null;
        if (grupoFiltro != null && !"".equals(grupoFiltro))
        {
            grupos = Integer.valueOf(grupoFiltro);
        }

        if (grupos != null)
        {
            hql.adicionarSql("INNER JOIN categoriaTalento.gruposCategoriaTalento gruposCategoriaTalento");
            hql.adicionarSql("WHERE gruposCategoriaTalento.codigo IN (:grupos)");
            hql.comParametro("grupos", grupos);
        }

        // Recupera os dados
        return obterTotalRegistrosPorHQL(hql);
    }

    /**
     * Obtm um registro a partir da chave
     * 
     * @param strChave
     *            Chave do registro a ser obtido
     * 
     * @return Object POJO representando o registro obtido
     * 
     * @throws DAOException
     *             se ocorrer algum erro relacionado com o acesso a banco de
     *             dados
     * 
     */
    public Object obterPelaChave(String chave) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("FROM CategoriaTalento categoriaTalento");
        hql.adicionarSql("WHERE categoriaTalento.identificador = :id");
        hql.comParametro("id", Integer.valueOf(chave));

        return (CategoriaTalento) obterObjetoPorHQL(hql);
    }

    /**
     * Obtm os grupos disponveis para determinada categoria de talento
     * 
     * @param objCategoriaTalento Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterGruposDisponiveis(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " Grupo grupo"
                +

                " WHERE"
                + " grupo.codigo NOT IN"
                +

                " (SELECT categoriaTalento.gruposCategoriaTalento.elements"
                + " FROM"
                + " CategoriaTalento categoriaTalento"
                + " WHERE"
                + " categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador()
                + ")"
                +

                " ORDER BY"
                + " grupo.descricao ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm os grupos disponveis para determinada categoria de talento
     * 
     * @param objCategoriaTalento Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterGruposSelecionados(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " Grupo grupo"
                +

                " WHERE"
                + " grupo.codigo IN"
                +

                " (SELECT categoriaTalento.gruposCategoriaTalento.elements"
                + " FROM"
                + " CategoriaTalento categoriaTalento"
                + " WHERE"
                + " categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador()
                + ")"
                +

                " ORDER BY"
                + " grupo.descricao ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm o ltimo sequencial de ordenao
     *
     * @return int contendo o valor requerido
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public int obterUltimoSequencialOrdenacao() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        //Declaraes
        int intRetorno = 0;

        // Monta query
        String strConsulta = " SELECT " + " MAX(categoriaTalento.sequencialOrdenacao)" +

                " FROM" + " CategoriaTalento categoriaTalento";

        // Consulta
        List lstResultado = obter(strConsulta);
        if (!lstResultado.isEmpty() && lstResultado.get(0) != null)
        {
            intRetorno = Integer.parseInt(String.valueOf(lstResultado.get(0)));
        }
        else
        {
            intRetorno = 1;
        }

        // Retorna
        return intRetorno;
    }

    /**
     * Exclui um registro
     *
     * @param Object POJO representando o objeto a ser excludo
     * 
     * @return String Contendo a consulta de excluso 
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public String excluirImpl(Object objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Retorna consulta de excluso
        return " FROM" + " CategoriaTalento categoriaTalento" +

                " WHERE" + " categoriaTalento.identificador = " + ((CategoriaTalento) objCategoriaTalento).getIdentificador();
    }

    /**
     * Obtm total de registros da consulta
     *
     * @param Consulta objeto contendo os filtros para consulta
     *  
     * @return List contendo o resultado da pesquisa
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros(Consulta objConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        int intRetorno = 0;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " SELECT " + " COUNT(*)" +

                    " FROM" + " CategoriaTalento categoriaTalento";

            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }

            // Recupera os dados
            intRetorno = super.obterTotalRegistros(strConsulta);

            // Retorna
            return intRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     * Mtodo utilizado para efetuar consultas genricas, por pgina
     * 
     * @param objConsulta Objeto de consulta contendo os parmetros necessrios para montar a query
     * @param intNumeroPagina Nmero da pgina a ser exibida
     * @param intMaximoPagina Nmero total de registros da pgina
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public List consultar(Consulta objConsulta, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        List lstRetorno = null;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " FROM" + " CategoriaTalento categoriaTalento";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }
            strConsulta += " ORDER BY" + " categoriaTalento.nome ASC";

            // Recupera os dados 
            lstRetorno = obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);

            // Retorna
            return lstRetorno;

        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     * 
     * Verifica se a categoria tem atributos relacionados com formao de descrio marcada
     * 
     * @param objCategoriaTalento Categoria de talento a ser verificada
     * 
     * @return boolean Com a verificao
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public boolean verificarExistenciaAtributoTalento(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT " + " COUNT(*)" +

                " FROM" + " CategoriaAtributoTalento categoriaAtributoTalento" +

                " WHERE" + " categoriaAtributoTalento.categoriaTalento.identificador = " + objCategoriaTalento.getIdentificador();

        // Retorna
        int intTotalRegistros = obterTotalRegistros(strConsulta);
        if (intTotalRegistros > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * 
     * Verifica se a categoria tem atributos relacionados com formao de descrio marcada
     * 
     * @param objCategoriaTalento Categoria de talento a ser verificada
     * 
     * @return boolean Com a verificao
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public boolean verificarAtributosAssociadosComFormacaoDescricao(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT "
                + " COUNT(*)"
                +

                " FROM"
                + " CategoriaAtributoTalento categoriaAtributoTalento"
                +

                " WHERE"
                + " categoriaAtributoTalento.categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador()
                + " AND categoriaAtributoTalento.formacaoDescricao = 'S'";

        // Retorna
        int intTotalRegistros = obterTotalRegistros(strConsulta);
        if (intTotalRegistros > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade gruposCategoriaTalento, que  lazy
     * 
     * @param objCategoriaTalento Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarGruposCategoriaTalento(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        inicializarRelacionamento(objCategoriaTalento.getGruposCategoriaTalento());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade gruposCategoriaTalento, que  lazy
     * 
     * @param objCategoriaTalento Lista com os objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarGruposCategoriaTalento(List lstCategoriasTalento) throws DAOException
    {
        Iterator itrCategoriasTalento = lstCategoriasTalento.iterator();
        while (itrCategoriasTalento.hasNext())
        {
            inicializarGruposCategoriaTalento((CategoriaTalento) itrCategoriasTalento.next());
        }
    }
}
