/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.dao;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.biblioteca.util.DataNova;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.ManipuladorSql;
import br.gov.camara.negocio.bancotalentos.facade.ConsultaTalentoFacade;
import br.gov.camara.negocio.bancotalentos.pojo.ConsultaTalento;
import br.gov.camara.negocio.bancotalentos.pojo.ParametroConsultaTalento;
import br.gov.camara.negocio.bancotalentos.pojo.Pessoa;
import br.gov.camara.negocio.bancotalentos.util.BancoTalentosUtil;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.seguranca.UsuarioAutenticado;

/**
 * Classe DAO para montar a consulta de talentos
 */
public class ConsultaTalentoDAO extends DAO
{
    private static final int PREENCHIMENTO_REALIZADO = 1;
    private static final int PREENCHIMENTO_NAO_REALIZADO = 2;

    private static final int CATEGORIA_DADOS_PESSOAIS = -1;

    // Variveis de instncia
    private static Log log = LogFactory.getLog(ConsultaTalentoDAO.class.getName());

    /**
     * Construtor sem argumentos 
     */
    public ConsultaTalentoDAO(String nomeConexao)
    {
        super("Consulta de talentos", nomeConexao);
    }

    /**
     * No implementado
     * 
     */
    public List obterTodos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Dipara exceo
        throw new DAOException("Mtodo no implementado");
    }

    /**
     * Mtodo no implementado
     * 
     */
    public String excluirImpl(Object objAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Dipara exceo
        throw new DAOException("Mtodo no implementado");
    }

    /**
     * Efetua consulta de talentos
     * 
     * @param mapConsulta Contendo os parmetros de consulta
     * @param objPessoa Pessoa que est realizando a consulta
     * @param lstPerfisSistema Perfis de sistema os quais a pessoa pertence
     * @param intMaximoPermitido Mximo de registros permitidos para o retorno da consulta
     * @param stbTotalRetornado Total de registros retornados
     * 
     * @return List Contendo o resultado da consulta, se o nmero mximo no for ultrapassado
     * 
     * @throws DAOException se ocorrer um erro relacionado com banco de dados 
     */

    public List consultar(Map mapConsulta, UsuarioAutenticado objPessoa, int intMaximoPermitido, StringBuilder stbTotalRetornado, boolean blnFiltroGrupo) throws DAOException
    {
        // Log
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Retorno
        Map mapRetorno = new HashMap<Integer, Pessoa>();

        // Declaraes
        List<Integer> gruposPermitidos = null;
        List<ManipuladorSql> lstQuery = new ArrayList<ManipuladorSql>();
        ManipuladorSql lstFromTalento = new ManipuladorSql();
        ManipuladorSql lstWhereTalento = new ManipuladorSql();
        int intSequencialPessoa = 0;
        int intSequencialTalento = 0;
        int intSequencialAtributoTalentoValorado = 0;

        // Verifica se h filtro por grupo
        if (blnFiltroGrupo)
        {
            FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
            gruposPermitidos = objFiltroConsultaDAO.obterListaCodigosGruposPermitidos(objPessoa.obterIdentificador().toString());
        }

        lstFromTalento = criarParteFromDaSubQuery(intSequencialPessoa);
        lstWhereTalento = criarParteWhereDaSubQuery(gruposPermitidos, intSequencialPessoa);

        // Cria query de consulta de talentos
        Map<String, ConsultaTalento> parametrosConsulta = (Map<String, ConsultaTalento>) mapConsulta;

        // Itera critrios de consulta
        int contador = 0;
        for (String chave : parametrosConsulta.keySet())
        {
            // Obtm beaConsulta
            ConsultaTalento criterioConsultaTalento = (ConsultaTalento) mapConsulta.get(chave);

            // Cria query de pessoa, se houver um UNION (OR)
            if (criterioConsultaTalento.getCriterioConsultaTalento().getIdentificador().equals(2) && contador++ > 0)
            {
                lstQuery.add(new ManipuladorSql().adicionarManipulador(lstFromTalento).adicionarManipulador(lstWhereTalento));

                lstFromTalento = criarParteFromDaSubQuery(intSequencialPessoa);
                lstWhereTalento = criarParteWhereDaSubQuery(gruposPermitidos, intSequencialPessoa);
            }

            // Se a categoria for DADOS PESSOAIS
            if (criterioConsultaTalento.getCategoriaTalento().getIdentificador().intValue() == CATEGORIA_DADOS_PESSOAIS)
            {
                Iterator itrPametros = criterioConsultaTalento.getListaParametrosConsulta().iterator();

                // Itera consulta
                while (itrPametros.hasNext())
                {
                    // Obtm dados
                    ParametroConsultaTalento objParametroConsultaTalento = (ParametroConsultaTalento) itrPametros.next();

                    if (objParametroConsultaTalento.getCategoriaAtributoTalento().getAtributoTalento().getIdentificador().equals(ConsultaTalentoFacade.ID_ATRIBUTO_NOME))
                    {
                        // Monta opes para nome
                        String conectorWhereOuAnd = lstWhereTalento.gerarSqlFinal().isEmpty() ? " WHERE " : " AND ";

                        lstWhereTalento.adicionarSql(conectorWhereOuAnd
                                + " UPPER(pessoa"
                                + intSequencialPessoa
                                + ".nome) LIKE '%"
                                + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao().toUpperCase().replaceAll("'", "''")
                                + "%'");
                    }
                    else if (objParametroConsultaTalento.getCategoriaAtributoTalento().getAtributoTalento().getIdentificador().equals(ConsultaTalentoFacade.ID_ATRIBUTO_IDADE))
                    {
                        String strDataInicial = null;
                        String strDataFinal = null;

                        // Monta opes para idade inicial
                        if (objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao() != null
                                && !"".equals(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao()))
                        {
                            Calendar calDataInicial = new GregorianCalendar();
                            calDataInicial.add(Calendar.YEAR, -(Integer.parseInt(String.valueOf(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao()))));
                            strDataInicial = "'" + formatarParametroData(calDataInicial.getTime()) + "'";
                        }

                        // Monta opes para idade final
                        if (objParametroConsultaTalento.getOpcaoComplementar() != null && !"".equals(objParametroConsultaTalento.getOpcaoComplementar()))
                        {
                            Calendar calDataFinal = new GregorianCalendar();
                            calDataFinal.add(Calendar.YEAR, -(Integer.parseInt(objParametroConsultaTalento.getOpcaoComplementar())));
                            calDataFinal.add(Calendar.DAY_OF_MONTH, 1);
                            calDataFinal.add(Calendar.YEAR, -1);
                            strDataFinal = "'" + formatarParametroData(calDataFinal.getTime()) + "'";
                        }

                        String conectorWhereOuAnd = lstWhereTalento.gerarSqlFinal().isEmpty() ? " WHERE " : " AND ";
                        lstWhereTalento.adicionarSql(conectorWhereOuAnd + " pessoa" + intSequencialPessoa + ".dataNascimento");

                        if (strDataInicial != null && strDataFinal != null)
                        {
                            lstWhereTalento.adicionarSql(" BETWEEN " + strDataFinal + " AND " + strDataInicial);
                        }
                        else if (strDataFinal != null)
                        {
                            lstWhereTalento.adicionarSql(" >= " + strDataFinal);
                        }
                        else if (strDataInicial != null)
                        {
                            lstWhereTalento.adicionarSql(" <= " + strDataInicial);
                        }
                    }
                    else if (objParametroConsultaTalento.getCategoriaAtributoTalento().getAtributoTalento().getIdentificador().equals(ConsultaTalentoFacade.ID_ATRIBUTO_SEXO))
                    {
                        // Monta opes para sexo
                        String conectorWhereOuAnd = lstWhereTalento.gerarSqlFinal().isEmpty() ? " WHERE " : " AND ";
                        lstWhereTalento.adicionarSql(conectorWhereOuAnd
                                + " pessoa"
                                + intSequencialPessoa
                                + ".sexo = '"
                                + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao().substring(0, 1).toUpperCase()
                                + "'");
                    }
                }
            }
            // Se POSSUI preenchimento
            else if (criterioConsultaTalento.getTipoConsultaTalento().getIdentificador().equals(PREENCHIMENTO_REALIZADO))
            {
                // Monta novo talento
                lstFromTalento.adicionarSql(", Talento talento" + ++intSequencialTalento);
                String conectorWhereOuAnd = lstWhereTalento.gerarSqlFinal().isEmpty() ? " WHERE " : " AND ";
                lstWhereTalento.adicionarSql(conectorWhereOuAnd
                        + " talento"
                        + intSequencialTalento
                        + ".categoriaTalento.identificador = "
                        + criterioConsultaTalento.getCategoriaTalento().getIdentificador()
                        + " AND talento"
                        + intSequencialTalento
                        + ".pessoa.identificador = pessoa"
                        + intSequencialPessoa
                        + ".identificador");

                if (criterioConsultaTalento.getListaParametrosConsulta() != null)
                {
                    Iterator itrPametros = criterioConsultaTalento.getListaParametrosConsulta().iterator();

                    // Itera consulta
                    while (itrPametros.hasNext())
                    {
                        // Obtm dados
                        ParametroConsultaTalento objParametroConsultaTalento = (ParametroConsultaTalento) itrPametros.next();

                        lstFromTalento.adicionarSql(", AtributoTalentoValorado atributoTalentoValorado" + ++intSequencialAtributoTalentoValorado);
                        lstWhereTalento.adicionarSql(" AND talento"
                                + intSequencialTalento
                                + ".identificador = atributoTalentoValorado"
                                + intSequencialAtributoTalentoValorado
                                + ".talento.identificador"
                                + " AND atributoTalentoValorado"
                                + intSequencialAtributoTalentoValorado
                                + ".categoriaAtributoTalento.identificador = "
                                + objParametroConsultaTalento.getCategoriaAtributoTalento().getIdentificador());

                        if (objParametroConsultaTalento.getAtributoTalentoOpcao() != null)
                        {
                            if (objParametroConsultaTalento.getOpcaoComplementar() == null)
                            {
                                if (objParametroConsultaTalento.getAtributoTalentoOpcao().getIdentificador() == null)
                                {
                                    lstWhereTalento.adicionarSql(" AND UPPER(atributoTalentoValorado"
                                            + intSequencialAtributoTalentoValorado
                                            + ".valoracao) LIKE '%"
                                            + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao().toUpperCase().replaceAll("'", "''")
                                            + "%'");
                                }
                                else
                                {
                                    lstWhereTalento.adicionarSql(" AND atributoTalentoValorado"
                                            + intSequencialAtributoTalentoValorado
                                            + ".atributoTalentoOpcao.identificador = "
                                            + objParametroConsultaTalento.getAtributoTalentoOpcao().getIdentificador());
                                }
                            }
                            else
                            {
                                boolean isData = false;
                                if (DataNova.validarData(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        || DataNova.validarData(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    isData = true;
                                }

                                if (!"".equals(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        && !"".equals(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    if (isData)
                                    {
                                        lstWhereTalento.adicionarSql(" AND ("
                                                + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "atributoTalentoValorado"
                                                        + intSequencialAtributoTalentoValorado
                                                        + ".valoracao")
                                                + " BETWEEN '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao()))
                                                + "' AND '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getOpcaoComplementar()))
                                                + "')");
                                    }
                                    else
                                    {
                                        lstWhereTalento.adicionarSql(" AND atributoTalentoValorado"
                                                + intSequencialAtributoTalentoValorado
                                                + ".valoracao BETWEEN "
                                                + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao()
                                                + " AND "
                                                + objParametroConsultaTalento.getOpcaoComplementar());
                                    }
                                }
                                else if ("".equals(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        && !"".equals(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    if (isData)
                                    {
                                        lstWhereTalento.adicionarSql(" AND ("
                                                + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "atributoTalentoValorado"
                                                        + intSequencialAtributoTalentoValorado
                                                        + ".valoracao")
                                                + " <= '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getOpcaoComplementar()))
                                                + "')");
                                    }
                                    else
                                    {
                                        lstWhereTalento.adicionarSql(" AND atributoTalentoValorado"
                                                + intSequencialAtributoTalentoValorado
                                                + ".valoracao <= "
                                                + objParametroConsultaTalento.getOpcaoComplementar());
                                    }
                                }
                                else if (!"".equals(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        && "".equals(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    if (isData)
                                    {
                                        lstWhereTalento.adicionarSql(" AND ("
                                                + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "atributoTalentoValorado"
                                                        + intSequencialAtributoTalentoValorado
                                                        + ".valoracao")
                                                + " >= '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao()))
                                                + "')");
                                    }
                                    else
                                    {
                                        lstWhereTalento.adicionarSql(" AND atributoTalentoValorado"
                                                + intSequencialAtributoTalentoValorado
                                                + ".valoracao >= "
                                                + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao());
                                    }
                                }

                            }
                        }

                    }
                }

            }
            // Se NO POSSUI preenchimento
            else if (criterioConsultaTalento.getTipoConsultaTalento().getIdentificador().equals(PREENCHIMENTO_NAO_REALIZADO))
            {
                ManipuladorSql lstWhereNotExists = new ManipuladorSql();

                String conectorWhereOuAnd = lstWhereTalento.gerarSqlFinal().isEmpty() ? " WHERE " : " AND ";
                lstWhereTalento.adicionarSql(conectorWhereOuAnd
                        + " NOT EXISTS (SELECT pessoa"
                        + ++intSequencialTalento
                        + ".identificador"
                        + " FROM Pessoa pessoa"
                        + intSequencialTalento);
                lstWhereTalento.adicionarSql(", Talento talento" + intSequencialTalento);
                lstWhereNotExists.adicionarSql(" WHERE pessoa" + intSequencialPessoa + ".identificador = pessoa" + intSequencialTalento + ".identificador");
                lstWhereNotExists.adicionarSql(" AND talento"
                        + intSequencialTalento
                        + ".pessoa.identificador = pessoa"
                        + intSequencialPessoa
                        + ".identificador"
                        + " AND talento"
                        + intSequencialTalento
                        + ".categoriaTalento.identificador = "
                        + criterioConsultaTalento.getCategoriaTalento().getIdentificador());

                if (criterioConsultaTalento.getListaParametrosConsulta() != null)
                {
                    Iterator itrPametros = criterioConsultaTalento.getListaParametrosConsulta().iterator();

                    // Itera consulta
                    while (itrPametros.hasNext())
                    {
                        // Obtm dados
                        ParametroConsultaTalento objParametroConsultaTalento = (ParametroConsultaTalento) itrPametros.next();

                        lstWhereTalento.adicionarSql(", AtributoTalentoValorado atributoTalentoValorado" + ++intSequencialAtributoTalentoValorado);
                        lstWhereNotExists.adicionarSql(" AND talento"
                                + intSequencialTalento
                                + ".identificador = atributoTalentoValorado"
                                + intSequencialAtributoTalentoValorado
                                + ".talento.identificador"
                                + " AND atributoTalentoValorado"
                                + intSequencialAtributoTalentoValorado
                                + ".categoriaAtributoTalento.identificador = "
                                + objParametroConsultaTalento.getCategoriaAtributoTalento().getIdentificador());
                        if (objParametroConsultaTalento.getAtributoTalentoOpcao() != null)
                        {

                            if (objParametroConsultaTalento.getOpcaoComplementar() == null)
                            {
                                if (objParametroConsultaTalento.getAtributoTalentoOpcao().getIdentificador() == null)
                                {
                                    lstWhereNotExists.adicionarSql(" AND UPPER(atributoTalentoValorado"
                                            + intSequencialAtributoTalentoValorado
                                            + ".valoracao) LIKE '%"
                                            + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao().toUpperCase().replaceAll("'", "''")
                                            + "%'");
                                }
                                else
                                {
                                    lstWhereNotExists.adicionarSql(" AND atributoTalentoValorado"
                                            + intSequencialAtributoTalentoValorado
                                            + ".atributoTalentoOpcao.identificador = "
                                            + objParametroConsultaTalento.getAtributoTalentoOpcao().getIdentificador());
                                }
                            }
                            else
                            {
                                boolean isData = false;
                                if (DataNova.validarData(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        || DataNova.validarData(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    isData = true;
                                }

                                if (!"".equals(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        && !"".equals(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    if (isData)
                                    {
                                        lstWhereNotExists.adicionarSql(" AND ("
                                                + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "atributoTalentoValorado"
                                                        + intSequencialAtributoTalentoValorado
                                                        + ".valoracao")
                                                + " BETWEEN '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao()))
                                                + "' AND '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getOpcaoComplementar()))
                                                + "')");
                                    }
                                    else
                                    {
                                        lstWhereNotExists.adicionarSql(" AND atributoTalentoValorado"
                                                + intSequencialAtributoTalentoValorado
                                                + ".valoracao BETWEEN "
                                                + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao()
                                                + " AND "
                                                + objParametroConsultaTalento.getOpcaoComplementar());
                                    }
                                }
                                else if ("".equals(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        && !"".equals(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    if (isData)
                                    {
                                        lstWhereNotExists.adicionarSql(" AND ("
                                                + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "atributoTalentoValorado"
                                                        + intSequencialAtributoTalentoValorado
                                                        + ".valoracao")
                                                + " <= '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getOpcaoComplementar()))
                                                + "')");
                                    }
                                    else
                                    {
                                        lstWhereNotExists.adicionarSql(" AND atributoTalentoValorado"
                                                + intSequencialAtributoTalentoValorado
                                                + ".valoracao <= "
                                                + objParametroConsultaTalento.getOpcaoComplementar());
                                    }
                                }
                                else if (!"".equals(objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao())
                                        && "".equals(objParametroConsultaTalento.getOpcaoComplementar()))
                                {
                                    if (isData)
                                    {
                                        lstWhereNotExists.adicionarSql(" AND ("
                                                + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "atributoTalentoValorado"
                                                        + intSequencialAtributoTalentoValorado
                                                        + ".valoracao")
                                                + " >= '"
                                                + formatarParametroData(DataNova.converter(objParametroConsultaTalento.getOpcaoComplementar()))
                                                + "')");
                                    }
                                    else
                                    {
                                        lstWhereNotExists.adicionarSql(" AND atributoTalentoValorado"
                                                + intSequencialAtributoTalentoValorado
                                                + ".valoracao >= "
                                                + objParametroConsultaTalento.getAtributoTalentoOpcao().getDescricao());
                                    }
                                }

                            }
                        }
                    }

                }
                lstWhereTalento.adicionarManipulador(lstWhereNotExists);
                lstWhereTalento.adicionarSql(")");
            }
        }

        // Monta a consulta de talentos
        int totalEncontradoAcumulado = 0;
        lstQuery.add(new ManipuladorSql().adicionarManipulador(lstFromTalento).adicionarManipulador(lstWhereTalento));
        for (ManipuladorSql objQueries : lstQuery)
        {
            // Monta query
            ManipuladorSql strQuery = new ManipuladorSql("SELECT COUNT(DISTINCT pessoa" + intSequencialPessoa + ".identificador)");
            strQuery.adicionarManipulador(objQueries);

            // Verifica o total de registros da query
            int intTotalRetornado = obterTotalRegistrosPorHQL(strQuery);
            totalEncontradoAcumulado += intTotalRetornado;
            if ((intMaximoPermitido == -1) || (totalEncontradoAcumulado <= intMaximoPermitido))
            {
                // Efetua consulta
                ManipuladorSql strQueryRetornoTemp = new ManipuladorSql("SELECT DISTINCT pessoa" + intSequencialPessoa + " ");
                strQueryRetornoTemp.adicionarManipulador(objQueries);
                strQueryRetornoTemp.adicionarSql(" ORDER BY pessoa" + intSequencialPessoa + ".nome");
                List lstRetornoTemp = obterListaPorHQL(strQueryRetornoTemp);
                for (int i = 0; i < lstRetornoTemp.size(); i++)
                {
                    if (mapRetorno.get(((Pessoa) lstRetornoTemp.get(i)).getIdentificador()) == null)
                    {
                        mapRetorno.put(((Pessoa) lstRetornoTemp.get(i)).getIdentificador(), ((Pessoa) lstRetornoTemp.get(i)));
                    }
                }
            }
            else
            {
                mapRetorno = null;
                break;
            }
        }

        // Retorno
        if (mapRetorno != null)
        {
            stbTotalRetornado.setLength(0);
            stbTotalRetornado.append(mapRetorno.size());

            return new ArrayList(mapRetorno.values());
        }

        stbTotalRetornado.setLength(0);
        stbTotalRetornado.append(totalEncontradoAcumulado);
        return new ArrayList(1);
    }

    private ManipuladorSql criarParteFromDaSubQuery(int intSequencialPessoa)
    {
        ManipuladorSql subqueryFrom = new ManipuladorSql();

        // Cria query de pessoa com talentos cadastrados
        subqueryFrom.adicionarSql("FROM Talento talento JOIN talento.pessoa AS pessoa" + intSequencialPessoa);

        return subqueryFrom;
    }

    private ManipuladorSql criarParteWhereDaSubQuery(List<Integer> gruposPermitidos, int intSequencialPessoa)
    {
        ManipuladorSql subqueryWhere = new ManipuladorSql();

        if (gruposPermitidos != null)
        {
            subqueryWhere.adicionarSql("WHERE talento.pessoa.grupo.codigo IN (:filtroPorGrupo)");
            subqueryWhere.comParametro("filtroPorGrupo", gruposPermitidos);
        }

        return subqueryWhere;
    }

}
