/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.dao;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.autenticacaoperfil.dao.UsuarioSistemaDAO;
import br.gov.camara.negocio.autenticacaoperfil.pojo.Grupo;
import br.gov.camara.negocio.autenticacaoperfil.pojo.UsuarioSistema;
import br.gov.camara.negocio.bancotalentos.pojo.FiltroConsulta;
import br.gov.camara.negocio.bancotalentos.pojo.TipoFiltroConsulta;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.negocio.util.Consulta;

/**
 * Classe DAO para a tabela FiltroConsulta
 */

public class FiltroConsultaDAO extends DAO
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(FiltroConsultaDAO.class.getName());

    /**
     * Construtor sem argumentos
     * 
     */
    public FiltroConsultaDAO(String nomeConexao)
    {
        super("FiltroConsulta", nomeConexao);
    }

    /**
     * Obtm todos os registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " FiltroConsulta filtroConsulta" +

                " ORDER BY" + " filtroConsulta.identificador ASC";

        return super.obter(strConsulta);
    }

    /**
     * Obtm todos os registros pelo tipo de filtro de consulta
     * 
     * @param TipoFiltroConsulta Objeto contendo os dados do tipo de filtro de consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodosPorTipoFiltroConsulta(TipoFiltroConsulta objTipoFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " FiltroConsulta filtroConsulta"
                + " WHERE filtroConsulta.tipoFiltroConsulta.identificador = "
                + objTipoFiltroConsulta.getIdentificador()
                + " ORDER BY"
                + " filtroConsulta.identificador ASC";

        return super.obter(strConsulta);
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Object POJO representando o registro obtido
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public Object obterPelaChave(String strChave) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        String strConsulta = " FROM" + " FiltroConsulta filtroConsulta" +

                " WHERE " + " filtroConsulta.identificador = " + strChave;

        return (FiltroConsulta) obter(strConsulta).get(0);
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Total de registro
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT COUNT(*)" +

                " FROM" + " FiltroConsulta filtroConsulta";

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm o total de registros pelo tipo de filtro da consulta
     * 
     * @return TipoFiltroConsulta Objeto contendo os dados do tipo de filtro de consulta
     * 
     * @return int Total de registros
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistrosPorTipoFiltroConsulta(TipoFiltroConsulta objTipoFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT COUNT(*)"
                + " FROM"
                + " FiltroConsulta filtroConsulta"
                + " WHERE filtroConsulta.tipoFiltroConsulta.identificador = "
                + objTipoFiltroConsulta.getIdentificador();

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm os registros de determinada pgina
     *
     * @param int Contendo o nmero da pgina
     * @param int Contendo o mximo de registros por pgina
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        // Monta consulta
        String strConsulta = " FROM" + " FiltroConsulta filtroConsulta" +

                " ORDER BY" + " filtroConsulta.identificador ASC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Obtm os registros de determinada pgina
     * 
     * @param TipoFiltroConsulta Objeto contendo os dados do tipo de filtro de consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorTipoFiltroConsultaPorPagina(TipoFiltroConsulta objTipoFiltroConsulta, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        // Monta consulta
        String strConsulta = " FROM"
                + " FiltroConsulta filtroConsulta"
                + " WHERE filtroConsulta.tipoFiltroConsulta.identificador = "
                + objTipoFiltroConsulta.getIdentificador()
                + " ORDER BY"
                + " filtroConsulta.identificador ASC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Obtm total de registros da consulta
     *
     * @param Consulta objeto contendo os filtros para consulta
     *  
     * @return List contendo o resultado da pesquisa
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros(Consulta objConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        int intRetorno = 0;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " SELECT COUNT(*)" + " FROM" + " FiltroConsulta filtroConsulta";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }

            // Recupera os dados
            intRetorno = obterTotalRegistros(strConsulta);

            // Retorna
            return intRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     * Exclui um registro
     *
     * @param Object POJO representando o objeto a ser excludo
     * 
     * @return String Contendo a consulta de excluso 
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public String excluirImpl(Object objPojo) throws DAOException
    {
        return " FROM FiltroConsulta filtroConsulta" + " WHERE filtroConsulta.identificador = " + ((FiltroConsulta) objPojo).getIdentificador();
    }

    /**
     * Mtodo que pede a inicializao da propriedade grupoCriterioConsulta, que  lazy
     * 
     * @param FiltroConsulta Classe que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarGrupoCriterioConsulta(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        inicializarRelacionamento(objFiltroConsulta.getGrupoCriterioConsulta());
    }

    /* Mtodo que pede a inicializao da propriedade grupoCriterioConsulta, que  lazy
     * 
     * @param List Lista contendo o objetos do tipo GrupoCriterioConsulta
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarGrupoCriterioConsulta(List lstPerfisSistema) throws DAOException
    {
        Iterator itrGrupoCriterioConsulta = lstPerfisSistema.iterator();
        while (itrGrupoCriterioConsulta.hasNext())
        {
            inicializarRelacionamento(((FiltroConsulta) itrGrupoCriterioConsulta.next()).getGrupoCriterioConsulta());
        }
    }

    /**
     * Mtodo que pede a inicializao da propriedade filtroConsultaGrupo, que  lazy
     * 
     * @param FiltroConsulta Classe que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarFiltroConsultaGrupo(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        inicializarRelacionamento(objFiltroConsulta.getFiltroConsultaGrupo());
    }

    /* Mtodo que pede a inicializao da propriedade filtroConsultaGrupo, que  lazy
     * 
     * @param List Contm objetos do tipo FiltroConsultaGrupo
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarFiltroConsultaGrupo(List lstFiltroConsulta) throws DAOException
    {
        Iterator itrFiltroConsultaGrupo = lstFiltroConsulta.iterator();
        while (itrFiltroConsultaGrupo.hasNext())
        {
            inicializarRelacionamento(((FiltroConsulta) itrFiltroConsultaGrupo.next()).getFiltroConsultaGrupo());
        }
    }

    /**
     * Mtodo que pede a inicializao da propriedade filtroConsultaUsuario, que  lazy
     * 
     * @param FiltroConsulta Classe que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarFiltroConsultaUsuario(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        inicializarRelacionamento(objFiltroConsulta.getFiltroConsultaUsuario());
        /* TODO
         * Cgigo para fazer a inicializao do pojo. O mtodo acima: inicializarRelacionamento no est fazendo
         * a inicializao devida. Por isso que foi feito o Iterator da coleo.
         */
        Iterator itrFiltroConsultaUsuario = objFiltroConsulta.getFiltroConsultaUsuario().iterator();
        while (itrFiltroConsultaUsuario.hasNext())
        {
            String strNome = ((UsuarioSistema) itrFiltroConsultaUsuario.next()).getNome();
        }

    }

    /**
     * Mtodo que pede a inicializao da propriedade tipofiltroConsulta, que 
     * lazy
     * 
     * @param FiltroConsulta
     *            Classe que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException
     *             Se ocorrer algum erro relacionado com o acesso a banco de
     *             dados
     */
    public void inicializarTipoFiltroConsulta(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        inicializarRelacionamento(objFiltroConsulta.getTipoFiltroConsulta());
    }

    /**
     * Obtm os grupos disponveis para determinado Filtro de Consulta
     * 
     * @param objFiltroConsulta
     *            Filtro de Consulta a ser verificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     * 
     * @throws DAOException
     *             se ocorrer algum erro relacionado com o acesso a banco de
     *             dados
     * 
     */
    public List obterGruposDisponiveisFiltroConsultaGrupo(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " Grupo grupo"
                +

                " WHERE"
                + " grupo.codigo NOT IN"
                +

                " (SELECT filtroConsulta.filtroConsultaGrupo.elements"
                + " FROM"
                + " FiltroConsulta filtroConsulta"
                + " WHERE"
                + " filtroConsulta.identificador = "
                + objFiltroConsulta.getIdentificador()
                + ")"
                +

                " ORDER BY"
                + " grupo.descricao ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm os grupos disponveis para determinado Filtro de Consulta 
     * 
     * @param objFiltroConsulta Filtro de Consulta a ser verificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterGruposDisponiveisCriterioConsulta(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " Grupo grupo"
                +

                " WHERE"
                + " grupo.codigo NOT IN"
                +

                " (SELECT filtroConsulta.grupoCriterioConsulta.elements"
                + " FROM"
                + " FiltroConsulta filtroConsulta"
                + " WHERE"
                + " filtroConsulta.identificador = "
                + objFiltroConsulta.getIdentificador()
                + ")"
                +

                " ORDER BY"
                + " grupo.descricao ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm os grupos disponveis para determinado Filtro de Consulta
     * 
     * @param objFiltroConsulta Filtro de Consulta a ser verificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterGruposSelecionadosFiltroConsultaGrupo(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " Grupo grupo"
                +

                " WHERE"
                + " grupo.codigo IN"
                +

                " (SELECT filtroConsulta.filtroConsultaGrupo.elements"
                + " FROM"
                + " FiltroConsulta filtroConsulta"
                + " WHERE"
                + " filtroConsulta.identificador = "
                + objFiltroConsulta.getIdentificador()
                + ")"
                +

                " ORDER BY"
                + " grupo.descricao ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm os grupos disponveis para determinado Filtro de Consulta
     * 
     * @param objFiltroConsulta Filtro de Consulta a ser verificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterGruposSelecionadosCriterioConsulta(FiltroConsulta objFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM"
                + " Grupo grupo"
                +

                " WHERE"
                + " grupo.codigo IN"
                +

                " (SELECT filtroConsulta.grupoCriterioConsulta.elements"
                + " FROM"
                + " FiltroConsulta filtroConsulta"
                + " WHERE"
                + " filtroConsulta.identificador = "
                + objFiltroConsulta.getIdentificador()
                + ")"
                +

                " ORDER BY"
                + " grupo.descricao ASC";

        return obter(strConsulta);

    }

    /**
     * Mtodo que retorna os grupos habilitados ao usurio passado como
     * parmetro
     * 
     * @param String
     *            Identificador do usurio
     * 
     * @throws DAOException
     *             Se ocorrer algum erro relacionado com o acesso a banco de
     *             dados
     */
    public String obterGruposHabilitadosConsultaCurriculo(String strIdentificadorUsuario) throws DAOException
    {
        return obterConjuntoCodigosGruposPermitidos(strIdentificadorUsuario, ", ");
    }

    public String obterConjuntoCodigosGruposPermitidos(String strIdentificadorUsuario, String separador) throws DAOException
    {
        StringBuilder grupos = new StringBuilder();

        List<Integer> listaCodigosGrupos = obterListaCodigosGruposPermitidos(strIdentificadorUsuario);
        if (!listaCodigosGrupos.isEmpty())
        {
            Iterator<Integer> itrGrupoCriterioConsulta = listaCodigosGrupos.iterator();
            while (itrGrupoCriterioConsulta.hasNext())
            {
                Integer codigoGrupo = itrGrupoCriterioConsulta.next();
                grupos.append(codigoGrupo);
                grupos.append(separador);
            }
            int tamanho = grupos.length();
            grupos.delete(tamanho - separador.length(), tamanho);
        }

        return grupos.toString();
    }

    public List<Integer> obterListaCodigosGruposPermitidos(String strIdentificadorUsuario) throws DAOException
    {
        Set<Integer> gruposHabilitados = new HashSet<Integer>();

        // Obtm usurio logado
        UsuarioSistemaDAO objUsuario = new UsuarioSistemaDAO(obterNomeConexao());

        // Obtm grupos de consulta para o usurio
        UsuarioSistema objUsuarioSistema = (UsuarioSistema) objUsuario.obterPelaChave(strIdentificadorUsuario);
        if (objUsuarioSistema != null)
        {
            List<Grupo> lstGrupoCriterioConsulta = null;

            lstGrupoCriterioConsulta = (List<Grupo>) obterCriterioConsultaPorUsuario(objUsuarioSistema.getIdentificador().toString(), "ConsultaCurriculo");
            if (lstGrupoCriterioConsulta != null)
            {
                Iterator<Grupo> itrGrupoCriterioConsulta = lstGrupoCriterioConsulta.iterator();
                while (itrGrupoCriterioConsulta.hasNext())
                {
                    Grupo objGrupoCriterio = (Grupo) itrGrupoCriterioConsulta.next();
                    gruposHabilitados.add(objGrupoCriterio.getCodigo());
                }
            }

            if (objUsuarioSistema.getGrupo() != null)
            {
                lstGrupoCriterioConsulta = (List<Grupo>) obterCriterioConsultaPorGrupo(objUsuarioSistema.getGrupo().getCodigo().toString(), "ConsultaCurriculo");
                if (lstGrupoCriterioConsulta != null)
                {
                    Iterator<Grupo> itrGrupoCriterioConsulta = lstGrupoCriterioConsulta.iterator();
                    while (itrGrupoCriterioConsulta.hasNext())
                    {
                        Grupo objGrupoCriterio = (Grupo) itrGrupoCriterioConsulta.next();
                        gruposHabilitados.add(objGrupoCriterio.getCodigo());
                    }
                }
            }
        }

        if (gruposHabilitados.isEmpty())
        {
            throw new DAOException("O usurio no tem permisso para visualizar nenhum grupo.");
        }

        return new ArrayList<Integer>(gruposHabilitados);
    }

    /**
     * Mtodo que pede a inicializao da propriedade filtroConsultaUsuario, que  lazy
     * 
     * @param UsuarioSistema Classe que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    /*
     * public void inicializarFiltroConsultaUsuario(UsuarioSistema
     * objUsuarioSistema) throws DAOException {
     * inicializarRelacionamento(objUsuarioSistema.getFiltroConsultaUsuario());
     * }
     */

    /**
     *  Mtodo que pede a inicializao da propriedade filtroConsultaUsuario, que  lazy
     * 
     * @param List  Lista de objetos que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    /*
     * public void inicializarFiltroConsultaUsuario(List
     * lstFiltroConsultaUsuario) throws DAOException { Iterator
     * itrFiltroConsultaUsuario = lstFiltroConsultaUsuario.iterator(); while
     * (itrFiltroConsultaUsuario.hasNext()) {
     * inicializarRelacionamento(((UsuarioSistema)
     * itrFiltroConsultaUsuario.next()).getFiltroConsultaUsuario()); } }
     */
    /**
     * Obtm todos os critrios de consulta (Grupos) associados ao grupo e ao tipo de filtro consulta especificados
     * 
     * @param TipoFiltroConsulta Objeto contendo os dados do tipo de filtro de consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    private List obterCriterioConsultaPorGrupo(String codGrupo, String nomObjetoControladoTipoFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT criterio"
                + " FROM"
                + " FiltroConsulta as filtro"
                + " join filtro.grupoCriterioConsulta as criterio"
                + " join filtro.filtroConsultaGrupo as filtroGrupo"
                + " join filtro.tipoFiltroConsulta as tipoFiltroConsulta"
                + " WHERE tipoFiltroConsulta.nomeObjetoControlado = '"
                + nomObjetoControladoTipoFiltroConsulta
                + "'"
                + " AND filtroGrupo IN"
                + " (FROM Grupo grupo"
                + " WHERE grupo.codigo = "
                + codGrupo
                + ")";

        return super.obter(strConsulta);
    }

    /**
     * Obtm todos os critrios de consulta (Grupos) associados ao grupo e ao tipo de filtro consulta especificados
     * 
     * @param TipoFiltroConsulta Objeto contendo os dados do tipo de filtro de consulta
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    private List obterCriterioConsultaPorUsuario(String ideUsuario, String nomObjetoControladoTipoFiltroConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT criterio"
                + " FROM"
                + " FiltroConsulta as filtro"
                + " join filtro.grupoCriterioConsulta as criterio"
                + " join filtro.filtroConsultaUsuario as filtroUsuario"
                + " join filtro.tipoFiltroConsulta as tipoFiltroConsulta"
                + " WHERE tipoFiltroConsulta.nomeObjetoControlado = '"
                + nomObjetoControladoTipoFiltroConsulta
                + "'"
                + " AND filtroUsuario IN"
                + " (FROM UsuarioSistema usuario"
                + " WHERE usuario.identificador = "
                + ideUsuario
                + ")";

        return super.obter(strConsulta);
    }
}
