/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.dao;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.ManipuladorSql;
import br.gov.camara.negocio.ParametroDAO;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaAtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaTalento;
import br.gov.camara.negocio.bancotalentos.pojo.Pessoa;
import br.gov.camara.negocio.bancotalentos.pojo.Talento;
import br.gov.camara.negocio.bancotalentos.util.BancoTalentosUtil;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.negocio.util.Consulta;
import br.gov.camara.negocio.util.ConsultaComum;

/**
 * Classe DAO para a tabela Talento
 */
public class TalentoDAO extends DAO implements ConsultaComum
{
    private static Log log = LogFactory.getLog(TalentoDAO.class.getName());

    /**
     * Construtor sem argumentos
     */
    public TalentoDAO(String nomeConexao)
    {
        super("Talentos", nomeConexao);
    }

    /**
     * Obtm todos os registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterTodos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " FROM" + " Talento talento" +

                " ORDER BY" + " talento.identificador ASC";

        return obter(strConsulta);

    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Object POJO representando o registro obtido
     *
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public Object obterPelaChave(String strChave) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta consulta
        StringBuffer consulta = new StringBuffer();
        ParametroDAO parametroDAO = new ParametroDAO();

        consulta.append(" FROM Talento talento");
        consulta.append(" LEFT JOIN FETCH talento.categoriaTalento categoriaTalento ");
        consulta.append(" WHERE talento.identificador = :idTalento");
        parametroDAO.adicionar("idTalento", Integer.valueOf(strChave));

        // Efetua consulta
        Talento retorno = (Talento) obterObjetoPorHQL(consulta.toString(), parametroDAO);

        return retorno;

    }

    /**
     * Obtm o total de registros por pessoa
     * 
     * @param objPessoa Pessoa desejada
     * 
     * @return int Contendo o total
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistrosPorPessoa(Pessoa objPessoa) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT " + "COUNT(*)" +

                " FROM" + " Talento talento" +

                " WHERE" + " talento.pessoa.identificador = " + objPessoa.getIdentificador();

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm total de pessoas com talentos cadastrados
     * 
     * @return int Contendo o total
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalPessoasComTalentos() throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = "SELECT COUNT(DISTINCT talento.pessoa.identificador) FROM Talento talento";

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm pessoas com talentos cadastrados
     * 
     * @return int Contendo o total
     * 
     * @throws DAOException
     *             Se ocorrer algum erro relacionado com o acesso a banco de
     *             dados
     * 
     */
    public List obterPessoasComTalentosPorPagina(int numeroPagina, int maximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String consulta = "SELECT pessoa.identificador as identificadorpessoa, pessoa.grupo.codigo as codigogrupo, MAX(talento.dataLancamento) as dataultimolancamento FROM Talento talento INNER JOIN talento.pessoa pessoa GROUP BY pessoa.identificador, pessoa.grupo.codigo ORDER BY pessoa.identificador";

        // Recupera os dados
        return obterListaComMapasPorHQLPorPagina(numeroPagina, maximoPagina, consulta, null);
    }

    /**
     * Obtm o total de registros por pessoa e categoria de talento
     * 
     * @param objPessoa Pessoa desejada
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistrosPorPessoaCategoriaTalento(Pessoa objPessoa, CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT "
                + "COUNT(*)"
                + " FROM"
                + " Talento talento"
                + " WHERE"
                + " talento.pessoa.identificador = "
                + objPessoa.getIdentificador()
                + " AND talento.categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador();

        // Recupera os dados
        return obterTotalRegistros(strConsulta);
    }

    /**
     * Obtm os registros da pessoa desejada de determinada pgina
     * 
     * @param objPessoa Pessoa desejada
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPessoaPorPagina(int identificadorPessoa, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        // Monta consulta
        String strConsulta = " FROM" + " Talento talento" +

                " WHERE" + " talento.pessoa.identificador = " + identificadorPessoa +

                " ORDER BY" + " talento.categoriaTalento.sequencialOrdenacao ASC," + " talento.dataLancamento DESC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Obtm os registros da pessoa desejada 
     * 
     * @param objPessoa Pessoa desejada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPessoa(Pessoa objPessoa) throws DAOException
    {
        // Monta consulta
        StringBuffer consulta = new StringBuffer();
        consulta.append(" FROM Talento talento");
        consulta.append(" LEFT JOIN FETCH talento.categoriaTalento categoriaTalento ");
        consulta.append(" WHERE talento.pessoa.identificador = :idPessoa");
        consulta.append(" ORDER BY talento.categoriaTalento.sequencialOrdenacao");

        ParametroDAO parametro = new ParametroDAO();
        parametro.adicionar("idPessoa", objPessoa.getIdentificador());

        // Recupera os dados
        return obterListaPorHQL(consulta.toString(), parametro);
    }

    /**
     * Obtm os registros da pessoa desejada e pela categoria escolhida, de determinada pgina
     * 
     * @param objPessoa Pessoa desejada
     * @param objCategoriaTalento Categoria de talento desejada
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterPorPessoaCategoriaTalentoPorPagina(Pessoa objPessoa, CategoriaTalento objCategoriaTalento, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        // Monta consulta
        String strConsulta = " FROM"
                + " Talento talento"
                +

                " WHERE"
                + " talento.pessoa.identificador = "
                + objPessoa.getIdentificador()
                + " AND talento.categoriaTalento.identificador = "
                + objCategoriaTalento.getIdentificador()
                +

                " ORDER BY"
                + " talento.categoriaTalento.sequencialOrdenacao ASC,"
                + " talento.dataLancamento DESC";

        // Recupera os dados
        return obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);
    }

    /**
     * Verifica se a pessoa j tem um talento cadastrado para a categoria especificada
     * 
     * @param objPessoa Pessoa desejada
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo a verificao
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public boolean verificarExistenciaPorPessoaCategoriaTalento(Pessoa objPessoa, CategoriaTalento objCategoriaTalento) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("SELECT COUNT(*)");
        hql.adicionarSql("FROM Talento talento");
        hql.adicionarSql("WHERE talento.pessoa.identificador = :idPessoa");
        hql.adicionarSql("AND talento.categoriaTalento.identificador = :idCategoriaTalento");
        hql.comParametro("idPessoa", objPessoa.getIdentificador());
        hql.comParametro("idCategoriaTalento", objCategoriaTalento.getIdentificador());

        // Conta registros
        int intTotalRegistros = obterTotalRegistrosPorHQL(hql);

        if (intTotalRegistros > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Exclui um registro
     *
     * @param Object POJO representando o objeto a ser excludo
     * 
     * @return String Contendo a consulta de excluso 
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public String excluirImpl(Object objTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Retorna consulta de excluso
        return " FROM" + " Talento talento" +

                " WHERE" + " talento.identificador = " + ((Talento) objTalento).getIdentificador();
    }

    /**
     * Exclui registros pela sua categoria/atributo
     *
     * @param CategoriaAtributoTalento Com a categoria/atributo desejado
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public void excluirPorCategoriaTalento(CategoriaTalento objCategoriaTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta consulta de excluso
        String strConsulta = " FROM" + " Talento talento" +

                " WHERE" + " talento.categoriaTalento.identificador = " + objCategoriaTalento.getIdentificador();

        //Executa
        excluir(strConsulta);
    }

    /**
     * Exclui registros pela sua categoria/atributo
     *
     * @param CategoriaAtributoTalento Com a categoria/atributo desejado
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public void excluirPorCategoriaAtributoTalento(CategoriaAtributoTalento objCategoriaAtributoTalento) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta consulta de excluso
        String strConsulta = " FROM" + " Talento talento" +

                " WHERE" + " talento.categoriaAtributoTalento.identificador = " + objCategoriaAtributoTalento.getIdentificador();

        //Executa
        excluir(strConsulta);
    }

    /**
     * Obtm total de registros da consulta
     *
     * @param Consulta objeto contendo os filtros para consulta
     *  
     * @return List contendo o resultado da pesquisa
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalRegistros(Consulta objConsulta) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        int intRetorno = 0;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " SELECT COUNT(*)" + " FROM" + " Talento talento";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }

            // Recupera os dados
            intRetorno = obterTotalRegistros(strConsulta);

            // Retorna
            return intRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     * Obtm AtributoTalento de acordo com filtro passado
     *
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * @param Consulta objeto contendo os filtros para consulta
     *  
     * @return List contendo o resultado da pesquisa
     * 
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List consultar(Consulta objConsulta, int intNumeroPagina, int intMaximoPagina) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Declaraes
        List lstRetorno = null;

        try
        {
            String strFiltro = objConsulta.montarFiltro();

            // Monta query
            String strConsulta = " FROM" + " Talento talento";
            if (!strFiltro.trim().equals(""))
            {
                strConsulta += " WHERE " + strFiltro;
            }
            strConsulta += " ORDER BY" + " talento.categoria.ordenacao ASC";

            // Recupera os dados 
            lstRetorno = obterPorPagina(intNumeroPagina, intMaximoPagina, strConsulta);

            // Retorna
            return lstRetorno;
        }
        catch (CDException cde)
        {
            throw new DAOException("Ocorreu um erro ao consultar registros na classe " + strNomeClasse, cde);
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade categoriaTalento, que  lazy
     * 
     * @param  objTalento Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarCategoriaTalento(Talento objTalento) throws DAOException
    {
        inicializarRelacionamento(objTalento.getCategoriaTalento());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade atributoTalentoOpcao, que  lazy
     * 
     * @param lstTalentos Lista com os objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarCategoriaTalento(List lstTalentos) throws DAOException
    {
        Iterator itrTalentos = lstTalentos.iterator();
        while (itrTalentos.hasNext())
        {
            inicializarCategoriaTalento((Talento) itrTalentos.next());
        }
    }

    /**
     *  Mtodo que pede a inicializao da propriedade pessoa, que  lazy
     * 
     * @param  objTalento Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarPessoa(Talento objTalento) throws DAOException
    {
        inicializarRelacionamento(objTalento.getPessoa());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade pessoa, que  lazy
     * 
     * @param  objTalento Objeto que tem o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarGrupo(Talento objTalento) throws DAOException
    {
        inicializarRelacionamento(objTalento.getPessoa().getGrupo());
    }

    /**
     *  Mtodo que pede a inicializao da propriedade pessoa, que  lazy
     * 
     * @param lstTalentos Lista com os objetos que tm o atributo que vai ser recuperado
     * 
     * @see br.gov.camara.negocio.DAO#inicializarRelacionamento(java.lang.Object)
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public void inicializarPessoa(List lstTalentos) throws DAOException
    {
        Iterator itrTalentos = lstTalentos.iterator();
        while (itrTalentos.hasNext())
        {
            Talento talento = (Talento) itrTalentos.next();
            inicializarPessoa(talento);
            inicializarGrupo(talento);
        }
    }

    /**
     * Obtm o total de registros por intervalo de data / categoria / grupo
     * A formatao de data  realizada atravs do mecanismo de substituio do Hibernate. 
     * As funes especficas do banco devem ser especificadas na chave "hibernate.query.substitutions" 
     * que se encontram no arquivo de configurao do Hibernate "hibernate-config.xml".
     * 
     * @param objPessoa Pessoa desejada
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List realizarConsultaEstatistica(String strDataInicial, String strDataTermino, List lstOrdenacao, String strGruposHabilitados) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        String strOrdenacao = "";
        if (lstOrdenacao != null)
        {
            for (int i = 0; i < lstOrdenacao.size(); i++)
            {
                String strQuebra = (String) lstOrdenacao.get(i);
                if (strQuebra.equalsIgnoreCase("grupo"))
                {
                    strOrdenacao += " talento.pessoa.grupo.codigo" + ",";
                }
                else if (strQuebra.equalsIgnoreCase("categoria"))
                {
                    strOrdenacao += " talento.categoriaTalento.nome" + ",";
                }
                else if (strQuebra.equalsIgnoreCase("data"))
                {
                    strOrdenacao += " talento.dataLancamento" + ",";
                }
            }
            strOrdenacao += " talento.pessoa.nome" + ",";

            if (!"".equals(strOrdenacao))
            {
                strOrdenacao = strOrdenacao.substring(1, strOrdenacao.length() - 1);
            }
        }
        // Monta query
        String strConsulta = " FROM " + " Talento talento ";

        String strLigacao = " WHERE ";

        if (strDataInicial != null && !"".equals(strDataInicial))
        {
            strConsulta += strLigacao
                    + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento")
                    + ">="
                    + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "'" + strDataInicial + "'");
            strLigacao = " AND ";
        }
        if (strDataTermino != null && !"".equals(strDataTermino))
        {
            strConsulta += strLigacao
                    + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento")
                    + "<="
                    + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "'" + strDataTermino + "'");
            strLigacao = " AND ";
        }
        if (strGruposHabilitados != null && !"".equals(strGruposHabilitados))
        {
            strConsulta += strLigacao + "talento.pessoa.grupo.codigo in (" + strGruposHabilitados + ")";
            strLigacao = " AND ";
        }
        if (strOrdenacao != null && !"".equals(strOrdenacao))
        {
            strConsulta += " ORDER BY " + strOrdenacao;
        }
        else
        {
            strConsulta += " ORDER BY talento.pessoa.identificador";
        }

        // Recupera os dados
        return obter(strConsulta);

    }

    /**
     * Obtm o total de registros por intervalo de data / categoria / grupo
     * A formatao de data  realizada atravs do mecanismo de substituio do Hibernate. 
     * As funes especficas do banco devem ser especificadas na chave "hibernate.query.substitutions" 
     * que se encontram no arquivo de configurao do Hibernate "hibernate-config.xml".
     * 
     * @param objPessoa Pessoa desejada
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public List obterListagemConsulta(Date strDataInicial, Date strDataTermino, Integer strGrupo, String strCategoria, Date strData, List<Integer> strGruposHabilitados) throws DAOException
    {
        ManipuladorSql hql = new ManipuladorSql();
        hql.adicionarSql("FROM Talento talento");
        hql.adicionarSql("LEFT JOIN FETCH talento.categoriaTalento categoriaTalento");
        hql.adicionarSql("LEFT JOIN FETCH categoriaTalento.gruposCategoriaTalento grupos");
        hql.adicionarSql("LEFT JOIN FETCH talento.pessoa pessoa");
        hql.adicionarSql("LEFT JOIN FETCH pessoa.grupo grupo");
        hql.adicionarSql("LEFT JOIN FETCH pessoa.lotacoes lotacoes");

        hql.adicionarSql("WHERE 1=1");

        if (strDataInicial != null)
        {
            hql.adicionarSql("AND " + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento") + " >= " + ":dataInicial");
            hql.comParametro("dataInicial", strDataInicial);
        }
        if (strDataTermino != null)
        {
            hql.adicionarSql("AND " + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento") + " <= " + ":dataTermino");
            hql.comParametro("dataTermino", strDataTermino);
        }
        if (strGrupo != null)
        {
            hql.adicionarSql("AND talento.pessoa.grupo.codigo = :grupo");
            hql.comParametro("grupo", Integer.valueOf(strGrupo));
        }
        if (strCategoria != null && !"".equals(strCategoria))
        {
            hql.adicionarSql("AND talento.categoriaTalento.identificador = :idCategoriaTalento");
            hql.comParametro("idCategoriaTalento", Integer.valueOf(strCategoria));
        }
        if (strData != null)
        {
            hql.adicionarSql("AND " + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento") + " = " + ":data");
            hql.comParametro("data", strData);
        }
        if (strGruposHabilitados != null && !strGruposHabilitados.isEmpty())
        {
            hql.adicionarSql("AND talento.pessoa.grupo.codigo in (:idsGruposHabilitados)");
            hql.comParametro("idsGruposHabilitados", strGruposHabilitados);
        }

        hql.adicionarSql("ORDER BY talento.pessoa.nome");

        // Recupera os dados
        return obterListaPorHQL(hql);

    }

    /**
     * Obtm o total de registros por intervalo de data / categoria / grupo. 
     * A formatao de data  realizada atravs do mecanismo de substituio do Hibernate. 
     * As funes especficas do banco devem ser especificadas na chave "hibernate.query.substitutions" 
     * que se encontram no arquivo de configurao do Hibernate "hibernate-config.xml".
     * 
     * @param objPessoa Pessoa desejada
     * @param objCategoriaTalento Categoria de talento desejada
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws DAOException Se ocorrer algum erro relacionado com o acesso a banco de dados
     * 
     */
    public int obterTotalListagemConsulta(String strDataInicial, String strDataTermino, String strGrupo, String strCategoria, String strData, String strGruposHabilitados) throws DAOException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Monta query
        String strConsulta = " SELECT COUNT(DISTINCT talento.pessoa.identificador)" + " FROM" + " Talento talento ";

        String strLigacao = " WHERE ";

        if (strDataInicial != null && !"".equals(strDataInicial))
        {
            strConsulta += strLigacao
                    + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento")
                    + ">="
                    + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "'" + strDataInicial + "'");
            strLigacao = " AND ";
        }
        if (strDataTermino != null && !"".equals(strDataTermino))
        {
            strConsulta += strLigacao
                    + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento")
                    + "<="
                    + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "'" + strDataTermino + "'");
            strLigacao = " AND ";
        }
        if (strGrupo != null && !"".equals(strGrupo))
        {
            strConsulta += strLigacao + " talento.pessoa.grupo.codigo = " + strGrupo;
            strLigacao = " AND ";
        }
        if (strCategoria != null && !"".equals(strCategoria))
        {
            strConsulta += strLigacao + " talento.categoriaTalento.identificador = " + strCategoria;
            strLigacao = " AND ";
        }
        if (strData != null && !"".equals(strData))
        {
            strConsulta += strLigacao
                    + BancoTalentosUtil.utilizarFuncaoBDRemocaoHorasDeData(this, "talento.dataLancamento")
                    + "="
                    + BancoTalentosUtil.utilizarFuncaoBDConversaoStringEmData(this, "'" + strData + "'");
            strLigacao = " AND ";
        }
        if (strGruposHabilitados != null && !"".equals(strGruposHabilitados))
        {
            strConsulta += strLigacao + "talento.pessoa.grupo.codigo in (" + strGruposHabilitados + ")";
            strLigacao = " AND ";
        }

        // Recupera os dados
        return obterTotalRegistros(strConsulta);

    }

}
