/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.facade;

import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.biblioteca.util.ExpressaoRegular;
import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoOpcaoDAO;
import br.gov.camara.negocio.bancotalentos.dao.TabelaApoioMMDAO;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalento;
import br.gov.camara.negocio.comum.dao.TipoHTMLDAO;
import br.gov.camara.negocio.comum.pojo.TipoHTML;
import br.gov.camara.negocio.exception.NegocioException;

/**
 * Facade para atributo de talento
 */
public class AtributoTalentoFacade extends Facade
{
    // Variveis de instncia
    private static Log log = LogFactory.getLog(AtributoTalentoFacade.class.getName());

    // Para trabalhar com objetos Mock
    private AtributoTalentoDAO objAtributoTalentoDAO;

    public AtributoTalentoFacade(String nomeConexao)
    {
        super(nomeConexao);
        setObjAtributoTalentoDAO(new AtributoTalentoDAO(nomeConexao));
    }

    public void setObjAtributoTalentoDAO(AtributoTalentoDAO objAtributoTalentoDAO)
    {
        this.objAtributoTalentoDAO = objAtributoTalentoDAO;
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistros() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina

        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objAtributoTalentoDAO.obterTotalRegistros();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm os registros de determinada pgina
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPaginaComTipoHtml(int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        // Instancia DAO e obtm os registros da pgina

        List lstRetorno = null;
        try
        {
            lstRetorno = objAtributoTalentoDAO.obterPorPaginaComTipoHtml(intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {

            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return AtributoTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public AtributoTalento obterPelaChave(String strChave) throws CDException
    {
        // Instancia DAO e obtm o registro pela chave

        AtributoTalento objAtributoTalento = null;
        try
        {
            objAtributoTalento = (AtributoTalento) objAtributoTalentoDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objAtributoTalento;
    }

    /**
     * Obtm candidatos a pai, ou seja, todos os atributos exceto o que est sendo passado
     * 
     * @param objAtributoTalento Atributo de talento que se deseja excluir
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCandidatosAPai(AtributoTalento objAtributoTalento) throws CDException
    {
        // Instancia DAO e obtm o registro pela chave

        List lstCandidatosAPai = null;
        try
        {
            lstCandidatosAPai = objAtributoTalentoDAO.obterCandidatosAPai(objAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstCandidatosAPai;
    }

    /**
     * Obtm os tipos de HTML
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTiposHTML() throws CDException
    {
        // Instancia DAO e obtm os registros
        TipoHTMLDAO objTipoHTMLDAO = new TipoHTMLDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objTipoHTMLDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm tabelas de apoio de meta dados
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTabelasApoioMM() throws CDException
    {
        // Instancia DAO e obtm
        TabelaApoioMMDAO objTabelaApoioMMDAO = new TabelaApoioMMDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objTabelaApoioMMDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Inclui um registro
     *
     * @param objAtributoTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     * @return retorna a chave do objeto que foi incluido
     *
     */
    public Integer incluir(AtributoTalento objAtributoTalento) throws CDException
    {
        TipoHTMLDAO objTipoHTMLDAO = new TipoHTMLDAO(obterNomeConexao());
        Integer strChave = null;
        try
        {
            //  Verifica se o atributo tipo HTML do atributo pai indica
            // que o atributo aceita multivalorao, o que no  permitido
            if (objAtributoTalento.getAtributoTalentoPai() != null)
            {
                AtributoTalento objAtributoTalentoPai = (AtributoTalento) objAtributoTalentoDAO.obterPelaChave(String.valueOf(objAtributoTalento.getAtributoTalentoPai().getIdentificador()));
                if (!"U".equals(objAtributoTalentoPai.getTipoHTML().getMultiplicidade()))
                {
                    throw new NegocioException("No  permitido que um atributo que possui filhos possa ser de um tipo HTML "
                            + "que permita mltiplos valores ou que no possua opes");
                }
            }

            // Se a descrio de pesquisa estiver preenchida, o indicativo de pesquisa tambm dever estar
            if (objAtributoTalento.getDescricaoPesquisa() != null && !"".equals(objAtributoTalento.getDescricaoPesquisa()))
            {
                if ("N".equals(objAtributoTalento.getIndicativoPesquisa()))
                {
                    throw new NegocioException("No  permitido informar uma dica de pesquisa" + " se o atributo no for de pesquisa");
                }
            }

            // Se o atributo for de pesquisa, o tipo HTML deve admitir opes, ou seja, ser dos tipos
            // U (aceita uma nica opo) ou M (aceita mltiplas opes)
            TipoHTML objTipoHTML = (TipoHTML) objTipoHTMLDAO.obterPelaChave(String.valueOf(objAtributoTalento.getTipoHTML().getIdentificador()));
            if ("S".equals(objAtributoTalento.getIndicativoPesquisa()))
            {
                if (!"U".equals(objTipoHTML.getMultiplicidade()) && !"M".equals(objTipoHTML.getMultiplicidade()))
                {
                    throw new NegocioException("No  permitido que um atributo que possa ser pesquisado no seja " + "de um tipo HTML que no permita opes");
                }
            }

            // Se houver mscara cadastrada, o tipo de HTML no pode aceitar opes
            if (objAtributoTalento.getMascara() != null && !"".equals(objAtributoTalento.getMascara()))
            {
                if ("U".equals(objTipoHTML.getMultiplicidade()) || "M".equals(objTipoHTML.getMultiplicidade()))
                {
                    throw new NegocioException("No  permitido informar mscara para um atributo que seja " + "de um tipo HTML que permita opes");
                }
            }

            // Valida mscara
            if (objAtributoTalento.getMascara() != null && !"".equals(objAtributoTalento.getMascara()))
            {
                if (!ExpressaoRegular.compilar(objAtributoTalento.getMascara()))
                {
                    throw new NegocioException("Mscara invlida");
                }
            }

            // Inclui
            strChave = Integer.valueOf(objAtributoTalentoDAO.incluir(objAtributoTalento).toString());
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return strChave;
    }

    /**
     * Altera um registro
     *
     * @param objAtributoTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterar(AtributoTalento objAtributoTalento) throws CDException
    {
        // Instancia DAO e altera registro

        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        TipoHTMLDAO objTipoHTMLDAO = new TipoHTMLDAO(obterNomeConexao());
        try
        {
            //  Verifica se o atributo tipo HTML do atributo pai indica
            // que o atributo aceita multivalorao, o que no  permitido
            if (objAtributoTalento.getAtributoTalentoPai() != null)
            {
                AtributoTalento objAtributoTalentoPai = (AtributoTalento) objAtributoTalentoDAO.obterPelaChave(String.valueOf(objAtributoTalento.getAtributoTalentoPai().getIdentificador()));
                if ("M".equals(objAtributoTalentoPai.getTipoHTML().getMultiplicidade()))
                {
                    throw new NegocioException("No  permitido que um atributo que possui filhos possa ser de um tipo HTML "
                            + "que permita mltiplos valores");
                }
            }

            // Valida se h referncia circular
            if (objAtributoTalento.getAtributoTalentoPai() != null)
            {
                if (objAtributoTalentoDAO.verificarReferenciaCircular(objAtributoTalento))
                {
                    throw new NegocioException("No pode existir referncia circular entre os atributos, isto ,"
                            + " um atributo no pode referenciar outro que j o referencia");
                }
            }

            // Valida se o tipo HTML tem multiplicidade igual a L (Livre preenchimento) e tem opes, o que no
            //  permitido
            TipoHTML objTipoHTML = (TipoHTML) objTipoHTMLDAO.obterPelaChave(String.valueOf(objAtributoTalento.getTipoHTML().getIdentificador()));
            if ("L".equals(objTipoHTML.getMultiplicidade()) && objAtributoTalentoOpcaoDAO.verificarExistenciaPorAtributoTalento(objAtributoTalento))
            {
                throw new NegocioException("No  permitido que esse atributo seja de livre preenchimento, " + "pois ele j possui opes cadastradas");
            }

            // Se a descrio de pesquisa estiver preenchida, o indicativo de pesquisa tambm dever estar
            if (objAtributoTalento.getDescricaoPesquisa() != null && !"".equals(objAtributoTalento.getDescricaoPesquisa()))
            {
                if ("N".equals(objAtributoTalento.getIndicativoPesquisa()))
                {
                    throw new NegocioException("No  permitido informar uma dica de pesquisa" + " se o atributo no for de pesquisa");
                }
            }

            // Se o atributo for de pesquisa, o tipo HTML deve admitir opes, ou seja, ser dos tipos
            // U (aceita uma nica opo) ou M (aceita mltiplas opes)
            if ("S".equals(objAtributoTalento.getIndicativoPesquisa()))
            {
                if (!"U".equals(objTipoHTML.getMultiplicidade()) && !"M".equals(objTipoHTML.getMultiplicidade()))
                {
                    throw new NegocioException("No  permitido que um atributo que possa ser pesquisado no seja " + "de um tipo HTML que no permita opes");
                }
            }

            // Se o atributo tiver uma mscara definida, o mesmo no pode ter opes
            if (objAtributoTalento.getMascara() != null && !"".equals(objAtributoTalento.getMascara()))
            {
                if (objAtributoTalentoOpcaoDAO.obterPeloAtributo(objAtributoTalento) != null)
                {
                    throw new NegocioException("No  permitido que um atributo com mscara de " + "preenchimento definida possua opes");
                }
            }

            // Se houver mscara cadastrada, o tipo de HTML no pode aceitar opes
            if (objAtributoTalento.getMascara() != null && !"".equals(objAtributoTalento.getMascara()))
            {
                if ("U".equals(objTipoHTML.getMultiplicidade()) || "M".equals(objAtributoTalento.getTipoHTML().getMultiplicidade()))
                {
                    throw new NegocioException("No  permitido informar mscara para um atributo que seja " + "de um tipo HTML que permita opes");
                }
            }

            // Valida mscara
            if (objAtributoTalento.getMascara() != null && !"".equals(objAtributoTalento.getMascara()))
            {
                Pattern.compile(objAtributoTalento.getMascara());
            }

            // Altera
            objAtributoTalentoDAO.alterar(objAtributoTalento);
        }
        catch (PatternSyntaxException pse)
        {
            throw new NegocioException("Mscara invlida");
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Exclui um registro
     *
     * @param objAtributoTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void excluir(AtributoTalento objAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {
            objAtributoTalentoDAO.excluir(objAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

}
