/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.facade;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoValoradoDAO;
import br.gov.camara.negocio.bancotalentos.dao.CategoriaAtributoTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.CategoriaTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.TalentoDAO;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoValorado;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaAtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaTalento;
import br.gov.camara.negocio.exception.NegocioException;

/**
 * Facade para atributo de talento
 */
public class CategoriaAtributoTalentoFacade extends Facade
{
    // Variveis de instncia
    private static Log log = LogFactory.getLog(CategoriaAtributoTalentoFacade.class.getName());

    public CategoriaAtributoTalentoFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistros() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objCategoriaAtributoTalentoDAO.obterTotalRegistros();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm o total de registros relacionados a determinada categoria de talento
     *
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistrosPorCategoriaTalento(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objCategoriaAtributoTalentoDAO.obterTotalRegistrosPorCategoriaTalento(objCategoriaTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm os registros de determinada pgina
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objCategoriaAtributoTalentoDAO.obterPorPagina(intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os registros de determinada pgina relacionados a determinada categoria de talento
     *
     * @param objCategoriaTalento Categoria de talento desejado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorCategoriaTalentoPorPagina(CategoriaTalento objCategoriaTalento, int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objCategoriaAtributoTalentoDAO.obterPorPaginaPorCategoriaTalento(objCategoriaTalento, intNumeroPagina, intMaximoPagina);
            objCategoriaAtributoTalentoDAO.inicializarAtributoTalento(lstRetorno);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaAtributoTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public CategoriaAtributoTalento obterPelaChaveComAtributoTalentoECategoriaTalento(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        CategoriaAtributoTalento objCategoriaAtributoTalento = null;
        try
        {
            objCategoriaAtributoTalento = (CategoriaAtributoTalento) objCategoriaAtributoTalentoDAO.obterPelaChaveComAtributoTalentoETipoHtml(strChave);
            objCategoriaAtributoTalentoDAO.inicializarCategoriaTalento(objCategoriaAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaAtributoTalento;
    }

    /**
     * Obtm uma categoria de talento a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public CategoriaTalento obterCategoriaTalentoPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        CategoriaTalento objCategoriaTalento = null;
        try
        {
            objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaTalento;
    }

    /**
     * Obtm os atributos de talento
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterAtributosTalento() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        AtributoTalentoDAO objAtributoTalentoDAO = new AtributoTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objAtributoTalentoDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm todos os registros relacionados a determinada categoria de talento
     *
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTodosPorCategoriaTalento(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objCategoriaAtributoTalentoDAO.obterPorCategoriaTalento(objCategoriaTalento);
            objCategoriaAtributoTalentoDAO.inicializarAtributoTalento(lstRetorno);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm hierarquia superior da categoria/atributo indicada
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterHierarquiaSuperior(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstRetorno = new ArrayList();
        try
        {
            if (objCategoriaAtributoTalento.getAtributoTalento().getAtributoTalentoPai() != null)
            {
                CategoriaAtributoTalento objCategoriaAtributoTalentoPai = (CategoriaAtributoTalento) objCategoriaAtributoTalentoDAO.obterPorCategoriaTalentoAtributoTalento(objCategoriaAtributoTalento.getCategoriaTalento(), objCategoriaAtributoTalento.getAtributoTalento().getAtributoTalentoPai());
                if (objCategoriaAtributoTalentoPai != null)
                {
                    lstRetorno = obterHierarquiaSuperior(objCategoriaAtributoTalentoPai);
                    lstRetorno.add(objCategoriaAtributoTalentoPai);
                }
            }
            objCategoriaAtributoTalentoDAO.inicializarAtributoTalento(lstRetorno);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm hierarquia inferior da categoria/atributo indicada
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterHierarquiaInferior(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstRetorno = new ArrayList();
        try
        {
            List lstCategoriaAtributosTalentoFilhos = objCategoriaAtributoTalentoDAO.obterTodosOsFilhos(objCategoriaAtributoTalento);
            if (lstCategoriaAtributosTalentoFilhos != null)
            {
                Iterator itrCategoriaAtributosTalentoFilhos = lstCategoriaAtributosTalentoFilhos.iterator();
                while (itrCategoriaAtributosTalentoFilhos.hasNext())
                {
                    CategoriaAtributoTalento objCategoriaAtributoTalentoFilho = (CategoriaAtributoTalento) itrCategoriaAtributosTalentoFilhos.next();
                    lstRetorno = obterHierarquiaInferior(objCategoriaAtributoTalentoFilho);
                    lstRetorno.add(objCategoriaAtributoTalentoFilho);
                }
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm toda a hierarquia da categoria/atributo indicada (inclusive ela)
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterHierarquia(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Obtm hierarquias inferior e superior e adiciona a passada
        List lstRetorno = new ArrayList();

        try
        {
            lstRetorno.addAll(obterHierarquiaSuperior(objCategoriaAtributoTalento));
            lstRetorno.add(objCategoriaAtributoTalento);
            lstRetorno.addAll(obterHierarquiaInferior(objCategoriaAtributoTalento));
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Inclui um registro
     *
     * @param objCategoriaAtributoTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     * @return retorna a chave do objeto que foi incluido
     *
     */
    public Integer incluir(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        Integer strChave = null;

        // Instancia DAO e inclui registro
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        // AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new
        // AtributoTalentoValoradoDAO();
        AtributoTalentoDAO objAtributoTalentoDAO = new AtributoTalentoDAO(obterNomeConexao());
        try
        {
            // Se o atributo formar descrio, o mesmo deve ser obrigatrio
            if ("S".equals(objCategoriaAtributoTalento.getFormacaoDescricao()))
            {
                if ("N".equals(objCategoriaAtributoTalento.getIndicativoObrigatoriedade()))
                {
                    throw new NegocioException("No  permitido que um atributo faa parte da descrio do talento " + "se o mesmo no for obrigatrio");
                }
            }

            // Se o atributo formar descrio, a categoria a que ele pertence no pode ser perfil
            CategoriaTalento objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(String.valueOf(objCategoriaAtributoTalento.getCategoriaTalento().getIdentificador()));
            if ("S".equals(objCategoriaTalento.getIndicativoUnicidade()))
            {
                if ("S".equals(objCategoriaAtributoTalento.getFormacaoDescricao()))
                {
                    throw new NegocioException("No  permitido que um atributo faa parte da descrio do talento "
                            + "se a categoria a qual ele pertence  uma categoria perfil");
                }
            }

            // Se o atributo em questo tiver um pai, ou algum filho, no pode existir o mesmo atributo 
            // mais de uma vez para a categoria
            if (objCategoriaAtributoTalento.getAtributoTalento().getAtributoTalentoPai() != null
                    || objAtributoTalentoDAO.verificarExistenciaFilhos(objCategoriaAtributoTalento.getAtributoTalento()))
            {
                if (objCategoriaAtributoTalentoDAO.verificarExistenciaAtributoRelacionado(objCategoriaTalento, objCategoriaAtributoTalento.getAtributoTalento()))
                {
                    throw new NegocioException("No  permitido que um atributo faa parte de uma hierarquia "
                            + "seja relacionado mais de uma vez para a mesma categoria");
                }
            }

            // Verifica se o atributo possui um pai, e se possuir, o inclui tambm, caso ele no esteja associado
            AtributoTalento objAtributoTalento = (AtributoTalento) objAtributoTalentoDAO.obterPelaChave(String.valueOf(objCategoriaAtributoTalento.getAtributoTalento().getIdentificador()));
            if (objAtributoTalento.getAtributoTalentoPai() != null)
            {
                if (!objCategoriaAtributoTalentoDAO.verificarExistenciaAtributoRelacionado(objCategoriaTalento, objAtributoTalento.getAtributoTalentoPai()))
                {
                    CategoriaAtributoTalento objCategoriaAtributoTalentoPai = new CategoriaAtributoTalento();
                    objCategoriaAtributoTalentoPai.setCategoriaTalento(objCategoriaTalento);
                    objCategoriaAtributoTalentoPai.setAtributoTalento(objAtributoTalento.getAtributoTalentoPai());
                    objCategoriaAtributoTalentoPai.setIndicativoObrigatoriedade(objCategoriaAtributoTalento.getIndicativoObrigatoriedade());
                    objCategoriaAtributoTalentoPai.setFormacaoDescricao(objCategoriaAtributoTalento.getFormacaoDescricao());
                    objCategoriaAtributoTalentoPai.setDicaPreenchimento("");
                    objCategoriaAtributoTalentoPai.setApelido("");
                    incluir(objCategoriaAtributoTalentoPai);
                }
            }

            // Seta ordenao
            objCategoriaAtributoTalento.setSequencialOrdenacao(new Integer(objCategoriaAtributoTalentoDAO.obterUltimoSequencialOrdenacao(objCategoriaAtributoTalento.getCategoriaTalento())
                    + 1));

            // Inclui
            strChave = Integer.valueOf(objCategoriaAtributoTalentoDAO.incluir(objCategoriaAtributoTalento).toString());
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return strChave;
    }

    /**
     * Ordena as categorias/atributos de talento de acordo com a lista passada 
     * 
     * @param lstOrdenacao Lista contendo as categorias/atributos de talentos a serem ordenadas
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public void ordenar(List lstOrdenacao) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        try
        {

            // Instancia DAO 
            CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());

            // Itera lista
            Iterator itrOrdenacao = lstOrdenacao.iterator();
            int intSequencialOrdenacao = 0;
            while (itrOrdenacao.hasNext())
            {
                CategoriaAtributoTalento objCategoriaAtributoTalento = (CategoriaAtributoTalento) itrOrdenacao.next();
                objCategoriaAtributoTalento = (CategoriaAtributoTalento) objCategoriaAtributoTalentoDAO.obterPelaChave(String.valueOf(objCategoriaAtributoTalento.getIdentificador()));
                objCategoriaAtributoTalento.setSequencialOrdenacao(new Integer(++intSequencialOrdenacao));
                objCategoriaAtributoTalentoDAO.alterar(objCategoriaAtributoTalento);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Altera um registro
     *
     * @param objCategoriaAtributoTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterar(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e inclui registro
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        try
        {
            // Se o atributo formar descrio, o mesmo deve ser obrigatrio
            if ("S".equals(objCategoriaAtributoTalento.getFormacaoDescricao()))
            {
                if ("N".equals(objCategoriaAtributoTalento.getIndicativoObrigatoriedade()))
                {
                    throw new NegocioException("No  permitido que um atributo faa parte da descrio do talento " + "se o mesmo no for obrigatrio");
                }
            }

            // Se o atributo formar descrio, a categoria a que ele pertence no pode ser perfil
            CategoriaTalento objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(String.valueOf(objCategoriaAtributoTalento.getCategoriaTalento().getIdentificador()));
            if ("S".equals(objCategoriaTalento.getIndicativoUnicidade()))
            {
                if ("S".equals(objCategoriaAtributoTalento.getFormacaoDescricao()))
                {
                    throw new NegocioException("No  permitido que um atributo faa parte da descrio do talento "
                            + "se a categoria a qual ele pertence  uma categoria perfil");
                }
            }

            // Se o atributo for obrigatrio verifica se h valoraes para ele, o que no permitir 
            // que ele se torne obrigatrio
            if (objCategoriaAtributoTalento.isBlnMudancaIndicativoObrigatoriedade() && "S".equals(objCategoriaAtributoTalento.getIndicativoObrigatoriedade()))
            {
                if (objAtributoTalentoValoradoDAO.obterTotalRegistrosPorCategoriaAtributoTalento(objCategoriaAtributoTalento) > 0)
                {
                    throw new NegocioException("No  permitido marcar um atributo como obrigatrio se existem" + " valoraes para ele");
                }
            }

            // Altera
            objCategoriaAtributoTalentoDAO.alterar(objCategoriaAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Exclui um registro
     *
     * @param objCategoriaAtributoTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void excluir(CategoriaAtributoTalento objCategoriaAtributoTalento, String strConfirma) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        // AtributoTalentoDAO objAtributoTalentoDAO = new AtributoTalentoDAO();
        try
        {
            // Verifica se o atributo possui algum filho associado, e se possuir, no permite a excluso
            objCategoriaAtributoTalento = (CategoriaAtributoTalento) objCategoriaAtributoTalentoDAO.obterPelaChave(String.valueOf(objCategoriaAtributoTalento.getIdentificador()));
            if (objCategoriaAtributoTalentoDAO.verificarExistenciaCategoriaAtributosTalentoFilhos(objCategoriaAtributoTalento))
            {
                throw new NegocioException("Para excluir este atributo relacionado,  necessrio antes " + "excluir o atributo pai tambm relacionado");
            }

            // Verifica a quantidade de talentos valorados para o atributo a ser excludo
            int intTotalTalentosValorados = 0;
            Iterator itrCagoriaAtributoTalento = objCategoriaAtributoTalento.getAtributosTalentoValorados().iterator();
            while (itrCagoriaAtributoTalento.hasNext())
            {
                // Verifica se o atributo associado possui valorao
                AtributoTalentoValorado objAtributoTalentoValorado = (AtributoTalentoValorado) itrCagoriaAtributoTalento.next();
                if (objAtributoTalentoValorado.getValoracao() != null && !"".equals(objAtributoTalentoValorado.getValoracao()))
                {
                    intTotalTalentosValorados++;
                }
            }

            // Verifica se existem talentos valorados associados ao atributo e se est confirmando a excluso
            if ((intTotalTalentosValorados > 0) && (strConfirma == null))
            {
                throw new NegocioException("Este atributo possui "
                        + intTotalTalentosValorados
                        + " talento(s) associado(s), "
                        + "caso seja excludo informaes sero perdidas - Confirma excluso?");
            }

            // Inicia transao
            DAO.iniciarTransacao(obterNomeConexao());

            // Exclui valoraes correspondentes a esta categoria/atributo
            objAtributoTalentoValoradoDAO.excluirPorCategoriaAtributoTalento(objCategoriaAtributoTalento);

            // Verifica se ainda h atributos para a categoria atual, se no houver, exclui os talentos associados
            if (objCategoriaAtributoTalentoDAO.obterTotalRegistrosPorCategoriaTalento(objCategoriaAtributoTalento.getCategoriaTalento()) == 0)
            {
                objTalentoDAO.excluirPorCategoriaAtributoTalento(objCategoriaAtributoTalento);
            }

            // Exclui categoria/atributo
            objCategoriaAtributoTalentoDAO.excluir(objCategoriaAtributoTalento);

            // Finaliza transao
            DAO.realizarTransacao(obterNomeConexao());
        }
        catch (Exception daoe)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }
}
