/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.facade;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.autenticacaoperfil.dao.GrupoDAO;
import br.gov.camara.negocio.bancotalentos.dao.CategoriaTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.TalentoDAO;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaTalento;
import br.gov.camara.negocio.exception.NegocioException;

/**
 * Facade para atributo de talento
 */
public class CategoriaTalentoFacade extends Facade
{
    // Variveis de instncia
    private static Log log = LogFactory.getLog(CategoriaTalentoFacade.class.getName());

    public CategoriaTalentoFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterQtdeAssociadasAAtributosPorGrupo(String grupoFiltro) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objCategoriaTalentoDAO.obterQtdeTodasPorGrupo(grupoFiltro);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm os registros de determinada pgina
     *
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterAssociadasAAtributosPorGrupoPorPagina(int intNumeroPagina, int intMaximoPagina, String grupo) throws CDException
    {
        // Instancia DAO e obtm os registros da pgina
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            List<Integer> grupos = new ArrayList<Integer>();
            if (grupo != null && !"".equals(grupo))
            {
                grupos.add(Integer.valueOf(grupo));
            }
            lstRetorno = objCategoriaTalentoDAO.obterAssociadasAAtributosPorGrupoPorPagina(grupos, intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    public List obterTodasPorGrupoPorPagina(int intNumeroPagina, int intMaximoPagina, String grupo) throws CDException
    {
        // Instancia DAO e obtm os registros da pgina
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            List<Integer> grupos = new ArrayList<Integer>();
            if (grupo != null && !"".equals(grupo))
            {
                grupos.add(Integer.valueOf(grupo));
            }
            lstRetorno = objCategoriaTalentoDAO.obterTodasPorGrupoPorPagina(grupos, intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    public int obterQtdeTodasPorGrupo(String grupoFiltro) throws CDException
    {
        // Instancia DAO e obtm os registros da pgina
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objCategoriaTalentoDAO.obterQtdeTodasPorGrupo(grupoFiltro);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public CategoriaTalento obterPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        CategoriaTalento objCategoriaTalento = null;
        try
        {
            objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaTalento;
    }

    /**
     * Inclui um registro
     *
     * @param objCategoriaTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     * @return retorna a chave do objeto que foi incluido
     *
     */
    public Integer incluir(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e inclui registro
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        Integer strChave = null;
        try
        {
            objCategoriaTalento.setSequencialOrdenacao(new Integer(objCategoriaTalentoDAO.obterUltimoSequencialOrdenacao() + 1));
            strChave = Integer.valueOf(objCategoriaTalentoDAO.incluir(objCategoriaTalento).toString());
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return strChave;
    }

    /**
     * Obtm os grupos disponveis
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterGrupos() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        GrupoDAO objGrupoDAO = new GrupoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objGrupoDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os grupos disponveis para determinada categoria de talento
     * 
     * @param objCategoriaTalento Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterGruposDisponiveis(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objCategoriaTalentoDAO.obterGruposDisponiveis(objCategoriaTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os grupos selecionados para determinada categoria de talento
     * 
     * @param objCategoriaTalento Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterGruposSelecionados(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objCategoriaTalentoDAO.obterGruposSelecionados(objCategoriaTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm todos os registros 
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTodos() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm todos os registros
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objCategoriaTalentoDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Ordena as categorias de talento de acordo com a lista passada 
     * 
     * @param lstOrdenacao Lista contendo as categorias de talentos a serem ordenadas
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public void ordenar(List lstOrdenacao) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        try
        {
            // Instancia DAO 
            CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());

            // Itera lista
            Iterator itrOrdenacao = lstOrdenacao.iterator();
            int intSequencialOrdenacao = 0;
            while (itrOrdenacao.hasNext())
            {
                CategoriaTalento objCategoriaTalento = (CategoriaTalento) itrOrdenacao.next();
                objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(String.valueOf(objCategoriaTalento.getIdentificador()));
                objCategoriaTalento.setSequencialOrdenacao(new Integer(++intSequencialOrdenacao));
                objCategoriaTalentoDAO.alterar(objCategoriaTalento);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Altera um registro
     *
     * @param objCategoriaTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterar(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e altera registro
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        try
        {
            // Se a categoria for perfil,  verifica se h atributos relacionados  categoria com formao de 
            // descrio marcada, o que no  permitido
            if ("S".equals(objCategoriaTalento.getIndicativoUnicidade())
                    && objCategoriaTalentoDAO.verificarAtributosAssociadosComFormacaoDescricao(objCategoriaTalento))
            {
                throw new NegocioException("No  permitido que uma categoria seja perfil,"
                        + " se existem atributos que faam parte da descrio do talento para a categoria em questo");
            }

            // Altera
            objCategoriaTalentoDAO.alterar(objCategoriaTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Exclui um registro
     *
     * @param objCategoriaTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void excluir(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e inclui registro
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        try
        {
            if (objCategoriaTalentoDAO.verificarExistenciaAtributoTalento(objCategoriaTalento))
            {
                throw new NegocioException("No  permitido que uma categoria que tenha atributos seja excluda");
            }
            objTalentoDAO.excluirPorCategoriaTalento(objCategoriaTalento);
            objCategoriaTalentoDAO.excluir(objCategoriaTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

}
