/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.facade;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.biblioteca.util.DataNova;
import br.gov.camara.biblioteca.util.StringUtil;
import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.bancotalentos.dao.FiltroConsultaDAO;
import br.gov.camara.negocio.bancotalentos.dao.TalentoDAO;
import br.gov.camara.negocio.bancotalentos.pojo.Estatistica;
import br.gov.camara.negocio.bancotalentos.pojo.Talento;

/**
 * Facade para a consulta de talentos
 */
public class ConsultaEstatisticaCadastroFacade extends Facade
{

    // Variveis de instncia
    private static Log log = LogFactory.getLog(ConsultaEstatisticaCadastroFacade.class.getName());

    public ConsultaEstatisticaCadastroFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Efetua consulta de estatsticas
     * 
     * @param strDataInicio Data de incio da consulta
     * @param strDataTermino Data de trmino da consulta
     * @param lstQuebras Quebras selecionadas para consulta
     * 
     * @return ArrayList Contendo o resultado da consulta
     * 
     * @throws CDException se ocorrer um erro relacionado com banco de dados 
     */
    public List consultar(String strIdentificadorUsuario, boolean blnFiltroGrupo, String strDataInicio, String strDataTermino, List lstQuebras, String[] strIdentacao) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }
        // Instancia DAO
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        List lstResultado = null;

        try
        {
            // Obtm grupos habilitados ao usurio logado consultar
            String strGrupos = "";
            if (blnFiltroGrupo)
            {
                strGrupos = objFiltroConsultaDAO.obterGruposHabilitadosConsultaCurriculo(strIdentificadorUsuario);
            }
            // Obtm lista de talentos de acordo com o filtro
            List lstTalentos = objTalentoDAO.realizarConsultaEstatistica(strDataInicio, strDataTermino, lstQuebras, strGrupos);

            // Inicializa objetos lazy
            objTalentoDAO.inicializarCategoriaTalento(lstTalentos);
            objTalentoDAO.inicializarPessoa(lstTalentos);

            // Coleo que armazena os objetos Estatistica, apenas para auxiliar
            // a montagem da lista de resultado.
            Map mapEstatistica = new HashMap();

            // Coleo que armazena as pessoas, apenas para obter a quantidade
            // total de pessoas.
            Map mapQtdPessoas = new HashMap();

            // Lista contendo o resultado da consulta
            lstResultado = new ArrayList();

            // Descrio da Quebra
            String strDescricao = "";

            // Caminho completo da quebra
            String strCaminho = "";

            // Contm os parametros que seram usados posteriormente para
            // listagem das pessoas que pertencem a quebra.
            String strParametro = "";

            Talento objTalento = null;

            for (int i = 0; i < lstTalentos.size(); i++)
            {
                // Recupera objeto.
                objTalento = (Talento) lstTalentos.get(i);

                // Recupera identificador da pessoa
                Integer intIdentificadorPessoa = objTalento.getPessoa().getIdentificador();

                if (mapQtdPessoas.get(intIdentificadorPessoa) == null)
                {
                    mapQtdPessoas.put(intIdentificadorPessoa, intIdentificadorPessoa);
                }

                // Inicializa variveis.
                strCaminho = "";
                strParametro = "&dtIni=" + strDataInicio + "&dtFim=" + strDataTermino;

                // Monta o resultado de acordo com a quebra selecionada
                for (int j = 0; j < lstQuebras.size(); j++)
                {
                    // Monta a descrio e o parametro de cada quebra

                    // Verifica se a quebra  por grupo
                    if (((String) lstQuebras.get(j)).equalsIgnoreCase("grupo"))
                    {
                        strDescricao = objTalento.getPessoa().getGrupo().getDescricao();
                        strParametro += "&grupo=" + objTalento.getPessoa().getGrupo().getCodigo();
                    }
                    // Verifica se a quebra  por categoria
                    else if (((String) lstQuebras.get(j)).equalsIgnoreCase("categoria"))
                    {
                        strDescricao = objTalento.getCategoriaTalento().getNome();
                        strParametro += "&categoria=" + objTalento.getCategoriaTalento().getIdentificador();
                    }
                    // Verifica se a quebra  por data
                    else if (((String) lstQuebras.get(j)).equalsIgnoreCase("data"))
                    {
                        strDescricao = objTalento.getDataLancamentoFormatada();
                        strParametro += "&data=" + objTalento.getDataLancamentoFormatada();
                    }

                    // Caminho da quebra
                    strCaminho += "/" + strDescricao;

                    Estatistica objEstatistica = null;
                    if (mapEstatistica.get(strCaminho) == null)
                    {
                        // Instancia objeto estatstica e atribui valores
                        objEstatistica = new Estatistica();
                        objEstatistica.setDescricao(strIdentacao[j] + strDescricao);
                        objEstatistica.setQuantidade(1);
                        objEstatistica.setPessoa(objTalento.getPessoa().getIdentificador().intValue());
                        objEstatistica.setNivel(j);
                        objEstatistica.setParametro(strParametro);
                        mapEstatistica.put(strCaminho, objEstatistica);
                        // Put que serve para identificao posterior da incluso do servidor no caminho indicado
                        mapEstatistica.put(intIdentificadorPessoa.intValue() + strCaminho, objEstatistica);

                        // Adiciona objeto de estatistica na lista de resultado da consulta
                        lstResultado.add(objEstatistica);
                    }
                    else
                    {
                        // Identifica se a pessoa j foi includo no caminho especificado
                        objEstatistica = (Estatistica) mapEstatistica.get(intIdentificadorPessoa.intValue() + strCaminho);
                        // Caso a pessoa no tenha sido inclusa, a quantidade  incrementada
                        if (objEstatistica == null)
                        {
                            objEstatistica = (Estatistica) mapEstatistica.get(strCaminho);
                            objEstatistica.setQuantidade(objEstatistica.getQuantidade() + 1);
                            // Put que serve para identificao posterior da incluso do servidor no caminho indicado
                            mapEstatistica.put(intIdentificadorPessoa.intValue() + strCaminho, objEstatistica);
                        }
                    }
                }
            }
            // Inclui o total de pessoas no geral        	
            Estatistica objEstatistica = new Estatistica();
            objEstatistica.setDescricao("<b>Total de pessoas no intervalo<b>");
            objEstatistica.setNivel(1);
            objEstatistica.setQuantidade(mapQtdPessoas.size());
            objEstatistica.setParametro("&dtIni=" + strDataInicio + "&dtFim=" + strDataTermino);
            lstResultado.add(objEstatistica);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return lstResultado;
    }

    /**
     * Repassa o pedido de obteno do nmero total de registros da consulta
     *
     * @param strDataInicial Data de incio da consulta
     * @param strDataTermino Data de trmino da consulta
     * @param strGrupo       Grupo a ser consultado 
     * @param strCategoria   Categoria a ser consultada
     * @param strData        Data a ser consultada
     *
     * @return int Nmero que representa o nmero de registros que satisfazem os criterios
     *
     * @throws CDException Se ocorrer algum erro relacionado com o acesso a banco de dados
     *
     */
    public int obterTotalRegistros(String strIdentificadorUsuario, boolean blnFiltroGrupo, String strDataInicial, String strDataTermino, String strGrupo, String strCategoria, String strData) throws CDException
    {
        // Instancia DAO/Bean
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());

        int retorno = 0;

        try
        {
            // Obtm grupos habilitados ao usurio logado consultar
            String strGrupos = "";
            if (blnFiltroGrupo)
            {
                strGrupos = objFiltroConsultaDAO.obterGruposHabilitadosConsultaCurriculo(strIdentificadorUsuario);
            }
            retorno = objTalentoDAO.obterTotalListagemConsulta(strDataInicial, strDataTermino, strGrupo, strCategoria, strData, strGrupos);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return retorno;
    }

    /**
     * Repassa o pedido de obteno da listagem da consulta
     *
     * @param strDataInicial Data de incio da consulta
     * @param strDataTermino Data de trmino da consulta
     * @param strGrupo       Grupo a ser consultado 
     * @param strCategoria   Categoria a ser consultada
     * @param strData        Data a ser consultada
     * @param intPagina      Pagina que dever ser retornada
     * @param intMaximoPagina Mximo de registro que sero retornados
     *
     * @return List Lista contento o resultado da consulta
     *
     * @throws CDException Se ocorrer algum erro relacionado com o acesso a banco de dados
     *
     */
    public List consultarListagem(String strIdentificadorUsuario, boolean blnFiltroGrupo, String strDataInicial, String strDataTermino, String strGrupo, String strCategoria, String strData, int intPagina, int intMaximoPagina) throws CDException
    {

        // Instancia DAO/Bean
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());

        List<Talento> lstTalentos = null;
        Map<Integer, Talento> mapTalentos = new HashMap<Integer, Talento>();
        try
        {
            // Obtm grupos habilitados ao usurio logado consultar
            List<Integer> strGrupos = null;
            if (blnFiltroGrupo)
            {
                strGrupos = objFiltroConsultaDAO.obterListaCodigosGruposPermitidos(strIdentificadorUsuario);
            }

            Date dataInicial = StringUtil.isNulaOuVazia(strDataInicial) ? null : DataNova.formatarDate(strDataInicial);
            Date dataTermino = StringUtil.isNulaOuVazia(strDataTermino) ? null : DataNova.formatarDate(strDataTermino);
            Date data = StringUtil.isNulaOuVazia(strData) ? null : DataNova.formatarDate(strData);
            Integer grupo = StringUtil.isNulaOuVazia(strGrupo) ? null : Integer.valueOf(strGrupo);

            lstTalentos = (List<Talento>) objTalentoDAO.obterListagemConsulta(dataInicial, dataTermino, grupo, strCategoria, data, strGrupos);
            for (int i = 0; i < lstTalentos.size(); i++)
            {
                Talento objTalento = lstTalentos.get(i);
                if (mapTalentos.get(objTalento.getPessoa().getIdentificador()) == null)
                {
                    mapTalentos.put(objTalento.getPessoa().getIdentificador(), objTalento);
                }
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return new ArrayList<Talento>(mapTalentos.values());
    }
}
