/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.facade;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.bancotalentos.consultatalento.ResultadoConsultaTalento;
import br.gov.camara.negocio.bancotalentos.consultatalento.ResultadoConsultaTalentoFacade;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoOpcaoDAO;
import br.gov.camara.negocio.bancotalentos.dao.CategoriaAtributoTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.CategoriaTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.ConsultaTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.FiltroConsultaDAO;
import br.gov.camara.negocio.bancotalentos.dao.PessoaDAO;
import br.gov.camara.negocio.bancotalentos.dao.TalentoDAO;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoOpcao;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaAtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaTalento;
import br.gov.camara.negocio.bancotalentos.pojo.Pessoa;
import br.gov.camara.negocio.comum.pojo.TipoHTML;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.seguranca.UsuarioAutenticado;

/**
 * Facade para a consulta de talentos
 */
public class ConsultaTalentoFacade extends Facade
{

    // Variveis de instncia
    private static Log log = LogFactory.getLog(ConsultaTalentoFacade.class.getName());

    public static final Integer ID_CATEGORIA_DADOS_PESSOAIS = new Integer(-1);

    public static final Integer ID_ATRIBUTO_NOME = new Integer(-1);

    public static final Integer ID_ATRIBUTO_IDADE = new Integer(-2);

    public static final Integer ID_ATRIBUTO_SEXO = new Integer(-3);
    public static final Integer ID_OPCAO_SEXO_MASCULINO = new Integer(-1);
    public static final Integer ID_OPCAO_SEXO_FEMININO = new Integer(-2);

    public ConsultaTalentoFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    public List obterCategoriasTalento(String identificador, boolean filtraPorGrupo) throws CDException
    {
        // Instancia DAO e obtm os registros
        List lstRetorno = null;
        try
        {
            // Qdo maxPag = 0 => retorna todos
            lstRetorno = obterCategoriasTalentoPorPagina(identificador, filtraPorGrupo, 1, 0);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    private List obterCategoriasTalentoPorPagina(String identificador, boolean filtraPorGrupo, int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        // Instancia DAO e obtm os registros
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        List lstRetorno = new ArrayList();
        try
        {
            CategoriaTalento objCategoriaTalento = new CategoriaTalento();
            objCategoriaTalento.setIdentificador(ID_CATEGORIA_DADOS_PESSOAIS);
            objCategoriaTalento.setNome("Dados pessoais");
            lstRetorno.add(objCategoriaTalento);
            lstRetorno.addAll(objCategoriaTalentoDAO.obterDisponiveisParaConsultaPorPagina(identificador, filtraPorGrupo, intNumeroPagina, intMaximoPagina));

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm uma categoria de talento de consulta a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public CategoriaTalento obterCategoriaTalentoPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        CategoriaTalento objCategoriaTalento = null;
        try
        {
            if (ID_CATEGORIA_DADOS_PESSOAIS.toString().equals(strChave))
            {
                objCategoriaTalento = new CategoriaTalento();
                objCategoriaTalento.setIdentificador(ID_CATEGORIA_DADOS_PESSOAIS);
                objCategoriaTalento.setNome("Dados pessoais");
            }
            else
            {
                objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(strChave);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaTalento;
    }

    /**
     * Obtm os relacionamentos categoria/atributo disponveis para a consulta para a categoria informada
     *
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCategoriaAtributosTalento(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstCategoriaAtributosTalento = null;
        try
        {
            CategoriaAtributoTalento objCategoriaAtributoTalento = null;
            AtributoTalento objAtributoTalento = null;
            TipoHTML objTipoHTML = null;
            if (objCategoriaTalento.getIdentificador().equals(ID_CATEGORIA_DADOS_PESSOAIS))
            {
                // Instancia lista
                lstCategoriaAtributosTalento = new ArrayList();

                // Adiciona nome
                objCategoriaAtributoTalento = new CategoriaAtributoTalento();
                objAtributoTalento = new AtributoTalento();
                objTipoHTML = new TipoHTML();
                objCategoriaAtributoTalento.setIdentificador(ID_ATRIBUTO_NOME);
                objAtributoTalento.setIdentificador(ID_ATRIBUTO_NOME);
                objAtributoTalento.setNome("Nome");
                objAtributoTalento.setTipoDado("T");
                objTipoHTML.setDescricao("TEXT");
                objTipoHTML.setMultiplicidade("L");
                objAtributoTalento.setTipoHTML(objTipoHTML);
                objCategoriaAtributoTalento.setAtributoTalento(objAtributoTalento);
                objCategoriaAtributoTalento.setCategoriaTalento(objCategoriaTalento);
                lstCategoriaAtributosTalento.add(objCategoriaAtributoTalento);

                // Adiciona idade
                objCategoriaAtributoTalento = new CategoriaAtributoTalento();
                objAtributoTalento = new AtributoTalento();
                objTipoHTML = new TipoHTML();
                objCategoriaAtributoTalento.setIdentificador(ID_ATRIBUTO_IDADE);
                objAtributoTalento.setIdentificador(ID_ATRIBUTO_IDADE);
                objAtributoTalento.setNome("Idade");
                objAtributoTalento.setTipoDado("N");
                objTipoHTML.setDescricao("TEXT");
                objTipoHTML.setMultiplicidade("L");
                objAtributoTalento.setTipoHTML(objTipoHTML);
                objCategoriaAtributoTalento.setAtributoTalento(objAtributoTalento);
                objCategoriaAtributoTalento.setCategoriaTalento(objCategoriaTalento);
                lstCategoriaAtributosTalento.add(objCategoriaAtributoTalento);

                // Adiciona sexo
                objCategoriaAtributoTalento = new CategoriaAtributoTalento();
                objAtributoTalento = new AtributoTalento();
                objTipoHTML = new TipoHTML();
                objCategoriaAtributoTalento.setIdentificador(ID_ATRIBUTO_SEXO);
                objAtributoTalento.setIdentificador(ID_ATRIBUTO_SEXO);
                objAtributoTalento.setNome("Sexo");
                objAtributoTalento.setTipoDado("N");
                objTipoHTML.setDescricao("SELECT-ONE");
                objTipoHTML.setMultiplicidade("U");
                objAtributoTalento.setTipoHTML(objTipoHTML);
                objCategoriaAtributoTalento.setAtributoTalento(objAtributoTalento);
                objCategoriaAtributoTalento.setCategoriaTalento(objCategoriaTalento);
                lstCategoriaAtributosTalento.add(objCategoriaAtributoTalento);

            }
            else
            {
                lstCategoriaAtributosTalento = objCategoriaAtributoTalentoDAO.obterParaConsultaPorCategoriaTalento(objCategoriaTalento);
                objCategoriaAtributoTalentoDAO.inicializarCategoriaTalento(lstCategoriaAtributosTalento);
                objCategoriaAtributoTalentoDAO.inicializarAtributoTalento(lstCategoriaAtributosTalento);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstCategoriaAtributosTalento;
    }

    /**
     * Obtm as opes de determinado atributo de talento (desde que no tenha filhos) , se existirem
     *
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterOpcoesPorAtributoTalento(AtributoTalento objAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        List lstAtributoTalentoOpcoes = null;
        try
        {
            AtributoTalentoOpcao objAtributoTalentoOpcao = null;
            if (objAtributoTalento.getIdentificador().equals(ID_ATRIBUTO_SEXO))
            {
                // Instancia lista
                lstAtributoTalentoOpcoes = new ArrayList();

                // Sexo masculino
                objAtributoTalentoOpcao = new AtributoTalentoOpcao();
                objAtributoTalentoOpcao.setIdentificador(ID_OPCAO_SEXO_MASCULINO);
                objAtributoTalentoOpcao.setDescricao("Masculino");
                lstAtributoTalentoOpcoes.add(objAtributoTalentoOpcao);

                // Sexo feminino
                objAtributoTalentoOpcao = new AtributoTalentoOpcao();
                objAtributoTalentoOpcao.setIdentificador(ID_OPCAO_SEXO_FEMININO);
                objAtributoTalentoOpcao.setDescricao("Feminino");
                lstAtributoTalentoOpcoes.add(objAtributoTalentoOpcao);

            }
            else
            {
                lstAtributoTalentoOpcoes = objAtributoTalentoOpcaoDAO.obterPorAtributoTalento(objAtributoTalento);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributoTalentoOpcoes;
    }

    /**
     * Verifica se determinado atributo de talento tm filhos na relao categoria/atributo
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejado
     * 
     * @return boolean Com a verificao
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public boolean verificarExistenciaCategoriaAtributosFilhos(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        boolean blnExistenciaFilhos = false;
        try
        {
            blnExistenciaFilhos = objCategoriaAtributoTalentoDAO.verificarExistenciaCategoriaAtributosTalentoFilhos(objCategoriaAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return blnExistenciaFilhos;
    }

    /**
     * Obtm uma categoria/atributo de talento a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaAtributoTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public CategoriaAtributoTalento obterCategoriaAtributoTalentoPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        CategoriaAtributoTalento objCategoriaAtributoTalento = null;
        try
        {
            CategoriaTalento objCategoriaTalento = null;
            AtributoTalento objAtributoTalento = null;
            TipoHTML objTipoHTML = null;
            if (ID_ATRIBUTO_NOME.toString().equals(strChave))
            {
                // Nome
                objCategoriaAtributoTalento = new CategoriaAtributoTalento();
                objCategoriaTalento = new CategoriaTalento();
                objAtributoTalento = new AtributoTalento();
                objTipoHTML = new TipoHTML();
                objCategoriaTalento.setIdentificador(ID_CATEGORIA_DADOS_PESSOAIS);
                objCategoriaAtributoTalento.setIdentificador(ID_ATRIBUTO_NOME);
                objAtributoTalento.setIdentificador(ID_ATRIBUTO_NOME);
                objAtributoTalento.setNome("Nome");
                objAtributoTalento.setTipoDado("T");
                objTipoHTML.setDescricao("TEXT");
                objTipoHTML.setMultiplicidade("L");
                objAtributoTalento.setTipoHTML(objTipoHTML);
                objCategoriaAtributoTalento.setAtributoTalento(objAtributoTalento);
                objCategoriaAtributoTalento.setCategoriaTalento(objCategoriaTalento);
            }
            else if (ID_ATRIBUTO_IDADE.toString().equals(strChave))
            {
                // Idade
                objCategoriaAtributoTalento = new CategoriaAtributoTalento();
                objCategoriaTalento = new CategoriaTalento();
                objAtributoTalento = new AtributoTalento();
                objTipoHTML = new TipoHTML();
                objCategoriaTalento.setIdentificador(ID_CATEGORIA_DADOS_PESSOAIS);
                objCategoriaAtributoTalento.setIdentificador(ID_ATRIBUTO_IDADE);
                objAtributoTalento.setIdentificador(ID_ATRIBUTO_IDADE);
                objAtributoTalento.setNome("Idade");
                objAtributoTalento.setTipoDado("N");
                objTipoHTML.setDescricao("TEXT");
                objTipoHTML.setMultiplicidade("L");
                objAtributoTalento.setTipoHTML(objTipoHTML);
                objCategoriaAtributoTalento.setAtributoTalento(objAtributoTalento);
                objCategoriaAtributoTalento.setCategoriaTalento(objCategoriaTalento);
            }
            else if (ID_ATRIBUTO_SEXO.toString().equals(strChave))
            {
                // Sexo
                objCategoriaAtributoTalento = new CategoriaAtributoTalento();
                objCategoriaTalento = new CategoriaTalento();
                objAtributoTalento = new AtributoTalento();
                objTipoHTML = new TipoHTML();
                objCategoriaTalento.setIdentificador(ID_CATEGORIA_DADOS_PESSOAIS);
                objCategoriaAtributoTalento.setIdentificador(ID_ATRIBUTO_SEXO);
                objAtributoTalento.setIdentificador(ID_ATRIBUTO_SEXO);
                objAtributoTalento.setNome("Sexo");
                objAtributoTalento.setTipoDado("N");
                objTipoHTML.setDescricao("SELECT-ONE");
                objTipoHTML.setMultiplicidade("U");
                objAtributoTalento.setTipoHTML(objTipoHTML);
                objCategoriaAtributoTalento.setAtributoTalento(objAtributoTalento);
                objCategoriaAtributoTalento.setCategoriaTalento(objCategoriaTalento);
            }
            else
            {
                objCategoriaAtributoTalento = (CategoriaAtributoTalento) objCategoriaAtributoTalentoDAO.obterPelaChaveComAtributoTalentoETipoHtml(strChave);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaAtributoTalento;
    }

    /**
     * Obtm a hierarquia inferior da categoria/atributo desejada
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de atributo de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCategoriaAtributoTalentoHierarquiaInferior(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoFacade objCategoriaAtributoTalentoFacade = new CategoriaAtributoTalentoFacade(obterNomeConexao());
        List lstCategoriaAtributoTalentoHierarquiaInferior = new ArrayList();

        try
        {
            lstCategoriaAtributoTalentoHierarquiaInferior = objCategoriaAtributoTalentoFacade.obterHierarquiaInferior(objCategoriaAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return lstCategoriaAtributoTalentoHierarquiaInferior;
    }

    /**
     * Obtm os relacionamentos categoria/atributo que so filhos da categoria/atributo informado
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCategoriaAtributosTalentoFilhos(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstCategoriaAtributosTalentoFilhos = null;
        try
        {
            lstCategoriaAtributosTalentoFilhos = objCategoriaAtributoTalentoDAO.obterTodosOsFilhos(objCategoriaAtributoTalento);
            objCategoriaAtributoTalentoDAO.inicializarAtributoTalento(lstCategoriaAtributosTalentoFilhos);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstCategoriaAtributosTalentoFilhos;
    }

    /**
     * Obtm as opes filhas de determinada opo pai
     *
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterOpcoesPeloPaiEAtributoTalento(AtributoTalentoOpcao objAtributoTalentoOpcaoPai, AtributoTalento objAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        List lstAtributoTalentoOpcoes = null;
        try
        {
            lstAtributoTalentoOpcoes = objAtributoTalentoOpcaoDAO.obterPeloPaiEAtributoTalento(objAtributoTalentoOpcaoPai, objAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributoTalentoOpcoes;
    }

    /**
     * Obtm uma opo de talento de consulta a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return AtributoTalentoOpcao POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public AtributoTalentoOpcao obterAtributoTalentoOpcaoPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        AtributoTalentoOpcao objAtributoTalentoOpcao = null;
        try
        {
            if (ID_OPCAO_SEXO_MASCULINO.toString().equals(strChave))
            {
                objAtributoTalentoOpcao = new AtributoTalentoOpcao();
                objAtributoTalentoOpcao.setIdentificador(ID_OPCAO_SEXO_MASCULINO);
                objAtributoTalentoOpcao.setDescricao("Masculino");
            }
            else if (ID_OPCAO_SEXO_FEMININO.toString().equals(strChave))
            {
                objAtributoTalentoOpcao = new AtributoTalentoOpcao();
                objAtributoTalentoOpcao.setIdentificador(ID_OPCAO_SEXO_FEMININO);
                objAtributoTalentoOpcao.setDescricao("Feminino");
            }
            else
            {
                objAtributoTalentoOpcao = (AtributoTalentoOpcao) objAtributoTalentoOpcaoDAO.obterPelaChave(strChave);
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objAtributoTalentoOpcao;
    }

    /**
     * Obtm um registro de pessoa a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Pessoa POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public Pessoa obterPessoaPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        PessoaDAO objPessoaDAO = new PessoaDAO(obterNomeConexao());
        Pessoa objPessoa = null;
        try
        {
            objPessoa = (Pessoa) objPessoaDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objPessoa;
    }

    /**
     * Obtm o total de pessoas com talentos cadastrados
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalPessoasComTalentos() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objTalentoDAO.obterTotalPessoasComTalentos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    public int obterTotalPessoasComTalentos(String strIdentificadorUsuario, boolean blnFiltroGrupo) throws CDException
    {
        // Instancia DAO/Bean
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());

        int retorno = 0;

        try
        {
            // Obtm grupos habilitados ao usurio logado consultar
            String strGrupos = null;
            if (blnFiltroGrupo)
            {
                strGrupos = objFiltroConsultaDAO.obterGruposHabilitadosConsultaCurriculo(strIdentificadorUsuario);
            }
            retorno = objTalentoDAO.obterTotalListagemConsulta(null, null, null, null, null, strGrupos);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return retorno;
    }

    public List obterPessoasComTalentos(int numeroPagina, int maximoPagina) throws CDException
    {
        // Instancia DAO e obtm os registros da pgina
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        List resultado = null;
        try
        {
            resultado = objTalentoDAO.obterPessoasComTalentosPorPagina(numeroPagina, maximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return resultado;
    }

    /**
     * Efetua consulta de talentos
     * 
     * @param mapConsulta
     *            Contendo os parmetros de consulta
     * @param objPessoa
     *            Pessoa que est realizando a consulta
     * @param intMaximoPermitido
     *            Mximo de registros permitidos para o retorno da consulta
     * @param stbTotalRetornado
     *            Total de registros retornados
     * @param objSistema
     *            Sistema o qual devero ser buscados os perfis para consulta
     * @param blnFiltroGrupo
     *            Para indicar se dever ou no haver filtro por grupo
     * 
     * @return List Contendo o resultado da consulta, se o nmero mximo no for
     *         ultrapassado
     * 
     * @throws DAOException
     *             se ocorrer um erro relacionado com banco de dados
     */

    public List<ResultadoConsultaTalento> consultar(Map mapConsultasTalento, UsuarioAutenticado objPessoa, int intMaximoPermitido, StringBuilder stbTotalRetornado, boolean blnFiltroGrupo) throws CDException
    {
        List<ResultadoConsultaTalento> resultadoConsulta = null;

        // Instancia DAO e obtm o registro pela chave
        ConsultaTalentoDAO objConsultaTalentoDAO = new ConsultaTalentoDAO(obterNomeConexao());
        try
        {
            List lstConsultaTalentos = objConsultaTalentoDAO.consultar(mapConsultasTalento, objPessoa, intMaximoPermitido, stbTotalRetornado, blnFiltroGrupo);

            ResultadoConsultaTalentoFacade consultaTalentoFacade = new ResultadoConsultaTalentoFacade(obterNomeConexao());
            resultadoConsulta = consultaTalentoFacade.recuperarResultadoConsultaTalentoDasPessoas(lstConsultaTalentos);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return resultadoConsulta;
    }

}
