/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.facade;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.autenticacaoperfil.dao.PerfilSistemaDAO;
import br.gov.camara.negocio.autenticacaoperfil.dao.UsuarioSistemaDAO;
import br.gov.camara.negocio.autenticacaoperfil.pojo.PerfilSistema;
import br.gov.camara.negocio.autenticacaoperfil.pojo.Sistema;
import br.gov.camara.negocio.autenticacaoperfil.pojo.UsuarioSistema;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.FiltroConsultaDAO;
import br.gov.camara.negocio.bancotalentos.dao.TipoFiltroConsultaDAO;
import br.gov.camara.negocio.bancotalentos.pojo.FiltroConsulta;
import br.gov.camara.negocio.bancotalentos.pojo.TipoFiltroConsulta;
import br.gov.camara.negocio.exception.NegocioException;

/**
 * Facade para atributo de talento
 */
public class FiltroConsultaFacade extends Facade
{
    // Variveis de instncia
    private static Log log = LogFactory.getLog(FiltroConsultaFacade.class.getName());

    public FiltroConsultaFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Obtm o total de registros
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistros() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objFiltroConsultaDAO.obterTotalRegistros();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm o total de registros por tipo de filtro de consulta
     * 
     * @param TipoFiltroConsulta Objeto contendo os dados do tipo de filtro de consulta
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistrosPorTipoFiltroConsulta(TipoFiltroConsulta objTipoFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());

        // Recebe o total de registro
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objFiltroConsultaDAO.obterTotalRegistrosPorTipoFiltroConsulta(objTipoFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    public String obterConjuntoCodigosGruposPermitidos(String identificadorUsuario, String separador) throws CDException
    {
        String conjuntoCodigosGruposPermitidos = null;

        // Recebe o total de registro
        try
        {
            // Instancia DAO e obtm os registros da pgina
            FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
            conjuntoCodigosGruposPermitidos = objFiltroConsultaDAO.obterConjuntoCodigosGruposPermitidos(identificadorUsuario, separador);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return conjuntoCodigosGruposPermitidos;
    }

    public List<Integer> obterCodigosGruposHabilitadosConsultaCurriculo(String strIdentificadorUsuario) throws CDException
    {
        List<Integer> lstGruposHabilitados = null;

        // Recebe o total de registro
        try
        {
            // Instancia DAO e obtm os registros da pgina
            FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
            lstGruposHabilitados = objFiltroConsultaDAO.obterListaCodigosGruposPermitidos(strIdentificadorUsuario);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return lstGruposHabilitados;
    }

    /**
     * Obtm os registros de determinada pgina
     * 
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPagina(int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());

        // Recebe lista de retorno
        List lstRetorno = null;
        try
        {
            lstRetorno = objFiltroConsultaDAO.obterPorPagina(intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os registros de determinada pgina por tipo de filtro de consulta
     * 
     * @param TipoFiltroConsulta Objetos Contendos os dados para consulta
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorTipoFiltroConsultaPorPagina(TipoFiltroConsulta objTipoFiltroConsulta, int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());

        // Recebe a lista de retorno
        List lstRetorno = null;
        try
        {
            lstRetorno = objFiltroConsultaDAO.obterPorTipoFiltroConsultaPorPagina(objTipoFiltroConsulta, intNumeroPagina, intMaximoPagina);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return FiltroConsulta POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public FiltroConsulta obterPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        FiltroConsulta objFiltroConsulta = null;
        try
        {
            // Obtm objeto pela chave
            objFiltroConsulta = (FiltroConsulta) objFiltroConsultaDAO.obterPelaChave(strChave);
            // Inicializa objetos relacionados
            objFiltroConsultaDAO.inicializarFiltroConsultaGrupo(objFiltroConsulta);
            objFiltroConsultaDAO.inicializarFiltroConsultaUsuario(objFiltroConsulta);
            objFiltroConsultaDAO.inicializarTipoFiltroConsulta(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objFiltroConsulta;
    }

    /**
     * Obtm um registro de perfil de sistema a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return PerfilSistema POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public PerfilSistema obterPerfilSistemaPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        PerfilSistemaDAO objPerfilSistemaDAO = new PerfilSistemaDAO(obterNomeConexao());
        PerfilSistema objPerfilSistema = null;
        try
        {
            objPerfilSistema = (PerfilSistema) objPerfilSistemaDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objPerfilSistema;
    }

    /**
     * Obtm os atributos de talento
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterAtributosTalento() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        AtributoTalentoDAO objAtributoTalentoDAO = new AtributoTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objAtributoTalentoDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os perfis de sistema pelo sistema a que ele pertence
     * 
     * @param objSistema Sistema que pertence o perfil de sistema
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPerfisSistemaPeloSistema(Sistema objSistema) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        PerfilSistemaDAO objPerfilSistemaDAO = new PerfilSistemaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objPerfilSistemaDAO.obterPeloSistema(objSistema);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Inclui um registro
     *
     * @param FiltroConsulta POJO representando o objeto a ser includo
     *
     * @return retorna a chave do objeto que foi incluido
     * 
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public Integer incluir(FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e inclui registro
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        Integer strChave = null;

        try
        {
            strChave = Integer.valueOf(objFiltroConsultaDAO.incluir(objFiltroConsulta).toString());
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return strChave;
    }

    /**
     * Altera um registro
     *
     * @param objFiltroConsulta POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterar(FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e altera registro
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        try
        {
            objFiltroConsultaDAO.alterar(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Exclui um registro
     *
     * @param objFiltroConsulta POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void excluir(FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e exclui registro
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        try
        {
            objFiltroConsultaDAO.excluir(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Obtm os grupos disponveis para determinada categoria de talento
     * 
     * @param objFiltroConsulta Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterGruposDisponiveisCriterioConsulta(FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFiltroConsultaDAO.obterGruposDisponiveisCriterioConsulta(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os grupos disponveis para determinada categoria de talento
     * 
     * @param objFiltroConsulta Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterGruposDisponiveisFiltroConsultaGrupo(FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFiltroConsultaDAO.obterGruposDisponiveisFiltroConsultaGrupo(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os grupos selecionados para determinada categoria de talento
     * 
     * @param objFiltroConsulta Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterGruposSelecionadosFiltroConsultaGrupo(FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFiltroConsultaDAO.obterGruposSelecionadosFiltroConsultaGrupo(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os grupos selecionados para determinada categoria de talento
     * 
     * @param objFiltroConsulta Categoria de talento a ser verificada
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterGruposSelecionadosCriterioConsulta(FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        FiltroConsultaDAO objFiltroConsultaDAO = new FiltroConsultaDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objFiltroConsultaDAO.obterGruposSelecionadosCriterioConsulta(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm uma categoria de talento a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public TipoFiltroConsulta obterTipoFiltroConsultaPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        TipoFiltroConsultaDAO objTipoFiltroConsultaDAO = new TipoFiltroConsultaDAO(obterNomeConexao());
        TipoFiltroConsulta objTipoFiltroConsulta = null;
        try
        {
            objTipoFiltroConsulta = (TipoFiltroConsulta) objTipoFiltroConsultaDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objTipoFiltroConsulta;
    }

    /**
     * Exclui o filtro de consulta de usurio
     * 
     * @param CriterioAcessoLotacao Usurio a ser verificado
     * @param FiltroConsulta Filtro de Consulta a ser verificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public void excluir(UsuarioSistema objUsuarioSistema, FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no mtodo");
        }

        try
        {
            // Itera coleo para procurar o registro desejado na excluso
            Iterator itrFiltroConsultaUsuario = objFiltroConsulta.getFiltroConsultaUsuario().iterator();
            UsuarioSistema objUsuario = null;

            while (itrFiltroConsultaUsuario.hasNext())
            {
                objUsuario = (UsuarioSistema) itrFiltroConsultaUsuario.next();
                if (objUsuarioSistema.getIdentificador().equals(objUsuario.getIdentificador()))
                {
                    break;
                }
                objUsuario = null;
            }
            if (objUsuario != null)
            {
                objFiltroConsulta.getFiltroConsultaUsuario().remove(objUsuario);
            }

            alterar(objFiltroConsulta);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Obtm os grupos disponveis para determinado filtro de consulta de usurio
     * 
     * @param objFiltroConsulta Filtro de consulta a ser verificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public void incluir(UsuarioSistema objUsuarioSistema, FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no mtodo");
        }

        // Instancia DAO e obtm os registros
        UsuarioSistemaDAO objUsuarioSistemaDAO = new UsuarioSistemaDAO(obterNomeConexao());

        try
        {
            // Obtm usurio
            objUsuarioSistema = (UsuarioSistema) objUsuarioSistemaDAO.obterPelaChave(objUsuarioSistema.getIdentificador().toString());
            // Verifica se usurio  vlido
            if (objUsuarioSistema == null)
            {
                throw new NegocioException("Usurio no existe");
            }
            else
            {
                // Verifica se usurio j est cadastrado
                UsuarioSistema objUsuario = obterPelaChave(objUsuarioSistema.getIdentificador().toString(), objFiltroConsulta);
                if (objUsuario != null)
                {
                    throw new NegocioException("Usurio j cadastrado");
                }
                else
                {
                    // Inclui usurio atravs do mtodo alterar da facade Filtro Consulta 
                    objFiltroConsulta.getFiltroConsultaUsuario().add(objUsuarioSistema);
                    alterar(objFiltroConsulta);
                }
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Obtm os grupos disponveis para determinado filtro de consulta de usurio
     * 
     * @param objFiltroConsulta Filtro de consulta a ser verificado
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public UsuarioSistema obterPelaChave(String strChave, FiltroConsulta objFiltroConsulta) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        //FiltroConsultaGrupoDAO objFiltroConsultaGrupoDAO = new FiltroConsultaGrupoDAO(obterNomeConexao());
        UsuarioSistema objUsuarioSistema = null;

        try
        {
            Iterator itrFiltroConsultaUsuario = objFiltroConsulta.getFiltroConsultaUsuario().iterator();
            while (itrFiltroConsultaUsuario.hasNext())
            {
                UsuarioSistema objUsuarioSistemaTemp = (UsuarioSistema) itrFiltroConsultaUsuario.next();
                if (strChave.equals("" + objUsuarioSistemaTemp.getIdentificador()))
                {
                    objUsuarioSistema = objUsuarioSistemaTemp;
                    break;
                }
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return objUsuarioSistema;
    }

}
