/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.facade;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.biblioteca.util.Copia;
import br.gov.camara.biblioteca.util.DataNova;
import br.gov.camara.biblioteca.util.ExpressaoRegular;
import br.gov.camara.biblioteca.util.Numero;
import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.DAO;
import br.gov.camara.negocio.Facade;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoOpcaoDAO;
import br.gov.camara.negocio.bancotalentos.dao.AtributoTalentoValoradoDAO;
import br.gov.camara.negocio.bancotalentos.dao.CategoriaAtributoTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.CategoriaTalentoDAO;
import br.gov.camara.negocio.bancotalentos.dao.PessoaDAO;
import br.gov.camara.negocio.bancotalentos.dao.TalentoDAO;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoOpcao;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoValorado;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaAtributoTalento;
import br.gov.camara.negocio.bancotalentos.pojo.CategoriaTalento;
import br.gov.camara.negocio.bancotalentos.pojo.Pessoa;
import br.gov.camara.negocio.bancotalentos.pojo.Talento;
import br.gov.camara.negocio.bancotalentos.pojo.TalentoVisualizacao;
import br.gov.camara.negocio.bancotalentos.util.GerenciadorAtributoVirtual;
import br.gov.camara.negocio.comum.facade.DataHoraBDFacade;
import br.gov.camara.negocio.exception.DAOException;
import br.gov.camara.negocio.exception.NegocioException;

/**
 * Facade para atributo de talento
 */
public class TalentoFacade extends Facade
{
    // Variveis de instncia
    private static Log log = LogFactory.getLog(TalentoFacade.class.getName());

    public TalentoFacade(String nomeConexao)
    {
        super(nomeConexao);
    }

    /**
     * Obtm o total de registros por pessoa
     * 
     * @param objPessoa Pessoa desejada
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistrosPorPessoa(Pessoa objPessoa) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objTalentoDAO.obterTotalRegistrosPorPessoa(objPessoa);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm o total de registros por pessoa e categoria de talento
     * 
     * @param objPessoa Pessoa desejada
     * @param objCategoriatalento Categoria de talento desejada
     * 
     * @return int Contendo o total
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public int obterTotalRegistrosPorPessoaCategoriaTalento(Pessoa objPessoa, CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        int intTotalRegistros = 0;
        try
        {
            intTotalRegistros = objTalentoDAO.obterTotalRegistrosPorPessoaCategoriaTalento(objPessoa, objCategoriaTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return intTotalRegistros;
    }

    /**
     * Obtm os registros da pessoa desejada de determinada pgina 
     *
     * @param objPessoa Pessoa desejada
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPessoaPorPagina(int identificadorPessoa, int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        List lstRetorno = new ArrayList();
        try
        {
            List lstTalentos = objTalentoDAO.obterPorPessoaPorPagina(identificadorPessoa, intNumeroPagina, intMaximoPagina);
            Iterator itrTalentos = lstTalentos.iterator();
            while (itrTalentos.hasNext())
            {
                Talento objTalento = (Talento) itrTalentos.next();
                TalentoVisualizacao objTalentoVisualizacao = new TalentoVisualizacao();
                Copia.criar(objTalento, objTalentoVisualizacao);
                objTalentoVisualizacao.setDescricao(obterDescricao(objTalento));
                lstRetorno.add(objTalentoVisualizacao);
            }

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm os registros da pessoa desejada, de determinada categoria de talento e de determinada pgina 
     *
     * @param objPessoa Pessoa desejada
     * @param objCategoriaTalento Categoria de talento desejada
     * @param int Nmero da pgina a ser mostrada
     * @param int Quantidade de registros
     *  
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterPorPessoaCategoriaTalentoPorPagina(Pessoa objPessoa, CategoriaTalento objCategoriaTalento, int intNumeroPagina, int intMaximoPagina) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros da pgina
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        List lstRetorno = new ArrayList();
        try
        {
            List lstTalentos = objTalentoDAO.obterPorPessoaCategoriaTalentoPorPagina(objPessoa, objCategoriaTalento, intNumeroPagina, intMaximoPagina);
            Iterator itrTalentos = lstTalentos.iterator();
            while (itrTalentos.hasNext())
            {
                Talento objTalento = (Talento) itrTalentos.next();
                TalentoVisualizacao objTalentoVisualizacao = new TalentoVisualizacao();
                Copia.criar(objTalento, objTalentoVisualizacao);
                objTalentoVisualizacao.setDescricao(obterDescricao(objTalento));
                lstRetorno.add(objTalentoVisualizacao);
            }

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm a descrio de determinado talento
     *
     * @param objTalento Talento a ser descrito
     * 
     * @return String Contendo a descrio
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public String obterDescricao(Talento objTalento) throws CDException
    {
        StringBuilder descricao = new StringBuilder();
        try
        {
            if ("S".equals(objTalento.getCategoriaTalento().getIndicativoUnicidade()))
            {
                descricao.append(objTalento.getCategoriaTalento().getNome());
            }
            else
            {
                AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());
                List<AtributoTalentoValorado> lstRetorno = (List<AtributoTalentoValorado>) objAtributoTalentoValoradoDAO.obterPeloTalento(objTalento);

                for (AtributoTalentoValorado objAtributoTalentoValorado : lstRetorno)
                {
                    if ("S".equals(objAtributoTalentoValorado.getCategoriaAtributoTalento().getFormacaoDescricao()))
                    {
                        descricao.append(objAtributoTalentoValorado.getValoracao());
                        descricao.append("/");
                    }
                }

                if (descricao.length() == 0)
                {
                    descricao.append(objTalento.getCategoriaTalento().getNome());
                }
                else
                {
                    descricao.deleteCharAt(descricao.length() - 1);
                }
            }

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }

        return descricao.toString();
    }

    /**
     * Obtm um registro a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Talento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public Talento obterPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        Talento objTalento = null;
        try
        {
            objTalento = (Talento) objTalentoDAO.obterPelaChave(strChave);

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objTalento;
    }

    /**
     * Obtm um registro de pessoa a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return Pessoa POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public Pessoa obterPessoaPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        PessoaDAO objPessoaDAO = new PessoaDAO(obterNomeConexao());
        Pessoa objPessoa = null;
        try
        {
            objPessoa = (Pessoa) objPessoaDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objPessoa;
    }

    /**
     * Obtm um registro a partir da chave, incluindo tambm a descrio
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return TalentoVisualizacao POJO (extendido) representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public TalentoVisualizacao obterPelaChaveComDescricao(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        Talento objTalento = null;
        TalentoVisualizacao objTalentoVisualizacao = null;
        try
        {
            objTalento = (Talento) objTalentoDAO.obterPelaChave(strChave);
            if (objTalento != null)
            {
                objTalentoVisualizacao = new TalentoVisualizacao();
                Copia.criar(objTalento, objTalentoVisualizacao);
                objTalentoVisualizacao.setDescricao(obterDescricao(objTalento));
            }
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objTalentoVisualizacao;
    }

    /**
     * Obtm as categorias de talento disponveis para o usurio desejado
     * 
     * @param objPessoa Pessoa a buscar os talentos
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCategoriasTalentoAssociadasAoGrupo(int codigoGrupo) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm os registros
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objCategoriaTalentoDAO.obterAssociadasAAtributosPorGrupoPorPagina(Arrays.asList(codigoGrupo), 1, 0);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * Obtm uma categoria de talento a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public CategoriaTalento obterCategoriaTalentoPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        CategoriaTalento objCategoriaTalento = null;
        try
        {
            objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaTalento;
    }

    /**
     * Obtm uma categoria/atributo de talento a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return CategoriaAtributoTalento POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public CategoriaAtributoTalento obterCategoriaAtributoTalentoPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        CategoriaAtributoTalento objCategoriaAtributoTalento = null;
        try
        {
            objCategoriaAtributoTalento = (CategoriaAtributoTalento) objCategoriaAtributoTalentoDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaAtributoTalento;
    }

    /**
     * Obtm uma opo de talento a partir da chave
     *
     * @param strChave Chave do registro a ser obtido
     * 
     * @return AtributoTalentoOpcao POJO representando o registro obtido
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public AtributoTalentoOpcao obterAtributoTalentoOpcaoPelaChave(String strChave) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        AtributoTalentoOpcao objAtributoTalentoOpcao = null;
        try
        {
            objAtributoTalentoOpcao = (AtributoTalentoOpcao) objAtributoTalentoOpcaoDAO.obterPelaChave(strChave);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objAtributoTalentoOpcao;
    }

    /**
     * Obtm os relacionamentos categoria/atributo para a categoria informada
     *
     * @param objCategoriaTalento Categoria de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCategoriaAtributosTalento(CategoriaTalento objCategoriaTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstCategoriaAtributosTalento = null;
        try
        {
            lstCategoriaAtributosTalento = objCategoriaAtributoTalentoDAO.obterPorCategoriaTalento(objCategoriaTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstCategoriaAtributosTalento;
    }

    /**
     * Obtm os relacionamentos categoria/atributo que so filhos da categoria/atributo informado
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCategoriaAtributosTalentoFilhos(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        List lstCategoriaAtributosTalentoFilhos = null;
        try
        {
            lstCategoriaAtributosTalentoFilhos = objCategoriaAtributoTalentoDAO.obterTodosOsFilhos(objCategoriaAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstCategoriaAtributosTalentoFilhos;
    }

    /**
     * Obtm as valoraes do talento especificado
     *
     * @param objTalento Talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterValoracoesPorTalento(Talento objTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());
        List lstAtributosTalentoValorados = null;
        try
        {
            lstAtributosTalentoValorados = objAtributoTalentoValoradoDAO.obterPeloTalentoComAtributoTalentoEAtributoTalentoOpcoes(objTalento);
            objAtributoTalentoValoradoDAO.inicializarAtributoTalentoOpcao(lstAtributosTalentoValorados);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributosTalentoValorados;
    }

    /**
     * Obtm as opes filhas de determinada opo pai
     *
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterValoracoesPorTalentoComAtributoTalentoEAtributoTalentoOpcoes(Talento objTalento) throws CDException
    {
        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());
        List lstAtributosTalentoValorados = null;
        try
        {
            lstAtributosTalentoValorados = objAtributoTalentoValoradoDAO.obterPeloTalentoComAtributoTalentoEAtributoTalentoOpcoes(objTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributosTalentoValorados;
    }

    /**
     * Obtm as opes filhas de determinada opo pai
     * 
     * @param objAtributoTalento
     *            Atributo de talento desejado
     * 
     * @return List Contendo os registros
     * 
     * @throws CDException
     *             se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterOpcoesPeloPaiEAtributoTalento(AtributoTalentoOpcao objAtributoTalentoOpcaoPai, AtributoTalento objAtributoTalento) throws CDException
    {
        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        List lstAtributoTalentoOpcoes = null;
        try
        {
            lstAtributoTalentoOpcoes = objAtributoTalentoOpcaoDAO.obterPeloPaiEAtributoTalento(objAtributoTalentoOpcaoPai, objAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributoTalentoOpcoes;
    }

    /**
     * Obtm as opes filhas de determinada opo pai
     *
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterOpcoesPeloPai(AtributoTalentoOpcao objAtributoTalentoOpcaoPai) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        List lstAtributoTalentoOpcoes = null;
        try
        {
            lstAtributoTalentoOpcoes = objAtributoTalentoOpcaoDAO.obterPeloPai(objAtributoTalentoOpcaoPai);
        }
        catch (DAOException daoe)
        {
            throw new CDException(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributoTalentoOpcoes;
    }

    /**
     * Obtm as opes de determinado atributo de talento (desde que no tenha filhos) , se existirem
     *
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterOpcoesPorAtributoTalento(AtributoTalento objAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        List lstAtributoTalentoOpcoes = null;
        try
        {
            lstAtributoTalentoOpcoes = objAtributoTalentoOpcaoDAO.obterPorAtributoTalento(objAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributoTalentoOpcoes;
    }

    /**
     * Obtm as opes de um determinado atributo talento
     *
     * @param objAtributoTalento Atributo de talento desejado
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterOpcoesAtributoTalento(AtributoTalento objAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        List lstAtributoTalentoOpcoes = null;
        try
        {
            lstAtributoTalentoOpcoes = objAtributoTalentoOpcaoDAO.obterPorAtributoTalento(objAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributoTalentoOpcoes;
    }

    /**
     * Obtm a hierarquia superior da opo desejada (incluindo ela)
     *
     * @param objAtributoTalentoOpcao Opo de atributo de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterAtributoTalentoOpcaoHierarquiaSuperior(AtributoTalentoOpcao objAtributoTalentoOpcao) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        AtributoTalentoOpcaoFacade objAtributoTalentoOpcaoFacade = new AtributoTalentoOpcaoFacade(obterNomeConexao());
        List lstAtributoTalentoOpcoesHierarquiaSuperior = new ArrayList();
        try
        {
            objAtributoTalentoOpcao = (AtributoTalentoOpcao) objAtributoTalentoOpcaoDAO.obterPelaChave(String.valueOf(objAtributoTalentoOpcao.getIdentificador()));
            lstAtributoTalentoOpcoesHierarquiaSuperior = objAtributoTalentoOpcaoFacade.obterHierarquiaSuperior(objAtributoTalentoOpcao);
            lstAtributoTalentoOpcoesHierarquiaSuperior.add(objAtributoTalentoOpcao);

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstAtributoTalentoOpcoesHierarquiaSuperior;
    }

    /**
     * Obtm a hierarquia inferior da categoria/atributo desejada
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de atributo de talento desejada
     * 
     * @return List Contendo os registros
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterCategoriaAtributoTalentoHierarquiaInferior(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoFacade objCategoriaAtributoTalentoFacade = new CategoriaAtributoTalentoFacade(obterNomeConexao());
        List lstCategoriaAtributoTalentoHierarquiaInferior = new ArrayList();
        try
        {
            lstCategoriaAtributoTalentoHierarquiaInferior = objCategoriaAtributoTalentoFacade.obterHierarquiaInferior(objCategoriaAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstCategoriaAtributoTalentoHierarquiaInferior;
    }

    /**
     * Verifica se determinado atributo de talento tm filhos na relao categoria/atributo
     *
     * @param objCategoriaAtributoTalento Categoria/atributo de talento desejado
     * 
     * @return boolean Com a verificao
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public boolean verificarExistenciaCategoriaAtributosFilhos(CategoriaAtributoTalento objCategoriaAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm o registro pela chave
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        boolean blnExistenciaFilhos = false;
        try
        {
            blnExistenciaFilhos = objCategoriaAtributoTalentoDAO.verificarExistenciaCategoriaAtributosTalentoFilhos(objCategoriaAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return blnExistenciaFilhos;
    }

    /**
     * Obtm todos os registros 
     * 
     * @return List Contendo os POJOs representando os registro obtidos
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     * 
     */
    public List obterTodos() throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm todos os registros
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        List lstRetorno = null;
        try
        {
            lstRetorno = objTalentoDAO.obterTodos();
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return lstRetorno;
    }

    /**
     * 
     * Obtm categoria/atributo de acordo com a categoria e o atributo passados
     * 
     * @param objCategoriaTalento Categoria de talento a ser consultada
     * @param objAtributoTalento Atributo de talento a ser consultado
     * 
     * @return CategoriaAtributoTalento Contendo a categoria/atributo
     * 
     * @throws DAOException se ocorrer algum erro relacionado com o acesso a banco de dados
     */
    public CategoriaAtributoTalento obterCategoriaAtributoTalentoPorCategoriaTalentoAtributoTalento(CategoriaTalento objCategoriaTalento, AtributoTalento objAtributoTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e obtm todos os registros
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        CategoriaAtributoTalento objCategoriaAtributoTalento = null;
        try
        {
            objCategoriaAtributoTalento = objCategoriaAtributoTalentoDAO.obterPorCategoriaTalentoAtributoTalento(objCategoriaTalento, objAtributoTalento);
        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        return objCategoriaAtributoTalento;

    }

    /**
     * Inclui um registro
     *
     * @param objTalento POJO representando o objeto a ser includo
     * @param lstAtributosTalentoValoros List contendo as valoraes do talento a serem includas
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     * @return retorna a chave do objeto que foi incluido
     *
     */
    public Integer incluir(Talento objTalento, List lstAtributosTalentoValorados) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e inclui registro
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        CategoriaTalentoDAO objCategoriaTalentoDAO = new CategoriaTalentoDAO(obterNomeConexao());
        Integer strChave = null;
        try
        {
            // No  permitido que a pessoa tenha mais de uma categoria
            // cadastrada se ela for perfil
            CategoriaTalento objCategoriaTalento = (CategoriaTalento) objCategoriaTalentoDAO.obterPelaChave(String.valueOf(objTalento.getCategoriaTalento().getIdentificador()));
            if ("S".equals(objCategoriaTalento.getIndicativoUnicidade()))
            {
                if (objTalentoDAO.verificarExistenciaPorPessoaCategoriaTalento(objTalento.getPessoa(), objTalento.getCategoriaTalento()))
                {
                    throw new NegocioException("No  possvel inserir mais de um talento se a categoria " + "em questo  perfil");
                }
            }

            // Verifica as valoraes obrigatrios
            verificarValoracoesObrigatorias(objTalento.getCategoriaTalento(), lstAtributosTalentoValorados);

            // Inicia transao
            DAO.iniciarTransacao(obterNomeConexao());

            // Preenche dados de talento
            objTalento.setDataLancamento(DataNova.converter(DataHoraBDFacade.obterTimeStampBD()));

            // Inclui talento
            objTalentoDAO.incluir(objTalento);

            // Inclui valoraes
            incluirValoracoes(objTalento, lstAtributosTalentoValorados);

            // Realiza transao
            DAO.realizarTransacao(obterNomeConexao());

        }
        catch (Exception daoe)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
        // Retorna chave
        return strChave;
    }

    private void verificarValoracoesObrigatorias(CategoriaTalento objCategoriaTalento, List lstAtributosTalentoValorados) throws CDException
    {
        try
        {
            // Declaraes
            CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());

            // Obtm categoria/atributos da categoria
            List lstCategoriaAtributosTalento = objCategoriaAtributoTalentoDAO.obterPorCategoriaTalento(objCategoriaTalento);

            // Verifica os atributos de talento obrigatrios 
            Iterator itrCategoriaAtributosTalento = lstCategoriaAtributosTalento.iterator();
            while (itrCategoriaAtributosTalento.hasNext())
            {
                // Obtm categoria/atributo atual e marca a presena como falsa
                CategoriaAtributoTalento objCategoriaAtributoTalento = (CategoriaAtributoTalento) itrCategoriaAtributosTalento.next();
                boolean blnPresente = false;
                boolean blnPreenchido = false;

                // Verifica o nome do atributo de talento
                String strNomeCampo;
                if (objCategoriaAtributoTalento.getApelido() == null || "".equals(objCategoriaAtributoTalento.getApelido()))
                {
                    strNomeCampo = objCategoriaAtributoTalento.getAtributoTalento().getNome();
                }
                else
                {
                    strNomeCampo = objCategoriaAtributoTalento.getApelido();
                }

                // Verifica se o atributo est presente e foi preenchido
                Iterator itrAtributosTalentoValorados = lstAtributosTalentoValorados.iterator();
                while (itrAtributosTalentoValorados.hasNext())
                {
                    AtributoTalentoValorado objAtributoTalentoValorado = (AtributoTalentoValorado) itrAtributosTalentoValorados.next();
                    if (objCategoriaAtributoTalento.getIdentificador().equals(objAtributoTalentoValorado.getCategoriaAtributoTalento().getIdentificador()))
                    {
                        blnPresente = true;
                        if (objAtributoTalentoValorado.getValoracao() != null && !"".equals(objAtributoTalentoValorado.getValoracao()))
                        {
                            blnPreenchido = true;
                            break;
                        }
                    }
                }

                // Verifica se o atributo no est presente e  obrigatrio
                if (!blnPreenchido && "S".equals(objCategoriaAtributoTalento.getIndicativoObrigatoriedade()))
                {
                    // Se for obrigatrio, dispara um erro
                    throw new NegocioException("O atributo " + strNomeCampo + "  obrigatrio");
                }
                else if (!blnPresente)
                {
                    // Se no for obrigatrio, cria valorao vazia e insere na lista
                    AtributoTalentoValorado objAtributoTalentoValorado = new AtributoTalentoValorado();
                    objAtributoTalentoValorado.setCategoriaAtributoTalento(objCategoriaAtributoTalento);
                    objAtributoTalentoValorado.setValoracao("");
                    lstAtributosTalentoValorados.add(objAtributoTalentoValorado);
                }
            }

        }
        catch (Exception daoe)
        {
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    private void incluirValoracoes(Talento objTalento, List lstAtributosTalentoValorados) throws CDException
    {
        // Declaraes
        CategoriaAtributoTalentoDAO objCategoriaAtributoTalentoDAO = new CategoriaAtributoTalentoDAO(obterNomeConexao());
        AtributoTalentoOpcaoDAO objAtributoTalentoOpcaoDAO = new AtributoTalentoOpcaoDAO(obterNomeConexao());
        AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());

        try
        {
            DAO.iniciarTransacao(obterNomeConexao());

            // Inclui valoraes
            Iterator itrAtributosTalentoValorados = lstAtributosTalentoValorados.iterator();
            while (itrAtributosTalentoValorados.hasNext())
            {
                // Obtm valorao
                AtributoTalentoValorado objAtributoTalentoValorado = (AtributoTalentoValorado) itrAtributosTalentoValorados.next();

                // Se no foi preenchido o atributo em questo, ignorar este e pegar o prximo
                if (objAtributoTalentoValorado.getValoracao() == null || "".equals(objAtributoTalentoValorado.getValoracao()))
                {
                    continue;
                }

                // Seta talento
                objAtributoTalentoValorado.setTalento(objTalento);

                // Obtm categoria/atributo relacionado 
                CategoriaAtributoTalento objCategoriaAtributoTalento = (CategoriaAtributoTalento) objCategoriaAtributoTalentoDAO.obterPelaChaveComAtributoTalentoETipoHtml(String.valueOf(objAtributoTalentoValorado.getCategoriaAtributoTalento().getIdentificador()));
                objAtributoTalentoValorado.setCategoriaAtributoTalento(objCategoriaAtributoTalento);

                // Verifica o nome do atributo de talento
                String strNomeCampo;
                if (objCategoriaAtributoTalento.getApelido() == null || "".equals(objCategoriaAtributoTalento.getApelido()))
                {
                    strNomeCampo = objCategoriaAtributoTalento.getAtributoTalento().getNome();
                }
                else
                {
                    strNomeCampo = objCategoriaAtributoTalento.getApelido();
                }

                //Trata atributo virtual
                if (GerenciadorAtributoVirtual.isAtributoVirtual(objCategoriaAtributoTalento.getAtributoTalento().getNome()))
                {
                    //se for um atributo virtual j tem que ir direto para incluso
                }
                // Verifica se a categoria/atributo recebido tem opes, e se tiver, cria o objeto AtributoTalentoOpcao
                // e o relaciona, se no tiver, valida o tipo de dado
                else if ("U".equals(objCategoriaAtributoTalento.getAtributoTalento().getTipoHTML().getMultiplicidade())
                        || "M".equals(objCategoriaAtributoTalento.getAtributoTalento().getTipoHTML().getMultiplicidade()))
                {
                    AtributoTalentoOpcao objAtributoTalentoOpcao = new AtributoTalentoOpcao();
                    objAtributoTalentoOpcao.setIdentificador(Integer.valueOf(objAtributoTalentoValorado.getValoracao()));
                    objAtributoTalentoOpcao = (AtributoTalentoOpcao) objAtributoTalentoOpcaoDAO.obterPelaChave(String.valueOf(objAtributoTalentoOpcao.getIdentificador()));
                    objAtributoTalentoValorado.setAtributoTalentoOpcao(objAtributoTalentoOpcao);
                    objAtributoTalentoValorado.setValoracao(objAtributoTalentoOpcao.getDescricao());
                }
                else
                {
                    // Valida o tipo de dado
                    if (objCategoriaAtributoTalento.getAtributoTalento().getMascara() != null
                            && !"".equals(objCategoriaAtributoTalento.getAtributoTalento().getMascara()))
                    {
                        if (!ExpressaoRegular.validar(objCategoriaAtributoTalento.getAtributoTalento().getMascara(), objAtributoTalentoValorado.getValoracao()))
                        {
                            throw new NegocioException("O preenchimento do atributo "
                                    + strNomeCampo
                                    + " no segue o padro esperado ("
                                    + objCategoriaAtributoTalento.getAtributoTalento().getMascara()
                                    + ").");
                        }
                    }
                    else
                    {
                        if ("N".equals(objCategoriaAtributoTalento.getAtributoTalento().getTipoDado()))
                        {
                            if (!Numero.validarNumero(objAtributoTalentoValorado.getValoracao()))
                            {
                                throw new NegocioException("O atributo " + strNomeCampo + " deve ser um nmero vlido.");
                            }
                        }
                        else if ("D".equals(objCategoriaAtributoTalento.getAtributoTalento().getTipoDado()))
                        {
                            if (!DataNova.validarData(objAtributoTalentoValorado.getValoracao()))
                            {
                                throw new NegocioException("O atributo " + strNomeCampo + " deve ser uma data vlida");
                            }
                        }
                        // Se for "T", no existe nenhuma validao a ser feita.
                    }
                }

                // Inclui valorao
                objAtributoTalentoValoradoDAO.incluir(objAtributoTalentoValorado);

            }
            DAO.realizarTransacao(obterNomeConexao());
        }
        catch (Exception daoe)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Altera um registro
     *
     * @param objTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void alterar(Talento objTalento, List lstAtributosTalentoValorados) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e altera registro
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());
        try
        {
            // Verifica as valoraes obrigatrios
            verificarValoracoesObrigatorias(objTalento.getCategoriaTalento(), lstAtributosTalentoValorados);

            // Inicia transao
            DAO.iniciarTransacao(obterNomeConexao());

            // Preenche dados de talento
            objTalento.setDataLancamento(DataNova.converter(DataHoraBDFacade.obterTimeStampBD()));

            // Exclui valoraes anteriores
            objAtributoTalentoValoradoDAO.excluirPorTalento(objTalento);

            // Reinclui valoraes
            incluirValoracoes(objTalento, lstAtributosTalentoValorados);

            // Altera
            objTalentoDAO.alterar(objTalento);

            // Realiza transao
            DAO.realizarTransacao(obterNomeConexao());
        }
        catch (Exception daoe)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }

    /**
     * Exclui um registro
     *
     * @param objTalento POJO representando o objeto a ser includo
     *
     * @throws CDException se ocorrer algum erro relacionado ao negcio
     *
     */
    public void excluir(Talento objTalento) throws CDException
    {
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no metodo");
        }

        // Instancia DAO e inclui registro
        AtributoTalentoValoradoDAO objAtributoTalentoValoradoDAO = new AtributoTalentoValoradoDAO(obterNomeConexao());
        TalentoDAO objTalentoDAO = new TalentoDAO(obterNomeConexao());
        try
        {
            // Inicia transao
            DAO.iniciarTransacao(obterNomeConexao());

            // Exclui valoraes
            objAtributoTalentoValoradoDAO.excluirPorTalento(objTalento);

            // Exclui
            objTalentoDAO.excluir(objTalento);

            // Realiza transao
            DAO.realizarTransacao(obterNomeConexao());
        }
        catch (Exception daoe)
        {
            DAO.desfazerTransacao(obterNomeConexao());
            CDException.dispararExcecao(daoe);
        }
        finally
        {
            DAO.desconectar(obterNomeConexao());
        }
    }
}
