/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.br.BrazilianAnalyzer;
import org.apache.lucene.util.Version;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.exception.NegocioException;
import sigesp.comum.util.biblioteca.Arquivo;
import sigesp.comum.util.exception.ArquivoException;

public class ConfiguracaoDaIndexacaoTextual
{
    private static Log LOGGER = LogFactory.getLog(ConfiguracaoDaIndexacaoTextual.class.getName());

    protected static final String SUBDIRETORIO_COM_INDICES_TEXTUAIS = "indices";

    private static final String ARQUIVO_DE_PROPRIEDADES_DA_INDEXACAO_TEXTUAL = "indexacao.properties";
    private static final String NOME_PROPRIEDADE_DATA_ULTIMA_INDEXACAO_MANUAL = "data.ultima.indexacao.manual";
    private static final String NOME_PROPRIEDADE_DATA_INICIO_INDEXACAO_MANUAL = "data.inicio.indexacao.manual";
    private static final String NOME_PROPRIEDADE_USUARIO_INICIO_INDEXACAO_MANUAL = "usuario.inicio.indexacao.manual";
    private static final String NOME_PROPRIEDADE_INSTANCIA_INICIO_INDEXACAO_MANUAL = "instancia.inicio.indexacao.manual";
    private static final String NOME_PROPRIEDADE_QUANTIDADE_PROCESSADA_INDEXACAO_MANUAL = "quantidade.processada.indexacao.manual";
    private static final String NOME_PROPRIEDADE_QUANTIDADE_TOTAL_INDEXACAO_MANUAL = "quantidade.total.indexacao.manual";

    private static Version versaoImplementacao = Version.LUCENE_36;
    private static Analyzer analisadorSintatico = new BrazilianAnalyzer(versaoImplementacao);

    private static ConfiguracaoDaIndexacaoTextual instance = null;

    private String caminhoRaizDosDadosDaIndexacaoTextual = null; //indica o caminho para o arquivo de indice  

    private ConfiguracaoDaIndexacaoTextual(String caminhoRaizDosDadosDaIndexacaoTextual)
    {
        this.caminhoRaizDosDadosDaIndexacaoTextual = caminhoRaizDosDadosDaIndexacaoTextual;
    }

    public static ConfiguracaoDaIndexacaoTextual getInstance() throws CDException
    {
        return retornarOuCriarInstancia();
    }

    private static ConfiguracaoDaIndexacaoTextual retornarOuCriarInstancia() throws CDException
    {
        if (instance == null)
        {
            synchronized (ConfiguracaoDaIndexacaoTextual.class)
            {
                if (instance == null)
                {
                    ConfiguracaoDaIndexacaoTextual instanciaCriada = criarInstancia();
                    inicializarArquivoComDadosDaIndexacaoManual(instanciaCriada);
                    inicializarDiretorioDosIndicesTextuais(instanciaCriada);

                    instance = instanciaCriada;
                }
            }
        }

        return instance;
    }

    private static void inicializarDiretorioDosIndicesTextuais(ConfiguracaoDaIndexacaoTextual instancia) throws CDException
    {
        Arquivo.garantirDiretorioPara(instancia.caminhoRaizDosDadosDaIndexacaoTextual + SUBDIRETORIO_COM_INDICES_TEXTUAIS + File.separatorChar + ".");
    }

    private static void inicializarArquivoComDadosDaIndexacaoManual(ConfiguracaoDaIndexacaoTextual instancia) throws CDException
    {
        try
        {
            criarArquivoSeNaoExistir(instancia.caminhoRaizDosDadosDaIndexacaoTextual + ARQUIVO_DE_PROPRIEDADES_DA_INDEXACAO_TEXTUAL);
        }
        catch (IOException e)
        {
            throw new ArquivoException(e.getMessage());
        }
    }

    private static ConfiguracaoDaIndexacaoTextual criarInstancia() throws CDException
    {
        String caminhoRaizDosDadosDaIndexacaoTextual = CaminhoParaArquivosBancoTalentos.paraIndicesTextuais().getCaminhoRaiz();

        File file = null;
        if (caminhoRaizDosDadosDaIndexacaoTextual != null)
        {
            file = new File(caminhoRaizDosDadosDaIndexacaoTextual);
        }

        if (file == null)
        {
            throw new CDException("Diretrio especificado para os ndices textuais '" + caminhoRaizDosDadosDaIndexacaoTextual + "' no est acessvel!");
        }

        if (!file.exists())
        {
            file.mkdirs();
        }

        if (!file.exists() || !file.isDirectory() || !file.canWrite())
        {
            throw new CDException("Diretrio especificado para os ndices textuais '" + caminhoRaizDosDadosDaIndexacaoTextual + "' no est acessvel!");
        }

        String caminhoAbsoluto = file.getAbsolutePath() + File.separatorChar;
        LOGGER.info("Caminho utilizado para os indices textuais: " + caminhoAbsoluto);

        return new ConfiguracaoDaIndexacaoTextual(caminhoAbsoluto);
    }

    private static Properties lerArquivoDePropriedades() throws IOException, NegocioException
    {
        Properties propriedades = new Properties();
        InputStream input = null;

        try
        {
            // Caminho relativo
            String caminhoCompletoArquivoIndexacaoManual = instance.caminhoRaizDosDadosDaIndexacaoTextual + ARQUIVO_DE_PROPRIEDADES_DA_INDEXACAO_TEXTUAL;
            File arquivo = new File(caminhoCompletoArquivoIndexacaoManual);
            if (arquivo.exists())
            {
                input = new FileInputStream(arquivo);
            }

            if (input != null)
            {
                propriedades.load(input);
            }
            else
            {
                throw new NegocioException("O arquivo '" + caminhoCompletoArquivoIndexacaoManual + "' no foi localizado.");
            }
        }
        finally
        {
            if (input != null)
            {
                input.close();
            }
        }

        return (propriedades);
    }

    private static void criarArquivoSeNaoExistir(String arquivo) throws IOException
    {
        // Criar o arquivo vazio...
        File yourFile = new File(arquivo);
        if (!yourFile.exists())
        {
            yourFile.createNewFile();
        }
    }

    private static void gravarArquivoConfiguracao(Properties propriedades) throws IOException
    {
        FileOutputStream fos = null;
        try
        {
            fos = new FileOutputStream(instance.caminhoRaizDosDadosDaIndexacaoTextual + ARQUIVO_DE_PROPRIEDADES_DA_INDEXACAO_TEXTUAL, false);
            propriedades.store(fos, null);
        }
        finally
        {
            if (fos != null)
            {
                fos.close();
            }
        }
    }

    public String getDataUltimaIndexacaoManual() throws CDException
    {
        String data = null;
        try
        {
            data = lerArquivoDePropriedades().getProperty(NOME_PROPRIEDADE_DATA_ULTIMA_INDEXACAO_MANUAL);
        }
        catch (IOException e)
        {
            CDException.dispararExcecao(e);
        }
        return data;
    }

    public DadosIndexacaoManual lerDadosIndexacaoManual() throws CDException
    {
        DadosIndexacaoManual dadosInicioIndexacaoManual = new DadosIndexacaoManual();
        try
        {
            Properties propriedades = lerArquivoDePropriedades();
            dadosInicioIndexacaoManual.setDataInicio(propriedades.getProperty(NOME_PROPRIEDADE_DATA_INICIO_INDEXACAO_MANUAL));
            dadosInicioIndexacaoManual.setInstancia(propriedades.getProperty(NOME_PROPRIEDADE_INSTANCIA_INICIO_INDEXACAO_MANUAL));
            dadosInicioIndexacaoManual.setLoginUsuario(propriedades.getProperty(NOME_PROPRIEDADE_USUARIO_INICIO_INDEXACAO_MANUAL));
            dadosInicioIndexacaoManual.setQuantidadeProcessada(propriedades.getProperty(NOME_PROPRIEDADE_QUANTIDADE_PROCESSADA_INDEXACAO_MANUAL));
            dadosInicioIndexacaoManual.setQuantidadeTotal(propriedades.getProperty(NOME_PROPRIEDADE_QUANTIDADE_TOTAL_INDEXACAO_MANUAL));
        }
        catch (IOException e)
        {
            CDException.dispararExcecao(e);
        }
        return dadosInicioIndexacaoManual;
    }

    public void registrarDadosIndexacaoManual(DadosIndexacaoManual dadosInicioIndexacaoManual) throws CDException
    {
        try
        {
            Properties propriedades = lerArquivoDePropriedades();
            propriedades.setProperty(NOME_PROPRIEDADE_DATA_INICIO_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getDataInicio());
            propriedades.setProperty(NOME_PROPRIEDADE_INSTANCIA_INICIO_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getInstancia());
            propriedades.setProperty(NOME_PROPRIEDADE_USUARIO_INICIO_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getLoginUsuario());
            propriedades.setProperty(NOME_PROPRIEDADE_QUANTIDADE_PROCESSADA_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getQuantidadeProcessada());
            propriedades.setProperty(NOME_PROPRIEDADE_QUANTIDADE_TOTAL_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getQuantidadeTotal());
            gravarArquivoConfiguracao(propriedades);
        }
        catch (IOException e)
        {
            CDException.dispararExcecao(e);
        }
    }

    public void setConclusaoIndexacaoManual(String dataConclusao) throws CDException
    {
        try
        {
            Properties propriedades = lerArquivoDePropriedades();
            propriedades.setProperty(NOME_PROPRIEDADE_DATA_ULTIMA_INDEXACAO_MANUAL, dataConclusao);

            DadosIndexacaoManual dadosInicioIndexacaoManual = new DadosIndexacaoManual();
            propriedades.setProperty(NOME_PROPRIEDADE_DATA_INICIO_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getDataInicio());
            propriedades.setProperty(NOME_PROPRIEDADE_INSTANCIA_INICIO_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getInstancia());
            propriedades.setProperty(NOME_PROPRIEDADE_USUARIO_INICIO_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getLoginUsuario());
            propriedades.setProperty(NOME_PROPRIEDADE_QUANTIDADE_PROCESSADA_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getQuantidadeProcessada());
            propriedades.setProperty(NOME_PROPRIEDADE_QUANTIDADE_TOTAL_INDEXACAO_MANUAL, dadosInicioIndexacaoManual.getQuantidadeTotal());
            gravarArquivoConfiguracao(propriedades);
        }
        catch (IOException e)
        {
            CDException.dispararExcecao(e);
        }
    }

    public boolean emExecucaoManual() throws CDException
    {
        DadosIndexacaoManual dadosInicioIndexacaoManual = lerDadosIndexacaoManual();

        return dadosInicioIndexacaoManual.emExecucaoManual();
    }

    public static Analyzer getAnalisadorSintatico()
    {
        return analisadorSintatico;
    }

    public static Version getVersaoImplentacao()
    {
        return versaoImplementacao;
    }

    public String getCaminhoParaIndicesTextuais()
    {
        return caminhoRaizDosDadosDaIndexacaoTextual + SUBDIRETORIO_COM_INDICES_TEXTUAIS + File.separatorChar;
    }
}