/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.BooleanClause.Occur;
import org.apache.lucene.search.BooleanQuery;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.bancotalentos.consultatalento.ResultadoConsultaTalento;
import br.gov.camara.negocio.bancotalentos.consultatalento.ResultadoConsultaTalentoFacade;
import br.gov.camara.negocio.bancotalentos.facade.FiltroConsultaFacade;
import br.gov.camara.negocio.bancotalentos.facade.TalentoFacade;
import br.gov.camara.negocio.bancotalentos.pojo.AtributoTalentoValorado;
import br.gov.camara.negocio.bancotalentos.pojo.Talento;
import br.gov.camara.negocio.comum.facade.DataHoraBDFacade;
import br.gov.camara.seguranca.UsuarioAutenticado;

/**
 *
 * Num processamento em batch o IndexWriter podera ser usado varias vezes antes de ser fechado. 
 * A obrigacao de fechar  de quem usa a classe IndexaTalento justamente por isso.
 *   
 *
 * @author suporte-urms
 */
public class ServicoDeIndexacaoDeTalento
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(ServicoDeIndexacaoDeTalento.class.getName());

    private static final String DOCUMENTO_PESSOA_IDENTIFICADOR = "identificador";
    private static final String DOCUMENTO_PESSOA_ULTIMA_INDEXACAO = "ultimaIndexacao";
    private static final String DOCUMENTO_PESSOA_GRUPO = "grupo";
    private static final String DOCUMENTO_PESSOA_TALENTOS = "talentos";

    private ServicoDePesquisaTextual servicoDePesquisaTextual = null;

    public ServicoDeIndexacaoDeTalento()
    {
        servicoDePesquisaTextual = new ServicoDePesquisaTextual(DOCUMENTO_PESSOA_TALENTOS);
    }

    /**
     * Indexa os talentos da pessoa informada.
     * @param pessoa Os dados obrigatrios so o identificador e o grupo da pessoa
     * @throws CDException
     */
    public void indexarTalentos(int identificadorPessoa, int codigoGrupo, String nomeConexao) throws CDException
    {
        String descricaoTalentos = concatenarTextosDosTalentos(identificadorPessoa, nomeConexao);

        indexarTalentosDeUmaPessoa(identificadorPessoa, codigoGrupo, descricaoTalentos);
    }

    private String concatenarTextosDosTalentos(int identificadorPessoa, String nomeConexao) throws CDException
    {
        TalentoFacade talentoFacade = new TalentoFacade(nomeConexao);
        List talentos = talentoFacade.obterPorPessoaPorPagina(identificadorPessoa, 1, 0); // Qdo max.pgina = 0 => recuperar TODOS 
        //varre os talentos inserindo no indice do Lucene
        Iterator itTalentos = talentos.iterator();

        //Vai acomodar uma string unica com todos os talentos de um usuario
        StringBuilder descricao_talento = new StringBuilder();

        while (itTalentos.hasNext())
        {
            Talento t = (Talento) itTalentos.next();

            //obtem as valoracoes de um talento
            List valoracoes = talentoFacade.obterValoracoesPorTalentoComAtributoTalentoEAtributoTalentoOpcoes(t);
            Iterator itValoracoes = valoracoes.iterator();

            while (itValoracoes.hasNext())
            {
                AtributoTalentoValorado v = (AtributoTalentoValorado) itValoracoes.next();

                String tipoDeDado = v.getCategoriaAtributoTalento().getAtributoTalento().getTipoDado();
                // S indexa atributos do tipo "T"exto (ignora aqueles do tipo "D"ata e "N"umero)
                if (tipoDeDado == null || "T".equals(tipoDeDado))
                {
                    descricao_talento.append(" " + v.getValoracao());
                }
            }

            valoracoes = null;
        } //quando termina esse while j varreu todos os talentos de um usuario em particular

        talentos = null;
        return descricao_talento.toString();
    }

    /*
     * Acrescenta um talento ao documento que representa os talentos de um usuario.
     * 
     * Nenhum metodo precisou ser serializado j que a classe IndexWriter  ThreadSafe
     */
    private void indexarTalentosDeUmaPessoa(int identificadorPessoa, int codigoGrupo, String talento) throws CDException
    {
        ServicoDeIndexacaoTextual servicoDeIndexacaoTextual = null;
        try
        {
            String sIdentificadorPessoa = Integer.toString(identificadorPessoa);
            String sCodigoGrupo = Integer.toString(codigoGrupo);

            Term termoIdentificacaoPessoa = criarTermoIdentificacaoPessoa(sIdentificadorPessoa);
            Document documentoTalento = criarDocumento(talento, sIdentificadorPessoa, sCodigoGrupo);

            servicoDeIndexacaoTextual = new ServicoDeIndexacaoTextual();
            servicoDeIndexacaoTextual.indexarDocumento(termoIdentificacaoPessoa, documentoTalento);
            servicoDeIndexacaoTextual.commit();
        }
        catch (IOException ioe)
        {
            CDException.dispararExcecao(ioe);
        }
        finally
        {
            if (servicoDeIndexacaoTextual != null)
            {
                try
                {
                    servicoDeIndexacaoTextual.fechar();
                }
                catch (IOException e)
                {
                    // Pacincia...
                }
            }
        }
    }

    private Term criarTermoIdentificacaoPessoa(String sIdentificadorPessoa)
    {
        return new Term(DOCUMENTO_PESSOA_IDENTIFICADOR, sIdentificadorPessoa);
    }

    private Document criarDocumento(String talento, String sIdentificadorPessoa, String sCodigoGrupo) throws CDException
    {
        Document doc = new Document();

        //Insere campo chave para recuperacao de um documento especifico 
        //VEja texto explicativo:

        //How do I delete documents from the index?
        //IndexWriter allows you to delete by Term or by Query. The deletes are buffered and then periodically flushed to the index, and made visible once commit() or close() is called.
        //IndexReader can also delete documents, by Term or document number, but you must close any open IndexWriter before using IndexReader to make changes (and, vice/versa). IndexReader also buffers the deletions and does not write changes to the index until close() is called, but if you use that same IndexReader for searching, the buffered deletions will immediately take effect. Unlike IndexWriter's delete methods, IndexReader's methods return the number of documents that were deleted.
        //Generally it's best to use IndexWriter for deletions, unless 1) you must delete by document number, 2) you need your searches to immediately reflect the deletions or 3) you must know how many documents were deleted for a given deleteDocuments invocation.
        //If you must delete by document number but would otherwise like to use IndexWriter, one common approach is to make a primary key field, that holds a unique ID string for each document. Then you can delete a single document by creating the Term containing the ID, and passing that to IndexWriter's deleteDocuments(Term) method.
        //Once a document is deleted it will not appear in TermDocs nor TermPositions enumerations, nor any search results. Attempts to load the document will result in an exception. The presence of this document may still be reflected in the docFreq statistics, and thus alter search scores, though this will be corrected eventually as segments containing deletions are merged.      

        //E esse:
        //What happens when you IndexWriter.add() a document that is already in the index? Does it overwrite the previous document?
        //No, there will be multiple copies of the same document in the index.      

        //Insere campo para recuperacao do documento. Este ser a matricula jah que em ultima instancia esta se buscando pelas pessoas 
        //cujos curriculos batem com a busca
        doc.add(new Field(DOCUMENTO_PESSOA_IDENTIFICADOR, sIdentificadorPessoa, Field.Store.YES, Field.Index.NOT_ANALYZED));//nao eh para indexar por esse valor ja que nao fara parte da busca do usuario
        doc.add(new Field(DOCUMENTO_PESSOA_ULTIMA_INDEXACAO, Long.toString(DataHoraBDFacade.obterTimeStampBD().getTime()), Field.Store.YES, Field.Index.NO));//nao eh para indexar por esse valor ja que nao fara parte da busca do usuario 
        doc.add(new Field(DOCUMENTO_PESSOA_GRUPO, sCodigoGrupo, Field.Store.YES, Field.Index.NOT_ANALYZED));//precisa indexar, mas sem analisar, para filtrar por grupo

        //ANALYZED - Index the tokens produced by running the field's value through an Analyzer.
        //ANALYZED_NO_NORMS - Expert: Index the tokens produced by running the field's value through an Analyzer, and also separately disable the storing of norms.
        //NO - Do not index the field value.
        //NOT_ANALYZED - Index the field's value without using an Analyzer, so it can be searched.
        //NOT_ANALYZED_NO_NORMS - Expert: Index the field's value without an Analyzer, and also disable the storing of norms.          

        doc.add(new Field(DOCUMENTO_PESSOA_TALENTOS, talento, Field.Store.YES, Field.Index.ANALYZED));
        return doc;
    }

    public void limparIndices() throws CDException
    {
        ServicoDeIndexacaoTextual servicoDeIndexacaoTextual = null;
        try
        {
            servicoDeIndexacaoTextual = new ServicoDeIndexacaoTextual();
            if (servicoDeIndexacaoTextual.isBloqueado())
            {
                log.info("Diretrio dos ndices bloqueado. Tentando desbloquar...");
                try
                {
                    servicoDeIndexacaoTextual.forcarRemocaoBloqueio();
                }
                catch (Exception e)
                {
                    log.error("No foi possvel desbloquar o diretrio dos ndices. Verificar eventuais problemas...");
                }
            }
            servicoDeIndexacaoTextual.zerarIndicesDaIndexacaoTextual();
        }
        catch (IOException ioe)
        {
            CDException.dispararExcecao(ioe);
        }
        finally
        {
            if (servicoDeIndexacaoTextual != null)
            {
                try
                {
                    servicoDeIndexacaoTextual.fechar();
                }
                catch (IOException e)
                {
                    // Pacincia...
                }
            }
        }
    }

    public List<ResultadoConsultaTalento> executarPesquisa(String queryString, UsuarioAutenticado objPessoa, int maximoRegistros, StringBuilder stbTotalRetornado, boolean blnFiltroGrupo, String nomeConexao) throws IOException, ParseException, CDException
    {
        ResultadoConsultaTalentoFacade consultaFacade = new ResultadoConsultaTalentoFacade(nomeConexao);

        List<ResultadoConsultaTalento> resultadoConsulta = null;

        BooleanQuery query = servicoDePesquisaTextual.criarCriterioPesquisaTextual(queryString, Occur.MUST);
        adicionarFiltroPorGrupo(objPessoa, blnFiltroGrupo, nomeConexao, query);

        int totalPessoasRetornado = servicoDePesquisaTextual.numeroTotalResultado(query);

        stbTotalRetornado.setLength(0);
        stbTotalRetornado.append(totalPessoasRetornado);

        if (totalPessoasRetornado <= maximoRegistros && totalPessoasRetornado > 0)
        {
            List<Integer> lstIdentificadores = new ArrayList<Integer>();
            List<Document> documentosRecuperados = servicoDePesquisaTextual.recuperarDocumentosAteOLimite(query, maximoRegistros);

            //obtem os usuarios com as respectivas matriculas
            for (Document documento : documentosRecuperados)
            {
                String identificador = documento.get(DOCUMENTO_PESSOA_IDENTIFICADOR);

                lstIdentificadores.add(Integer.valueOf(identificador));
            }

            resultadoConsulta = consultaFacade.recuperarResultadoConsultaTalentoDasMatriculas(lstIdentificadores);
        }
        else
        {
            resultadoConsulta = new ArrayList<ResultadoConsultaTalento>();
        }

        return resultadoConsulta;
    }

    private void adicionarFiltroPorGrupo(UsuarioAutenticado objPessoa, boolean blnFiltroGrupo, String nomeConexao, BooleanQuery query) throws CDException, ParseException, IOException
    {
        // Verifica se h filtro por grupo
        if (blnFiltroGrupo)
        {
            FiltroConsultaFacade filtroConsultaFacade = new FiltroConsultaFacade(nomeConexao);

            String gruposPermitidos = filtroConsultaFacade.obterConjuntoCodigosGruposPermitidos(objPessoa.obterIdentificador().toString(), " OR ");
            if (gruposPermitidos != null)
            {
                servicoDePesquisaTextual.adicionarCriterioPesquisaTextual(DOCUMENTO_PESSOA_GRUPO + ":(" + gruposPermitidos + ")", Occur.MUST, query);
            }
        }
    }

    public void indexarTalentosEmLoteComAtualizacaoDoAndamento(List<Integer[]> pontosEGruposDasPessoas, String nomeConexao, DadosIndexacaoManual dadosInicioIndexacaoManual) throws CDException
    {
        ServicoDeIndexacaoTextual servicoDeIndexacaoTextual = null;
        try
        {
            servicoDeIndexacaoTextual = new ServicoDeIndexacaoTextual();
            int processados = 0;
            try
            {
                processados = Integer.valueOf(dadosInicioIndexacaoManual.getQuantidadeProcessada());
            }
            catch (NumberFormatException e)
            {
                // Pacincia. Deve ser pela quantidade processada estar nula ou vazia, o que resulta em 0
            }

            for (Integer[] dadosPessoa : pontosEGruposDasPessoas)
            {
                String descricaoTalentos = concatenarTextosDosTalentos(dadosPessoa[0], nomeConexao);

                String sIdentificadorPessoa = Integer.toString(dadosPessoa[0]);
                String sCodigoGrupo = Integer.toString(dadosPessoa[1]);

                Term termoIdentificacaoPessoa = criarTermoIdentificacaoPessoa(sIdentificadorPessoa);
                Document documentoTalento = criarDocumento(descricaoTalentos, sIdentificadorPessoa, sCodigoGrupo);

                servicoDeIndexacaoTextual.indexarDocumento(termoIdentificacaoPessoa, documentoTalento);

                // Atualiza o "andamento" a cada MAXIMO_PESSOAS_PROCESSADAS_NUM_LOTE registros
                dadosInicioIndexacaoManual.setQuantidadeProcessada(++processados);
                try
                {
                    ConfiguracaoDaIndexacaoTextual.getInstance().registrarDadosIndexacaoManual(dadosInicioIndexacaoManual);
                }
                catch (CDException e)
                {
                    // Paciencia...
                }

            }

            servicoDeIndexacaoTextual.commit();
        }
        catch (IOException e)
        {
            CDException.dispararExcecao(e);
        }
        finally
        {
            if (servicoDeIndexacaoTextual != null)
            {
                try
                {
                    servicoDeIndexacaoTextual.fechar();
                }
                catch (IOException e)
                {
                    // Pacincia...
                }
            }
        }
    }

}