/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.util;

import java.io.File;
import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.index.IndexWriterConfig.OpenMode;
import org.apache.lucene.index.LogDocMergePolicy;
import org.apache.lucene.index.Term;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import br.gov.camara.exception.CDException;

class ServicoDeIndexacaoTextual
{
    private static Log log = LogFactory.getLog(ServicoDeIndexacaoTextual.class.getName());

    private static ConfiguracaoDaIndexacaoTextual configuracaoDaIndexacaoTextual = null;

    private IndexWriter indexWriter = null;
    private Directory diretorio = null;

    ServicoDeIndexacaoTextual() throws CDException
    {
        configuracaoDaIndexacaoTextual = ConfiguracaoDaIndexacaoTextual.getInstance();
    }

    private Directory getDirectory() throws IOException
    {
        if (diretorio == null)
        {
            diretorio = FSDirectory.open(new File(configuracaoDaIndexacaoTextual.getCaminhoParaIndicesTextuais()));
        }

        return diretorio;
    }

    private IndexWriter getIndexWriter() throws IOException
    {
        if (indexWriter == null)
        {
            indexWriter = new IndexWriter(getDirectory(), criarConfiguracaoDoIndexWriter());
        }

        return indexWriter;
    }

    void indexarDocumento(Term termo, Document documento) throws IOException
    {
        // Updates a document by first deleting the document(s) containing term and then adding the new document
        getIndexWriter().updateDocument(termo, documento);
    }

    void commit() throws IOException
    {
        getIndexWriter().commit();
    }

    void removerDocumento(Term termo) throws IOException
    {
        getIndexWriter().deleteDocuments(termo);
        commit();
    }

    //O IndexWriterConfig deve ser criado unicamente para cada instancia de IndexWriter
    private static IndexWriterConfig criarConfiguracaoDoIndexWriter()
    {
        //cria uma configuracao preparada para criar ou estender um indice
        IndexWriterConfig configuracao = new IndexWriterConfig(ConfiguracaoDaIndexacaoTextual.getVersaoImplentacao(), ConfiguracaoDaIndexacaoTextual.getAnalisadorSintatico()).setOpenMode(OpenMode.CREATE_OR_APPEND);

        //dependendo da politica de indexacao podem ser abertos muitos arquivos simultaneamente
        //isso pode ocorrer em erro de runtime de FileNotFoundException
        //para minimizar a abertura desses arquivos, deve ser setado Use Compound File para true
        //um efeito negativo disso  que a indexao fica mais lenta
        LogDocMergePolicy logMergePolicy = new LogDocMergePolicy();
        logMergePolicy.setUseCompoundFile(true);
        configuracao.setMergePolicy(logMergePolicy);

        return configuracao;
    }

    void zerarIndicesDaIndexacaoTextual() throws IOException
    {
        getIndexWriter().deleteAll();
        getIndexWriter().commit();
    }

    void fechar() throws IOException
    {
        if (indexWriter != null)
        {
            indexWriter.close();
        }

        if (diretorio != null)
        {
            diretorio.close();
        }
    }

    boolean isBloqueado() throws IOException
    {
        boolean bloqueado = IndexWriter.isLocked(getDirectory());
        return bloqueado;
    }

    void forcarRemocaoBloqueio() throws IOException
    {
        if (isBloqueado())
        {
            IndexWriter.unlock(getDirectory());
        }
    }

}