/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.bancotalentos.util;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.TotalHitCountCollector;
import org.apache.lucene.store.FSDirectory;

import br.gov.camara.exception.CDException;

public class ServicoDePesquisaTextual
{
    private static Log log = LogFactory.getLog(ServicoDePesquisaTextual.class.getName());

    private QueryParser parser = null;

    public ServicoDePesquisaTextual(String campoPadraoParaTermosDePesquisa)
    {
        parser = new QueryParser(ConfiguracaoDaIndexacaoTextual.getVersaoImplentacao(), campoPadraoParaTermosDePesquisa, ConfiguracaoDaIndexacaoTextual.getAnalisadorSintatico());
    }

    public Document recuperarDocumento(String campo, String conteudo) throws IOException, CDException
    {
        IndexSearcher searcher = null;
        try
        {
            searcher = inicializarPesquisa();

            TermQuery term = new TermQuery(new Term(campo, conteudo));
            TopDocs doc = searcher.search(term, 1);

            if (doc != null && doc.totalHits > 0 && doc.scoreDocs.length > 0)
            {
                return searcher.doc(doc.scoreDocs[0].doc);
            }
        }
        finally
        {
            finalizarPesquisa(searcher);
        }

        return null;
    }

    /*
     * Cria uma instncia de IndexSearcher.
     * ATENO: deve-se chamar o mtodo finalizarPesquisa aps o uso.
     */
    private IndexSearcher inicializarPesquisa() throws IOException, CDException
    {
        IndexSearcher searcher = new IndexSearcher(IndexReader.open(FSDirectory.open(new File(ConfiguracaoDaIndexacaoTextual.getInstance().getCaminhoParaIndicesTextuais()))));
        return searcher;
    }

    private void finalizarPesquisa(IndexSearcher searcher) throws IOException
    {
        if (searcher != null)
        {
            searcher.close();
        }
    }

    public BooleanQuery criarCriterioPesquisaTextual(String queryString, BooleanClause.Occur condicao) throws ParseException
    {
        Query query = parser.parse(queryString);

        BooleanQuery queryPesquisaTextual = new BooleanQuery();
        queryPesquisaTextual.add(query, condicao);
        return queryPesquisaTextual;
    }

    public BooleanQuery adicionarCriterioPesquisaTextual(String queryString, BooleanClause.Occur condicao, BooleanQuery queryPesquisaTextualExistente) throws ParseException
    {
        Query query = parser.parse(queryString);

        queryPesquisaTextualExistente.add(query, condicao);
        return queryPesquisaTextualExistente;
    }

    public int numeroTotalResultado(BooleanQuery query) throws IOException, CDException
    {
        IndexSearcher searcher = null;
        try
        {
            searcher = inicializarPesquisa();
            TotalHitCountCollector totalHitCountCollector = new TotalHitCountCollector();
            searcher.search(query, totalHitCountCollector);

            return totalHitCountCollector.getTotalHits();
        }
        finally
        {
            finalizarPesquisa(searcher);
        }
    }

    public List<Document> recuperarDocumentosAteOLimite(BooleanQuery query, int numeroMaximoDeResultado) throws IOException, CDException
    {
        IndexSearcher searcher = null;
        List<Document> resultado = null;
        try
        {
            searcher = inicializarPesquisa();
            TopDocs hits = searcher.search(query, numeroMaximoDeResultado);

            if (hits != null && hits.totalHits > 0 && hits.scoreDocs.length > 0)
            {
                resultado = new ArrayList<Document>(hits.scoreDocs.length);
                for (int i = 0; i < hits.scoreDocs.length; i++)
                {
                    resultado.add(searcher.doc(hits.scoreDocs[i].doc));
                }
            }
            else
            {
                resultado = new ArrayList<Document>(0);
            }
        }
        finally
        {
            finalizarPesquisa(searcher);
        }

        return resultado;

    }
}