/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio.exception;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Classe para recuperao de mensagens especficas para cada banco de dados
 */
public class Mensagem
{
    /**
    * Logger for this class
    */
    private static final Log log = LogFactory.getLog(Mensagem.class.getName());

    private static final String strCHAVE_DUPLICADA = "A informao que voc est tentando incluir j existe";
    private static final String strRESTRICAO_CHECAGEM = "O valor informado no  permitido";
    private static final String strREFERENCIA_INEXISTENTE = "A informao que voc est tentando incluir ou alterar est referenciando algo que no existe";
    private static final String strINTEGRIDADE_VIOLADA = "A informao que voc est tentando excluir est sendo referenciada em outra parte do sistema";
    private static final String strOBRIGATORIEDADE = "Voc est omitindo uma informao obrigatria";

    /**
     * Obtm a mensagem a ser exibida pelo usurio
     * 
     * @param strBanco Banco utilizado
     * @param intErro Cdigo do erro
     * @param strMensagem Mensagem gerada pelo banco
     * @return Mensagem a ser exibida
     */
    public static String obterMensagem(String strBanco, int intErro, String strMensagem)
    {
        String strMensagemRetorno = null;
        if (strBanco != null)
        {
            // Verifica qual o banco
            if (strBanco.toLowerCase().indexOf("oracle") > -1)
            {
                strMensagemRetorno = obterMensagemOracle(intErro, strMensagem);
            }
            else if (strBanco.toLowerCase().indexOf("ingres") > -1)
            {
                strMensagemRetorno = obterMensagemIngres(intErro, strMensagem);
            }
            else if (strBanco.toLowerCase().indexOf("mysql") > -1)
            {
                strMensagemRetorno = obterMensagemMySQL(intErro, strMensagem);
            }
        }

        if (strMensagemRetorno == null || "".equals(strMensagemRetorno))
        {
            strMensagemRetorno = "Erro desconhecido";
        }

        log.info(strMensagemRetorno + " -> " + strMensagem);

        return strMensagemRetorno;
    }

    private static String obterMensagemOracle(int intErro, String strMensagem)
    {
        // Declaraes
        String strRetorno = null;

        // Verifica erros conhecidos
        if (intErro == 1)
        {
            strRetorno = strCHAVE_DUPLICADA;
        }
        else if (intErro == 1400)
        {
            strRetorno = strOBRIGATORIEDADE;
        }
        else if (intErro == 2291)
        {
            strRetorno = strRESTRICAO_CHECAGEM;
        }
        else if (intErro == 2291)
        {
            strRetorno = strREFERENCIA_INEXISTENTE;
        }
        else if (intErro == 2292)
        {
            strRetorno = strINTEGRIDADE_VIOLADA;
        }
        else if (intErro == 20000)
        {
            strRetorno = strMensagem.substring(10, strMensagem.indexOf("ORA-", 10));
        }

        // Retorna
        return strRetorno;
    }

    private static String obterMensagemIngres(int intErro, String strMensagem)
    {
        // Declaraes
        String strRetorno = null;

        // Verifica erros conhecidos
        if (intErro == 1)
        {
            strRetorno = strMensagem.substring(32);
        }
        else if (intErro == 4500)
        {
            strRetorno = strCHAVE_DUPLICADA;
        }

        // Retorna
        return strRetorno;
    }

    private static String obterMensagemMySQL(int intErro, String strMensagem)
    {
        // Declaraes
        String strRetorno = null;

        // Verifica erros conhecidos
        if (strMensagem.toLowerCase().indexOf("foreign key constraint") > -1)
        {
            strRetorno = strINTEGRIDADE_VIOLADA;
        }

        // Retorna
        return strRetorno;
    }
}
