/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.servlet.timer;

import java.util.Map;
import java.util.Timer;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ProgramadorTarefas
{
    /**
     * Logger for this class
     */
    private static final Log log = LogFactory.getLog(ProgramadorTarefas.class);

    private boolean estadoExecucao = true;

    private static ProgramadorTarefas instanceProgramadorTarefas = null;
    private static Map<String, TarefaProgramavel> tarefasProgramadas = null;
    private static Map<String, Timer> poolTimers = null;

    private ProgramadorTarefas()
    {
        if (estadoExecucao)
        {
            if (tarefasProgramadas == null)
            {
                tarefasProgramadas = new ConcurrentHashMap<String, TarefaProgramavel>();
            }
            if (poolTimers == null)
            {
                poolTimers = new ConcurrentHashMap<String, Timer>();
            }
        }
    }

    public static ProgramadorTarefas getInstance()
    {
        if (instanceProgramadorTarefas == null)
        {
            synchronized (ProgramadorTarefas.class)
            {
                if (instanceProgramadorTarefas == null)
                {
                    instanceProgramadorTarefas = new ProgramadorTarefas();
                }
            }
        }
        return instanceProgramadorTarefas;
    }

    public void agendar(TarefaProgramavel tarefaProgramavel)
    {
        if (estadoExecucao)
        {
            agendar_Interno(tarefaProgramavel, tarefaProgramavel.getTempoEsperaParaPrimeiraExecucao());
        }
        else
        {
            log.warn("A tarefa '" + tarefaProgramavel.getId() + "' no foi agendada porque o ProgramadorTarefas est sendo ENCERRADO.");
        }

    }

    /**
     * Este mtodo reagenda uma tarefa, ou seja, ela:
     * 1) cancela a tarefa existente (se for o caso); e
     * 2) agenda uma nova instncia dessa tarefa. 
     * 
     * @param tarefaProgramavel
     */
    public void reagendarTarefa(TarefaProgramavel tarefaProgramavel)
    {
        if (estadoExecucao)
        {
            try
            {
                boolean podeReagendar = true;
                if (!tarefaProgramavel.isEstadoVirgem() || tarefaProgramavel.isEstadoAgendado())
                {
                    //  uma tarefa que j foi executada, cancelada ou j est agendada, portanto pegar uma nova instncia
                    cancelarTarefa(tarefaProgramavel);

                    if (tarefaProgramavel instanceof TarefaReprogramavel)
                    {
                        tarefaProgramavel = ((TarefaReprogramavel) tarefaProgramavel).getNovaInstancia();
                    }
                    else
                    {
                        log.error("ERRO: a tarefa " + tarefaProgramavel.getId() + " no  uma instncia da classe TarefaReprogramavel.");
                        podeReagendar = false;
                    }
                }

                if (podeReagendar)
                {
                    // Por fim, agendar a execuo da tarefa  
                    agendar_Interno(tarefaProgramavel, tarefaProgramavel.getTempoEsperaParaInicioExecucao());
                }
            }
            catch (Exception e)
            {
                log.error("Erro: " + e.getMessage());
            }
        }
        else
        {
            log.warn("A tarefa '" + tarefaProgramavel.getId() + "' no foi reagendada porque o ProgramadorTarefas est sendo ENCERRADO.");
        }
    }

    public void reagendarTarefa(String tarefaProgramavelId)
    {
        if (estadoExecucao)
        {
            try
            {
                TarefaProgramavel tarefaProgramavel = (TarefaProgramavel) tarefasProgramadas.get(tarefaProgramavelId);
                if (tarefaProgramavel != null)
                {
                    reagendarTarefa(tarefaProgramavel);
                }
                else
                {
                    log.error("Erro: A tarefa '" + tarefaProgramavelId + "' no foi encontrada na lista de tarefas programadas.");
                }
            }
            catch (Exception e)
            {
                log.error("Erro: " + e.getMessage());
            }
        }
        else
        {
            log.warn("A tarefa '" + tarefaProgramavelId + "' no foi reagendada porque o ProgramadorTarefas est sendo ENCERRADO.");
        }
    }

    /** 
     * Ateno, o mtodo chamador deve garantir a sincronia e o estado da execuo, visto que isso no  feito aqui.
     * @param tarefaProgramavel
     * @param proximaExecucaoEmMilisegundos
     */
    private void agendar_Interno(TarefaProgramavel tarefaProgramavel, long proximaExecucaoEmMilisegundos)
    {
        try
        {
            if (!tarefaProgramavel.isEstadoAgendado())
            {
                tarefaProgramavel.setEstadoAgendado();
                tarefasProgramadas.put(tarefaProgramavel.getId(), tarefaProgramavel);
                // Colocado um acrescimo para garantir a execuo
                getTimer(tarefaProgramavel).schedule(tarefaProgramavel, proximaExecucaoEmMilisegundos + 500);
            }
            else
            {
                log.error("A tarefa do tipo "
                        + tarefaProgramavel.getClass().getName()
                        + " identificada por "
                        + tarefaProgramavel.getId()
                        + " j est agendada.");
            }
        }
        catch (Exception e)
        {
            log.error("Erro: " + e.getMessage());
        }
    }

    private Timer getTimer(TarefaProgramavel tarefaProgramavel)
    {
        String nomeImplementacaoTarefaProgramavel = tarefaProgramavel.getClass().getName();
        Timer timerParaTarefaProgramavel = poolTimers.get(nomeImplementacaoTarefaProgramavel);
        if (timerParaTarefaProgramavel == null)
        {
            synchronized (poolTimers)
            {
                timerParaTarefaProgramavel = poolTimers.get(nomeImplementacaoTarefaProgramavel);
                if (timerParaTarefaProgramavel == null)
                {
                    timerParaTarefaProgramavel = new Timer("ProgramadorTarefasPara" + nomeImplementacaoTarefaProgramavel, true);
                    poolTimers.put(nomeImplementacaoTarefaProgramavel, timerParaTarefaProgramavel);
                }
            }
        }

        return timerParaTarefaProgramavel;
    }

    public TarefaProgramavel getTarefaAgendada(String idTarefa)
    {
        TarefaProgramavel retorno = null;
        if (estadoExecucao)
        {
            retorno = tarefasProgramadas.get(idTarefa);
        }
        else
        {
            log.warn("A tarefa '" + idTarefa + "' no est agendada porque o ProgramadorTarefas est sendo ENCERRADO.");
        }
        return retorno;
    }

    public boolean estaAgendada(String idTarefa)
    {
        boolean retorno = false;
        if (estadoExecucao)
        {
            retorno = (getTarefaAgendada(idTarefa) != null);
        }
        else
        {
            log.warn("A tarefa '" + idTarefa + "' no est agendada porque o ProgramadorTarefas est sendo ENCERRADO.");
        }
        return retorno;
    }

    /**
     * Encerra o programador de tarefas, mas antes cancela todas as tarefas gerenciadas por ele.
     */
    public void encerrarProgramador()
    {
        // Estado de encerramento...
        estadoExecucao = false;

        try
        {
            if (tarefasProgramadas != null)
            {
                for (TarefaProgramavel tarefa : tarefasProgramadas.values())
                {
                    cancelarTarefa(tarefa);
                }
            }
        }
        finally
        {
            if (poolTimers != null)
            {
                for (Timer timerPorTarefa : poolTimers.values())
                {
                    timerPorTarefa.cancel();
                }
            }
        }
    }

    /**
     * Cancela a tarefa especificada do agendamento, antes cancelando a mesma.
     * @param tarefaProgramavel
     */
    void cancelarTarefa(TarefaProgramavel tarefaProgramavel)
    {
        try
        {
            // Primeiro cancelamos a tarefa
            tarefaProgramavel.cancelar();
        }
        finally
        {
            // Em seguida, removemos da lista de tarefas gerenciadas por este programador de tarefas
            if (tarefasProgramadas != null)
            {
                tarefasProgramadas.remove(tarefaProgramavel.getId());
            }
        }
    }
}
