/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.servlet.timer;

import java.util.TimerTask;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.joda.time.LocalDate;
import org.joda.time.LocalDateTime;
import org.joda.time.LocalTime;

import br.gov.camara.exception.CDException;

public abstract class TarefaProgramavel extends TimerTask
{
    /**
     * Logger for this class
     */
    private static final Log log = LogFactory.getLog(TarefaProgramavel.class);

    private long tempoEsperaExecucao = 1;
    private boolean tarefaReagendada = false;

    public static int ESTADO_VIRGEM = 1;
    public static int ESTADO_EM_EXECUCAO = 2;
    public static int ESTADO_CANCELADO = 4;
    public static int ESTADO_AGENDADO = 8;
    private int estado = ESTADO_VIRGEM;

    public abstract String getId();

    protected abstract void executar() throws CDException;

    @Override
    public final void run()
    {
        try
        {
            if (isEstadoVirgem())
            {
                unsetEstadoVirgem();
                setEstadoEmExecucao();
                setReagendarAposExecucao(false);
                this.executar();
            }
            else
            {
                CDException.dispararExcecao("A tarefa programvel " + getId() + " no foi executada porque ela j foi utilizada (no est no estado virgem).");
            }
        }
        catch (Throwable te)
        {
            log.error("Ocorreu o erro '" + te.getMessage() + "' do tipo " + te.getClass());
            tratarExcecao(te);
        }
        finally
        {
            unsetEstadoEmExecucao();

            if (getReagendarAposExecucao())
            {
                ProgramadorTarefas.getInstance().reagendarTarefa(this.getId());
            }
            else
            {
                ProgramadorTarefas.getInstance().cancelarTarefa(this);
            }
        }
    }

    protected void tratarExcecao(Throwable te)
    {}

    public boolean getReagendarAposExecucao()
    {
        return tarefaReagendada;
    }

    public void setReagendarAposExecucao(boolean reagendar)
    {
        this.tarefaReagendada = reagendar;
    }

    public final boolean isEstadoEmExecucao()
    {
        return (estado & ESTADO_EM_EXECUCAO) == ESTADO_EM_EXECUCAO;
    }

    private void setEstadoEmExecucao()
    {
        estado |= ESTADO_EM_EXECUCAO;
        unsetEstadoVirgem();
    }

    private void unsetEstadoEmExecucao()
    {
        estado &= ~ESTADO_EM_EXECUCAO;
        unsetEstadoVirgem();
    }

    public final boolean isEstadoVirgem()
    {
        return (estado & ESTADO_VIRGEM) == ESTADO_VIRGEM;
    }

    protected void unsetEstadoVirgem()
    {
        estado &= ~ESTADO_VIRGEM;
    }

    public final boolean isEstadoCancelado()
    {
        return (estado & ESTADO_CANCELADO) == ESTADO_CANCELADO;
    }

    protected void setEstadoCancelado()
    {
        estado |= ESTADO_CANCELADO;
        unsetEstadoVirgem();
    }

    public final boolean isEstadoAgendado()
    {
        return (estado & ESTADO_AGENDADO) == ESTADO_AGENDADO;
    }

    protected void setEstadoAgendado()
    {
        estado |= ESTADO_AGENDADO;
    }

    public long getTempoEsperaParaPrimeiraExecucao()
    {
        return this.tempoEsperaExecucao;
    }

    public long getTempoEsperaParaInicioExecucao()
    {
        return this.tempoEsperaExecucao;
    }

    public void setTempoEsperaParaInicioExecucao(long tempoEmMilisegundosDeEsperaParaExecucao)
    {
        this.tempoEsperaExecucao = tempoEmMilisegundosDeEsperaParaExecucao;
    }

    public void setHorarioInicioDaExecucao(LocalTime horarioParaIniciarExecucao)
    {
        LocalTime horaAtual = LocalTime.now();
        if (horarioParaIniciarExecucao.isBefore(horaAtual))
        {
            LocalDate hoje = LocalDate.now();
            LocalDateTime amanhaNaHoraDaExecucao = hoje.plusDays(1).toLocalDateTime(horarioParaIniciarExecucao);
            LocalDateTime agora = hoje.toLocalDateTime(horaAtual);

            this.tempoEsperaExecucao = amanhaNaHoraDaExecucao.toDateTime().getMillis() - agora.toDateTime().getMillis();
        }
        else
        {
            this.tempoEsperaExecucao = horarioParaIniciarExecucao.getMillisOfDay() - horaAtual.getMillisOfDay();
        }

    }

    public final void cancelar()
    {
        if (!isEstadoCancelado())
        {
            setEstadoCancelado();
            this.cancel();
            executarAposCancelarAgendamento();
        }
    }

    protected void executarAposCancelarAgendamento()
    {}
}
