/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.talentos.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.biblioteca.util.CaminhoAbsoluto;
import br.gov.camara.biblioteca.util.ConteinerSingleton;
import br.gov.camara.exception.CDException;
import br.gov.camara.servlet.timer.ProgramadorTarefas;

public class IndexacaoTalentosUtil
{
    private static Log LOGGER = LogFactory.getLog(IndexacaoTalentosUtil.class.getName());

    private static final String KEY_PROPRIEDADES_ARQUIVO = "aplicacao.talentos.propriedades.arquivo";
    private static final String KEY_DESATIVAR_INDEXACAO_TEXTUAL = "aplicacao.talentos.desativar.indexacao.textual";

    private String arquivoConfiguracao = null;
    private static final String ARQUIVO_CONFIGURACAO_PADRAO = "resources/BancoTalentos.properties";

    private Properties propriedadesArquivoConfiguracao = null;

    private static IndexacaoTalentosUtil instanceIndexacaoTalentosUtil = null;

    public static IndexacaoTalentosUtil getInstance()
    {
        if (instanceIndexacaoTalentosUtil == null)
        {
            synchronized (IndexacaoTalentosUtil.class)
            {
                if (instanceIndexacaoTalentosUtil == null)
                {
                    instanceIndexacaoTalentosUtil = new IndexacaoTalentosUtil();
                }
            }
        }
        return instanceIndexacaoTalentosUtil;
    }

    private IndexacaoTalentosUtil()
    {
        setArquivoConfiguracao(ARQUIVO_CONFIGURACAO_PADRAO);
    }

    private IndexacaoTalentosUtil(String arquivoPropriedadesSeguranca)
    {
        setArquivoConfiguracao(arquivoPropriedadesSeguranca);
    }

    private void setArquivoConfiguracao(String arquivoConfiguracao)
    {
        this.arquivoConfiguracao = arquivoConfiguracao;
        propriedadesArquivoConfiguracao = (Properties) ConteinerSingleton.INSTANCIA.obterValorObjeto(KEY_PROPRIEDADES_ARQUIVO, arquivoConfiguracao);
        if (propriedadesArquivoConfiguracao == null)
        {
            try
            {
                propriedadesArquivoConfiguracao = lerArquivoConfiguracao(arquivoConfiguracao);
                ConteinerSingleton.INSTANCIA.adicionarPropriedade(KEY_PROPRIEDADES_ARQUIVO, propriedadesArquivoConfiguracao, arquivoConfiguracao);
            }
            catch (IOException e)
            {
                throw new RuntimeException(e);
            }
        }
    }

    private Properties lerArquivoConfiguracao(String nomeArquivoConfiguracao) throws IOException
    {
        Properties propriedades = new Properties();

        File arquivoConfiguracao = CaminhoAbsoluto.obterAPartirDoClasses(nomeArquivoConfiguracao);
        if (arquivoConfiguracao == null || !arquivoConfiguracao.exists())
        {
            throw (new FileNotFoundException("O arquivo de configurao do sistema '" + nomeArquivoConfiguracao + "' no foi localizado."));
        }

        FileInputStream inputDoArquivo = new FileInputStream(arquivoConfiguracao);
        propriedades.load(inputDoArquivo);

        return (propriedades);
    }

    public static void agendarIndexacaoTalentos(int identificadorPessoa, int codigoGrupo) throws CDException
    {
        if (getInstance().utilizarIndexacaoTextual())
        {
            TarefaIndexacaoTalentosPessoa tarefaIndexacaoTalentosPessoa = (TarefaIndexacaoTalentosPessoa) ProgramadorTarefas.getInstance().getTarefaAgendada(TarefaIndexacaoTalentosPessoa.gerarId(identificadorPessoa, codigoGrupo));
            if (tarefaIndexacaoTalentosPessoa == null)
            {
                // Agenda a reindexao do currculo desta pessoa
                LOGGER.info("Agendando a indexao textual dos talentos do ponto " + identificadorPessoa + " do grupo " + codigoGrupo);
                tarefaIndexacaoTalentosPessoa = new TarefaIndexacaoTalentosPessoa(identificadorPessoa, codigoGrupo);

                ProgramadorTarefas.getInstance().agendar(tarefaIndexacaoTalentosPessoa);
            }
            else
            {
                // Reprogramar o agendamento (adiar)
                LOGGER.info("Reagendando a indexao textual dos talentos do ponto " + identificadorPessoa + " do grupo " + codigoGrupo);
                ProgramadorTarefas.getInstance().reagendarTarefa(tarefaIndexacaoTalentosPessoa);
            }
        }
        else
        {
            // Indexao de talentos desativada
            LOGGER.info("A indexao textual dos talentos est DESATIVADA. Talentos no atualizados do ponto "
                    + identificadorPessoa
                    + " do grupo "
                    + codigoGrupo);
        }
    }

    public String obterValorArqCfg(String chave) throws IOException
    {
        return getArquivoConfiguracao().getProperty(chave);
    }

    private Properties getArquivoConfiguracao() throws IOException
    {
        return propriedadesArquivoConfiguracao;
    }

    public boolean utilizarIndexacaoTextual() throws CDException
    {
        boolean retorno = true;

        try
        {
            Boolean valorDesativaIndexacaoTextual = (Boolean) ConteinerSingleton.INSTANCIA.obterValorObjeto(KEY_DESATIVAR_INDEXACAO_TEXTUAL, arquivoConfiguracao);
            if (valorDesativaIndexacaoTextual == null)
            {
                String valorPropriedadeDesativaIndexacaoTextual = obterValorArqCfg(KEY_DESATIVAR_INDEXACAO_TEXTUAL);
                if (valorPropriedadeDesativaIndexacaoTextual != null && !"".equals(valorPropriedadeDesativaIndexacaoTextual))
                {
                    // Mas existe um valor na configurao, por isso, analisar esse valor
                    if ("sim".equalsIgnoreCase(valorPropriedadeDesativaIndexacaoTextual)
                            || "yes".equalsIgnoreCase(valorPropriedadeDesativaIndexacaoTextual)
                            || "true".equalsIgnoreCase(valorPropriedadeDesativaIndexacaoTextual)
                            || "1".equals(valorPropriedadeDesativaIndexacaoTextual))
                    {
                        valorPropriedadeDesativaIndexacaoTextual = "true";
                    }
                    else
                    {
                        valorPropriedadeDesativaIndexacaoTextual = "false";
                    }
                }
                else
                {
                    valorPropriedadeDesativaIndexacaoTextual = "false";
                }

                valorDesativaIndexacaoTextual = Boolean.parseBoolean(valorPropriedadeDesativaIndexacaoTextual);
                ConteinerSingleton.INSTANCIA.adicionarPropriedade(KEY_DESATIVAR_INDEXACAO_TEXTUAL, valorDesativaIndexacaoTextual, arquivoConfiguracao);
            }
            retorno = !valorDesativaIndexacaoTextual.booleanValue();
        }
        catch (IOException e)
        {
            throw new CDException(e.getMessage());
        }

        return retorno;
    }

}
