/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.talentos.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.bancotalentos.util.ConstantesBancoTalentos;
import br.gov.camara.negocio.bancotalentos.util.ServicoDeIndexacaoDeTalento;
import br.gov.camara.servlet.timer.TarefaProgramavel;
import br.gov.camara.servlet.timer.TarefaReprogramavel;

public class TarefaIndexacaoTalentosPessoa extends TarefaProgramavel implements TarefaReprogramavel
{
    /**
     * Logger for this class
     */
    private static final Log LOGGER = LogFactory.getLog(TarefaIndexacaoTalentosPessoa.class.getName());
    private static final int NUMERO_MAXIMO_TENTATIVAS = 3;

    private int identificadorPessoa = 0;
    private int codigoGrupo = 0;
    private int numeroTentativas = 0;

    public TarefaIndexacaoTalentosPessoa(int identificadorPessoa, int codigoGrupo)
    {
        this.identificadorPessoa = identificadorPessoa;
        this.codigoGrupo = codigoGrupo;
        this.numeroTentativas = NUMERO_MAXIMO_TENTATIVAS;
        this.setTempoEsperaParaInicioExecucao(5000); // 300000 milisegundos = 300 segundos = 5 minutos 
    }

    @Override
    public String getId()
    {
        return this.getClass().getName() + "_" + identificadorPessoa + "-" + codigoGrupo;
    }

    public static String gerarId(int identificadorPessoa, int codigoGrupo)
    {
        return TarefaIndexacaoTalentosPessoa.class.getName() + "_" + identificadorPessoa + "-" + codigoGrupo;
    }

    @Override
    protected void executar() throws CDException
    {
        ServicoDeIndexacaoDeTalento idx = new ServicoDeIndexacaoDeTalento();
        idx.indexarTalentos(identificadorPessoa, codigoGrupo, ConstantesBancoTalentos.CONEXAO_TALENTOS);
        LOGGER.info("Indexao textual dos talentos do ponto " + identificadorPessoa + " do grupo " + codigoGrupo + " realizada com sucesso");
    }

    @Override
    protected void tratarExcecao(Throwable e)
    {
        this.numeroTentativas--;
        if (numeroTentativas > 0)
        {
            LOGGER.warn("Na tentativa "
                    + (NUMERO_MAXIMO_TENTATIVAS - numeroTentativas)
                    + " ocorreu o seguinte erro ao tentar indexar os talentos do servidor "
                    + identificadorPessoa
                    + " do grupo "
                    + codigoGrupo
                    + ". REAGENDANDO NOVA TENTATIVA. Mensagem de erro: "
                    + e.getMessage());
            setReagendarAposExecucao(true);
        }
        else
        {
            LOGGER.error("Depois de "
                    + NUMERO_MAXIMO_TENTATIVAS
                    + " tentativas no foi possvel indexar os talentos do servidor "
                    + identificadorPessoa
                    + " do grupo "
                    + codigoGrupo
                    + ". ltima mensagem de erro: "
                    + e.getMessage());
            setReagendarAposExecucao(false);
        }
    }

    private void setNumeroTentativas(int numeroTentativas)
    {
        this.numeroTentativas = numeroTentativas;
    }

    public TarefaProgramavel getNovaInstancia()
    {
        TarefaIndexacaoTalentosPessoa tarefaProgramavel = new TarefaIndexacaoTalentosPessoa(identificadorPessoa, codigoGrupo);
        tarefaProgramavel.setTempoEsperaParaInicioExecucao(getTempoEsperaParaInicioExecucao());
        tarefaProgramavel.setNumeroTentativas(numeroTentativas);
        return tarefaProgramavel;
    }
}
