package br.gov.camara.util.displaytag;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

import org.apache.commons.lang.StringUtils;
import org.displaytag.exception.ObjectLookupException;
import org.displaytag.util.LookupUtil;
import org.displaytag.util.ParamEncoder;

public class DisplaytagUtil
{
    private DisplaytagUtil()
    {
        // No  instancivel 
    }

    private static String getEncodedPageParameterName(String idDisplaytag)
    {
        ParamEncoder paramEncoder = new ParamEncoder(idDisplaytag);
        return paramEncoder.encodeParameterName("p");
    }

    private static String getEncodedExportParameterName(String idDisplaytag)
    {
        ParamEncoder paramEncoder = new ParamEncoder(idDisplaytag);
        return paramEncoder.encodeParameterName("e");
    }

    public static void resetPageParameterValue(HttpServletRequest request, String idDisplaytag)
    {
        String encodedNomeTabela = getEncodedPageParameterName(idDisplaytag);
        request.removeAttribute(encodedNomeTabela);
        request.getSession().removeAttribute(encodedNomeTabela);
    }

    public static void setPageAttributeValue(HttpServletRequest request, String idDisplaytag, int pagina)
    {
        String encodedNomeTabela = getEncodedPageParameterName(idDisplaytag);
        request.setAttribute(encodedNomeTabela, pagina);
        request.getSession().setAttribute(encodedNomeTabela, pagina);
    }

    public static int modificarPaginaAtualSeMaiorQueQuantidadeMaximaDePaginas(HttpServletRequest request, String idDisplaytag, int totalDeRegistros, int quantidadePorPagina)
    {
        return modificarPaginaAtualSeMaiorQueQuantidadeMaximaDePaginas(request, idDisplaytag, totalDeRegistros, quantidadePorPagina, getIntPageValue(request, idDisplaytag));
    }

    public static int modificarPaginaAtualSeMaiorQueQuantidadeMaximaDePaginas(HttpServletRequest request, String idDisplaytag, int totalDeRegistros, int quantidadePorPagina, int paginaAtual)
    {
        int paginaMaxima = (int) Math.ceil((double) totalDeRegistros / quantidadePorPagina);
        if (paginaAtual > paginaMaxima)
        {
            resetPageParameterValue(request, idDisplaytag);
            setPageAttributeValue(request, idDisplaytag, paginaMaxima);
            return paginaMaxima;
        }

        return paginaAtual;
    }

    /**
     * Retorna o nmero da pgina para a tabela informada.
     * Procura o valor da pgina primeiro como parmetro do request e, se no encontrar, procura como um atributo.
     * @param request
     * @param idDisplaytag
     * @return
     */
    public static Object getPageValue(HttpServletRequest request, String idDisplaytag)
    {
        Object retorno = null;
        retorno = getPageParameterValue(request, idDisplaytag);
        if (retorno == null)
        {
            retorno = getPageAttributeValue(request, idDisplaytag);
        }
        return retorno;
    }

    /**
     * Retorna o nmero da pgina para a tabela informada.
     * Procura o valor da pgina primeiro como parmetro do request e, se no encontrar, procura como um atributo.
     * @param request
     * @param idDisplaytag
     * @return
     */
    public static int getIntPageValue(HttpServletRequest request, String idDisplaytag)
    {
        int intPagina = 1;
        Object valorPagina = DisplaytagUtil.getPageValue(request, idDisplaytag);
        if (valorPagina != null)
        {
            intPagina = Integer.parseInt(valorPagina.toString());
        }
        return intPagina;
    }

    /**
     * Retorna a pgina da tabela informada como atributo
     * @param request
     * @param idDisplaytag
     * @return
     */
    public static Object getPageAttributeValue(HttpServletRequest request, String idDisplaytag)
    {
        Object retorno = null;
        try
        {
            String encodedNomeTabela = getEncodedPageParameterName(idDisplaytag);
            retorno = evaluateExpression(request, encodedNomeTabela);
        }
        catch (ObjectLookupException ole)
        {
            // Ocorreu um erro... Ignorar e retornar nulo
        }
        return retorno;
    }

    /**
     * Retorna a pgina da tabela informada no parmetro do request
     * @param request
     * @param idDisplaytag
     * @return
     */
    public static Object getPageParameterValue(HttpServletRequest request, String idDisplaytag)
    {
        Object retorno = null;
        String encodedNomeTabela = getEncodedPageParameterName(idDisplaytag);
        retorno = request.getParameter(encodedNomeTabela);
        return retorno;
    }

    /**
     * Informa se a origem do request  a exportao dos dados da tabela informada
     * @param request
     * @param idDisplaytag
     * @return
     */
    public static boolean isExporting(HttpServletRequest request, String idDisplaytag)
    {
        boolean retorno = false;
        String encodedNomeTabela = getEncodedExportParameterName(idDisplaytag);
        if (request.getParameter(encodedNomeTabela) != null)
        {
            retorno = true;
        }
        return retorno;
    }

    /**
     * <p>
     * evaluate an expression in a way similar to LE in jstl.
     * </p>
     * <p>
     * the first token is supposed to be an object in the page scope (default scope) or one of the following:
     * </p>
     * <ul>
     * <li>pageScope</li>
     * <li>requestScope</li>
     * <li>sessionScope</li>
     * <li>applicationScope</li>
     * </ul>
     * <p>
     * Tokens after the object name are interpreted as javabean properties (accessed through getters), mapped or indexed
     * properties, using the jakarta common-beans library
     * </p>
     * @param expression expression to evaluate
     * @return Object result
     * @throws ObjectLookupException if unable to get a bean using the given expression
     */
    private static Object evaluateExpression(HttpServletRequest request, String expression) throws ObjectLookupException
    {

        String expressionWithoutScope = expression;

        // Para indicar que no foi especificado o escopo do bean        
        int scope = -1;

        if (expression.startsWith("requestScope.")) //$NON-NLS-1$
        {
            scope = PageContext.REQUEST_SCOPE;
            expressionWithoutScope = expressionWithoutScope.substring(expressionWithoutScope.indexOf('.') + 1);

        }
        else if (expression.startsWith("sessionScope.")) //$NON-NLS-1$
        {
            scope = PageContext.SESSION_SCOPE;
            expressionWithoutScope = expressionWithoutScope.substring(expressionWithoutScope.indexOf('.') + 1);

        }
        else if (expression.startsWith("applicationScope.")) //$NON-NLS-1$
        {
            scope = PageContext.APPLICATION_SCOPE;
            expressionWithoutScope = expressionWithoutScope.substring(expressionWithoutScope.indexOf('.') + 1);

        }

        return getBeanValue(request, expressionWithoutScope, scope);

    }

    /**
     * Read an object from the pagecontext with the specified scope and eventually lookup a property in it.
     * @param request PageContext
     * @param beanAndPropertyName String expression with bean name and attributes
     * @param scope One of the following values:
     * <ul>
     * <li>PageContext.REQUEST_SCOPE</li>
     * <li>PageContext.SESSION_SCOPE</li>
     * <li>PageContext.APPLICATION_SCOPE</li>
     * </ul>
     * @return Object
     * @throws ObjectLookupException for errors while retrieving a property in the bean
     */
    private static Object getBeanValue(HttpServletRequest request, String beanAndPropertyName, int scope) throws ObjectLookupException
    {

        String objectName = null;
        String beanProperty = null;

        Object objRetorno = null;

        if (beanAndPropertyName.indexOf('.') == -1)
        {
            // simple, only the javabean
            objectName = beanAndPropertyName;
        }
        else
        {
            // complex: property from a bean
            objectName = StringUtils.substringBefore(beanAndPropertyName, ".");
            beanProperty = StringUtils.substringAfter(beanAndPropertyName, ".");
        }

        if (objRetorno == null && (scope == -1 || scope == PageContext.REQUEST_SCOPE))
        {
            // Procurar no escope de request
            objRetorno = request.getAttribute(objectName);
        }

        if (objRetorno == null && (scope == -1 || scope == PageContext.SESSION_SCOPE))
        {
            // Procurar no escope de session
            objRetorno = request.getSession().getAttribute(objectName);
        }

        if (objRetorno == null && (scope == -1 || scope == PageContext.APPLICATION_SCOPE))
        {
            // Procurar no escope de application
            objRetorno = request.getSession().getServletContext().getAttribute(objectName);
        }

        if (objRetorno != null && beanProperty != null)
        {
            // complex: property from a bean

            // go get the property
            objRetorno = LookupUtil.getBeanProperty(objRetorno, beanProperty);
        }

        return objRetorno;
    }

}
