package br.gov.camara.util.struts.plugin;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Random;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.action.PlugIn;
import org.apache.struts.config.ModuleConfig;

import br.gov.camara.biblioteca.util.ConstantesSingleton;

/**
 * Plugin para a configurao do locale
 */
public class InicializacaoPlugin implements PlugIn
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(InicializacaoPlugin.class);

    private static boolean inicializado = false;

    // Implementao mtodo init
    public void init(ActionServlet servlet, ModuleConfig moduleConfig) throws ServletException
    {
        if (inicializado == false)
        {
            inicializar(servlet, moduleConfig);
        }
        else
        {
            if (log.isInfoEnabled())
            {
                log.info("Plug-in de inicializao da aplicao j executado.");
            }
        }
    }

    private synchronized void inicializar(ActionServlet servlet, ModuleConfig moduleConfig)
    {
        if (inicializado == true)
        {
            if (log.isInfoEnabled())
            {
                log.info("Plug-in de inicializao da aplicao j executado.");
            }
            return;
        }

        if (log.isInfoEnabled())
        {
            log.info("Executando o plug-in de inicializao da aplicao...");
        }

        ServletContext servletContext = servlet.getServletContext();

        String canonicalHostName = getCanonicalHostName();
        ConstantesSingleton.getInstance().setObjeto("canonicalHostName", canonicalHostName);
        servletContext.setAttribute("canonicalHostName", canonicalHostName);

        // Registra que foi inicializado
        inicializado = true;
    }

    /**
     * Gera um nmero nico que identifica a aplicao
     * @param nomeAplicacao
     * @param versaoBuild
     * @return
     */
    private int gerarHashVersaoAplicacao(String nomeAplicacao, String versaoBuild)
    {
        String versaoMaiorBuild = versaoBuild.split("\\.")[0]; // Pegar somente v123 e descartar o que vem depois.
        return (versaoMaiorBuild + nomeAplicacao).hashCode();
    }

    private String gerarHostNameNumericoAleatorio(String nomeMaquina)
    {
        // Se no foi possvel determinar o nome do servidor
        if (nomeMaquina == null)
        {
            nomeMaquina = Integer.toString((new Random()).nextInt());
        }
        return nomeMaquina;
    }

    private String getLocalHostName()
    {
        String nomeMaquina = null;

        try
        {
            nomeMaquina = InetAddress.getLocalHost().getHostName();
            if (nomeMaquina.indexOf(".") > -1)
            {
                nomeMaquina = nomeMaquina.substring(0, nomeMaquina.indexOf("."));
            }
        }
        catch (UnknownHostException unknownHostException)
        {
            // Pacincia...   
        }

        return nomeMaquina;
    }

    private String getCanonicalHostName()
    {

        String canonicalHostName = null;
        try
        {
            canonicalHostName = InetAddress.getLocalHost().getCanonicalHostName();
            if (canonicalHostName.indexOf(".") > -1)
            {
                canonicalHostName = canonicalHostName.substring(0, canonicalHostName.indexOf("."));
            }
        }
        catch (UnknownHostException unknownHostException)
        {
            // Pacincia...   
        }

        canonicalHostName = gerarHostNameNumericoAleatorio(canonicalHostName);

        return canonicalHostName;
    }

    // Implementao do mtodo destroy
    public void destroy()
    {
        if (log.isDebugEnabled())
        {
            log.debug("Finalizando o plug-in de inicializao da aplicao...");
        }
    }
}
