/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.util.strutsmenu;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpSession;

import net.sf.navigator.menu.MenuComponent;
import net.sf.navigator.menu.PermissionsAdapter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.seguranca.SegurancaWeb;
import br.gov.camara.seguranca.UsuarioAutenticado;
import br.gov.camara.seguranca.exception.SegurancaException;

/**
 * DOCUMENTAR!
 */
public class SegurancaPermissionsAdapter implements PermissionsAdapter
{

    // Inicializa o log
    private static Log log = LogFactory.getLog(SegurancaPermissionsAdapter.class.getName());

    private HttpSession session;

    private Map mapPermissoesUsuario;

    /**
     * Creates a new instance of SegurancaPermissionAdapter
     *
     * @param theMenuNames DOCUMENTAR!
     */
    public SegurancaPermissionsAdapter(HttpSession session)
    {
        if (log.isDebugEnabled())
        {
            log.debug("Inicializando SegurancaPermissionAdapter");
        }
        this.session = session;
        this.mapPermissoesUsuario = new HashMap();
    }

    /**
     * If the menu is allowed, this should return true.
     *
     * @param menu DOCUMENTAR!
     *
     * @return whether or not the menu is allowed.
     */
    public boolean isAllowed(MenuComponent menu)
    {

        boolean blnTemAutorizacao = false;

        SegurancaWeb segurancaWeb = SegurancaWeb.obterSegurancaWeb(session);

        if (segurancaWeb != null && segurancaWeb.temUsuarioLogado(session))
        {
            UsuarioAutenticado usuarioWeb = SegurancaWeb.obterSegurancaWeb(session).obterUsuarioAutenticado();

            Boolean blnResultadoCache = null;
            String strObjetoControlado = null;

            // Qual o item sendo validado ?
            String strNome = menu.getName();

            // A permisso j foi verificada ?...
            blnResultadoCache = (Boolean) mapPermissoesUsuario.get(strNome);
            if (blnResultadoCache == null)
            {
                // ...No, ela ainda no foi validada (no est no cache). Portanto: validar...

                // Recupera o objeto controlado
                String strPagina = menu.getPage();
                if ("".equals(strPagina))
                {
                    strPagina = null;
                }
                String strURL = menu.getUrl();
                if ("".equals(strURL))
                {
                    strURL = null;
                }

                if (strPagina == null && strURL == null)
                {
                    // No existe um link para uma pgina, portanto deve ser um menu agrupador

                    MenuComponent[] subMenus = menu.getMenuComponents();

                    if (subMenus.length > 0)
                    {
                        // Este item de menu tem filhos...

                        // Este usurio tem autorizao para algum item de menu filho ?
                        for (int i = 0; i < subMenus.length; i++)
                        {
                            if (isAllowed(subMenus[i]))
                            {
                                blnTemAutorizacao = true;
                            }
                        }

                        // Registra o log
                        if (log.isDebugEnabled())
                        {
                            if (blnTemAutorizacao)
                            {
                                log.debug("O usurio '"
                                        + usuarioWeb.obterLogin()
                                        + "' est logado e tem permisso para o menu agrupador '"
                                        + strObjetoControlado
                                        + "' (pois tem permisso para um de seus filhos)");
                            }
                            else
                            {
                                log.debug("O usurio '"
                                        + usuarioWeb.obterLogin()
                                        + "' est logado e no tem permisso para o menu agrupador '"
                                        + strObjetoControlado
                                        + "' (pois no tem permisso para nenhum de seus filhos)");
                            }
                        }
                    }
                    else
                    {
                        // Este item de menu no tem filhos...
                        // ... e como no tem pgina associada, ...              /* no exibir */
                        // ... EXIBIR, pois deve ser um item pai (Menu superior) /* Acrescentado em 19/07/2007 */

                        blnTemAutorizacao = true;

                        // Registra o log
                        if (log.isDebugEnabled())
                        {
                            log.debug("O usurio '"
                                    + usuarioWeb.obterLogin()
                                    + "' est logado e no tem permisso para o menu '"
                                    + strObjetoControlado
                                    + "' (pois ele no tem uma pgina associada e nem menus filhos)");
                        }
                    }

                }
                else
                {
                    String strAvaliacao = strPagina;
                    if (strAvaliacao == null)
                    {
                        strAvaliacao = strURL;
                    }

                    // Ser que  uma ao do Struts (contm .do)?
                    int delimitadorFinal = strAvaliacao.lastIndexOf(".do");
                    if (delimitadorFinal > -1)
                    {
                        //  uma ao do Struts, portanto devemos avaliar a questo de permisso
                        String auxiliar = strAvaliacao.substring(0, delimitadorFinal).trim();
                        if (auxiliar.length() > 0)
                        {
                            String[] subAuxiliar = auxiliar.split("[\'\"/ ]");
                            auxiliar = subAuxiliar[subAuxiliar.length - 1];

                            // Acrescentar a barra inicial
                            strObjetoControlado = "/" + auxiliar;
                        }
                        else
                        {
                            // No deveria acontecer, pois um link para '/.do' no faz muito sentido, mas liberar o acesso...
                            blnTemAutorizacao = true;
                        }
                    }
                    else
                    {
                        // No  uma ao do tipo Struts, portanto liberar o acesso
                        blnTemAutorizacao = true;
                    }

                    if (!blnTemAutorizacao)
                    {

                        // Verificar a autorizao no banco de dados...
                        try
                        {
                            blnTemAutorizacao = SegurancaWeb.obterSegurancaWeb(session).obterPermissao().temAutorizacaoFuncionalidade(SegurancaWeb.obterSegurancaWeb(session).obterUsuarioAutenticado(), strObjetoControlado);

                            if (blnTemAutorizacao)
                            {
                                if (log.isDebugEnabled())
                                {
                                    log.debug("O usurio '" + usuarioWeb.obterLogin() + "' est logado e tem permisso para o menu: " + strObjetoControlado);
                                }
                            }
                            else
                            {
                                if (log.isDebugEnabled())
                                {
                                    log.debug("O usurio '"
                                            + usuarioWeb.obterLogin()
                                            + "' est logado mas NO tem permisso para o menu: "
                                            + strObjetoControlado);
                                }
                            }
                        }
                        catch (SegurancaException ue)
                        {
                            blnTemAutorizacao = false;
                        }
                    }
                }

                // Registrar a autorizao para este item de menu no cache
                mapPermissoesUsuario.put(strNome, new Boolean(blnTemAutorizacao));
            }
            else
            {
                // Sim, ele j foi validado e o resultado ...

                blnTemAutorizacao = blnResultadoCache.booleanValue();

                if (log.isDebugEnabled())
                {
                    if (blnTemAutorizacao)
                    {
                        log.debug("A permisso do usurio '"
                                + usuarioWeb.obterLogin()
                                + "' est sendo recuperada do cache e ele tem permisso no objeto '"
                                + strObjetoControlado
                                + "'");
                    }
                    else
                    {
                        log.debug("A permisso do usurio '"
                                + usuarioWeb.obterLogin()
                                + "' est sendo recuperada do cache, mas ele no tem permisso no objeto '"
                                + strObjetoControlado
                                + "'");
                    }
                }
            }

        }
        else
        {
            if (log.isDebugEnabled())
            {
                log.debug("Como no tem usurio logado, retornar que no tem acesso para o menu: " + menu.getName());
            }
        }

        return blnTemAutorizacao;
    }
}
