package br.gov.camara.util.strutsmenu;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

import net.sf.navigator.displayer.MenuDisplayer;
import net.sf.navigator.displayer.MenuDisplayerMapping;
import net.sf.navigator.displayer.MessageResourcesMenuDisplayer;
import net.sf.navigator.menu.MenuRepository;
import net.sf.navigator.menu.PermissionsAdapter;
import net.sf.navigator.menu.RolesPermissionsAdapter;
import net.sf.navigator.taglib.UseMenuDisplayerTag;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.servlet.util.ServletUtil;

public class UseMenuDisplayerCDTag extends UseMenuDisplayerTag
{

    /**
     * 
     */
    private static final long serialVersionUID = -1588169987790241283L;

    private static Log log = LogFactory.getLog(UseMenuDisplayerCDTag.class);

    @Override
    public int doStartTag() throws JspException
    {
        if (getRepository() == null)
        {
            setRepository(MenuRepository.MENU_REPOSITORY_KEY);
        }

        if (log.isDebugEnabled())
        {
            log.debug("Looking for repository named '" + getRepository() + "'");
        }

        // get the menu repository
        MenuRepository rep = (MenuRepository) ServletUtil.findAttribute(getRepository(), pageContext);

        if (rep == null)
        {
            throw new JspException(messages.getString("menurepository.not.found"));
        }
        else
        {
            // set repository as a pageContext variable so that DisplayMenuTag
            // can grab it.
            if (log.isDebugEnabled())
            {
                log.debug("stuffing repository into pageContext...");
            }
            pageContext.setAttribute(PRIVATE_REPOSITORY, rep);
        }

        // get the displayer mapping
        MenuDisplayerMapping displayerMapping = rep.getMenuDisplayerMapping(this.name);

        if (displayerMapping == null)
        {
            throw new JspException(messages.getString("displayer.mapping.not.found"));
        }

        PermissionsAdapter permissions = getPermissionsAdapter();

        //get an instance of the menu displayer
        MenuDisplayer displayerInstance;

        try
        {
            displayerInstance = (MenuDisplayer) Class.forName(displayerMapping.getType()).newInstance();
            menuDisplayer = displayerInstance;
            // default to use the config on the mapping
            if (displayerMapping.getConfig() != null)
            {
                // this value (config) is set on the displayer below
                setConfig(displayerMapping.getConfig());
            }
        }
        catch (Exception e)
        {
            throw new JspException(e.getMessage());
        }

        if (bundleKey == null)
        {
            this.bundleKey = "org.apache.struts.action.MESSAGE";
        }

        // setup the displayerInstance
        // if the displayer is a MessageResourceMenuDisplayer
        // and a bundle is specified, then pass it the bundle (message resources) and
        // the locale
        if ((bundleKey != null && !"".equals(bundleKey)) && (displayerInstance instanceof MessageResourcesMenuDisplayer))
        {
            MessageResourcesMenuDisplayer mrDisplayerInstance = (MessageResourcesMenuDisplayer) displayerInstance;
            Locale locale;

            if (localeKey == null)
            {
                // default to Struts locale
                locale = (Locale) ServletUtil.findAttribute("org.apache.struts.action.LOCALE", pageContext);
                if (locale == null)
                {
                    locale = pageContext.getRequest().getLocale();
                }
            }
            else
            {
                locale = (Locale) ServletUtil.findAttribute(localeKey, pageContext);
            }
            mrDisplayerInstance.setLocale(locale);

            if (rb != null)
            {
                mrDisplayerInstance.setMessageResources(rb);
            }
            else
            {
                Object resources = ServletUtil.findAttribute(bundleKey, pageContext);

                if (resources == null)
                {
                    // try a simple ResourceBundle
                    try
                    {
                        rb = ResourceBundle.getBundle(bundleKey, locale);
                        mrDisplayerInstance.setMessageResources(rb);
                    }
                    catch (MissingResourceException mre)
                    {
                        log.error(mre.getMessage());
                    }
                }
                else
                {
                    mrDisplayerInstance.setMessageResources(resources);
                }
            }
        }

        displayerInstance.setConfig(getConfig());
        if (id != null)
        {
            pageContext.setAttribute("menuId", id);
        }

        displayerInstance.init(pageContext, displayerMapping);
        displayerInstance.setPermissionsAdapter(permissions);

        pageContext.setAttribute(DISPLAYER_KEY, displayerInstance);

        return (EVAL_BODY_INCLUDE);
    }

    @Override
    protected PermissionsAdapter getPermissionsAdapter() throws JspException
    {
        PermissionsAdapter adapter = null;

        if (getPermissions() != null)
        {
            // If set to "rolesAdapter", then create automatically
            if (getPermissions().equalsIgnoreCase(ROLES_ADAPTER))
            {
                adapter = new RolesPermissionsAdapter((HttpServletRequest) pageContext.getRequest());
            }
            else
            {
                adapter = (PermissionsAdapter) ServletUtil.findAttribute(getPermissions(), pageContext);

                if (adapter == null)
                {
                    throw new JspException(messages.getString("permissions.not.found"));
                }
            }
        }

        return adapter;
    }

}
