/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.visao.bancotalentosapoio.util;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.biblioteca.exception.DataException;
import br.gov.camara.biblioteca.util.DataNova;
import br.gov.camara.exception.CDException;
import br.gov.camara.negocio.bancotalentos.facade.ConsultaTalentoFacade;
import br.gov.camara.negocio.bancotalentos.util.ConfiguracaoDaIndexacaoTextual;
import br.gov.camara.negocio.bancotalentos.util.ConstantesBancoTalentos;
import br.gov.camara.negocio.bancotalentos.util.DadosIndexacaoManual;
import br.gov.camara.negocio.bancotalentos.util.ServicoDeIndexacaoDeTalento;
import br.gov.camara.negocio.comum.facade.DataHoraBDFacade;
import br.gov.camara.servlet.timer.TarefaProgramavel;

public class TarefaReindexacaoManual extends TarefaProgramavel
{
    /**
     * Logger for this class
     */
    private static final Log log = LogFactory.getLog(TarefaReindexacaoManual.class.getName());

    public static final int MAXIMO_PESSOAS_PROCESSADAS_NUM_LOTE = 200;

    private String loginUsuario = null;
    private String instanciaServidor = null;
    private boolean forcarReindexacaoGeral = false;
    DadosIndexacaoManual dadosInicioIndexacaoManual = null;

    public TarefaReindexacaoManual(String loginUsuario, String instanciaServidor, boolean forcarReindexacaoGeral)
    {
        this.loginUsuario = loginUsuario;
        this.instanciaServidor = instanciaServidor;
        this.forcarReindexacaoGeral = forcarReindexacaoGeral;
        this.setTempoEsperaParaInicioExecucao(1);

        try
        {
            this.dadosInicioIndexacaoManual = registrarInicioDaExecucao();
        }
        catch (Exception e)
        {
            log.error("Erro ao registrar incio da indexao textual. Abortando! ERRO: " + e.getMessage(), e);
        }
    }

    @Override
    public String getId()
    {
        return this.getClass().getName();
    }

    @Override
    protected void executar() throws CDException
    {
        if (dadosInicioIndexacaoManual == null)
        {
            // deve ter ocorrido um erro, portanto finalizar
            return;
        }

        Calendar dataUltimaReindexacaoManual = null;
        if (!forcarReindexacaoGeral)
        {
            try
            {
                dataUltimaReindexacaoManual = recuperarDataDaUltimaReindexacaoManual();
            }
            catch (Exception e)
            {
                log.error("Erro ao recuperar data da ltima reindexao manual. Abortando! ERRO: " + e.getMessage(), e);
                return;
            }
        }

        ServicoDeIndexacaoDeTalento servicoDeIndexacaoDeTalento = new ServicoDeIndexacaoDeTalento();
        try
        {
            ConsultaTalentoFacade consultaTalentoFacade = new ConsultaTalentoFacade(ConstantesBancoTalentos.CONEXAO_TALENTOS);

            if (forcarReindexacaoGeral)
            {
                // Como  uma reindexao geral, limpar todo o ndice.
                servicoDeIndexacaoDeTalento.limparIndices();
            }

            int totalPessoasComTalentos = consultaTalentoFacade.obterTotalPessoasComTalentos();
            dadosInicioIndexacaoManual.setQuantidadeTotal(totalPessoasComTalentos);

            int lote = 1;
            int processados = 0;
            while (processados < totalPessoasComTalentos)
            {
                List<Map<String, Object>> lstPessoas = consultaTalentoFacade.obterPessoasComTalentos(lote++, MAXIMO_PESSOAS_PROCESSADAS_NUM_LOTE);
                Iterator<Map<String, Object>> it = lstPessoas.iterator();

                List<Integer[]> listaRegistrosAAtualizar = new ArrayList<Integer[]>();
                while (it.hasNext())
                {
                    Map<String, Object> p = (Map<String, Object>) it.next();

                    if (forcarReindexacaoGeral
                            || dataUltimaReindexacaoManual == null
                            || ((Calendar) p.get("dataultimolancamento")).after(dataUltimaReindexacaoManual))
                    {
                        listaRegistrosAAtualizar.add(new Integer[] { (Integer) p.get("identificadorpessoa"), (Integer) p.get("codigogrupo") });
                    }
                }

                servicoDeIndexacaoDeTalento.indexarTalentosEmLoteComAtualizacaoDoAndamento(listaRegistrosAAtualizar, ConstantesBancoTalentos.CONEXAO_TALENTOS, dadosInicioIndexacaoManual);

                // Pode ser que algum currculo tenha sido excludo aps a leitura do total de pessoas, por isso, por garantia  melhor fazer a validao abaixo.
                if (lstPessoas.size() < MAXIMO_PESSOAS_PROCESSADAS_NUM_LOTE)
                {
                    break;
                }
            }
        }
        finally
        {
            ConfiguracaoDaIndexacaoTextual.getInstance().setConclusaoIndexacaoManual(dadosInicioIndexacaoManual.getDataInicio());
        }
    }

    private DadosIndexacaoManual registrarInicioDaExecucao() throws CDException
    {
        DadosIndexacaoManual dadosInicioIndexacaoManual = new DadosIndexacaoManual();
        dadosInicioIndexacaoManual.setDataInicio(new Date(DataHoraBDFacade.obterTimeStampBD().getTime() + getTempoEsperaParaInicioExecucao()));
        dadosInicioIndexacaoManual.setLoginUsuario(loginUsuario);
        dadosInicioIndexacaoManual.setInstancia(instanciaServidor);

        ConfiguracaoDaIndexacaoTextual.getInstance().registrarDadosIndexacaoManual(dadosInicioIndexacaoManual);

        return dadosInicioIndexacaoManual;
    }

    private Calendar recuperarDataDaUltimaReindexacaoManual() throws CDException, DataException
    {
        Calendar dataUltimaReindexacaoManual = null;
        String dataRecuperada = ConfiguracaoDaIndexacaoTextual.getInstance().getDataUltimaIndexacaoManual();
        if (dataRecuperada != null)
        {
            dataUltimaReindexacaoManual = DataNova.formatarCalendar(dataRecuperada);
        }
        return dataUltimaReindexacaoManual;
    }

    @Override
    protected void tratarExcecao(Throwable e)
    {
        log.error("Erro ao tentar executar a tarefa de indexao manual: " + e.getMessage(), e);
    }
}
