/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package sigesp.comum.util.bean;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.BasicDynaBean;
import org.apache.commons.beanutils.BasicDynaClass;
import org.apache.commons.beanutils.DynaBean;
import org.apache.commons.beanutils.DynaProperty;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import sigesp.comum.util.exception.FabricaBeansException;

/**
 * Classe FabricaBeans: permite a criao de javabeans dinamicamente.
 *
 * @author Andr Felipe Matos de Carvalho Camara dos Deputados 01/10/2002
 * @version 0.001
 */
public class FabricaBeans
{
    // Inicializa o log
    private static Log log = LogFactory.getLog(FabricaBeans.class.getName());

    /**
     * FabricaBeans: permite a criao de javabeans dinamicamente.
     *
     * @param itrColunas - ArrayList com os nomes dos atributos
     * @param arlValores - ArrayList de objetos (arlValores) de cada atributo.
     *
     * @return - Object javabean criado com os atributos setados.
     */
    public static Object criarBean(List itrColunas, List arlValores)
    {
        // Registra log
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no mtodo");
        }

        DynaBean objBean = null;
        int intCols = arlValores.size();

        try
        {
            DynaProperty[] dptPropriedades = new DynaProperty[intCols];

            for (int i = 0; i < intCols; i++)
            {
                dptPropriedades[i] = new DynaProperty("" + itrColunas.get(i));
            }

            BasicDynaClass dynaClass = new BasicDynaClass("registros", BasicDynaBean.class, dptPropriedades);

            objBean = dynaClass.newInstance();
            int i = 0;

            while (i < intCols)
            {
                objBean.set("" + itrColunas.get(i), arlValores.get(i));
                i++;
            }
        }
        catch (Exception erro)
        {
            if (log.isErrorEnabled())
            {
                log.error("Erro ao tentar criar Bean dinmico na classe FabricaBeans");
            }
        }

        return objBean;
    }

    /**
     * criarBean         Verso String[]  Recebe lista de atributos e arlValores para criar um objBean dinamico.
     *
     * @param itrColunas - String[] com os nomes dos atributos
     * @param arlValores - String[] com os arlValores de cada atributo.
     *
     * @return - Object javabean criado com os atributos setados.
     */
    public static Object criarBean(String[] itrColunas, String[] arlValores) throws FabricaBeansException
    {
        // Registra log
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no mtodo");
        }

        DynaBean objBean = null;

        int intCols = arlValores.length;

        try
        {
            DynaProperty[] dptPropriedades = new DynaProperty[intCols];

            for (int i = 0; i < intCols; i++)
            {
                dptPropriedades[i] = new DynaProperty("" + itrColunas[i]);
            }

            BasicDynaClass dynaClass = new BasicDynaClass("registros", BasicDynaBean.class, dptPropriedades);

            objBean = dynaClass.newInstance();
            int i = 0;

            while (i < intCols)
            {
                objBean.set("" + itrColunas[i], arlValores[i]);
                i++;
            }
        }
        catch (Exception erro)
        {
            if (log.isErrorEnabled())
            {
                log.error("Erro ao tentar criar Bean dinmico na classe FabricaBeans");
            }
            throw new FabricaBeansException("Erro ao tentar criar Bean dinmico na classe FabricaBeans");
        }

        return objBean;
    }

    /**
     * descreverBean:   retorna um ArrayList com os nomes dos atributos de um javabean dinmico.
     *
     * @param objBean :    Objeto javabean dinmico.
     *
     * @return :        ArrayList com os nomes dos atributos.
     *
     * @throws FabricaBeansException : No caso de erros na leitura dos atributos do javabean.
     */
    public static ArrayList descreverBean(Object objBean) throws FabricaBeansException
    {
        // Registra log
        if (log.isDebugEnabled())
        {
            log.debug("Entrada no mtodo");
        }

        Map mapMapa = null;
        Iterator itrColunas = null;

        ArrayList arlDescricaoBean = new ArrayList();

        try
        {
            // Recuperar todos os atributos que tenham mtodos de acesso para leitura:
            mapMapa = PropertyUtils.describe(objBean);

            // Enviar contedo do objBean para a linha recuperada:
            itrColunas = mapMapa.keySet().iterator();

            while (itrColunas.hasNext())
            {
                arlDescricaoBean.add("" + itrColunas.next());
            }
        }
        catch (NoSuchMethodException nsme)
        {
            if (log.isFatalEnabled())
            {
                log.fatal("No existem mtodos de acesso para o objBean " + objBean.toString());
            }
            throw new FabricaBeansException("No existem mtodos de acesso para o objBean " + objBean.toString());
        }
        catch (InvocationTargetException ite)
        {
            if (log.isFatalEnabled())
            {
                log.fatal("Problema ao invocar Target para o objBean " + objBean.toString());
            }
            throw new FabricaBeansException("Problema ao invocar Target para o objBean " + objBean.toString());
        }
        catch (IllegalAccessException iae)
        {
            if (log.isFatalEnabled())
            {
                log.fatal("Mtodo de acesso Ilegal para o objBean " + objBean.toString());
            }
            throw new FabricaBeansException("Mtodo de acesso Ilegal para o objBean " + objBean.toString());
        }
        catch (Exception e)
        {
            if (log.isFatalEnabled())
            {
                log.fatal("Erro inesperado em Persistencia.alterar( objBean , chave) ou no foi encontrado nenhum registro com a chave especificada"
                        + objBean.toString());
            }
            throw new FabricaBeansException("Erro inesperado em Persistencia.alterar( objBean , chave) ou no foi encontrado nenhum registro com a chave especificada"
                    + objBean.toString());
        }

        return arlDescricaoBean;
    }
}
