/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package sigesp.comum.util.biblioteca;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.camara.exception.CDException;
import sigesp.comum.util.exception.ArquivoException;

/**
 * Classe que contm rotinas para o tratamento de arquivos e diretrios
 */
public class Arquivo
{
    // Inicializa o log
    private static Log LOG = LogFactory.getLog("sigesp.comum.util.biblioteca.Arquivo");

    private Arquivo()
    {}

    /**
     * Verifica e cria, se no existir, o caminho para um dado arquivo
     *
     * @param strArquivoDestino Arquivo (com caminho) a ser validado
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void garantirDiretorioPara(String strArquivoDestino) throws ArquivoException
    {
        // Cria um objeto do tipo File apontando para o arquivo fleDestino
        File fleDestino = new File(strArquivoDestino);

        // Se o caminho para o arquivo fleDestino no existir, criar
        if (!fleDestino.getParentFile().exists())
        {
            // Registra log
            if (LOG.isDebugEnabled())
            {
                LOG.debug("Criando diretorio inexistente: " + fleDestino.getParentFile().toString());
            }

            if (!fleDestino.getParentFile().mkdirs())
            {
                if (LOG.isErrorEnabled())
                {
                    LOG.error("No foi possvel criar o diretrio " + fleDestino.getAbsolutePath());
                }
                throw new ArquivoException("No foi possvel criar o diretrio " + fleDestino.getAbsolutePath());
            }
        }
    }

    //    public static void garantirDiretorioAPartirDoClassesPara(String destino) throws ArquivoException
    //    {
    //        
    //        String[] partesDoDestino = destino.split(File.separator);
    //
    //        int i = 1;
    //        String caminhoEncontrado = partesDoDestino[i-1]; 
    //        for (; i < partesDoDestino.length; i++)
    //        {
    //            caminhoEncontrado
    //        }
    //        
    //        CaminhoAbsoluto.obterAPartirDoClasses(strArquivoDestino);
    //        // Cria um objeto do tipo File apontando para o arquivo fleDestino
    //        File fleDestino = new File(strArquivoDestino);
    //
    //        // Se o caminho para o arquivo fleDestino no existir, criar
    //        if (!fleDestino.getParentFile().exists())
    //        {
    //            // Registra log
    //            if (LOG.isDebugEnabled())
    //            {
    //                LOG.debug("Criando diretorio inexistente: " + fleDestino.getParentFile().toString());
    //            }
    //
    //            if (!fleDestino.getParentFile().mkdirs())
    //            {
    //                if (LOG.isErrorEnabled())
    //                {
    //                    LOG.error("No foi possvel criar o diretrio " + fleDestino.getAbsolutePath());
    //                }
    //                throw new ArquivoException("No foi possvel criar o diretrio " + fleDestino.getAbsolutePath());
    //            }
    //        }
    //    }

    /**
     * Copia o arquivo origem para o arquivo fleDestino especificado
     *
     * @param strArquivoOrigem Arquivo origem (com caminho)
     * @param strArquivoDestino Arquivo fleDestino (com caminho)
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void copiarArquivoBinario(String strArquivoOrigem, String strArquivoDestino) throws ArquivoException
    {

        // Garantir a existencia do diretrio fleDestino
        garantirDiretorioPara(strArquivoDestino);

        try
        {
            // Preparar buffers para a copia dos arquivos
            BufferedInputStream bisEntrada = new BufferedInputStream(new FileInputStream(strArquivoOrigem));
            BufferedOutputStream bosSaida = new BufferedOutputStream(new FileOutputStream(strArquivoDestino));

            // Varivel intByteLido
            int intByteLido;

            // Copiar byte a byte
            while ((intByteLido = bisEntrada.read()) != -1)
            {
                bosSaida.write(intByteLido);
            }

            // Fechar buffers
            bisEntrada.close();
            bosSaida.close();

            // Registra log
            if (LOG.isDebugEnabled())
            {
                LOG.debug("Arquivo " + strArquivoOrigem + " copiado para " + strArquivoDestino);
            }
        }
        catch (IOException ioe)
        {
            if (LOG.isErrorEnabled())
            {
                LOG.error(ioe.getMessage());
            }
            throw new ArquivoException("Ocorreu um erro tentando copiar o arquivo " + strArquivoOrigem + " para " + strArquivoDestino + ".");
        }
    }

    /**
     * Copia o contedo de um diretrio origem para um diretrio fleDestino
     *
     * @param strDiretorioOrigem Diretrio origem
     * @param strDiretorioDestino Diretrio fleDestino
     * @param blRecursivamente Indica se copia tambm os subdiretrios
     * @param strFiltro Especifica qual a sequncia de caracteres que o nome do item do diretrio deve ter para ser
     *        tratado (SEM CURINGAS!)
     * @param strLocalizar String a ser localizada na origem
     * @param strSubstituirPor String a ser gravada no lugar da strLocalizar
     * @param blnSubstituirTambemNoNome Indica se o nome do arquivo deve ser tambm alterado caso possua a string a ser
     *        localizada
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void copiarDiretorio(String strDiretorioOrigem, String strDiretorioDestino, boolean blRecursivamente, String strFiltro, String strLocalizar, String strSubstituirPor, boolean blnSubstituirTambemNoNome, String strFiltroExclusao) throws ArquivoException
    {

        // Preparar objetos / variveis
        File fleDirOrigem = new File(strDiretorioOrigem);
        File fleDirDestino = new File(strDiretorioDestino);
        File[] fleConteudoDiretorio;

        // Recuperar contedo no fleDirOrigem
        fleConteudoDiretorio = fleDirOrigem.listFiles();

        // Registra log
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Copiando o contedo do diretrio " + strDiretorioOrigem + " para " + strDiretorioDestino);
        }

        // Garantir a existencia do diretrio fleDestino
        garantirDiretorioPara(fleDirDestino.toString());

        // Analisar contedo no fleDirOrigem
        for (int i = 0; i < fleConteudoDiretorio.length; i++)
        {
            File fleItemDiretorio = fleConteudoDiretorio[i];

            // Verificar se o item satisfaz ao strFiltro especificado
            if ((fleItemDiretorio.getName().equalsIgnoreCase(strFiltro) || "".equals(strFiltro)) && !fleItemDiretorio.getName().matches(strFiltroExclusao))
            {
                // Se o item for um diretrio
                if (fleItemDiretorio.isDirectory())
                {
                    String strNovoNome = new String(fleItemDiretorio.getName());

                    if (blnSubstituirTambemNoNome)
                    {
                        strNovoNome = Conversao.substituirExpressaoString(strNovoNome, strLocalizar, strSubstituirPor);
                    }

                    // Tratar este diretrio
                    if (blRecursivamente)
                    {
                        copiarDiretorio(fleItemDiretorio.toString(), fleDirDestino.toString()
                                + "\\"
                                + strNovoNome, blRecursivamente, strFiltro, strLocalizar, strSubstituirPor, blnSubstituirTambemNoNome, strFiltroExclusao);
                    }
                }
                else
                {
                    // Verifica se deve ser feita alguma substituio ao copiar
                    if ("".equals(strLocalizar))
                    {
                        // Copiar arquivo sem substituio
                        copiarArquivoBinario(fleItemDiretorio.toString(), fleDirDestino.toString() + "\\" + fleItemDiretorio.getName());
                    }
                    else
                    {
                        // Copiar arquivo com substituio
                        copiarArquivoBinarioComSubstituicao(fleItemDiretorio.toString(), fleDirDestino.toString()
                                + "\\"
                                + fleItemDiretorio.getName(), strLocalizar, strSubstituirPor, blnSubstituirTambemNoNome);
                    }
                }
            }
        }
    }

    /**
     * Copia o contedo de um diretrio origem para um diretrio fleDestino
     *
     * @param strDiretorioOrigem Diretrio origem
     * @param strDiretorioDestino Diretrio fleDestino
     * @param blRecursivamente Indica se copia tambm os subdiretrios
     * @param strFiltro Especifica qual a sequncia de caracteres que o nome do item do diretrio deve ter para ser
     *        tratado (SEM CURINGAS!)
     * @param strLocalizar String a ser localizada na origem
     * @param strSubstituirPor String a ser gravada no lugar da strLocalizar
     * @param blnSubstituirTambemNoNome Indica se o nome do arquivo deve ser tambm alterado caso possua a string a ser
     *        localizada
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void copiarDiretorio(String strDiretorioOrigem, String strDiretorioDestino, boolean blRecursivamente, String strFiltro, String strLocalizar, String strSubstituirPor, boolean blnSubstituirTambemNoNome) throws ArquivoException
    {
        copiarDiretorio(strDiretorioOrigem, strDiretorioDestino, blRecursivamente, strFiltro, strLocalizar, strSubstituirPor, blnSubstituirTambemNoNome, "");
    }

    /**
     * Copia o contedo de um diretrio origem para um diretrio fleDestino
     *
     * @param strDiretorioOrigem Diretrio origem
     * @param strDiretorioDestino Diretrio fleDestino
     * @param blRecursivamente Indica se copia tambm os subdiretrios
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void copiarDiretorio(String strDiretorioOrigem, String strDiretorioDestino, boolean blRecursivamente) throws ArquivoException
    {

        // Copiar diretrio
        copiarDiretorio(strDiretorioOrigem, strDiretorioDestino, blRecursivamente, "", "", "", false);
    }

    /**
     * Copia o contedo de um diretrio origem para um diretrio fleDestino
     *
     * @param strDiretorioOrigem Diretrio origem
     * @param strDiretorioDestino Diretrio fleDestino
     * @param blRecursivamente Indica se copia tambm os subdiretrios
     * @param strFiltro Especifica qual a sequncia de caracteres que o nome do item do diretrio deve ter para ser
     *        tratado (SEM CURINGAS!)
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void copiarDiretorio(String strDiretorioOrigem, String strDiretorioDestino, boolean blRecursivamente, String strFiltro) throws ArquivoException
    {
        // Copiar diretrio
        copiarDiretorio(strDiretorioOrigem, strDiretorioDestino, blRecursivamente, strFiltro, "", "", false);
    }

    /**
     * Copia o arquivo origem para o arquivo fleDestino especificado substituindo uma sequncia do primeiro por uma outra
     * no segundo
     *
     * @param strArquivoOrigem Arquivo origem (com caminho)
     * @param strArquivoDestino Arquivo fleDestino (com caminho)
     * @param strLocalizar String a ser localizada na origem
     * @param strSubstituirPor String a ser gravada no lugar da strLocalizar
     * @param blnSubstituirTambemNoNome Indica se o nome do arquivo deve ser tambm alterado caso possua a string a ser
     *        localizada
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void copiarArquivoBinarioComSubstituicao(String strArquivoOrigem, String strArquivoDestino, String strLocalizar, String strSubstituirPor, boolean blnSubstituirTambemNoNome) throws ArquivoException
    {
        // Variveis
        File fleArquivoTemp = new File(strArquivoDestino);
        String strNovoNome = fleArquivoTemp.getName();

        if (blnSubstituirTambemNoNome)
        {
            strNovoNome = Conversao.substituirExpressaoString(strNovoNome, strLocalizar, strSubstituirPor);
        }

        // Garantir a existencia do diretrio fleDestino
        Arquivo.garantirDiretorioPara(fleArquivoTemp.getParent() + "\\" + strNovoNome);

        try
        {
            // Preparar buffers para a copia dos arquivos
            BufferedInputStream bisEntrada = new BufferedInputStream(new FileInputStream(strArquivoOrigem));
            BufferedOutputStream bosSaida = new BufferedOutputStream(new FileOutputStream(fleArquivoTemp.getParent() + "\\" + strNovoNome));

            // Variveis
            int intByteLido = -1;
            int intPosicao = 0;
            boolean blnContinuar = true;
            StringBuilder stbBufferString = new StringBuilder();

            // Ler primeiro byte e verificar se deve blnContinuar a leitura
            blnContinuar = ((intByteLido = bisEntrada.read()) != -1);

            if (blnContinuar)
            {
                stbBufferString.append((char) intByteLido);
            }

            // Copiar byte a byte
            while (blnContinuar)
            {
                if (strLocalizar.charAt(intPosicao) == stbBufferString.charAt(intPosicao))
                {
                    intPosicao++;

                    if (intPosicao >= strLocalizar.length())
                    {
                        // Gravar a string a substituir, no lugar da string a pesquisar
                        for (int j = 0; j < strSubstituirPor.length(); j++)
                        {
                            bosSaida.write((int) strSubstituirPor.charAt(j));
                        }

                        // Limpar buffer
                        stbBufferString.delete(0, stbBufferString.length());
                        intPosicao = 0;
                    }

                    // Ler proximo byte
                    intByteLido = bisEntrada.read();

                    // Validar byte lido
                    if (intByteLido != -1)
                    {
                        // Como no  fim de arquivo, blnContinuar processamento
                        blnContinuar = true;
                        stbBufferString.append((char) intByteLido);
                    }
                    else
                    {
                        // Fim de arquivo, no blnContinuar processamento
                        blnContinuar = false;

                        // Gravar buffer eventualmente existente
                        if (!"".equals(stbBufferString.toString()))
                        {
                            for (int j = 0; j < stbBufferString.length(); j++)
                            {
                                bosSaida.write((int) stbBufferString.charAt(j));
                            }
                            stbBufferString.delete(0, stbBufferString.length());
                        }
                    }
                }
                else
                {
                    // Gravar buffer eventualmente existente
                    if (!"".equals(stbBufferString.toString()))
                    {
                        for (int j = 0; j < stbBufferString.length(); j++)
                        {
                            bosSaida.write((int) stbBufferString.charAt(j));
                        }
                    }

                    // Limpar buffer
                    stbBufferString.delete(0, stbBufferString.length());
                    intPosicao = 0;

                    // Ler proximo byte
                    intByteLido = bisEntrada.read();

                    // Validar byte lido
                    if (intByteLido != -1)
                    {
                        // Como no  fim de arquivo, blnContinuar processamento
                        blnContinuar = true;
                        stbBufferString.append((char) intByteLido);
                    }
                    else
                    {
                        // Fim de arquivo, no blnContinuar processamento
                        blnContinuar = false;
                    }
                }
            }

            // Fechar buffers
            bisEntrada.close();
            bosSaida.close();

            // Registra log
            if (LOG.isDebugEnabled())
            {
                LOG.debug("Arquivo "
                        + strArquivoOrigem
                        + " copiado para "
                        + fleArquivoTemp.getParent()
                        + "\\"
                        + strNovoNome
                        + " substituindo as ocorrncias de "
                        + strLocalizar
                        + " por "
                        + strSubstituirPor);
            }
        }
        catch (IOException ioe)
        {
            if (LOG.isErrorEnabled())
            {
                LOG.error(ioe.getMessage());
            }

            throw new ArquivoException("Ocorreu um erro tentando copiar (substituindo) o arquivo "
                    + strArquivoOrigem
                    + " para "
                    + fleArquivoTemp.getParent()
                    + "\\"
                    + strNovoNome
                    + ".");
        }
    }

    /**
     * DOCUMENTAR!
     *
     * @param strArquivoOrigem DOCUMENTAR!
     * @param strArquivoDestino DOCUMENTAR!
     *
     * @throws ArquivoException DOCUMENTAR!
     */
    public static void moverArquivo(String strArquivoOrigem, String strArquivoDestino) throws ArquivoException
    {
        // Preparar objetos / variveis
        File fleArqOrigem = new File(strArquivoOrigem);

        Arquivo.copiarArquivoBinario(strArquivoOrigem.toString(), strArquivoDestino.toString());
        fleArqOrigem.delete();
    }

    /**
     * Copia o contedo de um diretrio origem para um diretrio fleDestino
     *
     * @param strDiretorioOrigem Diretrio origem
     * @param strDiretorioDestino Diretrio fleDestino
     *
     * @throws ArquivoException Exceo caso ocorra algum erro
     */
    public static void moverDiretorio(String strDiretorioOrigem, String strDiretorioDestino) throws ArquivoException
    {
        // Preparar objetos / variveis
        File fleDirOrigem = new File(strDiretorioOrigem);
        File fleDirDestino = new File(strDiretorioDestino);
        File[] fleConteudoDiretorio;

        // Recuperar contedo no fleDirOrigem
        fleConteudoDiretorio = fleDirOrigem.listFiles();

        // Registra log
        if (LOG.isDebugEnabled())
        {
            LOG.debug("Movendo o contedo do diretrio " + strDiretorioOrigem + " para " + strDiretorioDestino);
        }

        // Garantir a existencia do diretrio fleDestino
        garantirDiretorioPara(fleDirDestino.toString());

        // Analisar contedo no fleDirOrigem
        for (int i = 0; i < fleConteudoDiretorio.length; i++)
        {
            File fleItemDiretorio = fleConteudoDiretorio[i];

            // Se o item for um diretrio
            if (fleItemDiretorio.isDirectory())
            {
                moverDiretorio(fleItemDiretorio.toString(), fleDirDestino.toString() + "\\" + fleItemDiretorio.getName());
            }
            else
            {
                // Copiar arquivo sem substituio
                copiarArquivoBinario(fleItemDiretorio.toString(), fleDirDestino.toString() + "\\" + fleItemDiretorio.getName());
                fleItemDiretorio.delete();
            }
        }
        fleDirOrigem.delete();
    }

    public static void compactarArquivos(List<String> nomesDeArquivosACompactar, String nomeArquivo) throws CDException, IOException
    {
        if (nomesDeArquivosACompactar == null || nomesDeArquivosACompactar.isEmpty())
        {
            CDException.dispararExcecao("A lista de arquivos a compactar  nula ou est vazia.");
        }

        if (nomesDeArquivosACompactar != null && !nomesDeArquivosACompactar.isEmpty())
        {
            ZipOutputStream zip = null;
            try
            {
                Arquivo.garantirDiretorioPara(nomeArquivo);

                // Create the ZIP file
                zip = new ZipOutputStream(new FileOutputStream(nomeArquivo));
                // Define o tamanho do buffer
                byte[] buf = new byte[1024];

                for (String nomeArquivoCompactar : nomesDeArquivosACompactar)
                {
                    InputStream in = null;
                    try
                    {
                        in = new FileInputStream(nomeArquivoCompactar);

                        // Add ZIP entry to output stream.
                        zip.putNextEntry(new ZipEntry(nomeArquivoCompactar));

                        // Transfer bytes from the file to the ZIP file
                        int len;
                        while ((len = in.read(buf)) > 0)
                        {
                            zip.write(buf, 0, len);
                        }

                        // Complete the entry
                        zip.closeEntry();
                    }
                    finally
                    {
                        fechar(in);
                    }
                }
            }
            finally
            {
                fechar(zip);
            }
        }
    }

    public static void apagarArquivos(List<String> nomesDeArquivosAExcluir)
    {
        if (nomesDeArquivosAExcluir == null || nomesDeArquivosAExcluir.isEmpty())
        {
            return;
        }

        for (String nomeArquivoAExcluir : nomesDeArquivosAExcluir)
        {
            apagarArquivo(nomeArquivoAExcluir);
        }
    }

    public static void apagarArquivo(String nomeArquivoAExcluir)
    {
        if (nomeArquivoAExcluir == null)
        {
            return;
        }

        File arquivoExcluir = new File(nomeArquivoAExcluir);
        if (arquivoExcluir.exists())
        {
            arquivoExcluir.delete();
        }
    }

    public static void fecharLista(List<? super Closeable> objetosASeremFechados) throws IOException
    {
        if (objetosASeremFechados != null)
        {
            for (Object objetoASerFechado : objetosASeremFechados)
            {
                fechar((Closeable) objetoASerFechado);
            }
        }
    }

    public static void fechar(Closeable objetoASerFechado) throws IOException
    {
        if (objetoASerFechado != null)
        {
            objetoASerFechado.close();
        }
    }

}
