/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package sigesp.comum.util.biblioteca;

import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Classe que contm funes genricas de converso
 */
public class Conversao
{
	// Inicializa o log
	private static Log log = LogFactory.getLog(Conversao.class.getName());

	/**
	 * Funo que separa partes de uma expresso dado um delimitador
	 *
	 * @param strExpressao Expresso a ser separada
	 * @param strDelimitador Delimitador que ser utilizado para a separao
	 *
	 * @return Array de strings contendo as partes que foram separadas
	 */
	public static String[] split(String strExpressao, String strDelimitador)
	{
		// Registra log
		if (log.isDebugEnabled())
		{
			log.debug("Entrada no mtodo");
		}

		StringTokenizer stkTokenizer = new StringTokenizer(strExpressao,
				strDelimitador);
		String[] strTemp = new String[stkTokenizer.countTokens()];

		int i = 0;

		// Armazena tokens em um array
		while (stkTokenizer.hasMoreTokens())
		{
			strTemp[i++] = stkTokenizer.nextToken();
		}

		// Retorno
		return strTemp;
	}

	/**
	 * Funo que substitui numa string uma expresso por outra
	 *
	 * @param strString String contendo a expresso a ser substituda
	 * @param strExpressao Expresso a ser substituda
	 * @param strNovaExpressao Nova expresso
	 *
	 * @return Retorna uma nova string com contendo a expresso substituda
	 */
	public static String substituirExpressaoString(String strString,
		String strExpressao, String strNovaExpressao)
	{
		// Registra log
		if (log.isDebugEnabled())
		{
			log.debug("Entrada no mtodo");
		}

		int intPosicao = -1;
		StringBuffer stbNovaString = new StringBuffer(strString);

		while ((intPosicao = stbNovaString.indexOf(strExpressao)) >= 0)
		{
			stbNovaString.replace(intPosicao,
				intPosicao + strExpressao.length(), strNovaExpressao);
		}

		return stbNovaString.toString();
	}

	/**
	 * formatarSomentePrimeiraMaiuscula: recebe uma expresso e retorna o novo
	 * nome com apenas a primeira letra maiscula.
	 *
	 * @param strExpressao Expressao a formatar.
	 *
	 * @return Expressao formatada com a inicial em maisculo.
	 */
	public static String formatarSomentePrimeiraMaiuscula(String strExpressao)
	{
		// Registra log
		if (log.isDebugEnabled())
		{
			log.debug("Entrada no mtodo");
		}

		String strRestante = strExpressao.substring(1).toLowerCase();
		String strPrimeiraLetra = "" + strExpressao.charAt(0);
		strPrimeiraLetra = strPrimeiraLetra.toUpperCase();

		return (strPrimeiraLetra + strRestante);
	}
}
