/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package sigesp.comum.util.exception;

import java.text.DecimalFormat;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Classe padro de excees
 */
public class SigespException extends Exception
{
    private Log log = LogFactory.getLog(this.getClass());
    private int intIdentificadorErro = -1;
    protected String strMensagemUsuario = null;

    /**
     * Cria um novo objeto SigespException
     */
    public SigespException()
    {
        super("Erro inesperado no sistema");
        gerarLog();
    }

    /**
     * Cria um novo objeto SigespException
     *
     * @param strMensagem Mensagem da exceo
     */
    public SigespException(String strMensagem)
    {
        super(strMensagem);
        gerarLog();
    }

    /**
     * Cria um novo objeto SigespException
     *
     * @param strMensagem Mensagem da exceo
     * @param strMensagemUsuario Mensagem para o usurio
     */
    public SigespException(String strMensagem, String strMensagemUsuario)
    {
        super(strMensagem);
        this.strMensagemUsuario = strMensagemUsuario;
        gerarLog();
    }

    /**
     * Cria um novo objeto SigespException
     *
     * @param cause Causa da exceo
     */
    public SigespException(Throwable cause)
    {
        super("Erro inesperado", cause);
        intIdentificadorErro = ((SigespException) cause).getIdentificadorErro();
        gerarLog();
    }

    /**
     * Cria um novo objeto SigespException
     *
     * @param strMensagem Mensagem da exceo
     * @param cause Causa da exceo
     */
    public SigespException(String strMensagem, Throwable cause)
    {
        super(strMensagem, cause);
        intIdentificadorErro = ((SigespException) cause).getIdentificadorErro();
        strMensagemUsuario = ((SigespException) cause).getMensagemUsuario();
        gerarLog();
    }

    /**
     * Gera mensagem de log
     *
     */
    protected void gerarLog()
    {
        // Gera log
        if (log.isErrorEnabled())
        {
            DecimalFormat dcfFormato = new DecimalFormat("000000");

            if (getCause() == null)
            {
                intIdentificadorErro = 1 + (int) (Math.random() * 999999);
                StackTraceElement[] steException = getStackTrace();
                for (int i = 0; i < steException.length; i++)
                {
                    log.error(dcfFormato.format(intIdentificadorErro) + " | ST: " + steException[i]);
                }
            }
            log.error(dcfFormato.format(intIdentificadorErro) + " | GM: " + getMessage() + " | " + getClass());
        }
    }

    /**
     * Mtodo getter para mensagem de usurio
     *
     * @return String contendo a mensagem para o usurio
     */
    public String getMensagemUsuario()
    {
        return strMensagemUsuario;
    }

    /**
     * Mtodo getter para identificador do erro
     *
     * @return Identificado do erro
     */
    public int getIdentificadorErro()
    {
        return intIdentificadorErro;
    }

    /**
     * Obtm mensagem gerada para o usurio
     *
     * @return String contendo a mensagem
     */
    public String obterMensagem()
    {
        if (strMensagemUsuario != null)
        {
            return strMensagemUsuario;
        }
        else
        {
            return getMessage();
        }
    }

    /**
     * Obtm toda a pilha de excees ocorrida
     *
     * @return String com a mensagem da pilha
     */
    public String obterExcecoes()
    {
        String strRetorno = getMessage() + "\n";

        Throwable throwable = getCause();
        while (throwable != null)
        {
            strRetorno += (throwable.getMessage() + "\n");
            throwable = throwable.getCause();
        }

        return strRetorno;
    }
}
