/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

/*
 * Created on 19/11/2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package sigesp.comum.util.hibernate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * Conteiner dos dados associados com uma conexo:
 *   - nome da conexo 
 *   - propriedades da conexo
 *   - mdulos (arq. XML contendo os mapeamentos) associados  conexo
 *   - mapeamentos da conexo
 *   - substituies da conexo 
 */
public class DadosConexao
{
    private List dadosConexao = null;

    private static final int POSICAO_NOMES_CONEXAO = 0;
    private static final int POSICAO_PROPRIEDADES_CONEXAO = 1;
    private static final int POSICAO_MAPEAMENTOS_CONEXAO = 2;
    private static final int POSICAO_MODULOS_CONEXAO = 3;
    private static final int POSICAO_SUBSTITUICOES_CONEXAO = 4;
    private static final int POSICAO_INTERCEPTORS_CONEXAO = 5;
    private static final int POSICAO_CONTAINER_CONEXAO = 6;
    private static final int POSICAO_MAPEAMENTOS_TABELAS = 7;
    private static final int TAMANHO_POSICOES = 8;

    /**
     * Inicializa o conteiner da conexo e seus dados
     */
    public DadosConexao()
    {
        dadosConexao = new ArrayList(TAMANHO_POSICOES);
        for (int i = 0; i < TAMANHO_POSICOES; i++)
        {
            dadosConexao.add(null);
        }
        dadosConexao.set(POSICAO_NOMES_CONEXAO, new ArrayList());
        dadosConexao.set(POSICAO_PROPRIEDADES_CONEXAO, new Properties());
        dadosConexao.set(POSICAO_MAPEAMENTOS_CONEXAO, new ArrayList());
        dadosConexao.set(POSICAO_MODULOS_CONEXAO, new HashMap());
        dadosConexao.set(POSICAO_SUBSTITUICOES_CONEXAO, new Properties());
        dadosConexao.set(POSICAO_INTERCEPTORS_CONEXAO, new HashMap());
        dadosConexao.set(POSICAO_CONTAINER_CONEXAO, new HashMap());
        dadosConexao.set(POSICAO_MAPEAMENTOS_TABELAS, new ArrayList());
    }

    /**
     * Obtm o nome da conexo
     * 
     * @return String contendo o nome da conexo
     */
    public List getNomesConexao()
    {
        return (List) dadosConexao.get(POSICAO_NOMES_CONEXAO);
    }

    /**
     * Obtm as propriedades da conexo
     * 
     * @return Properties propriedades da conexo
     */
    public Properties getPropriedadesConexao()
    {
        return (Properties) dadosConexao.get(POSICAO_PROPRIEDADES_CONEXAO);
    }

    /**
     * Obtm os mapeamentos da conexo
     * 
     * @return List lista com os mapeamentos da conexo
     */
    public List getMapeamentosConexao()
    {
        return (List) dadosConexao.get(POSICAO_MAPEAMENTOS_CONEXAO);
    }

    /**
     * Obtm os mapeamentos das tabelas
     * 
     * @return List lista com os mapeamentos das tabelas
     */
    public List getMapeamentosTabelas()
    {
        return (List) dadosConexao.get(POSICAO_MAPEAMENTOS_TABELAS);
    }

    /**
     * Obtm os mdulos associados  conexo
     * 
     * @return List lista com os mdulos da conexo
     */
    public Map getModulosConexao()
    {
        return (Map) dadosConexao.get(POSICAO_MODULOS_CONEXAO);
    }

    /**
     * Obtm as substituies associadas  conexo
     * 
     * @return Properties propriedades com as substituies da conexo
     */
    public Properties getSubstituicoesConexao()
    {
        return (Properties) dadosConexao.get(POSICAO_SUBSTITUICOES_CONEXAO);
    }

    /**
     * Obtm os interceptors associados  conexo
     * 
     * @return Properties propriedades com os interceptors da conexo
     */
    private Map getInterceptorsConexao()
    {
        return (Map) dadosConexao.get(POSICAO_INTERCEPTORS_CONEXAO);
    }

    /**
     * Atribui o nome da conexo
     * 
     * @param nomeConexao
     *        Nome da conexo
     */
    public void setNomesConexao(List nomesConexao)
    {
        dadosConexao.set(POSICAO_NOMES_CONEXAO, nomesConexao);
    }

    /**
     * Atribui as propriedades da conexo
     * 
     * @param propriedadesConexao
     *        Propriedades da conexo
     */
    public void setPropriedadesConexao(Properties propriedadesConexao)
    {
        dadosConexao.set(POSICAO_PROPRIEDADES_CONEXAO, propriedadesConexao);
    }

    /**
     * Atribui os mapeamentos da conexo
     * 
     * @param mapeamentosConexao
     *        Mapeamentos da conexo
     */
    public void setMapeamentosConexao(List mapeamentosConexao)
    {
        dadosConexao.set(POSICAO_MAPEAMENTOS_CONEXAO, mapeamentosConexao);
    }

    /**
     * Atribui os mapeamentos das tabelas
     * 
     * @param mapeamentosConexao
     *        Mapeamentos da conexo
     */
    public void setMapeamentosTabelas(List mapeamentosTabelas)
    {
        dadosConexao.set(POSICAO_MAPEAMENTOS_TABELAS, mapeamentosTabelas);
    }

    /**
     * Atribui os mdulos (nomes de arquivo XML contento a relao de
     * mapeamentos) da conexo
     * 
     * @param modulosConexao
     */
    public void setModulosConexao(Map modulosConexao)
    {
        dadosConexao.set(POSICAO_MODULOS_CONEXAO, modulosConexao);
    }

    /**
     * Atribui as substituies da conexo
     * 
     * @param substituicoesConexao
     */
    public void setSubstituicoesConexao(Properties substituicoesConexao)
    {
        dadosConexao.set(POSICAO_SUBSTITUICOES_CONEXAO, substituicoesConexao);
    }

    /**
     * Atribui os interceptors da conexo
     * 
     * @param interceptorsConexao
     */
    public void addInterceptorConexao(String tipo, String nomeClasseInterceptor)
    {
        List interceptors = (List) (getInterceptorsConexao().get(tipo));
        if (interceptors == null)
        {
            interceptors = new ArrayList();
            getInterceptorsConexao().put(tipo, interceptors);

        }
        interceptors.add(nomeClasseInterceptor);
    }

    /**
     * Atribui os interceptors da conexo
     * 
     * @param interceptorsConexao
     */
    public List getInterceptorsPorTipo(String tipo)
    {
        List interceptors = (List) (getInterceptorsConexao().get(tipo));
        return interceptors;
    }

    public Object getObjetoConexao(String chave)
    {
        return ((Map) dadosConexao.get(POSICAO_CONTAINER_CONEXAO)).get(chave);
    }

    public void setObjetoConexao(String chave, Object objeto)
    {
        ((Map) dadosConexao.get(POSICAO_CONTAINER_CONEXAO)).put(chave, objeto);
    }
}
