/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package sigesp.comum.util.tag;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Classe Tag Handler para grade
 */
public class GradeTag extends TagSupport
{
    private static final long serialVersionUID = -8652307625184058320L;
    private String strLinha = "impar";
    private String strBorda = "0";
    private String strLargura = "";
    private String strLista = "";
    private String strQtdeMaxima = "0";
    private String strPagina = "";
    private int intPagina = 0;
    private int intTotalPaginas = 1;
    private List arlConteudo = new ArrayList();
    private List arlTitulo = new ArrayList();
    private List arlLarguraColuna = new ArrayList();

    /**
     * Obtm configurao de linha
     *
     * @return Contedo de linha
     */
    public String getLinha()
    {
        return strLinha;
    }

    /**
     * Atribui configurao de linha
     *
     * @param strLinha Contedo de linha
     */
    public void setLinha(String strLinha)
    {
        this.strLinha = strLinha;
    }

    /**
     * Obtm configurao de borda
     *
     * @return Contedo de borda
     */
    public String getBorda()
    {
        return strBorda;
    }

    /**
     * Atribui configurao de borda
     *
     * @param strBorda Contedo de borda
     */
    public void setBorda(String strBorda)
    {
        this.strBorda = strBorda;
    }

    /**
     * Obtm largura
     *
     * @return Contedo de largura
     */
    public String getLargura()
    {
        return strLargura;
    }

    /**
     * Atribui largura
     *
     * @param strLargura Contedo de largura
     */
    public void setLargura(String strLargura)
    {
        this.strLargura = strLargura;
    }

    /**
     * Obtm lista
     *
     * @return Lista
     */
    public String getLista()
    {
        return strLista;
    }

    /**
     * Atribui lista
     *
     * @param strLista Lista
     */
    public void setLista(String strLista)
    {
        this.strLista = strLista;
    }

    /**
     * Obtm configurao de quantidade de registros por pgina
     *
     * @return Contedo de quantidade de registros por pgina
     */
    public String getQtdeMaxima()
    {
        return strQtdeMaxima;
    }

    /**
     * Atribui configurao de quantidade de registros por pgina
     *
     * @param strQtdeMaxima Contedo de quantidade de registros por pgina
     */
    public void setQtdeMaxima(String strQtdeMaxima)
    {
        if (Integer.parseInt(strQtdeMaxima) < 0)
        {
            throw new RuntimeException("A quantidade mxima no pode ser negativa");
        }
        this.strQtdeMaxima = strQtdeMaxima;
    }

    /**
     * Obtm nome da pgina
     *
     * @return Contedo do nome da pgina
     */
    public String getPagina()
    {
        return strPagina;
    }

    /**
     * Atribui nome da pgina
     *
     * @param strPagina Contedo do nome da pgina
     */
    public void setPagina(String strPagina)
    {
        if ("".equals(strPagina))
        {
            throw new RuntimeException("A pgina no pode ser vazia");
        }

        String[] strURL = strPagina.split("?");
        this.strPagina = strURL[0] + "?";
        if (strURL.length == 2)
        {
            String[] strParam = strURL[1].split("&");
            for (int i = 0; i < strParam.length; i++)
            {
                String[] strValor = strParam[i].split("=");
                if ((strValor.length == 2) && !strValor[0].equals("pagina"))
                {
                    this.strPagina += (strValor[0] + "=" + strValor[1] + "&");
                }
            }
        }
    }

    /**
     * Obtm contedo a ser exibido
     *
     * @return Contedo a ser exibido
     */
    public List getConteudo()
    {
        return arlConteudo;
    }

    /**
     * Atribui contedo a ser exibido
     *
     * @param strConteudo Contedo a ser exibido
     */
    public void setConteudo(List arlConteudo)
    {
        this.arlConteudo = arlConteudo;
    }

    /**
     * Obtm ttulos a serem exibidos
     *
     * @return Contedo dos ttulos a serem exibidos
     */
    public List getTitulo()
    {
        return arlTitulo;
    }

    /**
     * Atribui ttulos a serem exibidos
     *
     * @param strTitulo Contedo dos ttulos a serem exibidos
     */
    public void setTitulo(List arlTitulo)
    {
        this.arlTitulo = arlTitulo;
    }

    /**
     * Obtm larguras de coluna
     *
     * @return Contedo das larguras de coluna
     */
    public List getLarguraColuna()
    {
        return arlLarguraColuna;
    }

    /**
     * Atribui larguras de coluna
     *
     * @param LarguraColuna Contedo das larguras de coluna
     */
    public void setLarguraColuna(ArrayList arlLarguraColuna)
    {
        this.arlLarguraColuna = arlLarguraColuna;
    }

    /**
     * Inicializao da tag
     *
     * @return Ao da tag
     */
    public int doStartTag()
    {
        try
        {
            JspWriter out = pageContext.getOut();
            out.print("<table class=\"table\" cellspacing=\"0\" cellpadding=\"4\" bordercolor=\"#FFFFFF\" "
                    + "bordercolorlight=\"#8D8D8D\" border=\" "
                    + strBorda
                    + "\" "
                    + ("".equals(strLargura) ? "" : (" width=\"" + strLargura + "\""))
                    + ">");
        }
        catch (IOException ioe)
        {}

        return (EVAL_BODY_INCLUDE);
    }

    /**
     * Finalizao da tag
     *
     * @return Ao da tag
     */
    public int doEndTag()
    {
        try
        {
            JspWriter out = pageContext.getOut();

            // Obtm colunas e linhas para impresso
            Iterator itrTitulo = arlTitulo.iterator();
            String[][] strTitulo = null;
            Iterator itrLarguraColuna = arlLarguraColuna.iterator();
            String[][] strLarguraColuna = null;
            Iterator itrConteudo = arlConteudo.iterator();
            String[][] strGrade = null;
            int intLinha = 0;
            int intColuna = 0;

            while (itrTitulo.hasNext())
            {
                // Verifica se j foi instanciado o array
                if (strTitulo == null)
                {
                    strTitulo = new String[1][arlTitulo.size()];
                }

                // Obtm valores dos ttulos
                strTitulo[0][intColuna++] = (String) itrTitulo.next();
            }

            intColuna = 0;
            while (itrLarguraColuna.hasNext())
            {
                // Verifica se j foi instanciado o array
                if (strLarguraColuna == null)
                {
                    strLarguraColuna = new String[1][arlLarguraColuna.size()];
                }

                // Obtm valores dos ttulos
                strLarguraColuna[0][intColuna++] = (String) itrLarguraColuna.next();
            }

            while (itrConteudo.hasNext())
            {
                // Obtm valores das colunas
                List arlColuna = (ArrayList) itrConteudo.next();
                Iterator itrColuna = arlColuna.iterator();
                int intColunaConteudo = 0;

                // Verifica se j foi instanciado o array
                if (strGrade == null)
                {
                    strGrade = new String[arlConteudo.size()][arlColuna.size()];
                }

                // Alimenta array
                while (itrColuna.hasNext())
                {
                    strGrade[intLinha][intColunaConteudo++] = itrColuna.next().toString();
                }

                // Incrementa linha
                intLinha++;
            }

            // Troca posio dos arrays
            String[][] strTemp = new String[strGrade[0].length][strGrade.length];
            for (int i = 0; i < strGrade.length; i++)
            {
                for (int j = 0; j < strGrade[i].length; j++)
                {
                    strTemp[j][i] = strGrade[i][j];
                }
            }
            strGrade = strTemp;

            // Verifica quantidade de registros para exibio
            if (Integer.parseInt(strQtdeMaxima) == 0)
            {
                strQtdeMaxima = String.valueOf(strGrade.length);
            }

            // Faz contagem de pginas
            if ((Integer.parseInt(strQtdeMaxima) > 0) && (pageContext.getRequest().getParameter("pagina") == null))
            {
                intTotalPaginas = strGrade.length / Integer.parseInt(strQtdeMaxima);
                if ((strGrade.length % Integer.parseInt(strQtdeMaxima)) > 0)
                {
                    intTotalPaginas++;
                }
                intPagina = 1;
            }

            // Seta pgina atual
            if (pageContext.getRequest().getParameter("pagina") != null)
            {
                intPagina = Integer.parseInt(pageContext.getRequest().getParameter("pagina"));
            }

            // Monta navegador de pginas
            String strNavegador = "";
            if (intTotalPaginas > 1)
            {
                String strAnterior = ((intPagina > 1) ? ("<a href=\"" + strPagina + "pagina=" + (intPagina - 1) + "\"><<</a>") : "");
                String strProximo = ((intPagina < intTotalPaginas) ? ("<a href=\"" + strPagina + "pagina=" + (intPagina + 1) + "\">>></a>") : "");
                strNavegador = " " + strAnterior + intPagina + " de " + intTotalPaginas + strProximo;
            }

            // Cria cabealho da tabela
            for (int i = 0; i < strTitulo.length; i++)
            {
                out.print("<tr class=\"tableRowHeader\">");

                for (int j = 0; j < strTitulo[i].length; j++)
                {
                    out.print("<td class=\"tableCell\" "
                            + ("".equals(strLarguraColuna[i][j]) ? "" : (" width=\"" + strLarguraColuna[i][j] + "\""))
                            + ">"
                            + strTitulo[i][j]
                            + "</td>");
                }
                out.print("</tr>");
            }

            // Cria linhas e colunas da tabela
            for (int i = (Integer.parseInt(strQtdeMaxima) * intPagina) - Integer.parseInt(strQtdeMaxima); i < (Integer.parseInt(strQtdeMaxima)
                    * intPagina); i++)
            {
                if (i < strGrade.length)
                {
                    if ("impar".equals(getLinha()))
                    {
                        out.print("<tr class=\"tableRowOdd\">");
                        setLinha("par");
                    }
                    else
                    {
                        out.print("<tr class=\"tableRowEven\">");
                        setLinha("impar");
                    }
                    for (int j = 0; j < strGrade[i].length; j++)
                    {
                        out.print("<td class=\"tableCell\">" + strGrade[i][j] + "</td>");
                    }
                }
                out.println("</tr>");
            }

            // Verifica se existe navegador
            if (!"".equals(strNavegador))
            {
                out.print("<tr class=\"tableRowHeader\"><td class=\"tableCell\" colspan=\""
                        + strGrade[0].length
                        + "\" align=\"right\">"
                        + strNavegador
                        + "</td></tr>");
            }
            out.print("</table>");

            // Limpa contedo
            strLinha = "impar";
            arlTitulo.clear();
            arlLarguraColuna.clear();
            arlConteudo.clear();
        }
        catch (IOException ioe)
        {}

        return (EVAL_PAGE);
    }
}
