CREATE TABLE IF NOT EXISTS TABELAAPOIOMM (
  IDETABELAAPOIOMM INT UNSIGNED NOT NULL,
  IDETABELAAPOIOMESTREMM INT UNSIGNED NULL,
  NOMIDENTIFICADORTABELAAPOIO VARCHAR(255) NULL,
  NOMDESCRITORTABELAAPOIO VARCHAR(255) NULL,
  NOMIDENTIFICADORTABELAMESTRE VARCHAR(255) NULL,
  NOMTABELAAPOIOMM VARCHAR(255) NULL,
  PRIMARY KEY(IDETABELAAPOIOMM),
  UNIQUE INDEX UKTABAPOMM_NOMTABAPOMM(NOMTABELAAPOIOMM),
  FOREIGN KEY(IDETABELAAPOIOMESTREMM)
    REFERENCES TABELAAPOIOMM(IDETABELAAPOIOMM)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS TABELAAPOIOMM_FKIndex1 ON TABELAAPOIOMM(IDETABELAAPOIOMESTREMM);

CREATE TABLE IF NOT EXISTS TIPOFILTROCONSULTA (
  IDETIPOFILTROCONSULTA INT UNSIGNED NOT NULL,
  NOMTIPOFILTROCONSULTA VARCHAR(255) NOT NULL,
  NOMOBJETOCONTROLADO VARCHAR(255) NOT NULL,
  PRIMARY KEY(IDETIPOFILTROCONSULTA),
  UNIQUE INDEX TIPOFILTROCONSULTA_index1858(NOMOBJETOCONTROLADO)
);

CREATE TABLE IF NOT EXISTS SISTEMA (
  IDESISTEMA INT UNSIGNED NOT NULL,
  NOMSISTEMA VARCHAR(255) NULL,
  DESSISTEMA VARCHAR(255) NULL,
  IDESISTEMAAGRUPADOR INT UNSIGNED NULL,
  NOMOBJETOCONTROLADO VARCHAR(255) NULL,
  PRIMARY KEY(IDESISTEMA),
  FOREIGN KEY(IDESISTEMAAGRUPADOR)
    REFERENCES SISTEMA(IDESISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS SISTEMA_FKIndex1 ON SISTEMA(IDESISTEMAAGRUPADOR);

CREATE TABLE IF NOT EXISTS GRUPO (
  CODGRUPO INT UNSIGNED NOT NULL,
  DESGRUPO VARCHAR(255) NULL,
  PRIMARY KEY(CODGRUPO)
);

CREATE TABLE IF NOT EXISTS CATEGORIATALENTO (
  IDECATEGORIA INT UNSIGNED NOT NULL,
  NOMCATEGORIA VARCHAR(255) NULL,
  DESCATEGORIA TEXT NULL,
  DESDICAPREENCHIMENTO VARCHAR(255) NULL,
  INDUNICIDADE CHAR(1) NULL DEFAULT 'N',
  SEQORDENACAO TINYINT UNSIGNED NULL,
  PRIMARY KEY(IDECATEGORIA)
);

CREATE TABLE IF NOT EXISTS TIPOHTML (
  IDETIPOHTML INT UNSIGNED NOT NULL,
  DESTIPOHTML VARCHAR(255) NULL,
  INDMULTIPLICIDADE CHAR(1) NULL,
  PRIMARY KEY(IDETIPOHTML),
  UNIQUE INDEX UKTIPOHTML_DESTIPOHTML(DESTIPOHTML)
);

CREATE TABLE IF NOT EXISTS USUARIOSISTEMA (
  IDEUSUARIO INT UNSIGNED NOT NULL,
  NOMUSUARIO VARCHAR(255) NOT NULL,
  DESLOGIN VARCHAR(255) NULL,
  NUMSENHAHASH VARCHAR(255) NULL,
  INDEXPIRACAOSENHA CHAR(1) NULL,
  DATATUALIZACAOSENHA DATETIME NULL,
  DATVALIDADE DATETIME NULL,
  DATDESLIGAMENTO DATETIME NULL,
  CODGRUPO INT UNSIGNED NULL,
  PRIMARY KEY(IDEUSUARIO),
  UNIQUE INDEX UKUSUARIOSISTEMA_DESLOGIN(DESLOGIN),
  FOREIGN KEY(CODGRUPO)
    REFERENCES GRUPO(CODGRUPO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS USUARIOSISTEMA_FKIndex1 ON USUARIOSISTEMA(CODGRUPO);

CREATE TABLE IF NOT EXISTS PERFILSISTEMA (
  IDEPERFILSISTEMA INT UNSIGNED NOT NULL,
  NOMPERFILSISTEMA VARCHAR(255) NULL,
  DESPERFILSISTEMA VARCHAR(255) NULL,
  IDESISTEMA INT UNSIGNED NULL,
  IDEPERFILAGRUPADOR INT UNSIGNED NULL,
  NOMOBJETOCONTROLADO VARCHAR(255) NULL,
  PRIMARY KEY(IDEPERFILSISTEMA),
  FOREIGN KEY(IDEPERFILAGRUPADOR)
    REFERENCES PERFILSISTEMA(IDEPERFILSISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDESISTEMA)
    REFERENCES SISTEMA(IDESISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS PERFILSISTEMA_FKIndex1 ON PERFILSISTEMA(IDEPERFILAGRUPADOR);
CREATE INDEX IF NOT EXISTS PERFILSISTEMA_FKIndex2 ON PERFILSISTEMA(IDESISTEMA);

CREATE TABLE IF NOT EXISTS FILTROCONSULTA (
  IDEFILTROCONSULTA INT UNSIGNED NOT NULL,
  IDETIPOFILTROCONSULTA INT UNSIGNED NOT NULL,
  NOMFILTROCONSULTA VARCHAR(255) NOT NULL,
  PRIMARY KEY(IDEFILTROCONSULTA),
  UNIQUE INDEX PERFILCONSULTA_Unique1(NOMFILTROCONSULTA),
  FOREIGN KEY(IDETIPOFILTROCONSULTA)
    REFERENCES TIPOFILTROCONSULTA(IDETIPOFILTROCONSULTA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS FILTROCONSULTA_FKIndex1 ON FILTROCONSULTA(IDETIPOFILTROCONSULTA);

CREATE TABLE IF NOT EXISTS FUNCIONALIDADESISTEMA (
  IDEFUNCIONALIDADESISTEMA INT UNSIGNED NOT NULL,
  NOMFUNCIONALIDADESISTEMA VARCHAR(255) NULL,
  DESFUNCIONALIDADESISTEMA VARCHAR(255) NULL,
  NOMOBJETOCONTROLADO VARCHAR(255) NULL,
  IDEFUNCIONALIDADEAGRUPADORA INT UNSIGNED NULL,
  IDESISTEMA INT UNSIGNED NULL,
  PRIMARY KEY(IDEFUNCIONALIDADESISTEMA),
  UNIQUE INDEX UKFUNSIS_IDENOMCON(IDESISTEMA, NOMOBJETOCONTROLADO),
  FOREIGN KEY(IDEFUNCIONALIDADEAGRUPADORA)
    REFERENCES FUNCIONALIDADESISTEMA(IDEFUNCIONALIDADESISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDESISTEMA)
    REFERENCES SISTEMA(IDESISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS FUNCIONALIDADESISTEMA_FKIndex1 ON FUNCIONALIDADESISTEMA(IDEFUNCIONALIDADEAGRUPADORA);
CREATE INDEX IF NOT EXISTS FUNCIONALIDADESISTEMA_FKIndex2 ON FUNCIONALIDADESISTEMA(IDESISTEMA);

CREATE TABLE IF NOT EXISTS SIGESPPESSOAL (
  IDEPESSOAL INT UNSIGNED NOT NULL,
  NOMPESSOAL VARCHAR(255) NULL,
  DATNASCIMENTO DATETIME NULL,
  INDSEXO CHAR(1) NULL DEFAULT 'M',
  NUMCPF CHAR(11) NULL,
  NUMRG VARCHAR(255) NULL,
  CODGRUPO INT UNSIGNED NULL,
  PRIMARY KEY(IDEPESSOAL),
  FOREIGN KEY(CODGRUPO)
    REFERENCES GRUPO(CODGRUPO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEPESSOAL)
    REFERENCES USUARIOSISTEMA(IDEUSUARIO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS SIGESPPESSOAL_NOMPESSOAL ON SIGESPPESSOAL(NOMPESSOAL);
CREATE INDEX IF NOT EXISTS SIGESPPESSOAL_FKIndex1 ON SIGESPPESSOAL(CODGRUPO);
CREATE INDEX IF NOT EXISTS SIGESPPESSOAL_FKIndex2 ON SIGESPPESSOAL(IDEPESSOAL);

CREATE TABLE IF NOT EXISTS USUARIOPERFILSISTEMA (
  IDEPERFILSISTEMA INT UNSIGNED NOT NULL,
  IDEUSUARIO INT UNSIGNED NOT NULL,
  INDGESTORPERFIL CHAR(1) NULL,
  PRIMARY KEY(IDEPERFILSISTEMA, IDEUSUARIO),
  FOREIGN KEY(IDEPERFILSISTEMA)
    REFERENCES PERFILSISTEMA(IDEPERFILSISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEUSUARIO)
    REFERENCES USUARIOSISTEMA(IDEUSUARIO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS USUARIOPERFILSISTEMA_FKIndex1 ON USUARIOPERFILSISTEMA(IDEPERFILSISTEMA);
CREATE INDEX IF NOT EXISTS USUARIOPERFILSISTEMA_FKIndex2 ON USUARIOPERFILSISTEMA(IDEUSUARIO);

CREATE TABLE IF NOT EXISTS TALENTO (
  IDETALENTO INT UNSIGNED NOT NULL,
  IDEPESSOAL INT UNSIGNED NOT NULL,
  IDECATEGORIA INT UNSIGNED NULL,
  DATLANCAMENTO DATETIME NULL,
  PRIMARY KEY(IDETALENTO),
  FOREIGN KEY(IDECATEGORIA)
    REFERENCES CATEGORIATALENTO(IDECATEGORIA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEPESSOAL)
    REFERENCES SIGESPPESSOAL(IDEPESSOAL)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS TALENTO_FKIndex1 ON TALENTO(IDECATEGORIA);
CREATE INDEX IF NOT EXISTS TALENTO_FKIndex2 ON TALENTO(IDEPESSOAL);

CREATE TABLE IF NOT EXISTS PERFILSISTEMAGRUPO (
  IDEPERFILSISTEMA INT UNSIGNED NOT NULL,
  CODGRUPO INT UNSIGNED NOT NULL,
  PRIMARY KEY(IDEPERFILSISTEMA, CODGRUPO),
  FOREIGN KEY(CODGRUPO)
    REFERENCES GRUPO(CODGRUPO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEPERFILSISTEMA)
    REFERENCES PERFILSISTEMA(IDEPERFILSISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS PERFILSISTEMAGRUPO_FKIndex1 ON PERFILSISTEMAGRUPO(CODGRUPO);
CREATE INDEX IF NOT EXISTS PERFILSISTEMAGRUPO_FKIndex2 ON PERFILSISTEMAGRUPO(IDEPERFILSISTEMA);

CREATE TABLE IF NOT EXISTS FILTROCONSULTAGRUPO (
  IDEFILTROCONSULTA INT UNSIGNED NOT NULL,
  CODGRUPO INT UNSIGNED NOT NULL,
  PRIMARY KEY(IDEFILTROCONSULTA, CODGRUPO),
  FOREIGN KEY(CODGRUPO)
    REFERENCES GRUPO(CODGRUPO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEFILTROCONSULTA)
    REFERENCES FILTROCONSULTA(IDEFILTROCONSULTA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS FILTROCONSULTAGRUPO_FKIndex1 ON FILTROCONSULTAGRUPO(CODGRUPO);
CREATE INDEX IF NOT EXISTS FILTROCONSULTAGRUPO_FKIndex2 ON FILTROCONSULTAGRUPO(IDEFILTROCONSULTA);

CREATE TABLE IF NOT EXISTS FILTROCONSULTAUSUARIO (
  IDEFILTROCONSULTA INT UNSIGNED NOT NULL,
  IDEUSUARIO INT UNSIGNED NOT NULL,
  PRIMARY KEY(IDEFILTROCONSULTA, IDEUSUARIO),
  FOREIGN KEY(IDEFILTROCONSULTA)
    REFERENCES FILTROCONSULTA(IDEFILTROCONSULTA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEUSUARIO)
    REFERENCES USUARIOSISTEMA(IDEUSUARIO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS FILTROCONSULTAUSUARIO_FKIndex1 ON FILTROCONSULTAUSUARIO(IDEUSUARIO);
CREATE INDEX IF NOT EXISTS FILTROCONSULTAUSUARIO_FKIndex2 ON FILTROCONSULTAUSUARIO(IDEFILTROCONSULTA);

CREATE TABLE IF NOT EXISTS ATRIBUTOTALENTO (
  IDEATRIBUTO INT UNSIGNED NOT NULL,
  IDEATRIBUTOPAI INT UNSIGNED NULL,
  NOMATRIBUTO VARCHAR(255) NULL,
  IDETIPOHTML INT UNSIGNED NULL,
  DESMASCARA VARCHAR(255) NULL,
  IDETABELAAPOIOMM INT UNSIGNED NULL,
  INDPESQUISA CHAR(1) NULL DEFAULT 'N',
  DESPESQUISA VARCHAR(255) NULL,
  INDTIPODADO CHAR(1) NULL,
  PRIMARY KEY(IDEATRIBUTO),
  UNIQUE INDEX UKATRIBUTOTALENTO_NOMATRIBUTO(NOMATRIBUTO),
  UNIQUE INDEX ATRIBUTOTALENTO_FKIndex3(IDETABELAAPOIOMM),
  FOREIGN KEY(IDETIPOHTML)
    REFERENCES TIPOHTML(IDETIPOHTML)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEATRIBUTOPAI)
    REFERENCES ATRIBUTOTALENTO(IDEATRIBUTO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDETABELAAPOIOMM)
    REFERENCES TABELAAPOIOMM(IDETABELAAPOIOMM)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS ATRIBUTOTALENTO_FKIndex1 ON ATRIBUTOTALENTO(IDETIPOHTML);
CREATE INDEX IF NOT EXISTS ATRIBUTOTALENTO_FKIndex2 ON ATRIBUTOTALENTO(IDEATRIBUTOPAI);

CREATE TABLE IF NOT EXISTS CATEGORIAATRIBUTOTALENTO (
  IDECATEGORIAATRIBUTO INT UNSIGNED NOT NULL,
  SEQPREENCHIMENTO TINYINT UNSIGNED NULL,
  IDECATEGORIA INT UNSIGNED NULL,
  IDEATRIBUTO INT UNSIGNED NULL,
  INDOBRIGATORIEDADE CHAR(1) NULL,
  INDFORMACAODESCRICAO CHAR(1) NULL,
  NOMAPELIDO VARCHAR(255) NULL,
  DESDICAPREENCHIMENTO VARCHAR(255) NULL,
  PRIMARY KEY(IDECATEGORIAATRIBUTO),
  FOREIGN KEY(IDEATRIBUTO)
    REFERENCES ATRIBUTOTALENTO(IDEATRIBUTO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDECATEGORIA)
    REFERENCES CATEGORIATALENTO(IDECATEGORIA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS CATEGORIAATRIBUTOTALENTO_FKIndex1 ON CATEGORIAATRIBUTOTALENTO(IDEATRIBUTO);
CREATE INDEX IF NOT EXISTS CATEGORIAATRIBUTOTALENTO_FKIndex2 ON CATEGORIAATRIBUTOTALENTO(IDECATEGORIA);

CREATE TABLE IF NOT EXISTS GRUPOCRITERIOCONSULTA (
  IDEFILTROCONSULTA INT UNSIGNED NOT NULL,
  CODGRUPO INT UNSIGNED NOT NULL,
  PRIMARY KEY(IDEFILTROCONSULTA, CODGRUPO),
  FOREIGN KEY(CODGRUPO)
    REFERENCES GRUPO(CODGRUPO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEFILTROCONSULTA)
    REFERENCES FILTROCONSULTA(IDEFILTROCONSULTA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS GRUPOCRITERIOCONSULTA_FKIndex1 ON GRUPOCRITERIOCONSULTA(CODGRUPO);
CREATE INDEX IF NOT EXISTS GRUPOCRITERIOCONSULTA_FKIndex2 ON GRUPOCRITERIOCONSULTA(IDEFILTROCONSULTA);

CREATE TABLE IF NOT EXISTS PERFILFUNCIONALIDADESISTEMA (
  IDEPERFILSISTEMA INT UNSIGNED NOT NULL,
  IDEFUNCIONALIDADESISTEMA INT UNSIGNED NOT NULL,
  PRIMARY KEY(IDEPERFILSISTEMA, IDEFUNCIONALIDADESISTEMA),
  FOREIGN KEY(IDEFUNCIONALIDADESISTEMA)
    REFERENCES FUNCIONALIDADESISTEMA(IDEFUNCIONALIDADESISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEPERFILSISTEMA)
    REFERENCES PERFILSISTEMA(IDEPERFILSISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS PERFILFUNCIONALIDADESISTEMA_FKIndex1 ON PERFILFUNCIONALIDADESISTEMA(IDEFUNCIONALIDADESISTEMA);
CREATE INDEX IF NOT EXISTS PERFILFUNCIONALIDADESISTEMA_FKIndex2 ON PERFILFUNCIONALIDADESISTEMA(IDEPERFILSISTEMA);

CREATE TABLE IF NOT EXISTS GESTORSISTEMA (
  IDESISTEMA INT UNSIGNED NOT NULL,
  IDEUSUARIO INT UNSIGNED NOT NULL,
  INDADMINISTRADOR CHAR(1) NULL,
  PRIMARY KEY(IDESISTEMA, IDEUSUARIO),
  FOREIGN KEY(IDESISTEMA)
    REFERENCES SISTEMA(IDESISTEMA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEUSUARIO)
    REFERENCES USUARIOSISTEMA(IDEUSUARIO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS GESTORSISTEMA_FKIndex1 ON GESTORSISTEMA(IDESISTEMA);
CREATE INDEX IF NOT EXISTS GESTORSISTEMA_FKIndex2 ON GESTORSISTEMA(IDEUSUARIO);

CREATE TABLE IF NOT EXISTS GRUPOCATEGORIATALENTO (
  CODGRUPO INT UNSIGNED NOT NULL,
  IDECATEGORIA INT UNSIGNED NOT NULL,
  PRIMARY KEY(CODGRUPO, IDECATEGORIA),
  FOREIGN KEY(IDECATEGORIA)
    REFERENCES CATEGORIATALENTO(IDECATEGORIA)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(CODGRUPO)
    REFERENCES GRUPO(CODGRUPO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS GRUPOCATEGORIATALENTO_FKIndex1 ON GRUPOCATEGORIATALENTO(IDECATEGORIA);
CREATE INDEX IF NOT EXISTS GRUPOCATEGORIATALENTO_FKIndex2 ON GRUPOCATEGORIATALENTO(CODGRUPO);

CREATE TABLE IF NOT EXISTS ATRIBUTOTALENTOOPCAO (
  IDEATRIBUTOOPCAO INT UNSIGNED NOT NULL,
  IDEATRIBUTO INT UNSIGNED NULL,
  IDEATRIBUTOOPCAOPAI INT UNSIGNED NULL,
  DESATRIBUTOOPCAO VARCHAR(255) NULL,
  IDETABELAAPOIOORIGEM INT UNSIGNED NULL,
  PRIMARY KEY(IDEATRIBUTOOPCAO),
  FOREIGN KEY(IDEATRIBUTO)
    REFERENCES ATRIBUTOTALENTO(IDEATRIBUTO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDEATRIBUTOOPCAOPAI)
    REFERENCES ATRIBUTOTALENTOOPCAO(IDEATRIBUTOOPCAO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS ATRIBUTOTALENTOOPCAO_FKIndex1 ON ATRIBUTOTALENTOOPCAO(IDEATRIBUTO);
CREATE INDEX IF NOT EXISTS ATRIBUTOTALENTOOPCAO_FKIndex2 ON ATRIBUTOTALENTOOPCAO(IDEATRIBUTOOPCAOPAI);

CREATE TABLE IF NOT EXISTS SIGESPPESSOALLOTACAO (
  IDEPESSOALLOTACAO INT UNSIGNED NOT NULL,
  IDEPESSOAL INT UNSIGNED NOT NULL,
  CODLOTACAO INT UNSIGNED NULL,
  DESLOTACAO VARCHAR(255) NULL,
  DATINICIO DATETIME NULL,
  DATTERMINO DATETIME NULL,
  PRIMARY KEY(IDEPESSOALLOTACAO),
  FOREIGN KEY(IDEPESSOAL)
    REFERENCES SIGESPPESSOAL(IDEPESSOAL)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS SIGESPPESSOALLOTACAO_FKIndex1 ON SIGESPPESSOALLOTACAO(IDEPESSOAL);

CREATE TABLE IF NOT EXISTS ATRIBUTOTALENTOVALORADO (
  IDEATRIBUTOVALORADO INT UNSIGNED NOT NULL,
  IDETALENTO INT UNSIGNED NOT NULL,
  IDECATEGORIAATRIBUTO INT UNSIGNED NULL,
  DESVALORACAO TEXT NULL,
  IDEATRIBUTOOPCAO INT UNSIGNED NULL,
  PRIMARY KEY(IDEATRIBUTOVALORADO),
  FOREIGN KEY(IDEATRIBUTOOPCAO)
    REFERENCES ATRIBUTOTALENTOOPCAO(IDEATRIBUTOOPCAO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDECATEGORIAATRIBUTO)
    REFERENCES CATEGORIAATRIBUTOTALENTO(IDECATEGORIAATRIBUTO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(IDETALENTO)
    REFERENCES TALENTO(IDETALENTO)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);
CREATE INDEX IF NOT EXISTS ATRIBUTOTALENTOVALORADO_FKIndex1 ON ATRIBUTOTALENTOVALORADO(IDEATRIBUTOOPCAO);
CREATE INDEX IF NOT EXISTS ATRIBUTOTALENTOVALORADO_FKIndex2 ON ATRIBUTOTALENTOVALORADO(IDECATEGORIAATRIBUTO);
CREATE INDEX IF NOT EXISTS ATRIBUTOTALENTOVALORADO_FKIndex3 ON ATRIBUTOTALENTOVALORADO(IDETALENTO);

