package br.gov.camara.testes.base;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import br.gov.camara.biblioteca.util.CaminhoAbsoluto;
import br.gov.camara.exception.CDException;
import junit.framework.TestCase;

public abstract class BaseCDTestCase extends TestCase
{
    List<String> caminhos = new ArrayList<String>();

    protected BaseCDTestCase()
    {
        configurarAmbiente();
        adicionarCaminhosAoClassLoader();
    }

    protected final void setUp() throws Exception
    {
        super.setUp();
        configurarAmbiente();
        executarAntesDeChamarMetodo();
    }

    protected final void tearDown() throws Exception
    {
        super.tearDown();
        executarDepoisDeChamarMetodo();
    }

    /**
     * Mtodo que  chamado antes de executar um mtodo
     * da classe de teste.
     * Substitui o setUp(), pois o nome deste  mais intuitivo. 
     */
    protected void executarAntesDeChamarMetodo() throws CDException
    {}

    /**
    * Mtodo que  chamado depois de executar um mtodo
    * da classe de teste.
    * Substitui o tearDown(), pois o nome deste  mais intuitivo.
    */
    protected void executarDepoisDeChamarMetodo() throws CDException
    {}

    private void adicionarCaminhosAoClassLoader()
    {
        // Adicionar caminhos no classpath
        try
        {
            List<String> caminhosParaAdicionar = obterCaminhosParaAcrescentarNoClassLoader();
            if (caminhosParaAdicionar.isEmpty())
            {
                return;
            }

            List<URL> urlsParaAdicionar = new ArrayList<URL>();
            for (String diretorio : caminhosParaAdicionar)
            {
                File diretorioParaAdicionar = CaminhoAbsoluto.obterAPartirDoClasses(diretorio);
                if (diretorioParaAdicionar == null)
                {
                    throw new RuntimeException("O caminho '" + diretorio + "' no foi encontrado.");
                }
                else
                {
                    URL url = new URL("file", "", diretorioParaAdicionar.getAbsolutePath().replaceAll("\\\\", "/") + "/");
                    urlsParaAdicionar.add(url);
                }
            }

            if (!urlsParaAdicionar.isEmpty())
            {
                ClassLoader classloader = new URLClassLoader(urlsParaAdicionar.toArray(new URL[0]), Thread.currentThread().getContextClassLoader());
                Thread.currentThread().setContextClassLoader(classloader);
            }
        }
        catch (Exception mue)
        {
            throw new RuntimeException("Erro ao inicializar o hibernate", mue);
        }
    }

    /**
     * @return Uma lista imutvel com os caminhos adicionados
     */
    protected List<String> obterCaminhosParaAcrescentarNoClassLoader()
    {
        return Collections.unmodifiableList(caminhos);
    }

    protected void adicionarCaminhoParaAcrescentarNoClassLoader(String caminho)
    {
        caminhos.add(caminho);
    }

    protected void configurarAmbiente()
    {}
}
